/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_set.c
 *   @brief This is the Module file for ChanServ SET Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ SET command.
 *
 *   The ChanServ SET Command allows the user to set various SET-Options<br/>
 *   for target Channel. (Controls FOUNDER/SUCCESSOR etc.)
 *
 *   Syntax is: /msg ChanServ SET #channel OPTION ...
 *
 *<table border="0">
 *    <tr><td> FOUNDER			</td><td> Syntax is: /msg ChanServ SET #channel FOUNDER			</td><td> < nick >							</td></tr>
 *    <tr><td> SUCSESSOR		</td><td> Syntax is: /msg ChanServ SET #channel SUCSESSOR		</td><td> < nick >							</td></tr>
 *    <tr><td> MLOCK			</td><td> Syntax is: /msg ChanServ SET #channel MLOCK			</td><td> < mlock (i.e: +ntlk-fc 10 rofl) >	</td></tr>
 *    <tr><td> DESCRIPTION		</td><td> Syntax is: /msg ChanServ SET #channel DESCRIPTION		</td><td> < new-description >				</td></tr>
 *    <tr><td> URL				</td><td> Syntax is: /msg ChanServ SET #channel URL				</td><td> < new-Url >						</td></tr>
 *    <tr><td> EMAIL			</td><td> Syntax is: /msg ChanServ SET #channel EMAIL			</td><td> < new-Email >						</td></tr>
 *    <tr><td> ENTRYMSG			</td><td> Syntax is: /msg ChanServ SET #channel ENTRYMSG		</td><td> < new-EntryMsg >					</td></tr>
 *    <tr><td> BANTYPE			</td><td> Syntax is: /msg ChanServ SET #channel BANTYPE			</td><td> < new-Bantype(0-3) >				</td></tr>
 *    <tr><td> KEEPTOPIC		</td><td> Syntax is: /msg ChanServ SET #channel KEEPTOPIC		</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> OPNOTICE			</td><td> Syntax is: /msg ChanServ SET #channel OPNOTICE		</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> PEACE			</td><td> Syntax is: /msg ChanServ SET #channel PEACE			</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> PRIVATE			</td><td> Syntax is: /msg ChanServ SET #channel PRIVATE			</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> RESTRICTED		</td><td> Syntax is: /msg ChanServ SET #channel RESTRICTED		</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> SECURE			</td><td> Syntax is: /msg ChanServ SET #channel SECURE			</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> SECUREOPS		</td><td> Syntax is: /msg ChanServ SET #channel SECUREOPS		</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> SECUREFOUNDER	</td><td> Syntax is: /msg ChanServ SET #channel SECUREFOUNDER	</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> SIGNKICK			</td><td> Syntax is: /msg ChanServ SET #channel SIGNKICK		</td><td> { ON | LEVEL | OFF }				</td></tr>
 *    <tr><td> TOPICLOCK		</td><td> Syntax is: /msg ChanServ SET #channel TOPICLOCK		</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> XOP				</td><td> Syntax is: /msg ChanServ SET #channel XOP				</td><td> { ON | OFF }						</td></tr>
 *    <tr><td> NOEXPIRE			</td><td> Syntax is: /msg ChanServ SET #channel NOEXPIRE		</td><td> { ON | OFF }						</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"	//orion_WRCtranslate_flood_mode_check();

#define SERVICE "cs"
#define MODUL "set"
#define MODUL_FILE "cs_set"
#define MODUL_VERSION "1.0"

/*(02:45) -ChanServ- FOUNDER Set the founder of a channel
(02:45) -ChanServ- SUCCESSOR Set the successor for a channel
(02:45) -ChanServ- DESC Set the channel description
(02:45) -ChanServ- URL Associate a URL with the channel
(02:45) -ChanServ- EMAIL Associate an E-mail address with the channel
(02:45) -ChanServ- ENTRYMSG Set a message to be sent to users when they
(02:45) -ChanServ- enter the channel
(02:45) -ChanServ- BANTYPE Set how Services make bans on the channel
(02:45) -ChanServ- MLOCK Lock channel modes on or off
(02:45) -ChanServ- KEEPTOPIC Retain topic when channel is not in use
(02:45) -ChanServ- OPNOTICE Send a notice when OP/DEOP commands are used
(02:45) -ChanServ- PEACE Regulate the use of critical commands
(02:45) -ChanServ- PRIVATE Hide channel from LIST command
(02:45) -ChanServ- RESTRICTED Restrict access to the channel
(02:45) -ChanServ- SECURE Activate ChanServ security features
(02:45) -ChanServ- SECUREOPS Stricter control of chanop status
(02:45) -ChanServ- SECUREFOUNDER Stricter control of channel founder status
(02:45) -ChanServ- SIGNKICK Sign kicks that are done with KICK command
(02:45) -ChanServ- TOPICLOCK Topic can only be changed with TOPIC
(02:45) -ChanServ- XOP Toggle the user privilege system*/

MOD_BOOL cs_set_founder(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_successor(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_description(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_mlock(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
/* There are many more but we skip here */

//--------------------------------------------------------------------------------------

MOD_BOOL cs_set_founder_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_successor_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_description_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_mlock_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
/* There are many more but we skip here */

MOD_BOOL cs_set_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_set_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"set * option[1]","set option[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"set",is_services_admin,is_services_user); /* just to add a displaylevel for /msg cs help
															  this should always be the lowest used Module level! */
															  
	addCommand(thread,&Module,"set founder",cs_set_founder,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"set successor",cs_set_successor,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"set desc",cs_set_description,is_services_admin,is_services_user,6,0);
	addCommand(thread,&Module,"set mlock",cs_set_mlock,is_services_admin,is_services_user,4,0);
	
	addCommand(thread,&Module,"help set founder",cs_set_founder_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set successor",cs_set_successor_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set desc",cs_set_description_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set mlock",cs_set_mlock_help,is_services_admin,is_services_user,5,5);
	
	addCommand(thread,&Module,"help set",cs_set_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_set_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Founder function for the ChanServ SET Command.
 * 
 * The ChanServ SET Founder Command allows the user to change the FounderNick<br/>
 *   to a new FounderNick
 *  
 * Syntax is: /msg ChanServ SET #channel FOUNDER < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_founder(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */

	/* Initialise Variables to (NULL) */

	/*
	 * Set Founder Command
	 */
	/* Proof if the command executors GLOBAL level is high enough to use this command */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set founder")){
		/* Updates the Threads Channel Management */
		/* Selects information about target channel (param[4]) and
		 * the channelaccess level for the given user (param[6]) in that channel */
		UpdateChannelInformation(thread,param[4],param[6]);
		
		/* If channel found, but suspended: */
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT))
			send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);

		/* Channel not suspended, and command executor has an alias AND the channel is registered for ChanServ */
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){

			/* If the command executor is Channel founder */
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) ||   /* OR */
			
			    /* The command executors channelaccess level is >= the Channelfounder-level
				 * (means we are a founder in the multifounder system - BUT not the one who registered the channel)
				 * AND
				 * Securefounder in the channelsettings ISNOT set */
				 (stoi(thread->c.chan.access.ownlevel)>=ACCESS_FOUNDER && ~thread->c.chan.settings&CS_SECUREFOUNDER_BIT) || /* OR */
				 
				 /* The command executors GLOBAL services access level is greater
				  * than the one in orion_init(); (specified for that command) */
				 thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set founder"))
			{
			
				/* Now we proof if our new founder is really registered */
				if(thread->c.aliases.authnick[0]!='\0'){

					/* Now we proof if the nick we expect as our new Channelfounder IS NOT forbidden */
    				if(!thread->c.aliases.forbid){


    				} else send_lang(thread,language,howtosend,snick,nick,"NICK_SUSPENDED_OR_FORBIDDEN",thread->c.aliases.nick);

				}else send_lang(thread,language,howtosend,snick,nick,"NO_AUTHNICK");
				
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
			
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Successor function for the ChanServ SET Command.
 * 
 * The ChanServ SET Successor Command allows the user to change the SuccessorNick<br/>
 *   to a new SuccessorNick
 *  
 * Syntax is: /msg ChanServ SET #channel SUCSESSOR < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_successor(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */

	/* Initialise Variables to (NULL) */

	/*
	 * Set Successor Command
	 */

	/* Proof if the command executors GLOBAL level is high enough to use this command */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set successor")){
		/* Updates the Threads Channel Management */
		/* Selects information about target channel (param[4]) and
		 * the channelaccess level for the given user (param[6]) in that channel */
		UpdateChannelInformation(thread,param[4],param[6]);
		
		/* If the channel is known by the Services and Suspended */
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT))
			send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
			
		/* If the command's executor is registered and target channel is registered for ChanServ */
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){

			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) ||
			
			    /* proof if the command's executor is Successor */
				 streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) ||
				 
				 /* if command executors channelaccess level is greater or equal to the founder access level
				  * AND
				  * Securefounder for that channel is off */
				 (stoi(thread->c.chan.access.ownlevel)>=ACCESS_FOUNDER && ~thread->c.chan.settings&CS_SECUREFOUNDER_BIT) ||
				 thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set successor"))
			{

				/* proof if our new founder is registered */
				if(thread->c.aliases.authnick[0]!='\0'){

					/* make sure the one we expect as a new founder IS NOT forbidden */
    				if(!thread->c.aliases.forbid){

    				}else send_lang(thread,language,howtosend,snick,nick,"NICK_SUSPENDED_OR_FORBIDDEN");
    				
                }else send_lang(thread,language,howtosend,snick,nick,"NO_AUTHNICK");
                
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
			
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Mlock function for the ChanServ SET Command.
 * 
 * The ChanServ SET Mlock Command allows the user to change the Mlock<br/>
 *   for target Channel
 *  
 * Syntax is: /msg ChanServ SET #channel MLOCK < mlock (i.e: +ntlk-fc 10 rofl) >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_mlock(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	char CBot[MAX_NICK_LEN];
	char modes[35];
	char positive[35];
	char negative[35];

	/* Initialise Variables to (NULL) */
	bzero(&CBot,sizeof(CBot));
	bzero(&modes,sizeof(modes));
	bzero(&positive,sizeof(positive));
	bzero(&negative,sizeof(negative));

	/*
	 * Set Mlock Command
	 */
	 //:shock PRIVMSG ChanServ :set #channel mlock +O
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set mlock")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){

/* I think you already know this stuff */
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) ||
				 streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) ||
				 stoi(thread->c.chan.access.ownlevel)>=ACCESS_FOUNDER ||
				 thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set mlock"))
			{

			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
			
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Description function for the ChanServ SET Command.
 * 
 * The ChanServ SET Description Command allows the user to change<br/>
 *   the Channel's Description
 *  
 * Syntax is: /msg ChanServ SET #channel DESCRIPTION < new-description >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_description(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Set Description Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set desc")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) ||
				streqnc(thread->sql.aliases.authnick,thread->c.chan.successor) ||
				stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set ||
				thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set desc"))
			{
				/* Update Memory, for caching */
				dynncpy(thread->c.chan.description,_dstrjoin(' ',param,6),sizeof(thread->c.chan.description));
				
				/* Update SQL as Live Update */
				domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`description`='%s' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),mysql_quote(thread,thread->c.chan.description),mysql_quote(thread,thread->c.chan.name));

				send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");

			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
			
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ SET Command (/msg ChanServ HELP SET)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set")) {
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET",snick,snick);
		
		if(thread->sql.access.rights>ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set"))
			send_lang(thread,language,howtosend,snick,nick,"CS_SERVADMIN_HELP_SET");
			
	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Founder Help function for the ChanServ SET Command (/msg ChanServ HELP SET FOUNDER)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET FOUNDER,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET FOUNDER
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_founder_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Founder Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set founder")) {
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_FOUNDER");

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Successor Help function for the ChanServ SET Command (/msg ChanServ HELP SET SUCCESSOR)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET SUCCESSOR,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET SUCCESSOR
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_successor_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Successor Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set successor")) {
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_SUCCESSOR",30);

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Mlock Help function for the ChanServ SET Command (/msg ChanServ HELP SET MLOCK)
 *
 * When the user tries to get help for this command, using /msg ChanServ HELP SET MLOCK,<br/>
 * this function is executed.
 *
 * Syntax: /msg ChanServ HELP SET MLOCK
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_mlock_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Mlock Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set mlock")) {
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_MLOCK",snick);

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS SET Descripton Help function for the ChanServ SET Command (/msg ChanServ HELP SET DESCRIPTION)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SET DESCRIPTION,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SET DESCRIPTION
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_description_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Descripton Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set desc")) {
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_SET_DESC");

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - SET part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SET command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_set_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Set (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_SET");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
