/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 370                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-26 17:17:04 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file datatypes.h
 *   @brief This is the datatypes header.
 *
 *   It contains all datatypes (used all over the program).
 */
/*  ************************************************************************************************  */

#ifndef STRUCTS_H
#define STRUCTS_H

#define ENABLE_THREAD_CACHING

#ifdef _WIN32
  #ifdef MODULE_COMPILE
    #define DLL __declspec( dllimport )
  #else
    #define DLL __declspec( dllexport )
  #endif
#else
  #define DLL extern
#endif

/* Linux defs:
 *   _REENTRANT to get thread-safe libs
 *   _POSIX_SOURCE to get POSIX semantics
 *   _P is a hack for LinuxThreads -- on my box,
 *      pthread.h includes sched.h. My sched.h
 *      (incorrectly) declares prototypes with
 *      _P instead of __P (which is what everything
 *      else uses... Maybe it's just me.
 */
#ifdef __linux__
  #ifndef _REENTRANT
    #define _REENTRANT
  #endif
  
  #ifndef _P
    #define _P __P
  #endif
#endif

#define E extern

#if INTTYPE_WORKAROUND
# undef int16
# undef int32
#endif

#include "sysconf.h"

#include <stdio.h>
#include <signal.h>					/* signal Handler */
#include <time.h>
#include <errno.h>
#include <limits.h>
#include <stdarg.h>
#include <ctype.h>

/* Some SUN fixs */
#ifdef __sun
/* Solaris specific code, types that do not exist in Solaris'
 * sys/types.h
 **/
  #undef u_int8_t
  #undef u_int16_t
  #undef u_int32_t
  #undef u_int_64_t
  #define u_int8_t uint8_t
  #define u_int16_t uint16_t
  #define u_int32_t uint32_t
  #define u_int64_t uint64_t

  #ifndef INADDR_NONE
    #define INADDR_NONE (-1)
  #endif
#endif

#ifdef HAVE_SYS_TYPES_H
  #include <sys/types.h>
#endif

#ifdef HAVE_SYS_STAT_H
  #include <sys/stat.h>	/* for umask() on some systems */
#endif

#ifdef HAVE_STDLIB_H
  #include <stdlib.h>
#endif

#ifdef HAVE_STRING_H
  #include <string.h>
#endif

#ifdef HAVE_MALLOC_H
  #include <malloc.h>
#endif

#ifdef USE_THREADS
  #include <pthread.h>				/* Thread Library (Maybe should be passed with Win32 conform Paths) */
#endif

#ifndef _WIN32
  #include <netdb.h>
  #include <netinet/in.h>
  #include <arpa/inet.h>
  #include <sys/socket.h>
  #include <sys/time.h>
  #include <sys/un.h>
  #include <grp.h>
  #include <dirent.h>
  
  #ifdef HAVE_UNISTD_H
    #include <unistd.h>
  #endif
  
  #ifdef HAVE_BACKTRACE
    #include <execinfo.h>
  #endif
#else
  //#include <winsock.h>
  #include <winsock2.h>
  //#include <afxwin.h>
  #include <windows.h>
  #include <sys/timeb.h>
  #include <direct.h>
  #include <io.h>
  
  	#define SIGHUP     1
	/* 2 is used for SIGINT on windows */
	#define SIGQUIT    3
	/* 4 is used for SIGILL on windows */
	#define SIGTRAP    5
	#define SIGIOT     6
	#define SIGBUS     7
	/* 8 is used for SIGFPE on windows */
	#define SIGKILL    9
	#define SIGUSR1    10
	/* 11 is used for SIGSEGV on windows */
	#define SIGUSR2    12
	#define SIGPIPE    13
	#define SIGALRM    14
	/* 15 is used for SIGTERM on windows */
	#define SIGSTKFLT  16
	#define SIGCHLD    17
	#define SIGCONT    18
	#define SIGSTOP    19
	#define SIGTSTP    20
	/* 21 is used for SIGBREAK on windows */
	/* 22 is used for SIGABRT on windows */
	#define SIGTTIN    23
	#define SIGTTOU    24
	#define SIGURG     25
	#define SIGXCPU    26
	#define SIGXFSZ    27
	#define SIGVTALRM  28
	#define SIGPROF    29
	#define SIGWINCH   30
#define SIGIO      31


#endif

#ifdef USE_MYSQL
  #ifdef MYSQL_HEADER_PREFIX
    #include <mysql/mysql.h>
  #else
    #include <mysql.h>
  #endif
#endif 

/*************************************************************************/
/* 
 * Typedef's which aren't structs
 */
 
//Trusts
typedef enum { DENY, TRUST } Trust_art;

//Servers
typedef enum {
	SSYNC_UNKNOWN       = 0,	/* We can't get the sync state   */
	SSYNC_IN_PROGRESS   = 1,	/* Sync is currently in progress */
	SSYNC_DONE          = 2		/* We're in sync                 */
} SyncState;

//Modules
typedef enum { 
	MOD_ERR_OK,
	MOD_ERR_MEMORY,
	MOD_ERR_PARAMS,
	MOD_ERR_EXISTS,
	MOD_ERR_NOEXIST,
	MOD_ERR_CMD_EXIST,
	MOD_ERR_CMD_NOEXIST,
	MOD_ERR_OPTIONS,
	MOD_ERR_NOTLOADED,
	MOD_ERR_NOTUNLOADED,
	MOD_ERR_NOUNIQUE,
	MOD_ERR_CANCELLED,
	MOD_ERR_NO_FUNCTARGET,
	MOD_ERR_FILE_IO,
	MOD_ERR_CALLBACK,
	MOD_ERR_SETTING_EXISTS,
	MOD_ERR_NOTUSEDINIT,
	MOD_ERR_END
} MOD_ERROR;

typedef enum {
	MOD_REPLY_NONE,
	MOD_REPLY_LESS_PARAMS,
	MOD_REPLY_TOO_MANY_PARAMS,
	MOD_REPLY_OPTIONS_NOT_LOGGED_IN,
	MOD_REPLY_OPTIONS_DEFCON_TOO_HIGH,
	MOD_REPLY_OPTIONS_READONLY,
	MOD_REPLY_OPTIONS_NOT_AN_IRCOP,
	MOD_REPLY_OPTIONS_NO_WEBACCESS_GRANTED
} MOD_REPLY;

/* These are all available module types
 * (specifying where they are used
 * and what for modules are needed - ENCRYPT is UNIQUE!) */
typedef enum { CORE, THIRD, EVENT, ENCRYPT } 						MODType;

/* These are all available position to what a module can be added
 * KNOWN_POSITIONS is used for <KNOWN_POSITIONS operations.
 * (makes changing very easy) */
typedef enum { MOD_UNIQUE, MOD_HEAD, MOD_TAIL, KNOWN_POSITIONS } 	MODposition;

/* These are all available access types
 * (specifying who has access to what module
 * and what level is needed) */
typedef enum { USER, ADMIN, BOTH, NOONE } 							MODACCESS;

/* These are all available languages
 * LASTLANG is used for <LASTLANG operations.
 * (makes changing very easy) */
typedef enum { EN, DE, FR, ES, IT, PL, LASTLANG } 					Languages;

/* These are to tell orion if his modules
 * are unloaded or only reloaded.
 * (Makes us able to reload modules without a single QUIT) */
typedef enum { DO_UNLOAD, DO_RELOAD } 								DO_ACTION;

/* These are all available module handling operations
 * They control running/stopping
 * if MOD_STOP is returned, we stop processing
 * (and skip other modules).
 * if MOD_CONT is returned, we continue processing
 * other modules */
typedef enum {
	MOD_STOP,
	MOD_CONT
}MOD_BOOL;

//Protocol
typedef enum { CMD_SUPPORTED, CMD_EXECUTED, CMD_NOT_SUPPORTED, CMD_ERROR } PROTOCOL_BOOL;

/* Windows defines a boolean type as an 
 * unsigned char. It does however need
 * true/false. -- codemastr
 */
#ifndef _WIN32
typedef enum { false, true } bool;
#else
 #ifndef bool
  #define bool int
 #endif
 #ifndef true
  #define true 1
 #endif
 #ifndef false
  #define false 0
 #endif
#endif /* _WIN32 */

/* Socket definitions for Windows
 * and for Linux -- shock
 */
#ifdef _WIN32
  typedef SOCKET			orion_socket_t;
  #define orion_sockread(fd, buf, len)		recv(fd, buf, len, 0)
  #define orion_sockwrite(fd, buf, len)	 	send(fd, buf, len, 0)
  #define orion_sockclose(fd)				closesocket(fd)
  #define orion_sockgeterr()				WSAGetLastError()
  #define orion_sockseterr(err)				WSASetLastError(err)
  /* orion_sockstrerror in sockutil.c */
  /* orion_socksetnonb in sockutil.c */
  #define orion_sockerrnonb(err)			(err == WSAEINPROGRESS || err == WSAEWOULDBLOCK)
  #define SOCKERR_EBADF						WSAENOTSOCK
  #define SOCKERR_EINTR						WSAEINTR
  #define SOCKERR_EINVAL					WSAEINVAL
  #define SOCKERR_EINPROGRESS				WSAEINPROGRESS
#else
  typedef	int32			orion_socket_t;
  #define orion_sockread(fd, buf, len)		read(fd, buf, len)
  #define orion_sockwrite(fd, buf, len) 	write(fd, buf, len)
  #define orion_sockclose(fd)				close(fd)
  #define orion_sockgeterr()				errno
  #define orion_sockseterr(err)				errno = err
  #define orion_sockstrerror(err)			strerror(err)
  #define orion_socksetnonb(fd)				fcntl(fd, F_SETFL, O_NONBLOCK)
  #define orion_sockerrnonb(err)			(err == EINPROGRESS)
  #define SOCKERR_EBADF						EBADF
  #define SOCKERR_EINTR						EINTR
  #define SOCKERR_EINVAL					EINVAL
  #define SOCKERR_EINPROGRESS				EINPROGRESS
#endif

/*********************************************************/

//#define THREAD_PROCESSES

/*
 * SQL Defines for Tables Chaching
 */
/* Common MySQL defines */
#define MAX_SQL_SELECT_ITEMS		5000
#define MAX_SQL_COLUMN_SELECT_LIST	6000
#define MAX_SQL_ROWS				30

/* MySQL Chaching tables */
#define NO_DIRECTTABLE				-1
#define MAX_BUFFER_TABLES			10
#define MAX_MYSQLTABLE_LEN 			70
#define MAX_MYSQLUSER_LEN 			80
#define MAX_MYSQLDATABASE_LEN 		70

/* SQL Buffers */
#define SQL_BUF 					8000
#define MYSQL_BUF					8192	//Buffer for outgoing MySQL Data
#define MYSQL_SET_BUF				8192
	/* Max Quotealloc's */
#define SQL_MAX_ITEM_QUOTE 			7000

/* Take in mind that direct buffered tables are much faster than dynamic buffered ones
   So we define the tables we want to buffer, and take care of them, since if we don't
   We would have big problems, that aren't easy to find anymore - so be careful */
#define SQL_BUFFER_CONDATA			0
#define SQL_BUFFER_CONDATA_NO_MODES	1
#define SQL_BUFFER_SJOIN			2

/******************************************************/

/*
 * SQL Defines for sql_chan_data();
 */
#define MAX_CHANDATA_TYPES			11

/* Identifiers */
#define CHANNEL						1
#define WITH_LEVELS					2
#define WITH_BOTSERV				4
#define COUNTUSERS					8
#define OWNACCLEVEL					16
#define OWNACCLEVEL_FROM_NICK		32
#define TARGETACCLEVEL				64
#define CONDATA						128
#define ALIASES						256
#define	ACCESS						512
#define ACCESS_FROM_NICK			1024
#define TARGETACCLEVEL_FROM_NICK	OWNACCLEVEL_FROM_NICK

/* Array Identifiers */
#define CHANNEL_ARR					0
#define WITH_LEVELS_ARR				1
#define WITH_BOTSERV_ARR			2
#define COUNTUSERS_ARR				3
#define OWNACCLEVEL_ARR				4
#define OWNACCLEVEL_FROM_NICK_ARR	5
#define TARGETACCLEVEL_ARR			6
#define CONDATA_ARR					7
#define ALIASES_ARR					8
#define	ACCESS_ARR					9
#define ACCESS_FROM_NICK_ARR		10

/* Combinations: */
#define CCLB		15				// CHANNEL + WITH_LEVELS + WITH_BOTSERV + COUNTUSERS
#define AAC			1408			// ACCESS_FROM_NICK + ALIASES + CONDATA

/* Complete channel selection macros */
#define SQL_THREAD_AAC	8,ACCESS_FROM_NICK,ALIASES,CONDATA,TARGETACCLEVEL_FROM_NICK,OWNACCLEVEL,COUNTUSERS,WITH_LEVELS,WITH_BOTSERV
#define SQL_THREAD_AAC2	7,ACCESS_FROM_NICK,ALIASES,CONDATA,TARGETACCLEVEL_FROM_NICK,COUNTUSERS,WITH_LEVELS,WITH_BOTSERV
#define SQL_THREAD_CCLB	3,COUNTUSERS,WITH_LEVELS,WITH_BOTSERV
#define SQL_THREAD		4,COUNTUSERS,OWNACCLEVEL,WITH_LEVELS,WITH_BOTSERV

/******************************************************/

/*
 * MAX Default Buffers - IRCd Specification (Defined by using Unreal)
 */
#define MAX_IRCD_MESSAGE_LEN		513		//Maximum allowed IRCD message length = 510 + \r\n = 512 + \0 = 513
#define MAX_MODE_PARAMS_IRCD		152
#define	MAX_CHANS					65
#define MAX_MODES					400
#define ARRAY_SIZE					257		//Half irc message (each param is only one charakter, worst case) + 1 to be sure =D

/* Network Buffers */
#define NET_BUFSIZE					65536	//Network Buffer real receive[NET_BUFSIZE]; data save
#define BUFFER						6000	//Buffer for reason lengths and topic lengths ......
#define PUTSERVER_BUF_LEN			8192*2	//Buffer for outgoing data
#define BUFSIZE						1024	//Buf pointers

/* Ensure that BUFSIZE > MAX_IRCD_MESSAGE_LEN */
#if MAX_IRCD_MESSAGE_LEN > BUFSIZE
 #undef BUFSIZE
 #define BUFSIZE MAX_IRCD_MESSAGE_LEN+100
#endif

/******************************************************/

/*
 * MAX Services (Buffer) Default Lengths
 */
/* Common length's (services) */
#define MAX_ENCRYPTION_MODUL_NAME_LEN 	20
#define MAX_PROTOCOLNAME_LEN 			30	//IRCd Protocol file length ("UnrealIRCd")

/* IRCd depending length's (RFC) */
#define MAX_EMAIL_LEN				100
#define MAX_NICK_LEN				35
#define MAX_HOST_LEN				64+1
#define MAX_IP_LEN					17 		//IPV4
#define MAX_SWHOIS_LEN				100
#define MAX_IDENT_LEN				15
#define MAX_GLINES					5000
#define MAX_SUBNET_LEN				64
#define MAX_NETWORKNAME_LEN			100
#define MAX_SERVER_LEN				100
/* users.c & servers.c (ID length's) */
#define MAX_USERID_LEN 				15		//For Servers using ID's to handle users/servers
#define MAX_SERVERID_LEN 			15		//For Servers using ID's to handle users/servers

/* Channel length's */
#define MAX_CHANNELNAME_LEN			80		//Referring to RFC1459 up to 200 chars!
#define MAX_CHANNELDESCRIPTION_LEN	308
#define MAX_KEY_LEN					50
#define MAX_TOPIC_LEN				308

/* NickServ Setting lengths */
#define MAX_GREET_LEN				255
#define MAX_URL_LEN					255
#define MAX_YAHOO_LEN				255
#define MAX_LOCATION_LEN			100
#define MAX_BIRTHDAY_LEN			30
#define MAX_REALNAME_LEN			308
#define MAX_INTERESTS_LEN			308
#define MAX_PROFESSION_LEN			100
#define MAX_SLOGAN_LEN				308
#define MAX_TYPE_LEN				60
#define MAX_HOMEPAGE_LEN 			100
#define GUEST_NUMBER_LEN	 		6		/* Number of numbers after "Guest" - generated randomly */

/* Message / Reason length's */
#define MAX_QUITMSG_LEN				255
#define MAX_MESSAGE_LEN 			300 	/* quitmsg, notices, globals and such stuff */
#define MAX_DESCRIPTION_LEN			255
#define MAX_REASON_LEN				308

/* Modes length's */
#define MAX_COMMON_LEN				45
#define MAX_CHAN_MODES				33
#define MAX_USER_MODES				33
#define MAX_CHAN_USER_MODES			20
#define MAX_MODES_LEN				33

/* Command / Modul length's */
#define MAX_COMMAND_LEN				100
#define MAX_COMMAND_CALL_LEN 		300
#define MAX_MODUL_AUTHOR_LEN 		70
#define MAX_MODUL_VERSION_LEN 		200

/* Filepath length's */
#define MAX_FILEPATH_LEN			750

/* Channel acclevel length */
#define MAX_CHANNEL_LEVELS 			50		/* Size for the LevelChanging struct */

/* Modules */
#define MAX_LANGSTRING_LEN          100
#define MAX_CONFSTRING_LEN          100
#define MAX_CONFIGVALUE_LEN         300
#define MAX_MODULENAME_LEN 			100
#define MAX_COMMANDNAME_LEN 		50
#define MAX_MODULEFILE_LEN 			100+MAX_NICK_LEN
#define MAX_FUNCTION_LEN 			50
#define NUM_LANGS					6+1

/******************************************************/

/*
 * Services Level Configuration
 */
#define ACCESS_FOUNDER			1000	/* Numeric level indicating founder access */
#define ACCESS_SUCCESSOR		999		/* Numeric level indicating successor access */
#define ACCESS_INVALID			-1000	/* Used in levels[] for disabled settings */

#define ACCESS_UNKNOWN_STRING	   "-1"
#define ACCESS_UNKNOWN				-1
#define ACCESS_UNKNOWN_END			-2
#define IDENTIFIED_NO_ACCESS		 0

/* Faketypes */
#define FAKETYPE_USER 			0
#define FAKETYPE_BOT 			1
#define FAKETYPE_SERVICE  		2
#define FAKETYPE_RELEASEUSER	3

/* Q,O,S Level Configuration */
#define O_SRA_LEVEL	1000
#define O_SA_LEVEL	870
#define O_SO_LEVEL	720

#define Q_SRA_LEVEL	1000
#define Q_SA_LEVEL	900
#define Q_SO_LEVEL	300

#define S_SRA_LEVEL	1000
#define S_SA_LEVEL	900
#define S_SO_LEVEL	300

/*********************************************************/

/*
 * XOP Level Configuration
 */
/* Common XOP configuration */
#define ACCESS_VOP			3
#define ACCESS_HOP			4
#define ACCESS_AOP			5
#define ACCESS_SOP			10

/* services.c (levelchanging) - to translate Qlevels to ChanServ levels and vice versa */
#define LEVELCHANGING_MAX_INDEX_LEN 	40
#define LEVELCHANGING_MAX_CS_LEVEL_LEN	40
#define LEVELCHANGING_MAX_Q_LEVEL_LEN	10

/*********************************************************/

/*
 * NickServ Default Lengths
 */
#define PASSWORD_LEN 		15
#define ENC_PASSMAX			40
#define MAX_PASSWORD_LEN	40
#define MAX_WELCOME_LEN		307
#define MAX_LEVEL_LEN		10
/* No encryption, meaning !? -> Ahh plain ^^ */
#define NO_ENCRYPT "plain"

#define DEFAULT_ENCRYPTION 		0
#define MAX_ENCRYPTION_MODULES 	2

/*********************************************************/

/*
 * NickServ Set Options (Settings Saved as bits)
 */
#define NS_SET_KILLPROTECT			0x00000001  /* Kill others who take this nick */
#define NS_SET_KILL_QUICK			0x00000002  /* Kill in 20 seconds instead of 60 */
#define NS_SET_KILL_IMMED			0x00000004  /* Kill immediately instead of in 60 sec */
#define NS_SET_MSG					0x00000008  /* Use PRIVMSGs instead of NOTICEs */
#define NS_SET_SECURE				0x00000010  /* NOT IMPLEMENTED - No Usage by now! */
#define NS_SET_PRIVATE				0x00000020  /* Don't show in LIST to non-servadmins */
#define NS_SET_HIDE_EMAIL			0x00000040  /* Don't show E-mail in INFO */
#define NS_SET_HIDE_MASK			0x00000080  /* Don't show last seen address in INFO */
#define NS_SET_HIDE_QUIT			0x00000100  /* Don't show last quit message in INFO */
#define NS_SET_HIDE_STATUS			0x00000200  /* Don't show services access status */
#define NS_SUSPENDED				0x00000400  /* Suspend state for an authnick */
#define NS_SET_AUTOOP				0x00000800  /* Set Autoop (ops you in all channels you have autoop access on identify) */
#define NS_SET_INCOGNITO			0x00001000  /* Set incognito (doesn't op you on identify and on joining channels with autoop) */
#define NS_NEW_ENCRYPTION			0x00002000  /* Password has newer encryption style */

/*********************************************************/

/*
 * ChanServ Default Levels
 */
#define CS_AUTOOP				5
#define CS_AUTOVOICE			3
#define CS_AUTODEOP				-1
#define CS_NOJOIN				-2
#define CS_INVITE				5
#define CS_AKICK				10
#define CS_SET					ACCESS_INVALID
#define CS_CLEAR				ACCESS_INVALID
#define CS_UNBAN				5
#define CS_OPDEOP				5
#define CS_ACCESS_LIST			1
#define CS_ACCESS_CHANGE		10
#define CS_MEMO					10
#define CS_ASSIGN				ACCESS_INVALID
#define CS_BADWORDS				10
#define CS_NOKICK				1
#define CS_FANTASIA				3
#define CS_SAY					5
#define CS_GREET				5
#define CS_GETKEY				5
#define CS_VOICEME				3
#define CS_VOICE				4
#define CS_AUTOHALFOP			4
#define CS_AUTOPROTECT			10
#define CS_OPDEOPME				5
#define CS_HALFOPME				4
#define CS_HALFOP				5
#define CS_PROTECTME			10
#define CS_PROTECT				ACCESS_INVALID
#define CS_KICKME				5
#define CS_KICK					5
#define CS_SIGNKICK				ACCESS_INVALID
#define CS_BANME				5
#define CS_BAN					5
#define CS_TOPIC				ACCESS_INVALID
#define CS_INFO					ACCESS_INVALID

/*********************************************************/

/*
 * ChanServ Set Options (Settings Saved as bits)
 */
#define CS_XOP_BIT					0x00000001
#define CS_TOPICLOCK_BIT			0x00000002
#define CS_KEEPTOPIC_BIT			0x00000004
#define CS_SECUREFOUNDER_BIT		0x00000008
#define CS_SECUREOPS_BIT			0x00000010
#define CS_SECURE_BIT				0x00000020
#define CS_RESTRICTED_BIT			0x00000040
#define CS_PRIVATE_BIT				0x00000080
#define CS_PEACE_BIT				0x00000100
#define CS_OPNOTICE_BIT				0x00000200
#define CS_SIGNKICK_1_BIT			0x00000400
#define CS_SIGNKICK_2_BIT			0x00000800
#define CS_BANTYPE_1_BIT			0x00001000
#define CS_BANTYPE_2_BIT			0x00002000
#define CS_FORBID_BIT				0x00004000
#define CS_SUSPEND_BIT				0x00008000
#define BS_SET_NOBOT_BIT			0x00010000
#define CS_INHABIT_BIT				0x00020000

/*********************************************************/

/*
 * BotServ defines
 */
#define MAX_FANTASYCHARACTER_LEN 	10 //Maximum length of Fantasy"character"

/*********************************************************/

/*
 * BotServ Set Options (Settings Saved as bits)
 */
#define BS_SET_DONTKICKOPS_BIT		0x00000001
#define BS_SET_DONTKICKVOICES_BIT	0x00000002
#define BS_SET_GREET_BIT			0x00000004
#define BS_SET_FANTASY_BIT			0x00000008
#define BS_SET_SYMBIOSIS_BIT		0x00000010
#define BS_KICK_BOLDS_BIT			0x00000020
#define BS_KICK_COLORS_BIT			0x00000040
#define BS_KICK_REVERSES_BIT		0x00000080
#define BS_KICK_UNDERLINES_BIT		0x00000100
#define BS_KICK_BADWORDS_BIT		0x00000200
#define BS_KICK_CAPS_BIT			0x00000400
#define BS_KICK_FLOOD_BIT			0x00000800
#define BS_KICK_REPEAT_BIT			0x00001000

/*********************************************************/

/*
 * Q Defines
 */
#define MAX_CHAN_ACCLEVEL_LEN	 	60


/*********************************************************/

/*
 * OperServ Defines
 */
/* RegEXP Glines */
#define MAX_GLINE_REGEXP_LEN 		400

/* Trusts and Trustgroups */
#define MAX_TRUSTGROUP_LEN 			50
#define MAX_TRUSTDESCRIPTION_LEN 	100
#define MAX_TRUSTLIMIT_LEN		 	35
#define MAX_EXPIRESTRING_LEN 		50

/*********************************************************/

#ifdef _WIN32
  #if defined(_MSC_VER) || defined(_MSC_EXTENSIONS)
    #define DELTA_EPOCH_IN_MICROSECS  11644473600000000Ui64
  #else
    #define DELTA_EPOCH_IN_MICROSECS  11644473600000000ULL
  #endif
 
  struct timezone 
  {
  	int  tz_minuteswest; /* minutes W of Greenwich */
  	int  tz_dsttime;     /* type of dst correction */
  };
#endif

/* Structs and Typedef's */
typedef struct users_ Users;
typedef struct mailinfo_ MailInfo;
typedef struct QUserlist_ QUserlist;
typedef struct lcolumn_ LColumn; //MySQL select LColumn struct -
typedef struct rcolumn_ RColumn; //MySQL select RColumn struct -
typedef struct aliases_ Aliases;
typedef struct access_ Access;
typedef struct acclist_ Acclist;
typedef struct bassign_ Bassign;
typedef struct bkick_ Bkick;
typedef struct channel_ Channel;
typedef struct channel2_ Channel2;
typedef struct channel3_ chan;
typedef struct channeldb_ ChannelDB;
typedef struct botserv_ BotServ;
typedef struct chanaccess_ ChanAccess;
typedef struct chanlevels_ ChanLevels;
typedef struct db3_ DB3;
typedef struct db4_ DB4;
typedef struct condata_ Condata;
typedef struct Trust_ Trust;
typedef struct BsChan_ BsChan;
typedef struct ChanCA_ ChanCA;
typedef struct SQL_ SQL;
typedef struct deepSQL_ deepSQL;
typedef struct MySQLBuf_ MySQLBuf;
typedef struct LevelChanging_ LevelChanging;
typedef struct threads_ Threads;
typedef struct socket_ Socket;

typedef struct Stays_ Stays;
typedef struct Limits_ Limits;
typedef struct UserChanMode_ UserChanModes;
typedef struct ulines_ Ulines;
typedef struct channels_ Uchannels;
typedef struct servers_ Servers;
typedef struct Trusts_ Trusts;
typedef struct Trustgroups_ Trustgroups;
typedef struct protocol_ Protocol;
typedef struct events_ Event;
typedef struct language_ SharedLanguage;

/* Modules.c */
typedef struct moduleconfig_ ModConfig;
typedef struct ModCheck_ ModCheck;
typedef struct ModInfo_ ModInfo;
typedef struct modules_ ModCo;
typedef struct commands_ CMD;

/* Config */
typedef struct Q_conf_ Q_conf;
typedef struct ns_conf_ ns_conf;
typedef struct cs_conf_ cs_conf;
typedef struct hs_conf_ hs_conf;
typedef struct ms_conf_ ms_conf;
typedef struct bs_conf_ bs_conf;
typedef struct g_conf_ g_conf;
typedef struct O_conf_ O_conf;
typedef struct sockscontrol_conf_ sockscontrol_conf;
typedef struct config_load_ config_load;

/* Config Q variables */
struct Q_conf_ {
	char nick[MAX_NICK_LEN];
	char ident[MAX_IDENT_LEN];
	char realname[MAX_REALNAME_LEN];
	char host[MAX_HOST_LEN];
	bool loaded;
};

struct ns_conf_ {
	char nick[MAX_NICK_LEN];
	char ident[MAX_IDENT_LEN];
	char realname[MAX_REALNAME_LEN];
	char host[MAX_HOST_LEN];
	size_t min_passwordlen;
	bool loaded;
};

struct cs_conf_ {
	char nick[MAX_NICK_LEN];
	char ident[MAX_IDENT_LEN];
	char realname[MAX_REALNAME_LEN];
	char host[MAX_HOST_LEN];
	bool loaded;
};

struct hs_conf_ {
	char nick[MAX_NICK_LEN];
	char ident[MAX_IDENT_LEN];
	char realname[MAX_REALNAME_LEN];
	char host[MAX_HOST_LEN];
	bool loaded;
};

struct ms_conf_ {
	char nick[MAX_NICK_LEN];
	char ident[MAX_IDENT_LEN];
	char realname[MAX_REALNAME_LEN];
	char host[MAX_HOST_LEN];
	bool loaded;
};

struct bs_conf_ {
	char nick[MAX_NICK_LEN];
	char ident[MAX_IDENT_LEN];
	char realname[MAX_REALNAME_LEN];
	char host[MAX_HOST_LEN];
	bool loaded;
};

struct g_conf_ {
	char nick[MAX_NICK_LEN];
	char ident[MAX_IDENT_LEN];
	char realname[MAX_REALNAME_LEN];
	char host[MAX_HOST_LEN];
	bool loaded;
};

/* Config O variables */
struct O_conf_ {
	char nick[MAX_NICK_LEN];
	char ident[MAX_IDENT_LEN];
	char realname[MAX_REALNAME_LEN];
	char host[MAX_HOST_LEN];
	bool loaded;
};

/* Config sockscontrol variables */
struct sockscontrol_conf_ {
	char nick[MAX_NICK_LEN];
	char ident[MAX_IDENT_LEN];
	char realname[MAX_REALNAME_LEN];
	char host[MAX_HOST_LEN];
	int		sockmode;
	int 	sockcons;
	time_t 	sockseconds;

	time_t reset_defcon_seconds;
	int auto_defcon_1;
	int auto_defcon_2;
	int auto_defcon_3;
	int auto_defcon_4;
	bool loaded;
};

/* Config variables */
struct config_load_ {
	time_t starttime;
	char *Numeric; //P10

	char server[MAX_HOST_LEN];
	char bindServicesIp[MAX_HOST_LEN];
	char WebFormIp[MAX_HOST_LEN];
	char bindWebFormIp[MAX_HOST_LEN];
	long port;
	long WebPort;
	char WebSocketPass[MAX_PASSWORD_LEN];
	char linkpass[MAX_PASSWORD_LEN];
	int serverlink;
	char servername[MAX_SERVER_LEN];
	char description[MAX_DESCRIPTION_LEN];
	char vhost[MAX_HOST_LEN];
	char modes[MAX_MODES_LEN];
	char host[MAX_HOST_LEN];
	char nick[MAX_NICK_LEN];
	char ident[MAX_IDENT_LEN];
	char homepage[MAX_HOMEPAGE_LEN];
	char LinkToWebform[MAX_HOMEPAGE_LEN];
	
	char protocol[MAX_PROTOCOLNAME_LEN];

	char LoadModulesConf[MAX_FILEPATH_LEN];
	char pidfilename[MAX_FILEPATH_LEN];
	int freelogs;
	int KeepLogs;
	char logfile[MAX_FILEPATH_LEN];
	char sqllogfile[MAX_FILEPATH_LEN];
	bool debuglogging;
	int debug;
	char debuglog[MAX_FILEPATH_LEN];
	char logquotes[MAX_FILEPATH_LEN];
	bool AutoRecognizeModules;

	char logchan[MAX_CHANNELNAME_LEN];
	char network[MAX_NETWORKNAME_LEN];

	bool StrictPasswords;
	int BadPassLimit;
	int SecureIdentify;
	int ReadTimeout;
	int ReadTimeout_us;
	bool UseWebForm;
	bool UseWhowas;
	time_t KeepWhowasLog;
	time_t LimitCheck;
	time_t TrustDumpTime;
	time_t ExpireCheckTime;
	time_t NickUsedCheck;
	bool ForceForbidReason;
	bool UsePrivmsg;
	bool UseStrictPrivMsg;
	bool GlobalOnCycle;
	char GlobalOnCycleMessage[MAX_MESSAGE_LEN];
	char GlobalOnCycleUp[MAX_MESSAGE_LEN];
	bool AnonymousGlobal;
	bool UseMail;
	char sendmailpath[MAX_FILEPATH_LEN];
	char SendFrom[MAX_HOST_LEN+MAX_IDENT_LEN];
	bool RestrictMail;
	bool DontQuoteAddresses;

	bool NSDefKill;
	bool NSDefKillQuick;
	bool NSDefKillImmed;
	bool NSDefSecure;
	bool NSDefPrivate;
	bool NSDefHideEmail;
	bool NSDefHideUsermask;
	bool NSDefHideQuit;
	bool NSDefHideStatus;
	bool NSDefMsg;
	bool NSDefAutoop;
	bool NSDefIncognito;
	bool NSDefMemoSignon;
	bool NSDefMemoReceive;

	int language;
	bool NSEmailReg;
	time_t NSRegDelay; // expire();
	time_t NSResendDelay; // expire();
	time_t NSRExpire;
	time_t NSWaitRegister;
	char NSExpire[50];
	time_t NSQuickKill;
	time_t NSImmedKill;
	int NSMaxGroups;
	int NSMaxAliases;
	int NSAccessMax;
	time_t NSReleaseTimeout;
	bool NSNoGroupChange;
	bool NSListOpersOnly;
	int NSListMax;
	char NSGuestNickPrefix[MAX_NICK_LEN-GUEST_NUMBER_LEN-1]; //perfix lenght
	bool NSSecureAdmins;
	bool NSRestrictGetPass;
	bool NSUseTempLinks;

	bool CSDefNone;
	bool CSDefKeepTopic;
	bool CSDefOpNotice;
	bool CSDefPeace;
	bool CSDefPrivate;
	bool CSDefRestricted;
	bool CSDefSecure;
	bool CSDefSecureOps;
	bool CSDefSecureFounder;
	bool CSDefSignKick;
	bool CSDefSignKickLevel;
	bool CSDefTopicLock;
	bool CSDefXOP;
	bool CSDefInhabit;

	int CSMaxReg;
	time_t CSWaitRegister;
	char CSExpire[MAX_EXPIRESTRING_LEN];
	int CSDefBantype;
	time_t CSStayChannel;
	int CSAccessMax;
	int CSAutokickMax;
	char CSAutokickReason[MAX_REASON_LEN];
	bool CSAutokickForceReason;
	bool CSListOpersOnly;
	int CSListMax;
	bool CSOpersOnly;

	int maxmemos;
	bool MSUsePriorities;

	bool BSDefDontKickOps;
	bool BSDefDontKickVoices;
	bool BSDefGreet;
	bool BSDefFantasy;
	bool BSDefSymbiosis;
	int BSMinUsers;
	int BSBadWordsMax;
	bool BSGentleBWReason;
	bool BSCaseSensitive;
	char BSFantasyCharacter[MAX_FANTASYCHARACTER_LEN];
	
	char HSWaitRequest[MAX_EXPIRESTRING_LEN];

	time_t socklinetime;
	char glineonclearchan[MAX_EXPIRESTRING_LEN];
	time_t GlineExpiry;

	bool OSForceTrusts;
	bool OSGlineOnTrustViolation;
	bool OSForceTrustDelete;
	bool OSUseRNlineIDline;
	
	bool OSMsgFloodKill; //(0/1)
	
	bool OSMsgFloodProt; //(0/1)
	bool OSMsgFloodlines;
	time_t OSMsgFloodtime;
	
	bool OSNickFloodProt; //(0/1)
	int OSNickFloodlines;
	time_t OSNickFloodtime;
	
	bool DisableRaw;
	
	bool WallBadOS;
	bool WallOSGlobal;
	bool WallOSMode;
	bool WallOSClearchan;
	bool WallOSKick;
	bool WallOSGline;
	bool WallOSNoOp;
	bool WallOSJupe;
	bool WallOSRaw;
	bool WallGetpass;
	bool WallSetpass;
	bool WallForbid;
	bool WallDrop;
	
	long maxconnections;
	char SessionLimitExceeded[MAX_MESSAGE_LEN];
	int MaxSessionKill;
	time_t SessionGlineExpiry;

	bool OSOpersOnly;
	char DefConChanModes[MAX_MODES_LEN];
	bool GlobalOnDefcon;
	char DefconMessage[MAX_MESSAGE_LEN];
	char DefConOffMessage[MAX_MESSAGE_LEN];
	char DefConKillReason[MAX_REASON_LEN];

	char mysqlsrv[MAX_HOST_LEN];
	char mysqluser[MAX_MYSQLUSER_LEN];
	char mysqlpass[MAX_PASSWORD_LEN];
	char mysqlDB[MAX_MYSQLDATABASE_LEN];
	long mysqlport;
	char mysqlsocket[MAX_FILEPATH_LEN];
	bool MySQLSuperUser;
	char MysqlMove[MAX_ENCRYPTION_MODUL_NAME_LEN];
	
	char NewEncrypt[MAX_ENCRYPTION_MODUL_NAME_LEN];
	char encrypt[MAX_ENCRYPTION_MODUL_NAME_LEN];

	/*### Traffic Settings ###*/

	double TrafficOut;
	double TrafficIn;
	double TrafficOut_bps;
	double TrafficIn_bps;
	double TrafficOut2_bps;
	double TrafficIn2_bps;
};
/**************************************************************************************/
/* Mail data */

struct mailinfo_ {
	FILE *pipe;
	char *sender;
	char *email;
	char *recipient;
	char *subject;
};

/* Q Userlist */
struct QUserlist_ {
	char **users;
	int count_users;
};

/* Trusts */
struct Trust_ {
	char rhost[MAX_HOST_LEN];
	char description[MAX_TRUSTDESCRIPTION_LEN];
	char limit[MAX_TRUSTLIMIT_LEN];
	int fields;
};

/* MySQL Column */
struct lcolumn_ {
	char *row[MAX_SQL_COLUMN_SELECT_LIST];
	int fields;
};

struct rcolumn_ {
	char *row[MAX_SQL_ROWS];
	int fields;
};

/* Access */
struct access_ {
	char authnick[MAX_NICK_LEN];
	char password[ENC_PASSMAX];
	char reason[MAX_REASON_LEN];
	int qlevel;
	int olevel;
	int slevel;
	int llevel;
	char email[MAX_EMAIL_LEN];
	char greet[MAX_GREET_LEN];
	char url[MAX_URL_LEN];
	long icq;
	char yahoo[MAX_YAHOO_LEN];
	char location[MAX_LOCATION_LEN];
	char birthday[MAX_BIRTHDAY_LEN];
	char realname[MAX_REALNAME_LEN];
	bool gender;
	char interests[MAX_INTERESTS_LEN];
	char profession[MAX_PROFESSION_LEN];
	char slogan[MAX_SLOGAN_LEN];
	int language;
	int rights;
	int settings;
	int maxmemos;
	
	time_t last_failed_login_time;
	char last_failed_login_password[MAX_PASSWORD_LEN];
	int failed_logins;
	
	int fields; //Entries in the MySQL **row array
};

/* Aliases */
struct aliases_ {
	char nick[MAX_NICK_LEN];
	char authnick[MAX_NICK_LEN];
	char last_vhost[MAX_HOST_LEN];
	char last_realname[MAX_REALNAME_LEN];
	char last_quit[MAX_QUITMSG_LEN];
	//char time_registered[200];
	time_t linked;
	time_t time_linked;
	time_t last_login;
	time_t last_used;
	bool expire;
	bool active;
	bool forbid;
	int fields; //Entries in the MySQL **row array
};

/* Acclist */
struct acclist_ {
	char serv[MAX_NICK_LEN];
	char authnick[MAX_HOST_LEN];  //could consist of a realhost
	char channel[MAX_CHANNELNAME_LEN];
	char level[MAX_CHAN_ACCLEVEL_LEN];
	int fields;
};

/* BotServ Assign */
struct bassign_ {
	char bot[MAX_NICK_LEN];
	char channel[MAX_CHANNELNAME_LEN];
	int set;
	int fields;
};

/* BotServ Kick */
struct bkick_ {
	char channel[MAX_CHANNELNAME_LEN];
	int caps_min;
	int caps_per;
	int flood_ln;
	time_t flood_secs;
	int repeat;
	int colors;
	int underlines;
	int reverses;
	int badwords;
	int bolds;
	int fields;
};

/* Channel */
struct channel_ {
	char serv[MAX_NICK_LEN]; //0
	char bot[MAX_NICK_LEN];
	char name[MAX_CHANNELNAME_LEN]; //1
	char founder[MAX_NICK_LEN]; //2
	char successor[MAX_NICK_LEN]; //4
	time_t datecreate; //5
	time_t regdate; //6
	char description[MAX_CHANNELDESCRIPTION_LEN]; //7
	char mode[MAX_CHAN_MODES]; //8
	char mlock_positive[MAX_CHAN_MODES]; //9
	char mlock_negative[MAX_CHAN_MODES]; //10
	long mlock_limit; //11
	char mlock_key[MAX_KEY_LEN]; //12
	char mlock_flood[MAX_COMMON_LEN]; //13
	char link[MAX_CHANNELNAME_LEN]; //14
	long limit; //15
	char key[MAX_KEY_LEN]; //16
	char flood[MAX_COMMON_LEN]; //17
	char last_topic[MAX_TOPIC_LEN]; //18
	char last_topic_setter[MAX_NICK_LEN+MAX_IDENT_LEN+MAX_HOST_LEN+3]; //19
	time_t last_topic_time; //20
	char url[MAX_URL_LEN]; //21
	long alimit; //22
	char flags[MAX_CHAN_MODES]; //23
	char welcome[MAX_WELCOME_LEN]; //24
	char mlock_redirect[MAX_CHANNELNAME_LEN]; //25
	time_t last_used; //26
	char reason[MAX_REASON_LEN]; //27
	char email[MAX_EMAIL_LEN]; //28
	long settings; //29
	bool expire; //30
	int maxmemos; //31
	int fields; //32
};

struct channeldb_ {
	char serv[MAX_NICK_LEN]; //0
	char bot[MAX_NICK_LEN];
	char name[MAX_CHANNELNAME_LEN]; //1
	char founder[MAX_NICK_LEN]; //2
	char successor[MAX_NICK_LEN]; //4
	time_t datecreate; //5
	time_t regdate; //6
	char description[MAX_CHANNELDESCRIPTION_LEN]; //7
	char mode[MAX_CHAN_MODES]; //8
	char mlock_positive[MAX_CHAN_MODES]; //9
	char mlock_negative[MAX_CHAN_MODES]; //10
	long mlock_limit; //11
	char mlock_key[MAX_KEY_LEN]; //12
	char mlock_flood[MAX_COMMON_LEN]; //13
	char link[MAX_CHANNELNAME_LEN]; //14
	long limit; //15
	char key[MAX_KEY_LEN]; //16
	char flood[MAX_COMMON_LEN]; //17
	char last_topic[MAX_TOPIC_LEN]; //18
	char last_topic_setter[MAX_NICK_LEN+MAX_IDENT_LEN+MAX_HOST_LEN+3]; //19
	time_t last_topic_time; //20
	char url[MAX_URL_LEN]; //21
	long alimit; //22
	char flags[MAX_CHAN_MODES]; //23
	char welcome[MAX_WELCOME_LEN]; //24
	char mlock_redirect[MAX_CHANNELNAME_LEN]; //25
	time_t last_used; //26
	char reason[MAX_REASON_LEN]; //27
	char email[MAX_EMAIL_LEN]; //28
	long settings; //29
	bool expire; //30
	int maxmemos; //31
	
	
	short int cs_autodeop;
	short int cs_autohalfop;
	short int cs_autoop;
	short int cs_autoprotect;
	short int cs_autovoice;
	short int cs_nojoin;
	short int cs_signkick;
	short int cs_acclist;
	short int cs_accchange;
	short int cs_akick;
	short int cs_set;
	short int cs_ban;
	short int cs_banme;
	short int cs_clear;
	short int cs_getkey;
	short int cs_halfop;
	short int cs_halfopme;
	short int cs_info;
	short int cs_kick;
	short int cs_kickme;
	short int cs_invite;
	short int cs_opdeop;
	short int cs_opdeopme;
	short int cs_protect;
	short int cs_protectme;
	short int cs_topic;
	short int cs_unban;
	short int cs_voice;
	short int cs_voiceme;
	short int cs_memo;
	short int cs_assign;
	short int cs_badwords;
	short int cs_fantasia;
	short int cs_greet;
	short int cs_nokick;
	short int cs_say;
	
	
	int 		bs_set;
	int			bs_caps_ttb;
	short int	bs_caps_min;//
	short int	bs_caps_per;//
	int			bs_flood_ttb;
	short int	bs_flood_ln;
	time_t		bs_flood_secs;
	int			bs_repeat_ttb;
	short int 	bs_repeat;
	int			bs_colors;//
	int			bs_underlines;//
	int 		bs_reverses;
	int			bs_badwords;//
	int			bs_bolds;//
	
	int fields;
};

struct botserv_ {
	int 		set;
	int			caps_ttb;
	short int	caps_min;//
	short int	caps_per;//
	int			flood_ttb;
	short int	flood_ln;
	time_t		flood_secs;
	short int	repeat_ttb;
	int			repeat;
	int			colors;//
	int			underlines;//
	int 		reverses;
	int			badwords;//
	int			bolds;//
	
	int fields;
};

struct chanaccess_ {
	char ownlevel[MAX_CHAN_ACCLEVEL_LEN];
	char targetlevel[MAX_CHAN_ACCLEVEL_LEN];
	
	char ownauthnick[MAX_NICK_LEN];
	int fields;
};

/* Channel  Levels */
struct chanlevels_ {
	char channel[MAX_CHANNELNAME_LEN]; //1
	short int autodeop;
	short int autohalfop;
	short int autoop;
	short int autoprotect;
	short int autovoice;
	short int nojoin;
	short int signkick;
	short int acclist;
	short int accchange;
	short int akick;
	short int set;
	short int ban;
	short int banme;
	short int clear;
	short int getkey;
	short int halfop;
	short int halfopme;
	short int info;
	short int kick;
	short int kickme;
	short int invite;
	short int opdeop;
	short int opdeopme;
	short int protect;
	short int protectme;
	short int topic;
	short int unban;
	short int voice;
	short int voiceme;
	short int memo;
	short int assign;
	short int badwords;
	short int fantasia;
	short int greet;
	short int nokick;
	short int say;
	int fields;
};

struct BsChan_ {
	char authnick[MAX_NICK_LEN];
	int qlevel;
	int olevel;
	int llevel;
	int slevel;
	int rights;
	bool active;
	
	char level[60];
	char cu_modes[MAX_USER_MODES];
	char cu_nick[MAX_NICK_LEN];
	char ident[MAX_IDENT_LEN];
	char vhost[MAX_HOST_LEN];
	char server[MAX_SERVER_LEN];

	char serv[MAX_NICK_LEN]; //0
	char bot[MAX_NICK_LEN];
	char name[MAX_CHANNELNAME_LEN]; //1
	char founder[MAX_NICK_LEN]; //2
	char successor[MAX_NICK_LEN]; //4
	time_t datecreate; //5
	time_t regdate; //6
	char description[MAX_CHANNELDESCRIPTION_LEN]; //7
	char mode[MAX_CHAN_MODES]; //8
	char mlock_positive[MAX_CHAN_MODES]; //9
	char mlock_negative[MAX_CHAN_MODES]; //10
	long mlock_limit; //11
	char mlock_key[MAX_KEY_LEN]; //12
	char mlock_flood[MAX_COMMON_LEN]; //13
	char link[MAX_CHANNELNAME_LEN]; //14
	long limit; //15
	char key[MAX_KEY_LEN]; //16
	char flood[MAX_COMMON_LEN]; //17
	char last_topic[MAX_TOPIC_LEN]; //18
	char last_topic_setter[MAX_NICK_LEN+MAX_IDENT_LEN+MAX_HOST_LEN+3]; //19
	time_t last_topic_time; //20
	char url[MAX_URL_LEN]; //21
	long  alimit; //22
	char flags[MAX_CHAN_MODES]; //23
	char welcome[MAX_WELCOME_LEN]; //24
	char mlock_redirect[MAX_CHANNELNAME_LEN]; //25
	time_t last_used; //26
	char reason[MAX_REASON_LEN]; //27
	char email[MAX_EMAIL_LEN]; //28
	long settings; //29
	bool expire; //30
	long maxmemos; //31
	
	
	short int cs_autodeop;
	short int cs_autohalfop;
	short int cs_autoop;
	short int cs_autoprotect;
	short int cs_autovoice;
	short int cs_nojoin;
	short int cs_signkick;
	short int cs_acclist;
	short int cs_accchange;
	short int cs_akick;
	short int cs_set;
	short int cs_ban;
	short int cs_banme;
	short int cs_clear;
	short int cs_getkey;
	short int cs_halfop;
	short int cs_halfopme;
	short int cs_info;
	short int cs_kick;
	short int cs_kickme;
	short int cs_invite;
	short int cs_opdeop;
	short int cs_opdeopme;
	short int cs_protect;
	short int cs_protectme;
	short int cs_topic;
	short int cs_unban;
	short int cs_voice;
	short int cs_voiceme;
	short int cs_memo;
	short int cs_assign;
	short int cs_badwords;
	short int cs_fantasia;
	short int cs_greet;
	short int cs_nokick;
	short int cs_say;
	
	
	int 		bs_set;
	int			bs_caps_ttb;//
	short int	bs_caps_min;//
	short int	bs_caps_per;//
	int			bs_flood_ttb;
	short int	bs_flood_ln;
	time_t		bs_flood_secs;
	int			bs_repeat_ttb;
	short int	bs_repeat;
	short int	bs_colors;//
	short int	bs_underlines;//
	short int	bs_reverses;
	short int	bs_badwords;//
	short int	bs_bolds;//
	
	int fields;
	int data;
};

struct condata_ {
	char nick[MAX_NICK_LEN];
	time_t time;
	time_t localtime;
	char rident[MAX_IDENT_LEN];
	char ident[MAX_IDENT_LEN];
	char rhost[MAX_HOST_LEN];
	char server[MAX_SERVER_LEN];
	char modes[MAX_USER_MODES];
	char vhost[MAX_HOST_LEN];
	char realname[MAX_REALNAME_LEN];
	char swhois[MAX_SWHOIS_LEN];
	char subnet[MAX_SUBNET_LEN];
	char ip[MAX_IP_LEN];
	time_t ns_warntime;
	int faketype;
	int fields;
};

struct channel2_ {
	char name[MAX_CHANNELNAME_LEN];
	char bot[MAX_NICK_LEN];
	char serv[MAX_NICK_LEN];
	char founder[MAX_NICK_LEN];
	char successor[MAX_NICK_LEN];
	time_t datecreate;
	time_t regdate;
	char description[MAX_CHANNELDESCRIPTION_LEN];
	char mode[MAX_CHAN_MODES];
	char mlock_positive[MAX_CHAN_MODES];
	char mlock_negative[MAX_CHAN_MODES];
	long mlock_limit;
	char mlock_key[MAX_KEY_LEN];
	char mlock_flood[MAX_COMMON_LEN];
	char mlock_redirect[MAX_CHANNELNAME_LEN];
	char link[MAX_CHANNELNAME_LEN];
	long limit;
	char key[MAX_KEY_LEN];
	char flood[MAX_COMMON_LEN];
	char last_topic[MAX_TOPIC_LEN];
	char last_topic_setter[MAX_NICK_LEN+MAX_IDENT_LEN+MAX_HOST_LEN+3]; //19
	time_t last_topic_time;
	char url[MAX_URL_LEN];
	long alimit;
	char flags[MAX_CHAN_MODES];
	char welcome[MAX_WELCOME_LEN];
	time_t last_used;
	char reason[MAX_REASON_LEN];
	char email[MAX_EMAIL_LEN];
	long settings;
	bool expire;
	int maxmemos;
	
	int countusers;
	int botinchan;

	ChanLevels levels;
	BotServ botserv;
	ChanAccess access;
	
	int fields; //31
};

struct db3_ {
	Access access;
	Aliases aliases;
	Condata Cdata;
	Channel2 chan;
	
	int fields;
};

struct db4_ {
	Access access;
	Aliases aliases;
	Condata Cdata;

	struct channel3_ {
		char name[MAX_CHANNELNAME_LEN];
		char bot[MAX_NICK_LEN];
		char serv[MAX_NICK_LEN];
		char founder[MAX_NICK_LEN];
		char successor[MAX_NICK_LEN];
		time_t datecreate;
		time_t regdate;
		char description[MAX_CHANNELDESCRIPTION_LEN];
		char mode[MAX_CHAN_MODES];
		char mlock_positive[MAX_CHAN_MODES];
		char mlock_negative[MAX_CHAN_MODES];
		long mlock_limit;
		char mlock_key[MAX_KEY_LEN];
		char mlock_flood[MAX_COMMON_LEN];
		char mlock_redirect[MAX_CHANNELNAME_LEN];
		char link[MAX_CHANNELNAME_LEN];
		long limit;
		char key[MAX_KEY_LEN];
		char flood[MAX_COMMON_LEN];
		char last_topic[MAX_TOPIC_LEN];
		char last_topic_setter[MAX_NICK_LEN+MAX_IDENT_LEN+MAX_HOST_LEN+3]; //19
		time_t last_topic_time;
		char url[MAX_URL_LEN];
		long alimit;
		char flags[MAX_CHAN_MODES];
		char welcome[MAX_WELCOME_LEN];
		time_t last_used;
		char reason[MAX_REASON_LEN];
		char email[MAX_EMAIL_LEN];
		long settings;
		bool expire;
		int maxmemos;
			
		int countusers;
		bool botinchan;
	
		ChanLevels levels;
		BotServ botserv;
		struct chanaccess2_ {
			char ownlevel[MAX_CHAN_ACCLEVEL_LEN];
			char ownauthnick[MAX_NICK_LEN];
				
			char targetlevel[MAX_CHAN_ACCLEVEL_LEN];
			char targetauthnick[MAX_NICK_LEN];
			int fields;
		}access;
			
		int fields; //31
	}chan;
	
	int fields;
};

struct ChanCA_ {
	char serv[MAX_NICK_LEN]; //0
	char bot[MAX_NICK_LEN]; //1
	char name[MAX_CHANNELNAME_LEN]; //1
	char founder[MAX_NICK_LEN]; //2
	char successor[MAX_NICK_LEN]; //4
	time_t datecreate;
	time_t regdate; //5 - consists suspend reasons
	char description[MAX_CHANNELDESCRIPTION_LEN]; //6
	char mode[MAX_CHAN_MODES]; //7
	char mlock_positive[MAX_CHAN_MODES]; //8
	char mlock_negative[MAX_CHAN_MODES]; //8
	long mlock_limit; //10
	char mlock_key[MAX_KEY_LEN]; //11
	char mlock_flood[MAX_COMMON_LEN]; //12
	char mlock_redirect[MAX_CHANNELNAME_LEN]; //12
	char link[MAX_CHANNELNAME_LEN]; //13
	long limit; //14
	char key[MAX_KEY_LEN]; //15
	char flood[MAX_COMMON_LEN]; //16
	char last_topic[MAX_TOPIC_LEN]; //18
	char last_topic_setter[MAX_NICK_LEN+MAX_IDENT_LEN+MAX_HOST_LEN+3]; //19
	time_t last_topic_time; //20
	char url[MAX_URL_LEN]; //21
	long alimit; //23
	char flags[MAX_CHAN_MODES]; //24
	char welcome[MAX_WELCOME_LEN]; //25
	time_t last_used; //27
	char email[MAX_EMAIL_LEN]; //29
	long settings; //30
	bool expire; //30
	int maxmemos;
	char reason[MAX_REASON_LEN]; //21
	int fields; //31
	
	char acclevel[MAX_CHAN_ACCLEVEL_LEN];
	char targetlevel[MAX_CHAN_ACCLEVEL_LEN];
};

struct SQL_ {
	Access access;
	Aliases aliases;
	Condata Cdata;
};

struct deepSQL_ {
	MYSQL_ROW  row;
	MYSQL_RES *res;
	int fields;
};

struct MySQLBuf_ {
	char buffer[MYSQL_BUF+50];
	char insertcache[BUFSIZE];
	char duplication[BUFSIZE];
	char table[MAX_MYSQLTABLE_LEN];
	char values[BUFSIZE*2];
	size_t length;
	time_t lastused;
};

struct Trusts_ {
	char host[MAX_HOST_LEN];
	char description[MAX_TRUSTDESCRIPTION_LEN];
	char limit[MAX_TRUSTDESCRIPTION_LEN];
	Trust_art art;
	int fields;
	struct Trusts_ *next, *prev;
};

struct Trustgroups_ {
	char group[MAX_TRUSTGROUP_LEN];
	long limit;
	int fields;
	struct Trustgroups_ *next, *prev;
};

struct threads_ {
#ifdef USE_THREADS
	pthread_t thread;
	pthread_attr_t attr;
#endif

	char index[200];
	
	char read_netbuf[NET_BUFSIZE];
	char *read_curpos; /* Next byte to return */
	char *read_bufend; /* Next position for data from socket */
	char *read_buftop;
	int32 total_read;
	
	char write_netbuf[NET_BUFSIZE];
	char *write_curpos;       /* Next byte to write to socket */
	char *write_bufend;       /* Next position for data to socket */
	char *write_buftop;
	int write_fd;
	int32 total_written;
	
	int32 lastchar;
	
	MYSQL mysql;              	/* MySQL Handler */
	//MYSQL_RES *res;           /* MySQL Result  */
	//MYSQL_FIELD *fields;      /* MySQL Fields  */
	//MYSQL_ROW row;            /* MySQL Row     */

	FILE *sqlloghandle;
	FILE *loghandle;
	FILE *debughandle;

	char inbuf[BUFSIZE];
	char buf[BUFSIZE];
	char webformOutput[NET_BUFSIZE];

#ifdef USE_THREADS
	/* dynamic string management (dynamic.c) */
	char djoin[BUFFER];
	char dstrjoin[PUTSERVER_BUF_LEN];
	char dstrjoin_det[PUTSERVER_BUF_LEN];
	
	char dynadd[BUFFER];
	
	/* services.c */
	char time1[200];
	
	/* string management (string.c) */
	char lc[BUFFER];
	char uc[BUFFER];
	
	/* mysql.c */
	char select[200];
	char aselect[200];
	char mysql_sel_var[300];
	char level[MAX_USER_MODES];
	char user[MAX_NICK_LEN];
	char modes[MAX_USER_MODES];
#endif

	char* sqlQuote[SQL_MAX_ITEM_QUOTE];
	int QUOTES;
	
	SQL sql;
	DB4 c;

	orion_socket_t Sock;
	orion_socket_t MSock;
	bool Webform;
	int webSuccess;

	struct threads_ *next, *prev;
};

struct socket_ {
	orion_socket_t 		Sock;
	struct in_addr		Ip;
	int 				Identified;
	struct socket_ *next, *prev;
};

struct servers_ {
	char server[MAX_SERVER_LEN];	/* Server name                        */
	char sid[MAX_SERVERID_LEN];
    short int hops;    				/* Hops between services and server   */
    char desc[MAX_DESCRIPTION_LEN];	/* Server description                 */
    short int flags;   				/* Some info flags, as defined below  */
    char *suid;     				/* Server Univeral ID                 */
    SyncState sync; 				/* Server sync state (see above)      */

    struct servers_ *links;			/* Linked list head for linked servers 	  */
    struct servers_ *uplink;		/* Server which pretends to be the uplink */
    
	struct servers_ *next, *prev;
};

#define SERVER_ISME  0x0001
#define SERVER_JUPED 0x0002

struct channels_ {
	char channel[MAX_CHANNELNAME_LEN];
	char lastmsg[BUFFER];
	
	int Repeat;
	time_t LastFloodCheck;
	int FloodLines;
	
	int kicked_repeat;
	int kicked_caps;
	int kicked_flood;

	int kicked_colors;
	int kicked_bolds;
	int kicked_underlines;
	int kicked_reverses;
	int kicked_badwords;
	
	int isin_chan;

	struct channels_ *next, *prev;
};

struct users_ {
	char nick[MAX_NICK_LEN];
	char host[MAX_HOST_LEN];
	char ip[MAX_IP_LEN];
	char vhost[MAX_HOST_LEN];
	char ident[MAX_IDENT_LEN];
	char vident[MAX_IDENT_LEN];
	char server[MAX_SERVER_LEN];
	char lastcommand[MAX_COMMAND_LEN];
	char uid[MAX_USERID_LEN];
	time_t time;
	int faketype;
		
	time_t LastJFloodCheck;
	time_t LastPFloodCheck;
	time_t LastNFloodCheck;
	time_t LastServicesMsg;
	time_t LastNSRegister;
	
	time_t Ignoretime;
	time_t ns_warntime;
	int registered;
	int rights;
	int settings;
	
	long JoinFloodLines;
	long PartFloodLines;
	long NickFloodLines;
	long ServicesMsgLines;
	long NSRegisterMsgs;
	
	struct channels_ *channels;

	struct users_ *next, *prev;
};

struct ulines_ {
	char server[MAX_SERVER_LEN];

	struct ulines_ *next, *prev;
};

struct UserChanMode_ {
	char nick[MAX_NICK_LEN];
	char authnick[MAX_NICK_LEN];
	char mode[MAX_MODES_LEN];
	
	char ident[MAX_IDENT_LEN];
	char vhost[MAX_HOST_LEN];
	char rhost[MAX_HOST_LEN];
	
	char banmask[MAX_HOST_LEN*2];
	char reason[MAX_REASON_LEN];
	char greet[MAX_GREET_LEN];
	
	bool active;
	bool getakick;
	char access[MAX_MODES_LEN];
	int qlevel;
	int olevel;
	int llevel;
	int slevel;
	int rights;
	int faketype;
	int settings;
};

struct LevelChanging_ {
	char index[LEVELCHANGING_MAX_INDEX_LEN];
	int type;
	char CSlevel[LEVELCHANGING_MAX_CS_LEVEL_LEN];
	int XOPlevel;
	char Qlevel[LEVELCHANGING_MAX_Q_LEVEL_LEN];
};

struct Limits_ {
	char channel[MAX_HOST_LEN];
	time_t date;
	bool flag;
	char serv[MAX_NICK_LEN];
	long alimit;
	struct Limits_ *next, *prev;
};

struct Stays_ {
	char channel[MAX_CHANNELNAME_LEN];
	char nick[MAX_NICK_LEN];
	int botjoin;
	bool csjoin;
	time_t addtime;
	struct Stays_ *next, *prev;
};

/* Protocol.c & Ptcsupport */
struct events_ {
   char call[MAX_COMMAND_CALL_LEN]; //Function called 'call' in Modul
   MOD_BOOL (*func) (Threads *thread, size_t ac, char **av);
   size_t check_param;
   struct events_ *next, *prev;
};

struct protocol_ {
	char Author[MAX_MODUL_AUTHOR_LEN]; //Author of the CommandModul
	char Version[MAX_MODUL_VERSION_LEN]; //Version of the CommandModul
	  
	char *NSMode;
	char *CSMode;
	char *MSMode;
	char *HSMode;
	char *OSMode;
	char *BSMode;
	char *HeServMode;
	char *GlobMode;
	char *QMode;
	char *SCMode;
	char *BSBotsMode;
	char ChannelModes[MAX_CHAN_MODES];
	char ParametricChannelModes[MAX_CHAN_MODES];
	char *MLockAbleModes;
	char *OnSuspend;
	char *OnUnSuspend;
	bool SupportSVS;
	bool SupportSVSJoin;
	bool SvsNick;
	bool VHost;
	bool HasOwner;
	bool HasProtect;
	char  OwnerMode;
	char  ProtectMode;
	char  OpMode;
	char  HOpMode;
	char  VMode;
	char  BanMode;
	char  ExceptMode;
	char SupportedUChanModes[MAX_CHAN_USER_MODES];
	char SupportedU2ChanModes[MAX_CHAN_USER_MODES+2];
	char SupportedUserModes[MAX_USER_MODES];
	char *ModeOnNickReg;
	char *ModeOnNickDrop;
	char *ModeOnChannelReg;
	char *ModeOnChannelDrop;
	char RegMode;
	bool ChanRegMode;
	bool NickRegMode;
	bool SupportSGlines;
	bool SupportSQlines;
	bool SupportSZlines;
	bool SupportHalfOp;
	bool SupportBanMode;
	bool SupportExceptMode;
	bool SupportProtectUmode;
	char *ProtectUmode;
	bool SupportInvisibleUMode;
	char *InvisibleUMode;
	bool SupportChanSQlines;
	bool DoesGlobalChangeAfterSVS;
	bool DoesGlobalChangeAfterSA;
	bool DoesQuitAfterKill;
	bool SupportVIdents;
	bool VHostOnNickChange;
	bool SupportRealnameChanges;
	bool SupportAdminOnlyChanMode;
	bool SupportOperOnlyChanMode;
	char *AdminOnlyCMode;
	char *OperOnlyCMode;
	char *DefaultMLOCK;
	char  VhostMode;
	bool SupportFloodMode;
	bool SupportLinkMode;
	char FloodCMode;
	char LinkCMode;
	char LimitCMode;
	char KeyCMode;
	char InviteCMode;
	bool SupportTokens;
	bool SupportSjoin;
	bool SupportSjoinBanprefix;
	bool SupportSjoinExceptprefix;
	char SjoinOwnerPrefix;
	char SjoinProtectPrefix;
	char SjoinOpPrefix;
	char SjoinHOpPrefix;
	char SjoinVoicePrefix;
	char SjoinBanPrefix;
	char SjoinExceptPrefix;
	bool SjoinHasIgnoreChar;
	char SjoinIgnoredChar;
	bool SupportHelperUMode;
	bool SupportSyncState;
	bool p10protocol;
	bool p10new;
   
	void *dl_handle; //Dynamic Library Handle with which the library was opened.
	MOD_BOOL (*func_finish) (void); //Finish function - is called before unloading the module
	Event *events;
   
	PROTOCOL_BOOL (*orion_cmd_initialize) (Threads *thread);
	PROTOCOL_BOOL (*orion_cmd_add_account) (Threads *thread, char *uid, char *authnick);
	PROTOCOL_BOOL (*orion_cmd_del_account) (Threads *thread, char *uid);
	PROTOCOL_BOOL (*orion_cmd_topic) (Threads *thread, char *whosets, char *chan, char *whosetit, char *topic, time_t when);
	PROTOCOL_BOOL (*orion_cmd_kill) (Threads *thread, char *source, char *user, char *buf);
	PROTOCOL_BOOL (*orion_cmd_gline) (Threads *thread, char *user, char *host, char *who, time_t when, time_t expires, char *reason);
	PROTOCOL_BOOL (*orion_cmd_ungline) (Threads *thread, char *unsetter, char *host, char *user, time_t time_in);
	PROTOCOL_BOOL (*orion_cmd_svskill) (Threads *thread, char *source, char *user, char *buf);
	PROTOCOL_BOOL (*orion_cmd_rawsvsmode) (Threads *thread, char *nick, char *target, char *modes);
	PROTOCOL_BOOL (*orion_cmd_svsmode) (Threads *thread, char *nick, char *target, char *modes);
	PROTOCOL_BOOL (*orion_cmd_svs2mode) (Threads *thread, char *nick, char *target, char *modes);
	PROTOCOL_BOOL (*orion_cmd_nick) (Threads *thread, char *nick, char *ident, char *host, char *name, char *modes);
	PROTOCOL_BOOL (*orion_cmd_mode) (Threads *thread, char *source, char *dest, time_t time_in, char *buf);
	PROTOCOL_BOOL (*orion_cmd_rawmode) (Threads *thread, char *source, char *dest, time_t time_in, char *buf);
	PROTOCOL_BOOL (*orion_cmd_kick) (Threads *thread, char *source, char *chan, char *user, char *buf);
	PROTOCOL_BOOL (*orion_cmd_notice) (Threads *thread, char *source, char *dest, char *buf);
	PROTOCOL_BOOL (*orion_cmd_privmsg) (Threads *thread, char *source, char *dest, char *buf);
	PROTOCOL_BOOL (*orion_cmd_serv_notice) (Threads *thread, char *source, char *dest, char *msg);
	PROTOCOL_BOOL (*orion_cmd_serv_privmsg) (Threads *thread, char *source, char *dest, char *msg);
	
	PROTOCOL_BOOL (*orion_cmd_quit) (Threads *thread, char *source, char *buf);
	PROTOCOL_BOOL (*orion_cmd_protoctl) (Threads *thread);
	PROTOCOL_BOOL (*orion_cmd_pass) (Threads *thread, char *pass);
	PROTOCOL_BOOL (*orion_cmd_server) (Threads *thread, char *servname, int hop, char *descript);
	PROTOCOL_BOOL (*orion_cmd_pong) (Threads *thread, char *servname, char *who);
	PROTOCOL_BOOL (*orion_cmd_ping) (Threads *thread, char *servname, char *who);
	PROTOCOL_BOOL (*orion_cmd_join) (Threads *thread, char *user, char *channel, time_t chantime);
	PROTOCOL_BOOL (*orion_cmd_services_join) (Threads *thread, char *user, char *channel, time_t chantime, int fake);

	PROTOCOL_BOOL (*orion_cmd_chghost) (Threads *thread, char *source, char *nick, char *vhost);
	PROTOCOL_BOOL (*orion_cmd_chgident) (Threads *thread, char *source, char *nick, char *vIdent);
	PROTOCOL_BOOL (*orion_cmd_invite) (Threads *thread, char *source, char *chan, char *nick);
	PROTOCOL_BOOL (*orion_cmd_part) (Threads *thread, char *nick, char *chan, char *buf);
	PROTOCOL_BOOL (*orion_cmd_global) (Threads *thread, char *source, char *buf);
	
	PROTOCOL_BOOL (*orion_cmd_squit) (Threads *thread, char *servname, char *message);
	PROTOCOL_BOOL (*orion_cmd_svso) (Threads *thread, char *source, char *nick, char *flag);
	PROTOCOL_BOOL (*orion_cmd_chg_nick) (Threads *thread, char *oldnick, char *newnick);
	PROTOCOL_BOOL (*orion_cmd_svsnick) (Threads *thread, char *source, char *oldnick, char *newnick, time_t when);
	PROTOCOL_BOOL (*orion_cmd_connect) (Threads *thread, bool how);
	PROTOCOL_BOOL (*orion_cmd_svshold) (Threads *thread, char *nick);
	PROTOCOL_BOOL (*orion_cmd_release_svshold) (Threads *thread, char *nick);
	
	PROTOCOL_BOOL (*orion_cmd_sqline) (Threads *thread, char *sender, char *mask, time_t time_in, time_t expire, char *reason);
	PROTOCOL_BOOL (*orion_cmd_unsqline) (Threads *thread, char *servername, char *nick, char *snick);
	
	PROTOCOL_BOOL (*orion_cmd_szline) (Threads *thread, char *servername, char *mask, char *snick, time_t when, time_t expire, char *reason);
	PROTOCOL_BOOL (*orion_cmd_unszline) (Threads *thread, char *servername, char *mask, char *snick, time_t in_time);
	
	PROTOCOL_BOOL (*orion_cmd_unban) (Threads *thread, char *name, char *nick);
	PROTOCOL_BOOL (*orion_cmd_svsmode_chan) (Threads *thread, char *name, char *mode, char *nick);
	PROTOCOL_BOOL (*orion_cmd_svsjoin) (Threads *thread, char *source, char *nick, char *chan, time_t date);
	PROTOCOL_BOOL (*orion_cmd_svspart) (Threads *thread, char *source, char *nick, char *chan);
	PROTOCOL_BOOL (*orion_cmd_sajoin) (Threads *thread, char *source, char *nick, char *chan, time_t date);
	PROTOCOL_BOOL (*orion_cmd_sapart) (Threads *thread, char *source, char *nick, char *chan);
	PROTOCOL_BOOL (*orion_cmd_wallops) (Threads *thread, char *source, char *fmt);
	
	PROTOCOL_BOOL (*orion_cmd_swhois) (Threads *thread, char *source, char *who, char *mask);
	PROTOCOL_BOOL (*orion_cmd_eob) (Threads *thread);
	PROTOCOL_BOOL (*orion_cmd_jupe) (Threads *thread, char *jserver, char *who, char *reason);
	PROTOCOL_BOOL (*orion_cmd_global_legacy) (Threads *thread, char *source, char *fmt);
	PROTOCOL_BOOL (*orion_cmd_raw) (Threads *thread, char *param[]);
	bool (*orion_valid_nick) (Threads *thread, char *nick);
	bool (*orion_valid_chan) (Threads *thread, char *chan);
	bool (*orion_flood_mode_check) (Threads *thread, char *value);
};

#endif
