/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 379                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2010-01-30 12:06:35 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file services.h
 *   @brief This is the main header.
 *
 *   It contains all functions which are used in the main moduls.
 */
/*  ************************************************************************************************  */

#ifndef SERVICES_H
#define SERVICES_H

#ifdef _WIN32
  #ifdef MODULE_COMPILE
    #define DLL __declspec( dllimport )
  #else
    #define DLL __declspec( dllexport )
  #endif
#else
  #define DLL extern
#endif

/* Define our macro's */
#define sfree(x)	if ((x) != NULL) free(x)


/* Some Linux boxes (or maybe glibc includes) require this for the
 * prototype of strsignal(). */
#ifndef _GNU_SOURCE
  #define _GNU_SOURCE
#endif

/*
 * Includes
 ************************
 * Windows does not have:
 * unistd.h, grp.h,
 * netdb.h, netinet/in.h,
 * sys/socket.h, sys/time.h
 *
 * Windows requires:
 * winsock.h
 ************************/
 
#if INTTYPE_WORKAROUND
# undef int16
# undef int32
#endif
 
#include "sysconf.h"
#include "global.h"
#include "datatypes.h"

#include <stdio.h>
#include <signal.h>					/* signal Handler */
#include <time.h>
#include <errno.h>
#include <limits.h>
#include <stdarg.h>
#include <ctype.h>

#ifdef HAVE_SYS_TYPES_H
  #include <sys/types.h>
#endif

#ifdef HAVE_SYS_STAT_H
  #include <sys/stat.h>	/* for umask() on some systems */
#endif

#ifdef HAVE_STDLIB_H
  #include <stdlib.h>
#endif

#ifdef HAVE_STRING_H
  #include <string.h>
#endif

#ifdef HAVE_MALLOC_H
  #include <malloc.h>
#endif

#ifdef USE_THREADS
  #include <pthread.h>				/* Thread Library (Maybe should be passed with Win32 conform Paths) */
#endif

#ifndef _WIN32
  #include <netdb.h>
  #include <netinet/in.h>
  #include <arpa/inet.h>
  #include <sys/socket.h>
  #include <sys/time.h>
  #include <sys/un.h>
  #include <grp.h>
  #include <dirent.h>
  
  #ifdef HAVE_UNISTD_H
    #include <unistd.h>
  #endif
  
  #ifdef HAVE_BACKTRACE
    #include <execinfo.h>
  #endif
  
  #define mygetpid()			getpid()
#else
  #include <winsock2.h>
  #include <windows.h>
  #include <sys/timeb.h>
  #include <direct.h>
  #include <io.h>
  
  #define vsnprintf             _vsnprintf
  #define mygetpid()			GetCurrentProcessId()
  #define kill(x,y)				raise(y)
#endif

#ifdef USE_MYSQL
  #ifdef MYSQL_HEADER_PREFIX
    #include <mysql/mysql.h>
  #else
    #include <mysql.h>
  #endif
#endif

#ifdef USE_THREADS
  #define THREAD_SAFE mysql_thread_safe()
  #define MYSQL_CONNECTION_HANDLE &mysql
#else
  #define THREAD_SAFE 0
  //#define MYSQL_CONNECTION_HANDLE &thread->mysql
  #define MYSQL_CONNECTION_HANDLE &mysql
#endif

#ifndef HAVE_BZERO
  #define bzero(x,y) memset(x,0,y)
#endif

/* Some SUN fixs */
#ifdef __sun
/* Solaris specific code, types that do not exist in Solaris'
 * sys/types.h
 **/
  #undef u_int8_t
  #undef u_int16_t
  #undef u_int32_t
  #undef u_int64_t
  #define u_int8_t uint8_t
  #define u_int16_t uint16_t
  #define u_int32_t uint32_t
  #define u_int64_t uint64_t

  #ifndef INADDR_NONE
    #define INADDR_NONE (-1)
  #endif
#endif

#ifdef __WINS__
#ifndef BKCHECK
#define BKCHECK
  extern "C" void __pfnBkCheck() {}
#endif
#endif

#define is_user 			0
#define is_services_user 	1
#define is_services_oper 	2
#define is_services_admin 	3
#define is_services_root 	4

#define VERSION "orion_xp2.9-stable.rc1.3[$Rev: 379 $]"
#define VERSION_NUMBER "2.9-stable.rc1.3[$Rev: 379 $]"
#define PROJECTNAME "orion"
#define MAINPROJECT "deepServices"
#define YEAR "2004-2010"

#define SQLrow MYSQL_ROW
#define SQLres MYSQL_RES

/************************ STRUCTS FOR TRUST-SESSIONSYSTEM******************************/
/* This is caused by MySQL - since MySQL isn't fast enough to run so many SELECTS due to
 * many users are connecting very fast (in case of ServerLinks for example)
 * So we had to manage some Lists to dump Information into on starting orion.
 * The dumps for this Lists come in a period of ~5minutes, or by updating through the
 * trust command @ O
 */

DLL void add_Trust(char *host, char* description, char* limit, Trust_art art, int fields);
DLL void add_Trustgroup(char *group, long limit, int fields);
DLL bool del_Trust(Trusts *trust);
DLL int del_Trust_by_Group(char *group, Trust_art T_art);
DLL int edit_Trust_by_Group(char *group, char *newgroup, Trust_art T_art);
DLL bool del_Trustgroup(Trustgroups *trustgroup);
DLL void DelTrustgroups(void);
DLL void DelTrusts(void);
DLL void GetTrustsSql(Threads *thread);
DLL void GetTrustgroupsSql(Threads *thread);
DLL Trusts *getTrust(char *host, Trust_art T_art);
DLL Trustgroups *getTrustgroup(char *group);
DLL long getHostLimit(char *host);

/**************************************************************************************/

void EndThread(void *in_thread);
bool KillThread(Threads *thread);
void NewThread(char *whatfor, void *(*func) (void *thread_in));
void RunThread(Threads *thread, void *(*func) (void *thread_in));
Threads *NewTData(char *whatfor);
DLL Threads *SeekThread(char *index);
#ifdef USE_THREADS  
  DLL int my_pthread_mutex_unlock(pthread_mutex_t *mutex);
  DLL int my_pthread_mutex_lock(pthread_mutex_t *mutex);
#endif

/**************************************************************************************/

/* Server data */

DLL Servers *findserver(Threads *thread, Servers * s, char *name);
DLL Servers *findserver_uid(Threads *thread, Servers * s, char *name);
DLL Servers *findserver_next(Threads *thread, Servers * s, char *name);
DLL Servers *new_server(Threads *thread, Servers * uplink, char *name, char *desc, short int flags, char *suid);
DLL void server(Threads *thread, char *server, char *uplink, short int hopcount);
DLL Servers* addServer(Threads *thread, char *source, char *servername, char *hops, char *descript, char *numeric);
DLL void ts6_uid_init(void);
DLL void ts6_uid_increment(unsigned int slot);
DLL char *ts6_uid_retrieve(void);

/*************************************************************************/

DLL Uchannels *GetUserChannel(Users *user, char *channel);
DLL Users *GetUser(char *nick);
DLL Users *GetUserByUID(char *uid);
DLL int GetUsersInChannel(char *channel);
DLL char *GetNewID(char *id);
DLL char *GetUserLastUID(char *sid);
DLL int GetUserIntID(char *id);

DLL bool DelUser(char *channel, char *nick);
DLL bool DelUserNick(char *nick);
DLL bool DelUserServer(char *server);
DLL bool DelChannelsForUser(char *nick);

DLL Users *ChangeUser(char *oldnick, char *newnick);
DLL Users *addUser(char *nick, char *server);
DLL Uchannels *addUserChannel(Users *user, char *channel);

DLL void PartAllChannels(Users *user);
DLL Uchannels *PartChannel(Users *user, char *channel);
DLL Uchannels *PartChannelNick(char *nick, char *channel);
//Uchannels *addUserChannelMore(Users *user, char *channel, char *lastmsg);

/**************************************************************************************/

DLL Ulines *addUline(Threads *thread, char *in_server);
DLL Ulines *getUline(Threads *thread, char *in_server);
DLL bool delUline(Threads *thread, char *in_server);
DLL bool delAllUlines(Threads *thread);

/**************************************************************************************/

int getModeNick(UserChanModes *ucmodes, size_t ucmodes_count, char *nick);
bool multiply_occurance(int nr, unsigned int max, char *param[]);

/**************************************************************************************/

DLL LevelChanging GetLevelvalueByIndex(LevelChanging *leveltable, char *index);
void FillLevel(LevelChanging *level, char *index, int type, char *CSlevel, int XOPlevel, char *Qlevel);

/**************************************************************************************/

DLL void addLimit(Threads *thread, char *channel, time_t date, bool flag, char* serv, long alimit);
DLL bool delLimit(Threads *thread, Limits *limit);
void PerformLimits(Threads *thread);
void delLimits(Threads *thread);

/**************************************************************************************/

//CServe.c
void do_backtrace(Threads *thread, bool show_segheader, int steps);
DLL void _orion_exit(void);
DLL void orion_exit(Threads *thread);
void remove_pidfile(void);
#ifdef _WIN32
DLL extern int serverid;
#else
DLL int serverid;
#endif

//movedb.c
void anope_to_orion(Threads *thread, bool blob);
void anopeFiles_to_orion(Threads *thread);


//mail.c
DLL MailInfo *o_email(Threads *thread, char *sender, char *recipient, char *email_address, char *subject);
DLL void c_email(Threads *thread, MailInfo *mail);
DLL void s_email(Threads *thread, void *language, FILE *pipe, char *message, ...);
DLL int MailValidate(const char *email);

//Stay.c
/**************************************************************************************/

DLL void addStayChannel(Threads *thread, char *channel, char *botnick, int bot_joined, int cs_had_to_join);
DLL bool delStayChannel(Threads *thread, char *channel);
DLL bool delStayNick(Threads *thread, char *nick);
DLL Stays *getStayChannel(Threads *thread, char *channel);
void PerformStay(Threads *thread);
DLL void DeleteStay(Stays *stay);

/**************************************************************************************/

//Base64.c
DLL const char *inttobase64(char *buf, unsigned int v, unsigned int count);
DLL unsigned int base64toIP(char *s);

//services.c
DLL bool check_expire(char *expirestring);
DLL time_t expire(char *time_in);
DLL void randomstring(char *array, size_t stellen);
DLL void randomnumber(char *array, size_t stellen);
DLL char *get_localtime(time_t time_in);

DLL bool deopall(Threads *thread, char *service, char *channel);
DLL bool unbanall(Threads *thread, char *service, char *channel);
DLL bool devoiceall(Threads *thread, char *service, char *channel);
DLL bool dehopall(Threads *thread, char *service, char *channel);
DLL void defconlev(Threads *thread, int defconlevel);
DLL bool do_mlock(Threads *thread, char *snick, char *channel, time_t date, char *modes, char *buf);

void get_last_function(char *ret, int steps);
DLL int NSCheckSettings(void);
DLL int BSCheckSettings(void);
DLL void CSCheckBot(char *newbot, char *bot, long set, size_t size);
DLL void QCheckBot(char *newbot, char *bot, long set, size_t size);
DLL char *CSCheckBotInPlace(char *bot, long set);
DLL char *QCheckBotInPlace(char *bot, long set);
DLL int CSCheckSettings(void);
DLL int CSAddBanType(int settings);
DLL int CSAnalyseBantype(int settings);
DLL int CSAnalyseSignKick(int settings);
DLL char *BuildBanMask(Threads *thread, Condata *Cdata, int type);
DLL void unknown_access(char *access, size_t len);
DLL void UpdateChannelInformation(Threads *thread,char *channel,char *target);
DLL void UpdateChannelInformation2(Threads *thread,char *channel,char *target);
DLL void UpdateChannelInformation3(Threads *thread,char *channel,char *target);

//log.c
#ifdef _WIN32
  DLL int gettimeofday(struct timeval *tv, struct timezone *tz);
#endif
DLL int get_logdate(char *date, int32 count, struct tm *tm);
bool open_logs(Threads *thread);
void close_logs(Threads *thread, char *lastmsg);
void remove_logs(Threads *thread);

bool open_all_logs(void);
void close_all_logs(char *message, ...);
void remove_all_logs(void);

DLL void alog(Threads *thread, const char *message, ...);
DLL void sqllog(Threads *thread, const char *message, ...);
DLL void dlog(Threads *thread, int priority, const char *message, ...);

//encrypt.c
DLL int orion_encrypt(const char *src, size_t len, char *dest, size_t size);
DLL int orion_encrypt_in_place(char *buf, size_t size);
DLL int orion_encrypt_check_len(size_t passlen, size_t bufsize);
DLL int orion_decrypt(const char *src, char *dest, size_t size);
DLL int orion_check_password(const char *plaintext, const char *password);
DLL char* orion_check_encryption(void);

DLL int orion_encrypt_specific(int index, const char *src, size_t len, char *dest, size_t size);
DLL int orion_encrypt_in_place_specific(int index, char *buf, size_t size);
DLL int orion_encrypt_check_len_specific(int index, size_t passlen, size_t bufsize);
DLL int orion_decrypt_specific(int index, const char *src, char *dest, size_t size);
DLL int orion_check_password_specific(int index, const char *plaintext, const char *password);
DLL char* orion_check_encryption_specific(int index);

void binary_to_hex(unsigned char *bin, char *hex, size_t length);

//string.c
DLL void CleanSocketString(char *string, size_t strsize);
DLL void strclean(char *receive);
DLL void c_lowercase(char *string);

#ifdef USE_THREADS
  #define _lc(x) 		lc(thread,x)
  #define _uc(x) 		uc(thread,x)

  DLL char* lc(Threads *thread, char *str);
  DLL char* uc(Threads *thread, char *str);
#else
  #define _lc(x) 		lc(x)
  #define _uc(x) 		uc(x)

  DLL char* lc(char *str);
  DLL char* uc(char *str);
#endif

DLL void strunshift(char *addchar, char *string, size_t strsize);

/* string manipulations */


DLL void strrepc(char *search_string, char nower, char newchar); //Replaces nower(char) with newchar(char) in the *search_string

DLL void strrepcof(char *search_string, char nower, char newchar); //Replaces nower(char) with newchar(char) in the *search_string but it replaces only the first found nower.

DLL size_t strsplit(char *search_string, char *param[], char repchar); //splits search_string to a *param[] array by the occurance of repchar (it's deleted), returns int (number of array items)


DLL char *sstrdup(const char *s); //duplicates a string
DLL void sort(char *string); //Sorts the string and overwrites it with the sorted result!
/************************/


/* Delete from a string */
DLL void substr(char *string, size_t position);
DLL void strdelc(char *search_string, char delchar);
DLL void strdelcof(char *search_string, char delchar);
DLL void strdels(char *search_string, char search, char *delchar);
/************************/

/* Add to a string */
DLL void straddc(char *string, char addchar); //Adds addchar to the end of the already existing *string
DLL void stradds(char *string, char *addstring); //Adds *addstring to the already existing *string (make sure there's enough space)
DLL void straddsni(char *string, char *addstring); //Adds *addstring to the already existing *string (make sure there's enough space)
/************************/

/* Add to a string (SAFE LIBRARY) */
DLL void straddnc(char *string, char addchar, size_t strsize); //Adds addchar to the end of the already existing *string
DLL void straddns(char *string, char *addstring, size_t strsize); //Adds *addstring to the already existing *string (make sure there's enough space)
DLL void straddnsni(char *string, char *addstring, size_t strsize); //Adds *addstring to the already existing *string (make sure there's enough space)
/************************/

/* Equalize strings */
DLL bool streq(char *str1, char *str2); //case equal
DLL bool streqnc(char *str1, char *str2); //no case equal
/********************/

/* Search in a String */
DLL bool strcor(char *string, char *chr); //case - strcor("abcdefghi","hko"); - return 1 cause of  "h" isn *string
DLL bool stricor(char *string, char *chr); //no case - strcor("abcdefghi","HKO"); - return 1 cause of  "H/h" isn *string

DLL bool strcin(char chr, char *str); //case - chr is in *str
DLL bool stricin(char chr, char *str); //no case - chr is in *str

DLL bool strins(char *string_to_search, char *search_string); //case - *string_to_search is in *search_string
DLL void strmatch(char *string_to_search, char *search_string, char *stringmatch, size_t strmatchsize);
DLL bool strinsnc(char *string_to_search, char *search_string); //no case - *string_to_search is in *search_string

DLL bool str_start(char *instring, char *parameter);
DLL bool str_end(char *instring, char *parameter);
DLL uint32 str_is_ip(char *str);

DLL bool strcsea(char *string, char *search_in); //case - strcsea("abcdefghi","a|b|c&d|e|f"); if (a b or c) and (d e or f) is in *string

DLL int stoi(char* convert);
DLL long stol(char* convert);

DLL char *myStrGetToken(const char *str, const char dilim, int token_number);
DLL char *myStrGetOnlyToken(const char *str, const char dilim, int token_number);
DLL char *myStrGetTokenRemainder(const char *str, const char dilim, int token_number);
DLL char *myStrSubString(const char *src, int start, int end);
DLL char *str_signed(unsigned char *str);
/**********************/
DLL int maxValue(int a, ...);
DLL int minValue(int a, ...);
DLL int findchar(char *mode_string, char findchar);
DLL int getoccurance_x(char *mode_string, char findchar, int x);
DLL int getposition(char *mode_string, char findchar);

/* string proofings and char manipultions */
DLL char lower(char value);
DLL bool digitcol(char *string);
DLL bool alphacol(char *string);
DLL bool is_valid_chars(char *text, char *allowed);
/******************************************/

#ifdef _WIN32
DLL char *
strtok_r(
	register char *s,
	register const char *delim,
	char **lasts);
#endif

//dynamic.c
DLL void dynsplice(char *value[], size_t x, size_t y);
DLL bool dynunshift(char *value[], char *string, size_t x, size_t size);
DLL void reverse(char *target, char *source);
DLL void join(char *receive, char *param[], char inputchar); /* You shouldn't use that function - it's unsafe */
#ifdef USE_THREADS
  #define _djoin(w,x) 				djoin(thread,w,x)
  #define _dstrjoin(w,x,y) 			dstrjoin(thread,w,x,y)
  #define _dstrjoin_det(w,x,y,z) 	dstrjoin_det(thread,w,x,y,z)
  #define _dynadd(...) 				dynadd(thread, __VA_ARGS__)

  DLL char* djoin(Threads *thread, char inputchar, char *param[]);
  DLL char* dstrjoin(Threads *thread, char inputchar, char *param[], size_t begin);
  DLL char* dstrjoin_det(Threads *thread, char inputchar, char *param[], size_t begin, size_t end);
  DLL char* dynadd(Threads *thread, const char *text, ...);
#else
  #define _djoin(w,x) 				djoin(w,x)
  #define _dstrjoin(w,x,y) 			dstrjoin(w,x,y)
  #define _dstrjoin_det(w,x,y,z) 	dstrjoin_det(w,x,y,z)
  #define _dynadd(...) 				dynadd(__VA_ARGS__)

  DLL char* djoin(char inputchar, char *param[]);
  DLL char* dstrjoin(char inputchar, char *param[], size_t begin);
  DLL char* dstrjoin_det(char inputchar, char *param[], size_t begin, size_t end);
  DLL char* dynadd(const char *text, ...);
#endif
DLL size_t dynlen(char *string);
DLL void dynncat(char *source, char *data, size_t len);
DLL void dyncpy(char *target, char *source);
DLL void dynncpy(char *target, char *source, size_t length);
DLL char* dyndup(char *source);
DLL size_t arraysize(char *param[]);

//misc.c
#ifdef _WIN32
char *GetWindowsVersion(void);
int SupportedWindowsVersion(void);
#endif

//mysql.c
DLL void mysql_connect(Threads *thread);
DLL void mysql_disconnect(Threads *thread);
DLL void mysql_free_result_myr(SQLres *res);
DLL int mysql_num_rows_myr(SQLres *res);

DLL char* mysql_quote(Threads *thread, char *sql);
DLL void mysql_free_quotes(Threads *thread);
DLL char *db_mysql_quote_buffer(char *sql, int size);
DLL char *db_mysql_secure(char *pass, int size);
DLL void ExecuteQuery(Threads *thread, char *query, size_t len);
DLL void ExecuteQuery_r(Threads *thread, char *query, size_t len);
DLL char *InlineExecuteQuery_r(Threads *thread, char *query, size_t len);

DLL char* mysql_sel_var(Threads *thread, const char *text, ...);
DLL void domysql(Threads *thread, const char *text, ...);
DLL int mysql_free(void);
DLL void mysql_sel(Threads *thread, char *nick, char *text);

DLL void sql_BsChan(Threads *thread, char *nick, char *channel, BsChan *bsChan);
DLL char* sql_BsChan2(Threads *thread, char *nick, char *target, char *channel, BsChan *bsChan);

DLL void sql_chanmodes(Threads *thread, char *channel, Channel *c, ChanLevels *cLev);
DLL ChanCA sql_chandata(Threads *thread, char *channel,char *authnick,char *target);
DLL ChannelDB sql_chan_db(Threads *thread, char *channel);
DLL DB3 sql_chan_data(Threads *thread, char *starttype, int parameters, ...);
DLL DB4 sql_thread_data(Threads *thread, char *starttype, int parameters, ...);
DLL Channel sql_chan(Threads *thread, char *channel);
DLL ChanLevels sql_chanlevels(Threads *thread, char *channel);

DLL void sql_mainAAC(Threads *thread, char *nick);
DLL Access sql_access(Threads *thread, char *authnick);
DLL Aliases sql_aliases(Threads *thread, char *aliasnick);
DLL Bassign sql_assign(Threads *thread, char *channel);
DLL Bkick sql_bkick(Threads *thread, char *channel);
DLL Acclist sql_acclist(Threads *thread, const char *serv, char *channel, char *authnick);
DLL Condata sql_condata(Threads *thread, char *nick);

DLL int count(Threads *thread, const char *serv, char *authnick);
DLL char* currmodes(Threads *thread, char *nick);
DLL char *sql_users(Threads *thread, char *chan, char *nick);
DLL char* acclevel(Threads *thread, char *serv, char *channel, char *authnick);
DLL char* accessselect(Threads *thread, const char *field, char *authnick);
DLL char* selectaliases(Threads *thread, const char *field, char *nnick);
DLL int SingleLevel(Threads *thread, const char *field, char *channel);
DLL Trust O_sql_trust(Threads *thread, char *host);
DLL QUserlist *Q_userlist(Threads *thread, char *pattern_in);
DLL SQLres *doSelectMySQL(Threads *thread, const char *text, ...);

DLL SQLres *AkickEnforce(Threads *thread, char *channel);
DLL SQLres *AkickOnJoin(Threads *thread, char *channel, char *nick);
DLL SQLres *AkickOnSJoin(Threads *thread, char *channel, char *nicks);

DLL size_t doBufferedInsert(Threads *thread, bool duplicate_update, size_t directtable, const char *input, ...);
DLL void MySQL_FlushInserts(Threads *thread, size_t begin, size_t end);
DLL void MySQL_CleanInserts(size_t begin, size_t end);

DLL void DropAuthnick(Threads *thread, char *authnick);

//modules.c
size_t getmodules(Threads *thread);

//config.c
DLL void loadconf(Threads *thread, char *modul);
DLL void update_config(Threads *thread, char *modul);
DLL void moduleLoadconfig(Threads *thread, ModCo *module, char *configfile);
void dumpconf_mysql(Threads *thread);
DLL bool proof_config(Threads *thread);
DLL void zero_config_values(Threads *thread);
void main_build(Threads *thread, char *desc,char *value);

//protocol.c & ptcsupport.c
DLL bool LoadProtocol(Threads *thread, Protocol *protocol, char *IRCd);
DLL bool on_protocol (Threads *thread, Protocol *protocol, size_t ac, char *param[]);

DLL void CreateProtocol(Threads *thread, Protocol *protocol, void *dl_handle);
DLL void event_init    (Threads *thread, Protocol *protocol, void *dl_handle, char *Author, char *Version);

DLL void createEvent(Threads *thread, Protocol *protocol,
	const char *call,
	MOD_BOOL (*func) (Threads *thread, size_t ac, char **av),
	size_t check_param
	);
	
//sharedlanguage.c
DLL void InitializeSharedLanguages(void);
DLL bool addModuleLanguage(Threads *thread, ModInfo *ModHandle, Languages lang, char *item, char *value);
DLL char *GetSharedLangstring(Threads *thread, int language, char *message);
DLL void delSharedLanguage(Threads *thread, char *modulefile);

/* Threads */
DLL void *ValidateNicks (void *in_thread);
#endif
