/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file bs_events.c
 *   @brief This is the Module file for BotServ EVENTS Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the BotServ EVENTS command.
 *
 *   Main functions - EVENTS ONLY
 *
 *   Syntax is: /msg BotServ EVENTS
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "webform.h"
#include "global.h"
#include "webform.h"

#define SERVICE "bs"
#define MODUL "events"
#define MODUL_FILE "bs_events"
#define MODUL_VERSION "1.2"

/*  ******************************************* Event List *****************************************  */

MOD_BOOL bs_on_message(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_on_channelmessage(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_on_protoctl(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_on_join(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL bs_primary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_secondary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_squit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

/*  **************************************** End of Event List **************************************  */

MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,EVENT_PROTOTYPE,EVENT_PROTOTYPE);

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addAccess (thread,&Module,NOONE);
	addMODType(thread,&Module,EVENT);

	addEventHook(thread,&Module,"on_message"		,bs_on_message);
	addEventHook(thread,&Module,"on_channelmessage"	,bs_on_channelmessage);
	addEventHook(thread,&Module,"on_protoctl"		,bs_on_protoctl);
	addEventHook(thread,&Module,"on_join"			,bs_on_join);

	addEventHook(thread,&Module,"on_connect_primary"	,bs_primary);
	addEventHook(thread,&Module,"on_connect_secondary"	,bs_secondary);
	addEventHook(thread,&Module,"on_squit"				,bs_squit);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	
/* Things we want to process "on load" */
	bs->loaded = 1;
	
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the BotServ PRIMARY Command.
 * 
 * The BotServ PRIMARY Command allows the user....
 *  
 * Syntax is: /msg BotServ PRIMARY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_primary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	deepSQL bots;
	
    do_kill(thread,m->servername,bs->nick,"nick used by orion");
   	do_sqline(thread,m->servername,bs->nick,0,time(NULL),"nick used by orion");
	createuser(thread,FAKETYPE_SERVICE,bs->nick,bs->ident,bs->host,IRC.BSMode,bs->realname);
    
	bots.res    = doSelectMySQL(thread,"SELECT `botnick`,`botident`,`bothost`,`botrealname`,`private`,`time` FROM `orion_bs_core`");
	bots.fields = mysql_num_rows(bots.res);

	if(bots.fields){
		while((bots.row = mysql_fetch_row(bots.res))){
			if(bots.row[0]!=NULL){	
				createuser(thread,FAKETYPE_BOT,bots.row[0],bots.row[1],bots.row[2],IRC.BSBotsMode,bots.row[3]);
			}
	    }
	}
    mysql_free_result_myr(bots.res);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the BotServ SECONDARY Command.
 * 
 * The BotServ SECONDARY Command allows the user....
 *  
 * Syntax is: /msg BotServ SECONDARY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_secondary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	//printf("INPUT - bs_secondary\n");
	deepSQL channels;
	
	if(m->logchan[0] != '\0'){
		do_services_join(thread,bs->nick,m->logchan,time(NULL),FAKETYPE_SERVICE);
	}
    
	channels.res    = doSelectMySQL(thread,"SELECT `botnick` , `orion_cs_core`.`channel` , `datecreate`, SUM(IF(`orion_chan_users`.`nick`=`orion_cs_core`.`botnick`,1,0)) as `botinchan`, COUNT(`orion_chan_users`.`nick`) as `count_all`, (COUNT(`orion_chan_users`.`nick`)-SUM(IF(`orion_chan_users`.`nick`=`orion_cs_core`.`botnick`,1,0))) as `count_normal_users`, `orion_cs_core`.`settings` FROM `orion_cs_core` LEFT JOIN `orion_chan_users` ON `orion_chan_users`.`channel` = `orion_cs_core`.`channel` WHERE `orion_cs_core`.`botnick` IS NOT NULL GROUP BY `orion_cs_core`.`channel`");
	channels.fields = mysql_num_rows(channels.res);

	if(channels.fields){
		while((channels.row = mysql_fetch_row(channels.res))){
			if(
				channels.row[0][0]!='\0' &&
				((~stol(channels.row[6])&CS_INHABIT_BIT && stol(channels.row[5])>=m->BSMinUsers) || (stol(channels.row[6])&CS_INHABIT_BIT && stol(channels.row[5])>=m->BSMinUsers+1)) &&
				stol(channels.row[3])==0 &&
				~stol(channels.row[6])&CS_SUSPEND_BIT
			){
				do_services_join(thread,channels.row[0],channels.row[1],stol(channels.row[2]) == 0 ? time(NULL) : (time_t)stol(channels.row[2]),FAKETYPE_BOT);
			}
	    }
	}
    mysql_free_result_myr(channels.res);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the BotServ SQUIT Command.
 * 
 * The BotServ SQUIT Command allows the user....
 *  
 * Syntax is: /msg BotServ SQUIT
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_squit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * bs_on_protoctl: BotServ Events: On Protoctl Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL bs_on_protoctl(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * bs_on_join: BotServ Events: On Join Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL bs_on_join(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	//@param - [0](nick), [1](channel), [2](time)
	/* Deklarate Variables */
	//DB3 c;
	deepSQL Akick;
	Stays *stay = NULL;
	bool csjoined = false;
	bool botjoined = false;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	bzero(&Akick,sizeof(Akick));
	csjoined = 0;
	Users *user = GetUser(nick);
	
	/*
	 * If it's a BotServ channel (Registered from ChanServ)
	 */
	if(user==NULL)
	    return MOD_CONT;
	
	/* Updates the Threads Channel Management - MAKE ME TARGET HERE! */
	if(user->rights>=is_services_user && !thread->c.aliases.active){
		thread->c = sql_thread_data(thread,param[1],SQL_THREAD_AAC2,nick);
	}else
		UpdateChannelInformation2(thread,param[1],nick);
	
	if(thread->c.chan.fields && streqnc(thread->c.chan.serv,"chanserv") && thread->c.chan.bot[0]!='\0'){
		thread->c.chan.datecreate = (time_t)stol(param[2]);
		
		/* We have to part ChanServ if we joined it before */
		stay = getStayChannel(thread, thread->c.chan.name);
		
		if(stay != NULL){
			if(thread->c.chan.countusers >= 2 && stay->csjoin)
				do_part(thread, cs->nick, stay->channel,NULL);
		}

		if(thread->c.chan.countusers > m->BSMinUsers)
			delStayChannel(thread,thread->c.chan.name);
		/***************************************************/

		/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
		if(thread->c.chan.settings&CS_INHABIT_BIT){
			Users 		*my_chanserv 		= GetUser(cs->nick);
			Uchannels 	*my_chanserv_channel= NULL;

			if(my_chanserv!=NULL)
				my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

			/* WOOHOOO ChanServ isn't in the Channel yet - if we will set a ban now, it will disappear without any user ...
			 and since ChanServ should be a channel inhabitant according to the channel settings -> it joins the channel here ;) */
			if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
				do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
				do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
			}
		}

		/* ChanServ isn't inhabitant, so if there is only one user in the channel (the one we want kick now)
		 we have to join the channel to make the ban "permanent" */
		else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers == m->BSMinUsers && getStayChannel(thread,thread->c.chan.name)==NULL && !thread->c.chan.botinchan){
			thread->c.chan.botinchan=1;
			botjoined=true;
			do_join(thread,FAKETYPE_BOT,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate);
        	do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",thread->c.chan.bot,thread->c.chan.bot,thread->c.chan.bot);
		}
		
		if(~thread->c.chan.botserv.set&BS_SET_SYMBIOSIS_BIT)
			dynncpy(thread->c.chan.bot,cs->nick,sizeof(thread->c.chan.bot));

		//if(thread->c.aliases.fields && thread->c.aliases.authnick[0]!='\0'){
		//	if(!thread->c.aliases.active) unknown_access(thread->c.chan.access.ownlevel,sizeof(thread->c.chan.access.ownlevel));
		//}
		
		//dlog(thread,2,"thread->c.chan.countusers = %d\n",thread->c.chan.countusers);
		
/* MLOCK FORCE */
			int j=0;
			if(~thread->c.chan.settings&CS_INHABIT_BIT && botjoined && thread->c.chan.countusers == 2 && thread->c.chan.serv[0]!='\0' && (thread->c.chan.mlock_positive[0]!='\0' || thread->c.chan.mlock_negative[0]!='\0')){
				char modes		 [200];
				char search		 [70];
				
				bzero(&modes,sizeof(modes));
				bzero(&search,sizeof(search));

				straddnc(search,'+',sizeof(search));
				if(!strcin(IRC.RegMode,thread->c.chan.mlock_positive) && IRC.ChanRegMode)
					straddnc(search,IRC.RegMode,sizeof(search));
					
				straddns(search,thread->c.chan.mlock_positive,sizeof(search));
				sort(search);
				straddnc(search,'-',sizeof(search));
				sort(thread->c.chan.mlock_negative);
				straddns(search,thread->c.chan.mlock_negative,sizeof(search));

				dynncpy(modes,search,sizeof(modes));

				while(search[j]!='\0'){
					if(search[j]==IRC.LimitCMode){
						straddnc(modes,' ',sizeof(modes));
						straddns(modes,_dynadd("%ld",thread->c.chan.mlock_limit),sizeof(modes));
					}
					else if(search[j]==IRC.FloodCMode){
						straddnc(modes,' ',sizeof(modes));
						straddns(modes,thread->c.chan.mlock_flood,sizeof(modes));
					}
					else if(search[j]==IRC.KeyCMode){
						straddnc(modes,' ',sizeof(modes));
						straddns(modes,thread->c.chan.mlock_key,sizeof(modes));
					}
					j++;
				}

				if(modes[0]!='\0') do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate == 0 ? time(NULL) : thread->c.chan.datecreate,"%s",modes);
			}
			else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.serv[0]!='\0' && thread->c.chan.countusers == 2 && IRC.ChanRegMode){
				do_mode(thread, thread->c.chan.bot, thread->c.chan.name, thread->c.chan.datecreate == 0 ? time(NULL) : thread->c.chan.datecreate, IRC.ModeOnChannelReg);
				/* This is to avoid channel desyncs */
				//domysql(thread,"UPDATE `orion_cs_core` SET `datecreate`='%ld',`mode`='%s',`link`='%s',`limit`='%d',`key`='%s',`flood`='%s' WHERE `channel` = '%s'",thread->c.chan.datecreate,mysql_quote(thread,thread->c.chan.link),thread->c.chan.limit,mysql_quote(thread,thread->c.chan.key),mysql_quote(thread,thread->c.chan.flood),mysql_quote(thread,thread->c.chan.name));
			}else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.serv[0]!='\0' && thread->c.chan.countusers == 2 && ~thread->c.chan.settings&CS_SUSPEND_BIT && thread->c.chan.settings&CS_KEEPTOPIC_BIT && thread->c.chan.last_topic[0]!='\0')
				do_topic(thread, thread->c.chan.bot, thread->c.chan.last_topic_setter[0]=='\0' ? thread->c.chan.bot : thread->c.chan.last_topic_setter, thread->c.chan.name, time(NULL), thread->c.chan.last_topic);
/* MLOCK FORCE END */
		
		
		if(thread->c.chan.name && (thread->c.chan.settings & CS_SUSPEND_BIT || thread->c.chan.settings & CS_FORBID_BIT) && thread->sql.access.rights<=is_services_admin){ //SO required
			/* Make the ban stay in channel - We proof the List for such channels later */

			/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
			if(thread->c.chan.settings&CS_INHABIT_BIT){
				Users 		*my_chanserv 		= GetUser(cs->nick);
				Uchannels 	*my_chanserv_channel= NULL;

				if(my_chanserv!=NULL)
					my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

				/* WOOHOOO ChanServ isn't in the Channel yet - if we will set a ban now, it will disappear without any user ...
				 and since ChanServ should be a channel inhabitant according to the channel settings -> it joins the channel here ;) */
				if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
					do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
					do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
				}
			}

			/* ChanServ isn't inhabitant, so if there is only one user in the channel (the one we want kick now)
			 we have to join the channel to make the ban "permanent" */
			else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers == m->BSMinUsers && !thread->c.chan.botinchan && !csjoined && getStayChannel(thread,thread->c.chan.name)==NULL){
				do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
				do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
				csjoined=1;
			}
			
			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,IRC.OnSuspend);
			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",BuildBanMask(thread,&thread->c.Cdata,CSAnalyseBantype(thread->c.chan.settings)));
			
			if(thread->c.chan.settings & CS_SUSPEND_BIT)	 do_kick_nopart(thread,thread->c.chan.bot,thread->c.chan.name,nick,"suspended [%s]",thread->c.chan.reason);
			else if(thread->c.chan.settings & CS_FORBID_BIT) do_kick_nopart(thread,thread->c.chan.bot,thread->c.chan.name,nick,thread->c.chan.reason[0]=='\0' ? "Forbidden" : thread->c.chan.reason);
			//thread->c.chan.countusers--;
		}else if(!streqnc(nick,cs->nick)){
			
			if(stoi(thread->c.chan.access.targetlevel)<=IDENTIFIED_NO_ACCESS && (thread->c.chan.settings&CS_RESTRICTED_BIT)){
				/* Make the ban stay in channel - We proof the List for such channels later */

				/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
				if(thread->c.chan.settings&CS_INHABIT_BIT){
					Users 		*my_chanserv 		= GetUser(cs->nick);
					Uchannels 	*my_chanserv_channel= NULL;

					if(my_chanserv!=NULL)
						my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

					/* WOOHOOO ChanServ isn't in the Channel yet - if we will set a ban now, it will disappear without any user ...
					 and since ChanServ should be a channel inhabitant according to the channel settings -> it joins the channel here ;) */
					if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
						do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
						do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
					}
				}

				/* ChanServ isn't inhabitant, so if there is only one user in the channel (the one we want kick now)
				 we have to join the channel to make the ban "permanent" */
				else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers == m->BSMinUsers && !thread->c.chan.botinchan && !csjoined && getStayChannel(thread,thread->c.chan.name)==NULL){
					do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
					do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
					csjoined=1;
				}
				
				do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",BuildBanMask(thread,&thread->c.Cdata,CSAnalyseBantype(thread->c.chan.settings)));
				do_kick_nopart(thread,thread->c.chan.bot,thread->c.chan.name,nick,"You are not permitted to be on this channel.");
				//thread->c.chan.countusers--;
			}
			
			else{
				/* ******************************* AKick Check **************************************** */
				Akick.res    = AkickOnJoin(thread,param[1],nick);
				Akick.fields = mysql_num_rows(Akick.res);
				
				if(Akick.fields){
					while((Akick.row = mysql_fetch_row(Akick.res))){
						if(Akick.row[1]!=NULL){
							/* Make the ban stay in channel - We proof the List for such channels later */
							
							/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
							if(thread->c.chan.settings&CS_INHABIT_BIT){
								Users 		*my_chanserv 		= GetUser(cs->nick);
								Uchannels 	*my_chanserv_channel= NULL;

								if(my_chanserv!=NULL)
									my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

								/* WOOHOOO ChanServ isn't in the Channel yet - if we will set a ban now, it will disappear without any user ...
								 and since ChanServ should be a channel inhabitant according to the channel settings -> it joins the channel here ;) */
								if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
									do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
									do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
								}
							}

							/* ChanServ isn't inhabitant, so if there is only one user in the channel (the one we want kick now)
							 we have to join the channel to make the ban "permanent" */
							else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers == m->BSMinUsers && !thread->c.chan.botinchan && !csjoined && getStayChannel(thread,thread->c.chan.name)==NULL){
								do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
					        	do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
					        	csjoined=1;
							}
							
							do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",Akick.row[1]);
							do_kick_nopart(thread,thread->c.chan.bot,thread->c.chan.name,nick,"Akicked: %s",Akick.row[2]);
							//thread->c.chan.countusers--;
						}
			        }
				}
				mysql_free_result_myr(Akick.res);
				/* ************************************************************************************ */
				
				if(stoi(thread->c.chan.access.targetlevel)<=thread->c.chan.levels.nojoin){
					/* Make the ban stay in channel - We proof the List for such channels later */

					/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
					if(thread->c.chan.settings&CS_INHABIT_BIT){
						Users 		*my_chanserv 		= GetUser(cs->nick);
						Uchannels 	*my_chanserv_channel= NULL;

						if(my_chanserv!=NULL)
							my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

						/* WOOHOOO ChanServ isn't in the Channel yet - if we will set a ban now, it will disappear without any user ...
						 and since ChanServ should be a channel inhabitant according to the channel settings -> it joins the channel here ;) */
						if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
							do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
							do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
						}
					}

					/* ChanServ isn't inhabitant, so if there is only one user in the channel (the one we want kick now)
					 we have to join the channel to make the ban "permanent" */
					else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers == m->BSMinUsers && !thread->c.chan.botinchan && !csjoined && getStayChannel(thread,thread->c.chan.name)==NULL){
						do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
						do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
						csjoined=1;
					}
					
					do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",BuildBanMask(thread,&thread->c.Cdata,CSAnalyseBantype(thread->c.chan.settings)));
					do_kick_nopart(thread,thread->c.chan.bot,thread->c.chan.name,nick,"matched NOJOIN levelsetting");
					//thread->c.chan.countusers--;
				}else{
					if(stoi(thread->c.chan.access.targetlevel)>IDENTIFIED_NO_ACCESS)
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%d' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),mysql_quote(thread,thread->c.chan.name));
					

					if(stoi(thread->c.chan.access.targetlevel)>=ACCESS_FOUNDER && ~thread->sql.access.settings&NS_SET_INCOGNITO)
						do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+qao %s %s %s",nick,nick,nick);
					else if(streqnc(nick,thread->c.chan.founder) && ~thread->sql.access.settings&NS_SET_INCOGNITO)
						do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+qao %s %s %s",nick,nick,nick);
						
					else if(stoi(thread->c.chan.access.targetlevel)>=thread->c.chan.levels.autoop && ((thread->sql.access.settings&NS_SET_INCOGNITO && thread->c.chan.levels.autoop<=IDENTIFIED_NO_ACCESS) || ~thread->sql.access.settings&NS_SET_INCOGNITO)){
						if(stoi(thread->c.chan.access.targetlevel)>=thread->c.chan.levels.autoprotect && ((thread->sql.access.settings&NS_SET_INCOGNITO && thread->c.chan.levels.autoprotect<=IDENTIFIED_NO_ACCESS) || ~thread->sql.access.settings&NS_SET_INCOGNITO))
							do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+oa %s %s",nick,nick);
						else do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+o %s",nick);
					}

					else if(stoi(thread->c.chan.access.targetlevel)>=thread->c.chan.levels.autohalfop && ((thread->sql.access.settings&NS_SET_INCOGNITO && thread->c.chan.levels.autohalfop<=IDENTIFIED_NO_ACCESS) || ~thread->sql.access.settings&NS_SET_INCOGNITO)){
						if(stoi(thread->c.chan.access.targetlevel)>=thread->c.chan.levels.autoprotect && ((thread->sql.access.settings&NS_SET_INCOGNITO && thread->c.chan.levels.autoprotect<=IDENTIFIED_NO_ACCESS) || ~thread->sql.access.settings&NS_SET_INCOGNITO))
							do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+ha %s %s",nick,nick);
						else do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+h %s",nick);
					}

					else if(stoi(thread->c.chan.access.targetlevel)>=thread->c.chan.levels.autovoice && ((thread->sql.access.settings&NS_SET_INCOGNITO && thread->c.chan.levels.autovoice<=IDENTIFIED_NO_ACCESS) || ~thread->sql.access.settings&NS_SET_INCOGNITO))
						do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+v %s",nick);
					
					if(thread->c.chan.botserv.set&BS_SET_SYMBIOSIS_BIT && stoi(thread->c.chan.access.targetlevel)>=thread->c.chan.levels.greet && thread->c.access.greet[0]!='\0'){
						do_privmsg(thread,thread->c.chan.bot,thread->c.chan.name,_dynadd("[%s] %s",nick,thread->c.access.greet));
					}
				}
			}
			if(thread->c.chan.fields && thread->c.chan.welcome[0]!='\0')
				msend(thread,0,thread->c.chan.bot,nick,"[%s] %s",thread->c.chan.name,thread->c.chan.welcome);
		}
		
		/* Manage our Bot */
		if((thread->c.chan.botinchan || csjoined) &&
			((~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers < m->BSMinUsers) || (thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers < m->BSMinUsers+1)))
		{
			addStayChannel(thread,thread->c.chan.name,thread->c.chan.bot,thread->c.chan.botinchan,csjoined);
		}
	}
	/*
	 * If it's a BotServ channel (Registered from Q)
	 */
	else if(thread->c.chan.fields && streqnc(thread->c.chan.serv,"q") && thread->c.chan.bot[0]!='\0'){
		
		/* We only have to delete the Channel (Since Q stays in Channel permanently) */
		if(thread->c.chan.countusers > m->BSMinUsers)
			delStayChannel(thread,thread->c.chan.name);
		/*****************************************************************************/
		
		/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
		if(thread->c.chan.settings&CS_INHABIT_BIT){
			Users 		*my_chanserv 		= GetUser(cs->nick);
			Uchannels 	*my_chanserv_channel= NULL;

			if(my_chanserv!=NULL)
				my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

			/* WOOHOOO ChanServ isn't in the Channel yet - if we will set a ban now, it will disappear without any user ...
			 and since ChanServ should be a channel inhabitant according to the channel settings -> it joins the channel here ;) */
			if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
				do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
				do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
			}
		}

		/* ChanServ isn't inhabitant, so if there is only one user in the channel (the one we want kick now)
		 we have to join the channel to make the ban "permanent" */
		else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers == m->BSMinUsers && getStayChannel(thread,thread->c.chan.name)==NULL && !thread->c.chan.botinchan){
			do_join(thread,FAKETYPE_BOT,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate);
        	do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",thread->c.chan.bot,thread->c.chan.bot,thread->c.chan.bot);
        	thread->c.chan.botinchan=1;
		}
		
		if(~thread->c.chan.botserv.set&BS_SET_SYMBIOSIS_BIT)
			dynncpy(thread->c.chan.bot,Q->nick,sizeof(thread->c.chan.bot));

		/* ******************************* AKick Check **************************************** */
		Akick.res    = AkickOnJoin(thread,param[1],nick);
		Akick.fields = mysql_num_rows(Akick.res);
				
		if(Akick.fields){
			while((Akick.row = mysql_fetch_row(Akick.res))){
				if(Akick.row[1]!=NULL){
					do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",Akick.row[1]);
					do_kick_nopart(thread,thread->c.chan.bot,thread->c.chan.name,nick,"Q-banned: %s",Akick.row[2]);
					//thread->c.chan.countusers--;
				}
	        }
		}
		mysql_free_result_myr(Akick.res);
		/* ************************************************************************************ */

		if(thread->c.chan.settings & CS_SUSPEND_BIT || thread->c.chan.settings & CS_FORBID_BIT){
			if(thread->c.access.qlevel<=100){
				do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,IRC.OnSuspend);
				do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",thread->c.Cdata.vhost);
				
				if(thread->c.chan.settings & CS_SUSPEND_BIT)	 do_kick_nopart(thread,thread->c.chan.bot,thread->c.chan.name,nick,"suspended [%s]",thread->c.chan.reason);
				else if(thread->c.chan.settings & CS_FORBID_BIT) do_kick_nopart(thread,thread->c.chan.bot,thread->c.chan.name,nick,thread->c.chan.reason[0]=='\0' ? "Forbidden" : thread->c.chan.reason);
				//thread->c.chan.countusers--;
			}
		}else if(!streqnc(nick,thread->c.chan.bot)){
			if(thread->c.chan.access.targetauthnick[0]!='\0' && thread->c.chan.access.targetlevel[0]!='\0'){
				if(strcor("b",thread->c.chan.access.targetlevel)){
					do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",thread->c.Cdata.vhost);
					do_kick_nopart(thread,thread->c.chan.bot,thread->c.chan.name,nick,"%s-banned",thread->c.chan.bot);
					//thread->c.chan.countusers--;
					
				}else if(strcor("a",thread->c.chan.access.targetlevel) && ~thread->sql.access.settings&NS_SET_INCOGNITO){
					if(strcor("o",thread->c.chan.access.targetlevel)){
						if(strcor("p",thread->c.chan.access.targetlevel)) do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+oa %s %s",nick,nick);
						else do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+o %s",nick);
					}else if(strcor("h",thread->c.chan.access.targetlevel)){
						if(strcor("p",thread->c.chan.access.targetlevel)) do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+ha %s %s",nick,nick);
						else do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+h %s",nick);
					}else if(strcor("v",thread->c.chan.access.targetlevel)) do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+v %s",nick);
				}
			}
			if(strcor("w",thread->c.chan.flags) && thread->c.chan.welcome[0]!='\0')
				msend(thread,0,thread->c.chan.bot,nick,"[%s] %s",thread->c.chan.name,thread->c.chan.welcome);
				
			if(strcor("c",thread->c.chan.flags) && thread->c.chan.alimit!=0)
				addLimit(thread,thread->c.chan.name,thread->c.chan.datecreate,strcor("c",thread->c.chan.flags),thread->c.chan.serv,thread->c.chan.alimit);
			
			if(strcor("o",thread->c.chan.flags))
				do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+o %s",nick);
				
			else if(strcor("v",thread->c.chan.flags))
				do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+v %s",nick);
		}
		
		/* Manage our Bot */
		if((thread->c.chan.botinchan || csjoined) && 
		    ((~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers < m->BSMinUsers) || (thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers < m->BSMinUsers+1)))
		{
			addStayChannel(thread,thread->c.chan.name,thread->c.chan.bot,thread->c.chan.botinchan,2); //3 tells us that it's a Q channel!
		}
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * bs_on_channelmessage: BotServ Events: On Channelmessage Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL bs_on_channelmessage(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	//@param - includes the whole message (including ":nick PRIVMSG #channel :")..
	//@nick  - is the Nickname (the user who sent the PRIVMSG)
	//BsChan bsChan;
	deepSQL badwords;
	//Condata Cdata;
	
	size_t caps_counter = 0;
	size_t msglength = 0;
	size_t a = 0;
	int i = 0, j = 0;
	int bansetting = 0;
	char targetlevel[MAX_MODES_LEN];
	char *cmd = &param[3][1];
	char *message = &_dstrjoin(' ',param,3)[1];

	//bzero(&bsChan,sizeof(bsChan));
	bzero(&badwords,sizeof(badwords));
	//bzero(&Cdata,sizeof(Cdata));
	bzero(&targetlevel,sizeof(targetlevel));

	if(GetUser(nick)==NULL)
	    return MOD_CONT;
	
	bzero(&thread->sql,sizeof(SQL));
	sql_mainAAC(thread,nick);
	
	if(param[4]){
		if(streqnc(cmd,"!op") || streqnc(cmd,"!deop") || streqnc(cmd,"!halfop") 
		 || streqnc(cmd,"!dehalfop") || streqnc(cmd,"!protect") || streqnc(cmd,"!deprotect")
		 || streqnc(cmd,"!voice") || streqnc(cmd,"!devoice") || streqnc(cmd,"!kick") || streqnc(cmd,"!kb")){
		 	/* Updates the Threads Channel Management */
			UpdateChannelInformation(thread,param[2],param[4]);
			dynncpy(targetlevel,thread->c.chan.access.targetlevel,sizeof(targetlevel));
		}else
			UpdateChannelInformation(thread,param[2],NULL);
	}else{
		//sql_BsChan(thread,nick,param[2],&bsChan);
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[2],NULL);
	}
	
	/* Introduce variabels */	
	//dynncpy(Cdata.nick,bsChan.cu_nick,sizeof(Cdata.nick));
	//dynncpy(Cdata.ident,bsChan.ident,sizeof(Cdata.ident));
	//dynncpy(Cdata.vhost,bsChan.vhost,sizeof(Cdata.vhost));
	//dynncpy(Cdata.server,bsChan.server,sizeof(Cdata.server));
	bansetting = CSAnalyseBantype(thread->c.chan.settings);
	/**********************************************/
	
	//do_kickban(thread,bsChan.bot,bsChan.name,bansetting,Cdata,
	
	if(thread->c.chan.bot[0]!='\0' && thread->c.chan.name[0]!='\0' && thread->c.chan.fields && (streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q"))){
		if(~thread->c.chan.botserv.set&BS_SET_SYMBIOSIS_BIT){
			if(streqnc(thread->c.chan.serv,"chanserv"))
				dynncpy(thread->c.chan.bot,cs->nick,sizeof(thread->c.chan.bot));
			else if(streqnc(thread->c.chan.serv,"q"))
				dynncpy(thread->c.chan.bot,Q->nick,sizeof(thread->c.chan.bot));
		}

		if(streqnc(thread->c.chan.serv,"chanserv")){
    		//************ MODES *****************
    		
    		dlog(thread,5,"my level = %d\n",stoi(thread->c.chan.access.ownlevel));
    		dlog(thread,5,"targetlevel = %d\n",stoi(targetlevel));
    		dlog(thread,5,"channel level = %d\n",thread->c.chan.levels.opdeopme);
    		dlog(thread,5,"FantasyChar = %s\n",m->BSFantasyCharacter);
    		dlog(thread,5,"cmd = %s\n",cmd);
    		dlog(thread,5,"wanted i.e. = \"%s\"\n",_dynadd("%sop",m->BSFantasyCharacter));
    		if(mysql_ping(MYSQL_CONNECTION_HANDLE))
    			dlog(thread,3,"Mysql: LOST CONNECTION!");
    		
    		if(thread->c.chan.botserv.set&BS_SET_FANTASY_BIT && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.fantasia){

	    		if(streqnc(cmd,_dynadd("%sop",m->BSFantasyCharacter)) && !param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.opdeopme)
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+o %s",nick);
	    		else if(streqnc(cmd,_dynadd("%sop",m->BSFantasyCharacter)) && param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.opdeop){
	    			if(thread->c.chan.settings&CS_PEACE_BIT){
						if(stoi(thread->c.chan.access.ownlevel)>stoi(targetlevel))
	    					do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+o %s",param[4]);
	    			}else if(~thread->c.chan.settings&CS_PEACE_BIT)
	    				do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+o %s",param[4]);
				}
	
	    		else if(streqnc(cmd,_dynadd("%shalfop",m->BSFantasyCharacter)) && !param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.halfopme)
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+h %s",nick);
	    		else if(streqnc(cmd,_dynadd("%shalfop",m->BSFantasyCharacter)) && param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.halfop){
	    			if(thread->c.chan.settings&CS_PEACE_BIT){
						if(stoi(thread->c.chan.access.ownlevel)>stoi(targetlevel))
	    					do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+h %s",param[4]);
	    			}else if(~thread->c.chan.settings&CS_PEACE_BIT)
	    				do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+h %s",param[4]);
				}
	
	    		else if(streqnc(cmd,_dynadd("%sprotect",m->BSFantasyCharacter)) && !param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.protectme)
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+a %s",nick);	
	    		else if(streqnc(cmd,_dynadd("%sprotect",m->BSFantasyCharacter)) && param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.protect){
	    			if(thread->c.chan.settings&CS_PEACE_BIT){
						if(stoi(thread->c.chan.access.ownlevel)>stoi(targetlevel))
	    					do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+a %s",param[4]);
	    			}else if(~thread->c.chan.settings&CS_PEACE_BIT)
	    				do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+a %s",param[4]);
				}
	    			
	    		else if(streqnc(cmd,_dynadd("%svoice",m->BSFantasyCharacter)) && !param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.voiceme)
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+v %s",nick);	
	    		else if(streqnc(cmd,_dynadd("%svoice",m->BSFantasyCharacter)) && param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.voice){
	    			if(thread->c.chan.settings&CS_PEACE_BIT){
						if(stoi(thread->c.chan.access.ownlevel)>stoi(targetlevel))
	    					do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+v %s",param[4]);
	    			}else if(~thread->c.chan.settings&CS_PEACE_BIT)
	    				do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+v %s",param[4]);
				}
	    			
	    		else if(streqnc(cmd,_dynadd("%skick",m->BSFantasyCharacter)) && param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.kick){
	    			if(thread->c.chan.settings&CS_PEACE_BIT){
						if(stoi(thread->c.chan.access.ownlevel)>stoi(targetlevel) && stoi(targetlevel)<thread->c.chan.levels.nokick){
							if(thread->c.Cdata.faketype>FAKETYPE_USER)
								do_kill(thread,thread->c.chan.bot,nick,"services abuse");
							else
								do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,param[4],"Requested");
						}
	    			}
					else if(~thread->c.chan.settings&CS_PEACE_BIT && stoi(targetlevel)<thread->c.chan.levels.nokick){
						if(thread->c.Cdata.faketype>FAKETYPE_USER)
							do_kill(thread,thread->c.chan.bot,nick,"services abuse");
						else
							do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,param[4],"Requested");
					}
				}
	    			
	    		else if(streqnc(cmd,_dynadd("%skb",m->BSFantasyCharacter)) && param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.kick && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.ban){
	    			if(thread->c.chan.settings&CS_PEACE_BIT){
						if(stoi(thread->c.chan.access.ownlevel)>stoi(targetlevel) && stoi(targetlevel)<thread->c.chan.levels.nokick){
							if(thread->c.Cdata.faketype>FAKETYPE_USER)
								do_kill(thread,thread->c.chan.bot,nick,"services abuse");
							else
	    						do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->c.Cdata,"Requested");
						}
	    			}
					else if(thread->c.Cdata.faketype==FAKETYPE_USER && ~thread->c.chan.settings&CS_PEACE_BIT && stoi(targetlevel)<thread->c.chan.levels.nokick){
						if(thread->c.Cdata.faketype>FAKETYPE_USER)
							do_kill(thread,thread->c.chan.bot,nick,"services abuse");
						else
							do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->c.Cdata,"Requested");
					}
				}
	    			
	    		//************* DE - MODES *****************
	
	    		else if(streqnc(cmd,_dynadd("%sdeop",m->BSFantasyCharacter)) && !param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.opdeopme)
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-o %s",nick);
	    		else if(streqnc(cmd,_dynadd("%sdeop",m->BSFantasyCharacter)) && param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.opdeop){
	    			if(thread->c.chan.settings&CS_PEACE_BIT){
						if(stoi(thread->c.chan.access.ownlevel)>stoi(targetlevel))
	    					do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-o %s",param[4]);
	    			}else if(~thread->c.chan.settings&CS_PEACE_BIT)
	    				do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-o %s",param[4]);
				}
	
	    		else if(streqnc(cmd,_dynadd("%sdehalfop",m->BSFantasyCharacter)) && !param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.halfopme)
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-h %s",nick);
	    		else if(streqnc(cmd,_dynadd("%sdehalfop",m->BSFantasyCharacter)) && param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.halfop){
	    			if(thread->c.chan.settings&CS_PEACE_BIT){
						if(stoi(thread->c.chan.access.ownlevel)>stoi(targetlevel))
	    					do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-h %s",param[4]);
	    			}else if(~thread->c.chan.settings&CS_PEACE_BIT)
	    				do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-h %s",param[4]);
				}
	
	    		else if(streqnc(cmd,_dynadd("%sdeprotect",m->BSFantasyCharacter)) && !param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.protectme)
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-a %s",nick);
	    		else if(streqnc(cmd,_dynadd("%sdeprotect",m->BSFantasyCharacter)) && param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.protect){
	    			if(thread->c.chan.settings&CS_PEACE_BIT){
						if(stoi(thread->c.chan.access.ownlevel)>stoi(targetlevel))
	    					do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-a %s",param[4]);
	    			}else if(~thread->c.chan.settings&CS_PEACE_BIT)
	    				do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-a %s",param[4]);
				}
	    			
	    		else if(streqnc(cmd,_dynadd("%sdevoice",m->BSFantasyCharacter)) && !param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.voiceme)
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-v %s",nick);
	    		else if(streqnc(cmd,_dynadd("%sdevoice",m->BSFantasyCharacter)) && param[4] && stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.voice){
	    			if(thread->c.chan.settings&CS_PEACE_BIT){
						if(stoi(thread->c.chan.access.ownlevel)>stoi(targetlevel))
	    					do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-v %s",param[4]);
	    			}else if(~thread->c.chan.settings&CS_PEACE_BIT)
	    				do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-v %s",param[4]);
				}
				else{
					/* Give all modules the event, that someone is doing a valid command via fantasy */
					_ExecuteEvent(nick,param,"on_valid_fantasymessage");
					/*********************************************************************************/
				}
			}
			
			if(thread->c.access.rights<=is_services_oper){
				Users *user			= GetUser(nick);
				Uchannels *channel	= GetUserChannel(user, thread->c.chan.name);
				if (getUline(thread,user->server)!=NULL)
					/* User is connecting from a ulined server, we ignore him =) */
					return MOD_CONT;
				
		        /*if (channel==NULL)
		            channel = addUserChannelMore(user, thread->c.chan.name, &_dstrjoin(' ',param,3)[1]);*/
				
				dlog(thread,5,"check started\n");
				
				if(thread->c.chan.botserv.set&BS_SET_DONTKICKVOICES_BIT && 
					((stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.autovoice) || 
					 (stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.voice) || 
					 (stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.voiceme) || 
					 thread->c.access.rights>=is_services_oper))
				{
					dlog(thread,5,"DONTKICKVOICES = on\n");
					return MOD_CONT;
				}
					
				else if(thread->c.chan.botserv.set&BS_SET_DONTKICKOPS_BIT &&
					((stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.autoop) || 
					 (stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.opdeop) || 
					 (stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.opdeopme) || 
					 
					 (stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.autohalfop) || 
					 (stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.halfop) || 
					 (stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.halfopme) || 

					 (stoi(thread->c.chan.access.ownlevel)>=ACCESS_FOUNDER) || 					 
					 (stoi(thread->c.chan.access.ownlevel)>=ACCESS_SUCCESSOR) ||
					 thread->c.access.rights>=is_services_oper || 
					 
					 (thread->c.aliases.active && streqnc(thread->c.aliases.authnick,thread->c.chan.founder)) || 
					 (thread->c.aliases.active && streqnc(thread->c.aliases.authnick,thread->c.chan.successor))))
				{
					dlog(thread,5,"DONTKICKOPS = on\n");
					return MOD_CONT;
				}
				
				if(thread->c.chan.botserv.set&BS_KICK_FLOOD_BIT) dlog(thread,5,"BS_KICK_FLOOD = on\n");
				if(thread->c.chan.botserv.set&BS_KICK_REPEAT_BIT) dlog(thread,5,"BS_KICK_REPEAT = on\n");
				
				if(thread->c.chan.botserv.set&BS_KICK_CAPS_BIT) dlog(thread,5,"BS_KICK_CAPS = on\n");
				if(thread->c.chan.botserv.set&BS_KICK_BADWORDS_BIT) dlog(thread,5,"BS_KICK_BADWORDS = on\n");
				if(thread->c.chan.botserv.set&BS_KICK_BOLDS_BIT) dlog(thread,5,"BS_KICK_BOLDS = on\n");
				if(thread->c.chan.botserv.set&BS_KICK_UNDERLINES_BIT) dlog(thread,5,"BS_KICK_UNDERLINES = on\n");
				if(thread->c.chan.botserv.set&BS_KICK_COLORS_BIT) dlog(thread,5,"BS_KICK_COLORS = on\n");
				if(thread->c.chan.botserv.set&BS_KICK_REVERSES_BIT) dlog(thread,5,"BS_KICK_REVERSES = on\n");

				badwords.res    = doSelectMySQL(thread,"SELECT `art`,`badword` FROM `orion_bs_badwords` WHERE `channel` = '%s'",mysql_quote(thread,thread->c.chan.name));
				badwords.fields = mysql_num_rows(badwords.res);
				
				caps_counter = 0;
				msglength = 0;
				
				for(i=3;param[i]!=NULL;i++){
					if(thread->c.chan.botserv.set&BS_KICK_CAPS_BIT){
						for(a=0;param[i][a]!='\0';a++){
							dlog(thread,5,"%c = %d\n",param[i][a],(int)param[i][a]);
							if((int)param[i][a] >= 65 && (int)param[i][a] <= 90) caps_counter++;
							msglength++;
						}
					}
					
					if(strcin('',param[i]) && (thread->c.chan.botserv.set&BS_KICK_BOLDS_BIT)){
						dlog(thread,3,"bolds kick - don't kick ops / don't kick voices OFF\n");
						if(thread->c.chan.botserv.bolds>-1 && channel->kicked_bolds >= thread->c.chan.botserv.bolds){
							do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kickbanned cause of using bolds");
							channel->kicked_bolds = 0;
						}else{
							do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using bolds");
							if(thread->c.chan.botserv.bolds>-1)
								channel->kicked_bolds++;
						}
						return MOD_CONT;
						
					}else if(strcin('',param[i]) && (thread->c.chan.botserv.set&BS_KICK_UNDERLINES_BIT)){
						dlog(thread,3,"underlines kick - don't kick ops / don't kick voices OFF\n");
						if(thread->c.chan.botserv.underlines>-1 && channel->kicked_underlines >= thread->c.chan.botserv.underlines){
							do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kickbanned cause of using underlines");
							channel->kicked_underlines = 0;
						}else{
							do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using underlines");
							if(thread->c.chan.botserv.underlines>-1)
								channel->kicked_underlines++;
						}
						return MOD_CONT;
						
					}else if(strcin('',param[i]) && (thread->c.chan.botserv.set&BS_KICK_COLORS_BIT)){
						dlog(thread,3,"colors kick - don't kick ops / don't kick voices OFF\n");
						if(thread->c.chan.botserv.colors>-1 && channel->kicked_colors >= thread->c.chan.botserv.colors){
							do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kickbanned cause of using colors");
							channel->kicked_colors = 0;
						}else{
							do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using colors");
							if(thread->c.chan.botserv.colors>-1)
								channel->kicked_colors++;
						}
						return MOD_CONT;
						
					}else if(strcin('',param[i]) && (thread->c.chan.botserv.set&BS_KICK_REVERSES_BIT)){
						dlog(thread,3,"reverses kick - don't kick ops / don't kick voices OFF\n");
						if(thread->c.chan.botserv.reverses>-1 && channel->kicked_reverses >= thread->c.chan.botserv.reverses){
							do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kickbanned cause of using reverses");
							channel->kicked_reverses = 0;
						}else{
							do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using reverses");
							if(thread->c.chan.botserv.reverses>-1)
								channel->kicked_reverses++;
						}
						return MOD_CONT;
					}
				}
				
				dlog(thread,5,"caps counter = %d\n",caps_counter);
				dlog(thread,5,"message length = %d\n",msglength);
				dlog(thread,5,"badword.fields = %d\n",badwords.fields);
	
				if((thread->c.chan.botserv.set&BS_KICK_CAPS_BIT) && thread->c.chan.botserv.caps_min>-1 && thread->c.chan.botserv.caps_per>-1 && (thread->c.chan.botserv.caps_min<=caps_counter && thread->c.chan.botserv.caps_per<(msglength/100)*caps_counter)){
					dlog(thread,3,"caps kick - don't kick ops / don't kick voices OFF\n");
					if(thread->c.chan.botserv.caps_ttb>-1 && channel->kicked_caps >= thread->c.chan.botserv.caps_ttb){
						do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kickbanned cause of using caps");
						channel->kicked_caps = 0;
					}else{
						do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using caps");
						if(thread->c.chan.botserv.caps_ttb>-1)
							channel->kicked_caps++;
					}
					return MOD_CONT;
				}
				
				if(thread->c.chan.botserv.set&BS_KICK_BADWORDS_BIT){
					dlog(thread,3,"kick badwords\n");
					for(j=0;j<badwords.fields;j++){
						badwords.row = mysql_fetch_row(badwords.res);
						
						if(stoi(badwords.row[0])==0){
							if((!m->BSCaseSensitive && strinsnc(message,badwords.row[1])) || (m->BSCaseSensitive && strins(message,badwords.row[1]))){
								//--> kick
								dlog(thread,3,"badwords kick MATCH - don't kick ops / don't kick voices OFF\n");
								if(thread->c.chan.botserv.badwords>-1 && channel->kicked_badwords >= thread->c.chan.botserv.badwords){
									if(!m->BSGentleBWReason)	do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of using a badword (%s)",badwords.row[1]);
									else						do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of using a badword");
									channel->kicked_badwords = 0;
								}else{
									if(!m->BSGentleBWReason)	do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using a badword (%s)",badwords.row[1]);
									else						do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using a badword");
										
									if(thread->c.chan.botserv.badwords>-1)
										channel->kicked_badwords++;
								}
								return MOD_CONT;
							}
						}else if(stoi(badwords.row[0])==1){
							for(i=0;param[i]!=NULL;i++){
								if((!m->BSCaseSensitive && streqnc(param[i],badwords.row[1])) || (m->BSCaseSensitive && streq(param[i],badwords.row[1]))){
									//--> kick
									dlog(thread,3,"badwords kick SINGLE - don't kick ops / don't kick voices OFF\n");
									if(thread->c.chan.botserv.badwords>-1 && channel->kicked_badwords >= thread->c.chan.botserv.badwords){
										if(!m->BSGentleBWReason)	do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of using a badword (%s)",badwords.row[1]);
										else						do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of using a badword");
										channel->kicked_badwords = 0;
									}else{
										if(!m->BSGentleBWReason)	do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using a badword (%s)",badwords.row[1]);
										else						do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using a badword");
										
										if(thread->c.chan.botserv.badwords>-1)
											channel->kicked_badwords++;
									}
									return MOD_CONT;
								}
							}
						}else if(stoi(badwords.row[0])==2){
							for(i=0;param[i]!=NULL;i++){
								if(str_start(param[i],badwords.row[1])){
									//--> kick
									dlog(thread,3,"badwords kick START - don't kick ops / don't kick voices OFF\n");
									if(thread->c.chan.botserv.badwords>-1 && channel->kicked_badwords >= thread->c.chan.botserv.badwords){
										if(!m->BSGentleBWReason)	do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of using a badword (%s)",badwords.row[1]);
										else						do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of using a badword");
										channel->kicked_badwords = 0;
									}else{
										if(!m->BSGentleBWReason)	do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using a badword (%s)",badwords.row[1]);
										else						do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using a badword");
										
										if(thread->c.chan.botserv.badwords>-1)
											channel->kicked_badwords++;
									}
									return MOD_CONT;
								}
							}
						}else if(stoi(badwords.row[0])==3){
							for(i=0;param[i]!=NULL;i++){
								if(str_end(param[i],badwords.row[1])){
									//--> kick
									dlog(thread,3,"badwords kick END - don't kick ops / don't kick voices OFF\n");
									if(thread->c.chan.botserv.badwords>-1 && channel->kicked_badwords >= thread->c.chan.botserv.badwords){
										if(!m->BSGentleBWReason)	do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of using a badword (%s)",badwords.row[1]);
										else						do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of using a badword");
										channel->kicked_badwords = 0;
									}else{
										if(!m->BSGentleBWReason)	do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using a badword (%s)",badwords.row[1]);
										else						do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using a badword");
										
										if(thread->c.chan.botserv.badwords>-1)
											channel->kicked_badwords++;
									}
									return MOD_CONT;
								}
							}
						}				
					}
				}
				
		        /* Flood kicker */
		        if((thread->c.chan.botserv.set&BS_KICK_FLOOD_BIT) && thread->c.chan.botserv.flood_ln>-1 && thread->c.chan.botserv.flood_secs>-1){
		            time_t now = time(NULL);
		
		            if (now - channel->LastFloodCheck > thread->c.chan.botserv.flood_secs) {
		                channel->LastFloodCheck = time(NULL);
		                channel->FloodLines = 0;
		            }
		
		            channel->FloodLines++;
		            if (channel->FloodLines >= thread->c.chan.botserv.flood_ln) {
						if(thread->c.chan.botserv.flood_ttb>-1 && channel->kicked_flood >= thread->c.chan.botserv.flood_ttb){
		                	do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of flooding");
		                	channel->kicked_flood = 0;
		                	channel->FloodLines = 0;
						}else{
							do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of flooding");
							if(thread->c.chan.botserv.flood_ttb>-1)
								channel->kicked_flood++;
		               		channel->FloodLines = 0;
						}
		                return MOD_CONT;
		            }
		        }
		
		        /* Repeat kicker */
		        if((thread->c.chan.botserv.set&BS_KICK_REPEAT_BIT) && thread->c.chan.botserv.repeat>-1){
					
			        if (channel->lastmsg[0]!='\0' && !streqnc(channel->lastmsg, &_dstrjoin(' ',param,3)[1])) {
			            //bzero(&channel->lastmsg,sizeof(channel->lastmsg));
			            dynncpy(channel->lastmsg,&_dstrjoin(' ',param,3)[1],sizeof(channel->lastmsg));
			            channel->Repeat = 0;
			        } else {
			            if (channel->lastmsg[0]=='\0')
			                dynncpy(channel->lastmsg,&_dstrjoin(' ',param,3)[1],sizeof(channel->lastmsg));
			            channel->Repeat++;
					}
			
					if (channel->Repeat-1 >= thread->c.chan.botserv.repeat) {
						if(thread->c.chan.botserv.repeat_ttb>-1 && channel->kicked_repeat >= thread->c.chan.botserv.repeat_ttb){
							do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of repeating");
							channel->kicked_repeat = 0;
							channel->Repeat = 0;
						}else{
							do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of repeating");
							if(thread->c.chan.botserv.repeat_ttb>-1)
								channel->kicked_repeat++;
			            	channel->Repeat = 0;
						}
			            return MOD_CONT;
			        }
				}
				/* END OF FLOOD / REPEAT KICKER */

				/* MySQL Free Result (BadWords) */
				mysql_free_result_myr(badwords.res);
			}
			
			
		}else if(streqnc(thread->c.chan.serv,"q")){
    		// MODES
    		//************ MODES *****************
    		
    		dlog(thread,5,"my level = %d\n",stoi(thread->c.chan.access.ownlevel));
    		dlog(thread,5,"channel level = %d\n",thread->c.chan.levels.opdeopme);
    		
    		if(thread->c.chan.botserv.set&BS_SET_FANTASY_BIT){
	    		if(streqnc(cmd,_dynadd("%sop",m->BSFantasyCharacter)) && !param[4] && strcor(thread->c.chan.access.ownlevel,"o"))
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+o %s",nick);
	    		else if(streqnc(cmd,_dynadd("%sop",m->BSFantasyCharacter)) && param[4] && strcor(thread->c.chan.access.ownlevel,"o"))
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+o %s",param[4]);
	
	    		else if(streqnc(cmd,_dynadd("%shalfop",m->BSFantasyCharacter)) && !param[4] && strcor(thread->c.chan.access.ownlevel,"h"))
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+h %s",nick);
	    		else if(streqnc(cmd,_dynadd("%shalfop",m->BSFantasyCharacter)) && param[4] && strcor(thread->c.chan.access.ownlevel,"h"))
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+h %s",param[4]);
	
	    		else if(streqnc(cmd,_dynadd("%sprotect",m->BSFantasyCharacter)) && !param[4] && strcor(thread->c.chan.access.ownlevel,"p"))
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+a %s",nick);
	    		else if(streqnc(cmd,_dynadd("%sprotect",m->BSFantasyCharacter)) && param[4] && strcor(thread->c.chan.access.ownlevel,"p"))
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+a %s",param[4]);
	    			
	    		else if(streqnc(cmd,_dynadd("%svoice",m->BSFantasyCharacter)) && !param[4] && strcor(thread->c.chan.access.ownlevel,"v"))
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+v %s",nick);
	    		else if(streqnc(cmd,_dynadd("%svoice",m->BSFantasyCharacter)) && param[4] && strcor(thread->c.chan.access.ownlevel,"v"))
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"+v %s",param[4]);
	    		else if(streqnc(cmd,_dynadd("%skick",m->BSFantasyCharacter)) && param[4] && strcor(thread->c.chan.access.ownlevel,"o")){
					if(thread->c.Cdata.faketype>FAKETYPE_USER)
					    do_kill(thread,thread->c.chan.bot,nick,"services abuse");
					else
						do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,param[4],"Requested");
				}
	    		else if(streqnc(cmd,_dynadd("%skb",m->BSFantasyCharacter)) && param[4] && strcor(thread->c.chan.access.ownlevel,"o")){
					if(thread->c.Cdata.faketype>FAKETYPE_USER)
					    do_kill(thread,thread->c.chan.bot,nick,"services abuse");
					else
						do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->c.Cdata,"Requested");
				}
	    		//************* DE - MODES *****************
	
	    		else if(streqnc(cmd,_dynadd("%sdeop",m->BSFantasyCharacter)) && !param[4] && strcor(thread->c.chan.access.ownlevel,"o"))
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-o %s",nick);
	    		else if(streqnc(cmd,_dynadd("%sdeop",m->BSFantasyCharacter)) && param[4] && strcor(thread->c.chan.access.ownlevel,"o"))
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-o %s",param[4]);
	
	    		else if(streqnc(cmd,_dynadd("%sdehalfop",m->BSFantasyCharacter)) && !param[4] && strcor(thread->c.chan.access.ownlevel,"h"))
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-h %s",nick);
	    		else if(streqnc(cmd,_dynadd("%sdehalfop",m->BSFantasyCharacter)) && param[4] && strcor(thread->c.chan.access.ownlevel,"h"))
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-h %s",param[4]);
	
	    		else if(streqnc(cmd,_dynadd("%sdeprotect",m->BSFantasyCharacter)) && !param[4] && strcor(thread->c.chan.access.ownlevel,"p"))
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-a %s",nick);
	    		else if(streqnc(cmd,_dynadd("%sdeprotect",m->BSFantasyCharacter)) && param[4] && strcor(thread->c.chan.access.ownlevel,"p"))
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-a %s",param[4]);
	    			
	    		else if(streqnc(cmd,_dynadd("%sdevoice",m->BSFantasyCharacter)) && !param[4] && strcor(thread->c.chan.access.ownlevel,"v"))
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-v %s",nick);
	    		else if(streqnc(cmd,_dynadd("%sdevoice",m->BSFantasyCharacter)) && param[4] && strcor(thread->c.chan.access.ownlevel,"v"))
	    			do_mode(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,"-v %s",param[4]);
			}
			if(thread->c.access.rights<=is_services_oper){
				Users *user			= GetUser(nick);
				Uchannels *channel	= GetUserChannel(user, thread->c.chan.name);
				
				if (getUline(thread,user->server)!=NULL)
					/* User is connecting from a ulined server, we ignore him =) */
					return MOD_CONT;
				
		        /*if (channel==NULL)
		            channel = addUserChannelMore(user, thread->c.chan.name, &_dstrjoin(' ',param,3)[1]);*/
				
				dlog(thread,5,"check started\n");
				
				if(thread->c.chan.botserv.set&BS_SET_DONTKICKVOICES_BIT && 
					((stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.autovoice) || 
					 (stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.voice) || 
					 (stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.voiceme) || 
					 thread->c.access.rights<=is_services_oper))
				{
					dlog(thread,5,"DONTKICKVOICES = on\n");
					return MOD_CONT;
				}
					
				else if(thread->c.chan.botserv.set&BS_SET_DONTKICKOPS_BIT &&
					((stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.autoop) || 
					 (stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.opdeop) || 
					 (stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.opdeopme) || 
					 
					 (stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.autohalfop) || 
					 (stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.halfop) || 
					 (stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.halfopme) || 

					 (stoi(thread->c.chan.access.ownlevel)>=ACCESS_FOUNDER) || 					 
					 (stoi(thread->c.chan.access.ownlevel)>=ACCESS_SUCCESSOR) || 
					 thread->c.access.rights<=is_services_oper || 
					 
					 (thread->c.aliases.active && streqnc(thread->c.aliases.authnick,thread->c.chan.founder)) || 
					 (thread->c.aliases.active && streqnc(thread->c.aliases.authnick,thread->c.chan.successor))))
				{
					dlog(thread,5,"DONTKICKOPS = on\n");
					return MOD_CONT;
				}

				badwords.res    = doSelectMySQL(thread,"SELECT `art`,`badword` FROM `orion_bs_badwords` WHERE `channel` = '%s'",mysql_quote(thread,thread->c.chan.name));
				badwords.fields = mysql_num_rows(badwords.res);
				
				caps_counter = 0;
				msglength = 0;
				
				for(i=3;param[i]!=NULL;i++){
					if(thread->c.chan.botserv.set&BS_KICK_CAPS_BIT){
						for(a=0;param[i][a]!='\0';a++){
							dlog(thread,5,"%c = %d\n",param[i][a],(int)param[i][a]);
							if((int)param[i][a] >= 65 && (int)param[i][a] <= 90) caps_counter++;
							msglength++;
						}
					}
					
					if(strcin('',param[i]) && (thread->c.chan.botserv.set&BS_KICK_BOLDS_BIT)){
						dlog(thread,3,"bolds kick - don't kick ops / don't kick voices OFF\n");
						if(thread->c.chan.botserv.bolds>-1 && channel->kicked_bolds >= thread->c.chan.botserv.bolds){
							do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kickbanned cause of using bolds");
							channel->kicked_bolds = 0;
						}else{
							do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using bolds");
							if(thread->c.chan.botserv.bolds>-1)
								channel->kicked_bolds++;
						}
						return MOD_CONT;
						
					}else if(strcin('',param[i]) && (thread->c.chan.botserv.set&BS_KICK_UNDERLINES_BIT)){
						dlog(thread,3,"underlines kick - don't kick ops / don't kick voices OFF\n");
						if(thread->c.chan.botserv.underlines>-1 && channel->kicked_underlines >= thread->c.chan.botserv.underlines){
							do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kickbanned cause of using underlines");
							channel->kicked_underlines = 0;
						}else{
							do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using underlines");
							if(thread->c.chan.botserv.underlines>-1)
								channel->kicked_underlines++;
						}
						return MOD_CONT;
						
					}else if(strcin('',param[i]) && (thread->c.chan.botserv.set&BS_KICK_COLORS_BIT)){
						dlog(thread,3,"colors kick - don't kick ops / don't kick voices OFF\n");
						if(thread->c.chan.botserv.colors>-1 && channel->kicked_colors >= thread->c.chan.botserv.colors){
							do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kickbanned cause of using colors");
							channel->kicked_colors = 0;
						}else{
							do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using colors");
							if(thread->c.chan.botserv.colors>-1)
								channel->kicked_colors++;
						}
						return MOD_CONT;
					}else if(strcin('',param[i]) && (thread->c.chan.botserv.set&BS_KICK_REVERSES_BIT)){
						dlog(thread,3,"reverses kick - don't kick ops / don't kick voices OFF\n");
						if(thread->c.chan.botserv.reverses>-1 && channel->kicked_reverses >= thread->c.chan.botserv.reverses){
							do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kickbanned cause of using reverses");
							channel->kicked_reverses = 0;
						}else{
							do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using reverses");
							if(thread->c.chan.botserv.reverses>-1)
								channel->kicked_reverses++;
						}
						return MOD_CONT;
					}
				}
				
				dlog(thread,5,"caps counter = %d\n",caps_counter);
				dlog(thread,5,"message length = %d\n",msglength);
				dlog(thread,5,"badwords fields = %d\n",badwords.fields);
	
				if((thread->c.chan.botserv.set&BS_KICK_CAPS_BIT) && thread->c.chan.botserv.caps_min>-1 && thread->c.chan.botserv.caps_per>-1 && (thread->c.chan.botserv.caps_min<=caps_counter && thread->c.chan.botserv.caps_per<(msglength/100)*caps_counter)){
					dlog(thread,3,"caps kick - don't kick ops / don't kick voices OFF\n");
					if(thread->c.chan.botserv.caps_ttb>-1 && channel->kicked_caps >= thread->c.chan.botserv.caps_ttb){
						do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kickbanned cause of using caps");
						channel->kicked_caps = 0;
					}else{
						do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using caps");
						if(thread->c.chan.botserv.caps_ttb>-1)
							channel->kicked_caps++;
					}
					return MOD_CONT;
				}
				
				if(thread->c.chan.botserv.set&BS_KICK_BADWORDS_BIT){
					dlog(thread,3,"kick badwords\n");
					for(j=0;j<badwords.fields;j++){
						badwords.row = mysql_fetch_row(badwords.res);
						
						if(stoi(badwords.row[0])==0){
							if((!m->BSCaseSensitive && strinsnc(message,badwords.row[1])) || (m->BSCaseSensitive && strins(message,badwords.row[1]))){
								//--> kick
								dlog(thread,3,"badwords kick MATCH - don't kick ops / don't kick voices OFF\n");
								if(thread->c.chan.botserv.badwords>-1 && channel->kicked_badwords >= thread->c.chan.botserv.badwords){
									if(!m->BSGentleBWReason)	do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of using a badword (%s)",badwords.row[1]);
									else						do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of using a badword");
									channel->kicked_badwords = 0;
								}else{
									if(!m->BSGentleBWReason)	do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using a badword (%s)",badwords.row[1]);
									else						do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using a badword");

									if(thread->c.chan.botserv.badwords>-1)
										channel->kicked_badwords++;
								}
								return MOD_CONT;
							}
						}else if(stoi(badwords.row[0])==1){
							for(i=0;param[i]!=NULL;i++){
								if((!m->BSCaseSensitive && streqnc(param[i],badwords.row[1])) || (m->BSCaseSensitive && streq(param[i],badwords.row[1]))){
									//--> kick
									dlog(thread,3,"badwords kick MATCH - don't kick ops / don't kick voices OFF\n");
									if(thread->c.chan.botserv.badwords>-1 && channel->kicked_badwords >= thread->c.chan.botserv.badwords){
										if(!m->BSGentleBWReason)	do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of using a badword (%s)",badwords.row[1]);
										else						do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of using a badword");
										channel->kicked_badwords = 0;
									}else{
										if(!m->BSGentleBWReason)	do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using a badword (%s)",badwords.row[1]);
										else						do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using a badword");

										if(thread->c.chan.botserv.badwords>-1)
											channel->kicked_badwords++;
									}
									return MOD_CONT;
								}
							}
						}else if(stoi(badwords.row[0])==2){
							for(i=0;param[i]!=NULL;i++){
								if(str_start(param[i],badwords.row[1])){
									//--> kick
									dlog(thread,3,"badwords kick START - don't kick ops / don't kick voices OFF\n");
									if(thread->c.chan.botserv.badwords>-1 && channel->kicked_badwords >= thread->c.chan.botserv.badwords){
										if(!m->BSGentleBWReason)	do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of using a badword (%s)",badwords.row[1]);
										else						do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of using a badword");
										channel->kicked_badwords = 0;
									}else{
										if(!m->BSGentleBWReason)	do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using a badword (%s)",badwords.row[1]);
										else						do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using a badword");

										if(thread->c.chan.botserv.badwords>-1)
											channel->kicked_badwords++;
									}
									return MOD_CONT;
								}
							}
						}else if(stoi(badwords.row[0])==3){
							for(i=0;param[i]!=NULL;i++){
								if(str_end(param[i],badwords.row[1])){
									//--> kick
									dlog(thread,3,"badwords kick END - don't kick ops / don't kick voices OFF\n");
									if(thread->c.chan.botserv.badwords>-1 && channel->kicked_badwords >= thread->c.chan.botserv.badwords){
										if(!m->BSGentleBWReason)	do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of using a badword (%s)",badwords.row[1]);
										else						do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of using a badword");
										channel->kicked_badwords = 0;
									}else{
										if(!m->BSGentleBWReason)	do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using a badword (%s)",badwords.row[1]);
										else						do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of using a badword");

										if(thread->c.chan.botserv.badwords>-1)
											channel->kicked_badwords++;
									}
									return MOD_CONT;
								}
							}
						}				
					}
				}
				
		        /* Flood kicker */
		        if((thread->c.chan.botserv.set&BS_KICK_FLOOD_BIT) && thread->c.chan.botserv.flood_ln>-1 && thread->c.chan.botserv.flood_secs>-1){
		            time_t now = time(NULL);
		
		            if (now - channel->LastFloodCheck > thread->c.chan.botserv.flood_secs) {
		                channel->LastFloodCheck = time(NULL);
		                channel->FloodLines = 0;
		            }
		
		            channel->FloodLines++;
		            if (channel->FloodLines >= thread->c.chan.botserv.flood_ln) {
						if(thread->c.chan.botserv.flood_ttb>-1 && channel->kicked_flood >= thread->c.chan.botserv.flood_ttb){
		                	do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of flooding");
		                	channel->kicked_flood = 0;
		                	channel->FloodLines = 0;
						}else{
							do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of flooding");
							if(thread->c.chan.botserv.flood_ttb>-1)
								channel->kicked_flood++;
		               		channel->FloodLines = 0;
						}
		                return MOD_CONT;
		            }
		        }
		
		        /* Repeat kicker */
		        if((thread->c.chan.botserv.set&BS_KICK_REPEAT_BIT) && thread->c.chan.botserv.repeat>-1){
					
			        if (channel->lastmsg[0]!='\0' && !streqnc(channel->lastmsg, &_dstrjoin(' ',param,3)[1])) {
			            //bzero(&channel->lastmsg,sizeof(channel->lastmsg));
			            dynncpy(channel->lastmsg,&_dstrjoin(' ',param,3)[1],sizeof(channel->lastmsg));
			            channel->Repeat = 0;
			        } else {
			            if (channel->lastmsg[0]=='\0')
			                dynncpy(channel->lastmsg,&_dstrjoin(' ',param,3)[1],sizeof(channel->lastmsg));
			            channel->Repeat++;
					}
			
					if (channel->Repeat-1 >= thread->c.chan.botserv.repeat) {
						if(thread->c.chan.botserv.repeat_ttb>-1 && channel->kicked_repeat >= thread->c.chan.botserv.repeat_ttb){
							do_kickban(thread,thread->c.chan.bot,thread->c.chan.name,thread->c.chan.datecreate,bansetting,&thread->sql.Cdata,"kicked cause of repeating");
							channel->kicked_repeat = 0;
							channel->Repeat = 0;
						}else{
							do_BSkick(thread,thread->c.chan.bot,thread->c.chan.name,user->nick,"kicked cause of repeating");
							if(thread->c.chan.botserv.repeat_ttb>-1)
								channel->kicked_repeat++;
			            	channel->Repeat = 0;
						}
			            return MOD_CONT;
			        }
				}
				/* END OF FLOOD / REPEAT KICKER */

				/* MySQL Free Result (BadWords) */
				mysql_free_result_myr(badwords.res);
			}
		}
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * bs_on_message: BotServ Events: On Message Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL bs_on_message(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* (language == NULL, howtosend = 0, snick == NULL) */
	
	if((m->UseStrictPrivMsg && streqnc(param[2],_dynadd("%s@%s",bs->ident,m->servername))) || (!m->UseStrictPrivMsg && (streqnc(param[2],bs->nick) || streqnc(param[2],_dynadd("%s@%s",bs->ident,m->servername))))){
		if(GetUser(nick)==NULL && (!thread->Webform && FindWebSocket(thread->Sock)==NULL))
	    	return MOD_CONT;

		ModCheck modcheck;
    	ModCo *modul = NULL;
    	char *cmd    = &param[3][1];
    	
    	bzero(&modcheck,sizeof(modcheck));
    	
    	bzero(&thread->sql,sizeof(SQL));
		sql_mainAAC(thread,nick);
		
    	/* Since language == NULL, howtosend = 0, snick == NULL - we wake them up now ;) */
    	snick		= bs->nick;
    	howtosend	= thread->sql.access.settings&NS_SET_MSG ? 1 : 0;
    	language	= languages[thread->sql.access.language];
    	/*********************************************************************************/
    	
	/* BS Commands */
	
		/* If help for a specific Command was requested */
    	if(streqnc(cmd,"help") && param[4]){
			
			/* Help with wildmasks */
    		if(strcor(param[4],"*"))
				call_wildview(thread, snick, nick, SERVICE, "help", param);
    		
			/* Normal help for a specific command (no wildmasks) */
			else{
    			modul 		= findModule(thread, SERVICE, param[4]);
  				modcheck 	= call_helpcommand(thread, modul, cmd, snick, nick, param);
  				
				if(!ModReply(thread, &modcheck, snick, nick, param))
					send_lang(thread, language, howtosend, snick, nick, "BS_UNKNOWN_COMMAND", snick);
    		}
    	}
		
		/* If Command was only "help" */
		else if(streqnc(cmd,"help") && !param[4]){
    		modul 		= findModule(thread, SERVICE, cmd);
    		modcheck 	= call_helpcommand(thread, modul, cmd, snick, nick, param);
    		
			if(!ModReply(thread, &modcheck, snick, nick, param))
    			send_lang(thread, language, howtosend, snick, nick, "BS_UNKNOWN_COMMAND", snick);
    	}
    	
		/* All other commands which aren't "help" */
		else if(!streqnc(cmd,"help")){
    		modul 		= findModule(thread, SERVICE, cmd);
  			modcheck 	= call_command(thread, modul, cmd, snick, nick, param);
			  	
			if(!ModReply(thread, &modcheck, snick, nick, param))
    			send_lang(thread, language, howtosend, snick, nick, "BS_UNKNOWN_COMMAND", snick);
    	}
		else send_lang(thread, language, howtosend, snick, nick, "BS_UNKNOWN_COMMAND", snick);
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_BOOL
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
    deepSQL bots;
    
    if(Action == DO_UNLOAD){
		bs->loaded = 0;

		bots.res    = doSelectMySQL(thread,"SELECT `botnick`,`botident`,`bothost`,`botrealname`,`private`,`time` FROM `orion_bs_core`");
		bots.fields = mysql_num_rows(bots.res);

		if(bots.fields){
			while((bots.row = mysql_fetch_row(bots.res))){
				if(bots.row[0]!=NULL){
                    delStayNick(thread,bots.row[0]);
					do_quit(thread,bots.row[0],"module unloaded");
				}
	    	}
		}
		mysql_free_quotes(thread);
		mysql_free_result_myr(bots.res);
    
		delStayNick(thread,bs->nick);
		do_quit(thread,bs->nick,"module unloaded");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */
