/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file bs_set.c
 *   @brief This is the Module file for BotServ SET Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the BotServ SET command.
 *
 *   The BotServ SET Command allows the user to set various SET-Options<br/>
 *   for target Channel. (Controls DONTKICKOPS/DONTKICKVOICES etc.)
 *
 *   Syntax is: /msg BotServ SET #channel OPTION { ON | OFF }
 *
 *<table border="0">
 *    <tr><td> DONTKICKOPS		</td><td> Syntax is: /msg BotServ SET #channel DONTKICKOPS		</td><td> { ON | OFF }	</td></tr>
 *    <tr><td> DONTKICKVOICES	</td><td> Syntax is: /msg BotServ SET #channel DONTKICKVOICES	</td><td> { ON | OFF }	</td></tr>
 *    <tr><td> GREET			</td><td> Syntax is: /msg BotServ SET #channel GREET			</td><td> { ON | OFF }	</td></tr>
 *    <tr><td> FANTASY			</td><td> Syntax is: /msg BotServ SET #channel FANTASY			</td><td> { ON | OFF }	</td></tr>
 *    <tr><td> SYMBIOSIS		</td><td> Syntax is: /msg BotServ SET #channel SYMBIOSIS		</td><td> { ON | OFF }	</td></tr>
 *    <tr><td> NOBOT			</td><td> Syntax is: /msg BotServ SET #channel NOBOT			</td><td> { ON | OFF }	</td></tr>
 *    <tr><td> PRIVATE			</td><td> Syntax is: /msg BotServ SET botnick  PRIVATE			</td><td> { ON | OFF }	</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "bs"
#define MODUL "set"
#define MODUL_FILE "bs_set"
#define MODUL_VERSION "1.2"

MOD_BOOL bs_set(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL bs_set_dontkickops(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_set_dontkickvoices(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_set_greet(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_set_fantasy(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_set_symbiosis(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_set_nobot(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_set_private(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL bs_set_dontkickops_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_set_dontkickvoices_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_set_greet_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_set_fantasy_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_set_symbiosis_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_set_nobot_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_set_private_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL bs_set_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_set_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"set * option[1]","set helpoption[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"set",is_services_admin,is_services_user); /* just to add a displaylevel for /msg bs help
															  this should always be the lowest used Module level! */

	addCommand(thread,&Module,"set dontkickops",bs_set_dontkickops,is_services_oper,is_services_user,6,6);
	addCommand(thread,&Module,"set dontkickvoices",bs_set_dontkickvoices,is_services_oper,is_services_user,6,6);
	addCommand(thread,&Module,"set greet",bs_set_greet,is_services_oper,is_services_user,6,6);
	addCommand(thread,&Module,"set fantasy",bs_set_fantasy,is_services_oper,is_services_user,6,6);
	addCommand(thread,&Module,"set symbiosis",bs_set_symbiosis,is_services_oper,is_services_user,6,6);
	addCommand(thread,&Module,"set nobot",bs_set_nobot,is_services_oper,INFINIT,6,6);
	addCommand(thread,&Module,"set private",bs_set_private,is_services_oper,INFINIT,6,6);

	addCommand(thread,&Module,"help set",bs_set_help,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help set dontkickops",bs_set_dontkickops_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set dontkickvoices",bs_set_dontkickvoices_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set greet",bs_set_greet_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set fantasy",bs_set_fantasy_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set symbiosis",bs_set_symbiosis_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set nobot",bs_set_nobot_help,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"help set private",bs_set_private_help,is_services_admin,is_services_user,5,5);
	
	addCommand(thread,&Module,"help",bs_set_help_listing,is_services_admin,is_services_user,3,3);
	
	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}
//> :shock PRIVMSG chanserv :set #lol founder cliebig

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the BotServ SET Command (/msg BotServ HELP SET)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP SET,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP SET
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_set_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /*
     * Set Help
   	 */
    if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set")){
    	send_lang(thread,language,howtosend,snick,nick,"BS_HELP_SET",snick);
    	if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"help set")) 
			send_lang(thread,language,howtosend,snick,nick,"BS_SERVADMIN_HELP_SET");
			
		setWebSuccess(thread); /* Indentify webform that changes were applied */
    }else send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
    return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS SET DontKickOps function for the BotServ SET Command.
 * 
 * The BotServ SET DontKickOps Command allows the user to activate the KICK/BAN exception<br/>
 *   for ChannelOps
 *  
 * Syntax is: /msg BotServ SET #channel DONTKICKOPS { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_set_dontkickops(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * DontKickOps Command
	 */
 	//:shock PRIVMSG BotServ :set #channel dontkickops ON/OFF
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set dontkickops")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.bot[0]=='\0')									send_lang(thread,language,howtosend,snick,nick,"BS_UNASSIGN_NOT_ASSIGNED_TO",param[4]);
		
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || strcor(thread->c.chan.access.ownlevel,"mn") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set dontkickops")){
				if(streqnc(param[6],"on")){
                    if(~thread->c.chan.botserv.set&BS_SET_DONTKICKOPS_BIT){
						thread->c.chan.botserv.set |= BS_SET_DONTKICKOPS_BIT;
                    	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,mysql_quote(thread,thread->c.chan.name));
					}
                    send_lang(thread,language,howtosend,snick,nick,"BS_SET_DONTKICKOPS_ON",param[4]);
                    
                    setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
                    if(thread->c.chan.botserv.set&BS_SET_DONTKICKOPS_BIT){
						thread->c.chan.botserv.set &= ~BS_SET_DONTKICKOPS_BIT;
                    	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,mysql_quote(thread,thread->c.chan.name));
					}
                    send_lang(thread,language,howtosend,snick,nick,"BS_SET_DONTKICKOPS_OFF",param[4]);
                    
                    setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS SET DontKickVoices function for the BotServ SET Command.
 * 
 * The BotServ SET DontKickVoices Command allows the user to activate the KICK/BAN exception<br/>
 *   for ChannelVoices
 *  
 * Syntax is: /msg BotServ SET #channel DONTKICKVOICES { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_set_dontkickvoices(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * DontKickVoices Command
	 */
 	//:shock PRIVMSG BotServ :set #channel dontkickvoices ON/OFF
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set dontkickvoices")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.bot[0]=='\0')									send_lang(thread,language,howtosend,snick,nick,"BS_UNASSIGN_NOT_ASSIGNED_TO",param[4]);
		
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || strcor(thread->c.chan.access.ownlevel,"mn") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set dontkickvoices")){
				if(streqnc(param[6],"on")){
                    if(~thread->c.chan.botserv.set&BS_SET_DONTKICKVOICES_BIT){
						thread->c.chan.botserv.set |= BS_SET_DONTKICKVOICES_BIT;
                    	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,mysql_quote(thread,thread->c.chan.name));
					}
                    send_lang(thread,language,howtosend,snick,nick,"BS_SET_DONTKICKVOICES_ON",param[4]);
                    
                    setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
                    if(thread->c.chan.botserv.set&BS_SET_DONTKICKVOICES_BIT){
						thread->c.chan.botserv.set &= ~BS_SET_DONTKICKVOICES_BIT;
                    	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,mysql_quote(thread,thread->c.chan.name));
					}
                    send_lang(thread,language,howtosend,snick,nick,"BS_SET_DONTKICKVOICES_OFF",param[4]);
                    
                    setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS SET Greet function for the BotServ SET Command.
 * 
 * The BotServ SET Greet Command allows the user to activate the Greet-Option<br/>
 *   for target Channel or not. (Bot will greet joining Users with thier NS Greet Message)
 *  
 * Syntax is: /msg BotServ SET #channel GREET { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_set_greet(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * Greet Command
	 */
 	//:shock PRIVMSG BotServ :set #channel greet ON/OFF
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set greet")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.bot[0]=='\0')									send_lang(thread,language,howtosend,snick,nick,"BS_UNASSIGN_NOT_ASSIGNED_TO",param[4]);
		
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || strcor(thread->c.chan.access.ownlevel,"mn") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set greet")){
				if(streqnc(param[6],"on")){
                    if(~thread->c.chan.botserv.set&BS_SET_GREET_BIT){
						thread->c.chan.botserv.set |= BS_SET_GREET_BIT;
                    	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,mysql_quote(thread,thread->c.chan.name));
					}
                    send_lang(thread,language,howtosend,snick,nick,"BS_SET_GREET",_uc(param[6]),param[4]);
                    
                    setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
                    if(thread->c.chan.botserv.set&BS_SET_GREET_BIT){
						thread->c.chan.botserv.set &= ~BS_SET_GREET_BIT;
                    	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,mysql_quote(thread,thread->c.chan.name));
					}
                    send_lang(thread,language,howtosend,snick,nick,"BS_SET_GREET",_uc(param[6]),param[4]);
                    
                    setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS SET Fantasy function for the BotServ SET Command.
 * 
 * The BotServ SET Fantasy Command allows the user to activate the Fantasy-Option<br/>
 *   for target Channel. (Bot will execute Commands passed to Channel i.e. !op shock)
 *  
 * Syntax is: /msg BotServ SET #channel FANTASY { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_set_fantasy(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * Fantasy Command
	 */
 	//:shock PRIVMSG BotServ :set #channel fantasy ON/OFF
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set fantasy")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.bot[0]=='\0')									send_lang(thread,language,howtosend,snick,nick,"BS_UNASSIGN_NOT_ASSIGNED_TO",param[4]);
		
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || strcor(thread->c.chan.access.ownlevel,"mn") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set fantasy")){
				if(streqnc(param[6],"on")){
                    if(~thread->c.chan.botserv.set&BS_SET_FANTASY_BIT){
						thread->c.chan.botserv.set |= BS_SET_FANTASY_BIT;
                    	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,mysql_quote(thread,thread->c.chan.name));
					}
                    send_lang(thread,language,howtosend,snick,nick,"BS_SET_FANTASY",_uc(param[6]),param[4]);
                    
                    setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
                    if(thread->c.chan.botserv.set&BS_SET_FANTASY_BIT){
						thread->c.chan.botserv.set &= ~BS_SET_FANTASY_BIT;
                    	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,mysql_quote(thread,thread->c.chan.name));
					}
                    send_lang(thread,language,howtosend,snick,nick,"BS_SET_FANTASY",_uc(param[6]),param[4]);
                    
                    setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS SET Symbiosis function for the BotServ SET Command.
 * 
 * The BotServ SET Symbiosis Command allows the user to activate the Symbiosis-Option<br/>
 *   for target Channel. (The Bot will do all what is normally done by ChanServ)
 *  
 * Syntax is: /msg BotServ SET #channel SYMBIOSIS { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_set_symbiosis(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * Symbiosis Command
	 */
 	//:shock PRIVMSG BotServ :set #channel fantasy ON/OFF
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set symbiosis")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.bot[0]=='\0')									send_lang(thread,language,howtosend,snick,nick,"BS_UNASSIGN_NOT_ASSIGNED_TO",param[4]);
		
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set || strcor(thread->c.chan.access.ownlevel,"mn") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set symbiosis")){
				if(streqnc(param[6],"on")){
                    if(~thread->c.chan.botserv.set&BS_SET_SYMBIOSIS_BIT){
						thread->c.chan.botserv.set |= BS_SET_SYMBIOSIS_BIT;
                    	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,mysql_quote(thread,thread->c.chan.name));
					}
                    send_lang(thread,language,howtosend,snick,nick,"BS_SET_SYMBIOSIS",_uc(param[6]),param[4]);
                    
                    setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
                    if(thread->c.chan.botserv.set&BS_SET_SYMBIOSIS_BIT){
						thread->c.chan.botserv.set &= ~BS_SET_SYMBIOSIS_BIT;
                    	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`bs_set`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.botserv.set,mysql_quote(thread,thread->c.chan.name));
					}
                    send_lang(thread,language,howtosend,snick,nick,"BS_SET_SYMBIOSIS",_uc(param[6]),param[4]);
                    
                    setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS SET Nobot function for the BotServ SET Command.
 * 
 * The BotServ SET NoBot Command allows the admin to prevent users assigning bots<br/>
 *   to target Channel. (BotServ will ignore assigning Requests for that Channel)
 *  
 * Syntax is: /msg BotServ SET #channel NOBOT { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_set_nobot(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
#ifdef USE_THREADS
	char botnick[MAX_NICK_LEN];
#endif
	
	/* Initialise Variables to (NULL) */
#ifdef USE_THREADS
	bzero(&botnick,sizeof(botnick));
#endif
	
	/*
	 * NoBot Command
	 */
 	//:shock PRIVMSG BotServ :set #channel fantasy ON/OFF
	if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set nobot")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
			if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set nobot")){
				if(streqnc(param[6],"on")){
                    if(~thread->c.chan.settings&BS_SET_NOBOT_BIT){
						thread->c.chan.settings |= BS_SET_NOBOT_BIT;
                    	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_set`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));

						if(thread->c.chan.bot){
							thread->c.chan.botserv.set = -1;
	    					domysql(thread,"UPDATE `orion_cs_core` SET `botnick`=NULL, `bs_set`='%d' WHERE `channel`='%s'",thread->c.chan.botserv.set,mysql_quote(thread,param[4]));
	    					domysql(thread,"DELETE FROM `orion_bs_badwords` WHERE `channel`='%s'",mysql_quote(thread,param[4]));
							if(thread->c.chan.botinchan){
	        					do_privmsg(thread,thread->c.chan.bot,param[4],"I was unassigned from that channel, bye ladies and gentleman");

								/* if we are threadsafe we retrieve channel information
								 * with "UpdateChannelInformation3" here again (we deleted the bot via update before)
								 * means that we don't have "thread->c.chan.bot" filled properly in and after do_part.
								 * (so we have to save it before here!) */
#ifdef USE_THREADS
								dynncpy(botnick,thread->c.chan.bot,sizeof(thread->c.chan.bot));
		    					do_part(thread,botnick,param[4],NULL);
	        					thread->c.chan.botinchan = 0;
#else
								/* We use threads so we select channel information there again..
								 * (we don't have to care about countusers and such stuff) */
		    					do_part(thread,thread->c.chan.bot,param[4],NULL);
#endif
							}
#ifdef USE_THREADS
							send_lang(thread,language,howtosend,snick,nick,"BS_UNASSIGN_BOT_X_UNASSIGNED_FROM_CHANNEL_X",botnick,param[4]);
#else
							send_lang(thread,language,howtosend,snick,nick,"BS_UNASSIGN_BOT_X_UNASSIGNED_FROM_CHANNEL_X",thread->c.chan.bot,param[4]);
#endif
							bzero(&thread->c.chan.bot,sizeof(thread->c.chan.bot));
	                    }
					}
                    send_lang(thread,language,howtosend,snick,nick,"BS_SET_NOBOT",_uc(param[6]),param[4]);
                    
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(streqnc(param[6],"off")){
                    if(thread->c.chan.settings&BS_SET_NOBOT_BIT){
						thread->c.chan.settings &= ~BS_SET_NOBOT_BIT;
                    	domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_set`='%d' WHERE `channel`='%s'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.name));
					}
                    send_lang(thread,language,howtosend,snick,nick,"BS_SET_NOBOT",_uc(param[6]),param[4]);
                    
                    setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS SET Private function for the BotServ SET Command.
 * 
 * The BotServ SET Private Command allows the admin to prevent users accessing<br/>
 *   target bots. (BotServ will "hide" target Bot from the Public - only ServicesOps/ServicesAdmins)<br/>
 *   will assign target Bot to any Channel.
 *  
 * Syntax is: /msg BotServ SET botnick PRIVATE { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_set_private(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * Private Command
	 */
 	//:shock PRIVMSG BotServ :set botnick PRIVATE ON/OFF
	if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set private")){
		if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_bs_core` WHERE `botnick`='%s'",mysql_quote(thread,param[4])))){
    		if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set private")){
    			if(streqnc(param[6],"on")){
    				domysql(thread,"UPDATE `orion_bs_core` SET `private`='%d' WHERE `botnick`='%s'",1,mysql_quote(thread,param[4]));
    				send_lang(thread,language,howtosend,snick,nick,"BS_SET_PRIVATE",param[4],_uc(param[6]));
    			}else if(streqnc(param[6],"off")){
    				domysql(thread,"UPDATE `orion_bs_core` SET `private`='%d' WHERE `botnick`='%s'",0,mysql_quote(thread,param[4]));
    				send_lang(thread,language,howtosend,snick,nick,"BS_SET_PRIVATE",param[4],_uc(param[6]));
    			}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION",param[6]);
    		}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
        }else send_lang(thread,language,howtosend,snick,nick,"BS_NO_SUCH_BOT",param[4]);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS SET DontKickOps Help function for the BotServ SET Command (/msg BotServ HELP SET DONTKICKOPS)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP SET DONTKICKOPS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP SET DONTKICKOPS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_set_dontkickops_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * DontKickOps Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set dontkickops"))
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_SET_DONTKICKOPS");
	else
	    send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
	
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS SET DontKickVoices Help function for the BotServ SET Command (/msg BotServ HELP SET DONTKICKVOICES)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP SET DONTKICKVOICES,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP SET DONTKICKVOICES
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_set_dontkickvoices_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * DontKickVoices Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set dontkvoices"))
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_SET_DONTKICKVOICES");
	else
	    send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
	
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS SET Greet Help function for the BotServ SET Command (/msg BotServ HELP SET GREET)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP SET GREET,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP SET GREET
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_set_greet_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Greet Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set greet"))
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_SET_GREET");
	else
	    send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
	
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS SET Fantasy Help function for the BotServ SET Command (/msg BotServ HELP SET FANTASY)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP SET FANTASY,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP SET FANTASY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_set_fantasy_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Fantasy Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set fantasy"))
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_SET_FANTASY");
	else
	    send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
	
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS SET Symbiosis Help function for the BotServ SET Command (/msg BotServ HELP SET SYMBIOSIS)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP SET SYMBIOSIS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP SET SYMBIOSIS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_set_symbiosis_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Symbiosis Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set symbiosis"))
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_SET_SYMBIOSIS",snick);
	else
	    send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
	
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS SET NoBot Help function for the BotServ SET Command (/msg BotServ HELP SET NOBOT)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP SET NOBOT,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP SET NOBOT
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_set_nobot_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * NoBot Help
	 */
	if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"help set nobot"))
		send_lang(thread,language,howtosend,snick,nick,"BS_SERVADMIN_HELP_SET_NOBOT");
	else
	    send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
	
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS SET Private Help function for the BotServ SET Command (/msg BotServ HELP SET PRIVATE)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP SET PRIVATE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP SET PRIVATE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_set_private_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Private Help
	 */
	if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"help set private"))
		send_lang(thread,language,howtosend,snick,nick,"BS_SERVADMIN_HELP_SET_PRIVATE");
	else
	    send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
	
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief BotServ Help List Command - SET part (/msg BotServ help)
 * 
 * When the user tries to get general BotServ help, using /msg BotServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SET command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_set_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_BS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Set (Help listing)
	 */
	if(Users_BS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_CMD_SET");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

