/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_events.c
 *   @brief This is the Module file for ChanServ EVENTS Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ EVENTS command.
 *
 *   Main functions - EVENTS ONLY
 *
 *   Syntax is: /msg ChanServ EVENTS
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"
#include "webform.h"

#define SERVICE "cs"
#define MODUL "events"
#define MODUL_FILE "cs_events"
#define MODUL_VERSION "1.2"

/*  ******************************************* Event List *****************************************  */

MOD_BOOL cs_on_netinfo(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_on_topic(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_on_join(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_on_message(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_on_valid_fantasymessage(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL cs_primary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_secondary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_squit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

/*  **************************************** End of Event List **************************************  */

MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,EVENT_PROTOTYPE,EVENT_PROTOTYPE);

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addAccess (thread,&Module,NOONE);
	addMODType(thread,&Module,EVENT);

	addEventHook(thread,&Module,"on_netinfo"				,cs_on_netinfo);
	addEventHook(thread,&Module,"on_topic"					,cs_on_topic);
	addEventHook(thread,&Module,"on_join"					,cs_on_join);
	addEventHook(thread,&Module,"on_message"				,cs_on_message);
	addEventHook(thread,&Module,"on_valid_fantasymessage"	,cs_on_valid_fantasymessage);

	addEventHook(thread,&Module,"on_connect_primary"	,cs_primary);
	addEventHook(thread,&Module,"on_connect_secondary"	,cs_secondary);
	addEventHook(thread,&Module,"on_squit"				,cs_squit);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	
/* Things we want to process "on load" */
	cs->loaded = 1;
	
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ PRIMARY Command.
 * 
 * The ChanServ PRIMARY Command allows the user....
 *  
 * Syntax is: /msg ChanServ PRIMARY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_primary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    do_kill(thread,m->servername,cs->nick,"nick used by orion");
	do_sqline(thread,m->servername,cs->nick,0,time(NULL),"nick used by orion");
	createuser(thread,FAKETYPE_SERVICE,cs->nick,cs->ident,cs->host,IRC.CSMode,cs->realname);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ SECONDARY Command.
 * 
 * The ChanServ SECONDARY Command allows the user....
 *  
 * Syntax is: /msg ChanServ SECONDARY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_secondary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	if(m->logchan[0] != '\0'){
		do_services_join(thread,cs->nick,m->logchan,time(NULL),FAKETYPE_SERVICE);
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ SQUIT Command.
 * 
 * The ChanServ SQUIT Command allows the user....
 *  
 * Syntax is: /msg ChanServ SQUIT
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_squit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * cs_on_netinfo: ChanServ Events: On Netinfo Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL cs_on_netinfo(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	deepSQL channels;
	deepSQL nicks;
	char modes[200];
	char search[70];
	int i = 0;
	size_t j = 0;
	bzero(&channels,sizeof(channels));
	
	channels.res    = doSelectMySQL(thread,
					/*"SELECT DISTINCT `orion_cs_core`.`channel`,`orion_cs_core`.`mlock_positive`,`orion_cs_core`.`mlock_negative`,`orion_cs_core`.`mlock_limit`, "
									"`orion_cs_core`.`mlock_flood`,`orion_cs_core`.`mlock_key`,`orion_cs_core`.`datecreate`,`orion_cs_core`.`settings`,"
									"`orion_cs_core`.`last_topic`,`orion_cs_core`.`last_topic_setter`,`orion_cs_core`.`last_topic_time`,`orion_cs_core`.`reason` "
									"FROM `orion_cs_core` LEFT JOIN `orion_chan_users` ON `orion_chan_users`.`channel`=`orion_cs_core`.`channel` "
									"WHERE `orion_chan_users`.`channel` IS NOT NULL AND `orion_cs_core`.`serv`='ChanServ'"*/
					"SELECT DISTINCT `orion_cs_core`.`channel`,`orion_cs_core`.`mlock_positive`,`orion_cs_core`.`mlock_negative`,"
									"`orion_cs_core`.`mlock_limit`,`orion_cs_core`.`mlock_flood`,`orion_cs_core`.`mlock_key`,"
									"`orion_cs_core`.`datecreate`,`orion_cs_core`.`settings`,`orion_cs_core`.`last_topic`,"
									"`orion_cs_core`.`last_topic_setter`,`orion_cs_core`.`last_topic_time`,`orion_cs_core`.`reason`,"
									"COUNT(`orion_chan_users`.`nick`) AS `counted`,`orion_cs_core`.`mode`"
									"FROM `orion_cs_core` "
									"LEFT JOIN `orion_chan_users` "
										"ON `orion_chan_users`.`channel`=`orion_cs_core`.`channel` "
									"WHERE `orion_cs_core`.`serv`='ChanServ' GROUP BY `orion_cs_core`.`channel`"
									);
	channels.fields = mysql_num_rows(channels.res);

	for(i=0;i<channels.fields;i++){
		channels.row = mysql_fetch_row(channels.res);
		
		if(stoi(channels.row[12])>0 || stoi(channels.row[7])&CS_INHABIT_BIT){

			if(stoi(channels.row[7])&CS_INHABIT_BIT)
				do_services_join(thread,cs->nick,channels.row[0],stol(channels.row[6]) == 0 ? time(NULL) : (time_t)stol(channels.row[6]),FAKETYPE_SERVICE);
		
			if((channels.row[1][0]!='\0' || channels.row[2][0]!='\0') && ~stol(channels.row[7])&CS_SUSPEND_BIT){
				bzero(&modes,sizeof(modes));
				bzero(&search,sizeof(search));
			
				straddnc(search,'+',sizeof(search));
				if(!strcin(IRC.RegMode,channels.row[1]) && IRC.ChanRegMode) straddnc(search,IRC.RegMode,sizeof(search));
				straddns(search,channels.row[1],sizeof(search));
				sort(search);
				straddnc(search,'-',sizeof(search));
				sort(channels.row[2]);
				straddns(search,channels.row[2],sizeof(search));
			
				dynncpy(modes,search,sizeof(modes));
			
				while(search[j]!='\0'){
					if(search[j]==IRC.LimitCMode){
						straddnc(modes,' ',sizeof(modes));
						straddns(modes,channels.row[3],sizeof(modes));
					}
					else if(search[j]==IRC.FloodCMode){
						straddnc(modes,' ',sizeof(modes));
						straddns(modes,channels.row[4],sizeof(modes));
					}
					else if(search[j]==IRC.KeyCMode){
						straddnc(modes,' ',sizeof(modes));
						straddns(modes,channels.row[5],sizeof(modes));
					}
					j++;
				}
				if(modes[0]!='\0') do_mode(thread,cs->nick,channels.row[0],stol(channels.row[6]) == 0 ? time(NULL) : (time_t)stol(channels.row[6]),"%s",modes);
			}
			else if(IRC.ChanRegMode){
				if(channels.row[14][0]!='\0' && !strcin(IRC.RegMode,channels.row[14]))
				do_mode(thread,cs->nick,channels.row[0],stol(channels.row[6]) == 0 ? time(NULL) : (time_t)stol(channels.row[6]),IRC.ModeOnChannelReg);
			}

			if(stol(channels.row[7])&CS_SUSPEND_BIT){
				bzero(&nicks,sizeof(nicks));
			
				nicks.res    = doSelectMySQL(thread,"SELECT `orion_chan_users`.`nick`,`orion_core_condata`.`vhost` FROM `orion_chan_users` LEFT JOIN `orion_core_condata` ON `orion_chan_users`.`nick`=`orion_core_condata`.`nick` WHERE `channel`='%s' AND `orion_core_condata`.`faketype`='0'",mysql_quote(thread,channels.row[0]));
				nicks.fields = mysql_num_rows(nicks.res);
		
				do_mode(thread,cs->nick,channels.row[0],thread->c.chan.datecreate,IRC.OnSuspend);
				
				for(i=0;i<nicks.fields;i++){
					nicks.row = mysql_fetch_row(nicks.res);
			
					do_mode(thread,cs->nick,channels.row[0],thread->c.chan.datecreate,"+b %s",nicks.row[1]);
					do_kick(thread,cs->nick,channels.row[0],nicks.row[0],"channel is suspended reason: %s",channels.row[11]);
				}
				/* mysql_free_result_myr(); - Chanaccess */
				mysql_free_result_myr(nicks.res);
			}
		
			if(~stoi(channels.row[7])&CS_SUSPEND_BIT && stoi(channels.row[7])&CS_KEEPTOPIC_BIT && channels.row[8][0]!='\0')
				do_topic(thread, cs->nick, channels.row[9], channels.row[0], time(NULL), channels.row[8]);
		}
	}
	mysql_free_result_myr(channels.res);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * cs_on_topic: ChanServ Events: On Topic Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL cs_on_topic(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	//@param - [0](setter), [1](channel), [2](date), [3](topic)
	//@nick  - setter
	
	/*printf("param[0] = %s\n",param[0]);
	printf("param[1] = %s\n",param[1]);
	printf("param[3] = %s\n",param[3]);
	printf("param[2] = %ld\n",stol(param[2]));*/
	
	/* Updates the Threads Channel Management */
	UpdateChannelInformation3(thread,param[1],NULL);
	printf("cs_on_topic - internal\n");
	if(thread->c.chan.fields && streqnc(thread->c.chan.serv,"chanserv")){
		printf("cs_on_topic - internal if/else\n");
		if(thread->c.chan.settings&CS_TOPICLOCK_BIT)
			//do_topic(thread,cs->nick,param[1],thread->c.chan.datecreate,thread->c.chan.last_topic);
			//else   We Will leave the Topic then to cache and SQL... (change: shock)
			do_rawtopic(thread, cs->nick, param[1], cs->nick, thread->c.chan.last_topic, time(NULL));
		else{
			printf("cs_on_topic - internal else\n");
			/* 	Update our Cache	*/
			//bzero(&thread->c.chan.last_topic,sizeof(thread->c.chan.last_topic));
			//bzero(&thread->c.chan.last_topic_setter,sizeof(thread->c.chan.last_topic_setter));
		
			dynncpy(thread->c.chan.last_topic,param[3],sizeof(thread->c.chan.last_topic));
			dynncpy(thread->c.chan.last_topic_setter,param[0],sizeof(thread->c.chan.last_topic_setter));
			thread->c.chan.last_topic_time = (time_t)stol(param[2]);
			/**			***			**/
		
			domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`last_topic`='%s',`last_topic_setter`='%s',`last_topic_time`='%ld' WHERE `channel`='%s'",time(NULL),mysql_quote(thread,thread->c.chan.last_topic),mysql_quote(thread,thread->c.chan.last_topic_setter),thread->c.chan.last_topic_time,mysql_quote(thread,param[1]));
		}
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * cs_on_join: ChanServ Events: On Join Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL cs_on_join(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	//@param - [0](nick), [1](channel), [2](time)
	/* Deklarate Variables */
	//DB3 c;
	deepSQL Akick;
	Stays *stay = NULL;
	bool csjoined = 0;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	bzero(&Akick,sizeof(Akick));
	csjoined = 0;
	Users *user = GetUser(nick);
	
	/*
	 * If it's a ChanServ channel (Registered from ChanServ)
	 */
	if(user==NULL)
	    return MOD_CONT;
	
	/* Updates the Threads Channel Management - MAKE ME TARGET HERE! */
	if(user->rights>=is_services_user && !thread->c.aliases.active){
		thread->c = sql_thread_data(thread,param[1],SQL_THREAD_AAC2,nick);
	}else
		UpdateChannelInformation2(thread,param[1],nick);
	 
	if(thread->c.chan.fields && streqnc(thread->c.chan.serv,"chanserv") && thread->c.chan.bot[0]=='\0'){
		 thread->c.chan.datecreate = (time_t)stol(param[2]);

		/* We have to part ChanServ if we joined it before */
		stay = getStayChannel(thread, thread->c.chan.name);
		
		if(stay != NULL){
			if(thread->c.chan.countusers >= 2 && stay->csjoin)
				do_part(thread, cs->nick, stay->channel,NULL);
		}

		if(thread->c.chan.countusers > m->BSMinUsers)
			delStayChannel(thread,thread->c.chan.name);
		/***************************************************/

		/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
		if(thread->c.chan.settings&CS_INHABIT_BIT){
			Users 		*my_chanserv 		= GetUser(cs->nick);
			Uchannels 	*my_chanserv_channel= NULL;

			if(my_chanserv!=NULL)
				my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

			/* WOOHOOO ChanServ isn't in the Channel yet - if we will set a ban now, it will disappear without any user ...
			 and since ChanServ should be a channel inhabitant according to the channel settings -> it joins the channel here ;) */
			if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
				do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
				do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
			}
		}

		dlog(thread,2, "thread->c.aliases.fields = %d, thread->c.aliases.authnick = %s\n",thread->c.aliases.fields,thread->c.aliases.authnick);
		dlog(thread,2, "thread->c.aliases.active = %d\n",thread->c.aliases.active);
		
		/*if(thread->c.aliases.fields && thread->c.aliases.authnick[0]!='\0'){
			dlog(thread,2, "entered if\n");
			if(!thread->c.aliases.active){
				dlog(thread,2, "!thread->c.aliases.active\n");
				unknown_access(thread->c.chan.access.targetlevel,sizeof(thread->c.chan.access.targetlevel));
				dlog(thread,2, "thread->c.chan.access.targetlevel = %d\n",thread->c.chan.access.targetlevel);
			}
		}*/
		
		
/* MLOCK FORCE */
			int j=0;
			if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers == 1 && thread->c.chan.serv[0]!='\0' && (thread->c.chan.mlock_positive[0]!='\0' || thread->c.chan.mlock_negative[0]!='\0')){
				char modes		 [200];
				char search		 [70];
				
				bzero(&modes,sizeof(modes));
				bzero(&search,sizeof(search));

				straddnc(search,'+',sizeof(search));
				if(!strcin(IRC.RegMode,thread->c.chan.mlock_positive) && IRC.ChanRegMode)
					straddnc(search,IRC.RegMode,sizeof(search));
					
				straddns(search,thread->c.chan.mlock_positive,sizeof(search));
				sort(search);
				straddnc(search,'-',sizeof(search));
				sort(thread->c.chan.mlock_negative);
				straddns(search,thread->c.chan.mlock_negative,sizeof(search));

				dynncpy(modes,search,sizeof(modes));

				while(search[j]!='\0'){
					if(search[j]==IRC.LimitCMode){
						straddnc(modes,' ',sizeof(modes));
						straddns(modes,_dynadd("%ld",thread->c.chan.mlock_limit),sizeof(modes));
					}
					else if(search[j]==IRC.FloodCMode){
						straddnc(modes,' ',sizeof(modes));
						straddns(modes,thread->c.chan.mlock_flood,sizeof(modes));
					}
					else if(search[j]==IRC.KeyCMode){
						straddnc(modes,' ',sizeof(modes));
						straddns(modes,thread->c.chan.mlock_key,sizeof(modes));
					}
					j++;
				}

				if(modes[0]!='\0') do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.countusers == 1 ? time(NULL) : thread->c.chan.datecreate,"%s",modes);
			}
			else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.serv[0]!='\0' && thread->c.chan.countusers == 1 && IRC.ChanRegMode){
				do_mode(thread, cs->nick, thread->c.chan.name, thread->c.chan.datecreate == 0 ? time(NULL) : thread->c.chan.datecreate, IRC.ModeOnChannelReg);
				/* This is to avoid channel desyncs */
				//domysql(thread,"UPDATE `orion_cs_core` SET `datecreate`='%ld',`mode`='%s',`link`='%s',`limit`='%d',`key`='%s',`flood`='%s' WHERE `channel` = '%s'",thread->c.chan.datecreate,mysql_quote(thread,thread->c.chan.link),thread->c.chan.limit,mysql_quote(thread,thread->c.chan.key),mysql_quote(thread,thread->c.chan.flood),mysql_quote(thread,thread->c.chan.name));
			}else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.serv[0]!='\0' && thread->c.chan.countusers == 1 && ~thread->c.chan.settings&CS_SUSPEND_BIT && thread->c.chan.settings&CS_KEEPTOPIC_BIT && thread->c.chan.last_topic[0]!='\0')
				do_topic(thread, cs->nick, thread->c.chan.last_topic_setter[0]=='\0' ? cs->nick : thread->c.chan.last_topic_setter, thread->c.chan.name, time(NULL), thread->c.chan.last_topic);
/* MLOCK FORCE END */
		
		
		if(thread->c.chan.name && (thread->c.chan.settings & CS_SUSPEND_BIT || thread->c.chan.settings & CS_FORBID_BIT) && thread->sql.access.rights<=is_services_admin){ //SO required
			/* Make the ban stay in channel - We proof the List for such channels later */

			/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
			if(thread->c.chan.settings&CS_INHABIT_BIT){
				Users 		*my_chanserv 		= GetUser(cs->nick);
				Uchannels 	*my_chanserv_channel= NULL;

				if(my_chanserv!=NULL)
					my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

				/* WOOHOOO ChanServ isn't in the Channel yet - if we will set a ban now, it will disappear without any user ...
				 and since ChanServ should be a channel inhabitant according to the channel settings -> it joins the channel here ;) */
				if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
					do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
					do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
				}
			}

			/* ChanServ isn't inhabitant, so if there is only one user in the channel (the one we want kick now)
			 we have to join the channel to make the ban "permanent" */
			else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers == 1 && getStayChannel(thread,thread->c.chan.name)==NULL){
				do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
				do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
				csjoined=1;
			}
			
			do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,IRC.OnSuspend);
			do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",BuildBanMask(thread,&thread->c.Cdata,CSAnalyseBantype(thread->c.chan.settings)));
			
			if(thread->c.chan.settings & CS_SUSPEND_BIT)	 do_kick_nopart(thread,cs->nick,thread->c.chan.name,nick,"suspended [%s]",thread->c.chan.reason);
			else if(thread->c.chan.settings & CS_FORBID_BIT) do_kick_nopart(thread,cs->nick,thread->c.chan.name,nick,thread->c.chan.reason[0]=='\0' ? "Forbidden" : thread->c.chan.reason);
			//thread->c.chan.countusers--;
		}
		
		else if(!streqnc(nick,cs->nick)){
			
			if(stoi(thread->c.chan.access.targetlevel)<=IDENTIFIED_NO_ACCESS && (thread->c.chan.settings&CS_RESTRICTED_BIT)){
				/* Make the ban stay in channel - We proof the List for such channels later */

				/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
				if(thread->c.chan.settings&CS_INHABIT_BIT){
					Users 		*my_chanserv 		= GetUser(cs->nick);
					Uchannels 	*my_chanserv_channel= NULL;

					if(my_chanserv!=NULL)
						my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

					/* WOOHOOO ChanServ isn't in the Channel yet - if we will set a ban now, it will disappear without any user ...
					 and since ChanServ should be a channel inhabitant according to the channel settings -> it joins the channel here ;) */
					if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
						do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
						do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
					}
				}

				/* ChanServ isn't inhabitant, so if there is only one user in the channel (the one we want kick now)
				 we have to join the channel to make the ban "permanent" */
				else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers == 1 && getStayChannel(thread,thread->c.chan.name)==NULL){
					do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
					do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
					csjoined=1;
				}
				
				do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",BuildBanMask(thread,&thread->c.Cdata,CSAnalyseBantype(thread->c.chan.settings)));
				do_kick_nopart(thread,cs->nick,thread->c.chan.name,nick,"You are not permitted to be on this channel.");
				//thread->c.chan.countusers--;
			}
			
			else{
				/* ******************************* AKick Check **************************************** */
				Akick.res    = AkickOnJoin(thread,param[1],nick);
				Akick.fields = mysql_num_rows(Akick.res);
				
				if(Akick.fields){
					while((Akick.row = mysql_fetch_row(Akick.res))){
						if(Akick.row[1]!=NULL){
							/* Make the ban stay in channel - We proof the List for such channels later */
							
							/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
							if(thread->c.chan.settings&CS_INHABIT_BIT){
								Users 		*my_chanserv 		= GetUser(cs->nick);
								Uchannels 	*my_chanserv_channel= NULL;

								if(my_chanserv!=NULL)
									my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

								/* WOOHOOO ChanServ isn't in the Channel yet - if we will set a ban now, it will disappear without any user ...
								 and since ChanServ should be a channel inhabitant according to the channel settings -> it joins the channel here ;) */
								if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
									do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
									do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
								}
							}

							/* ChanServ isn't inhabitant, so if there is only one user in the channel (the one we want kick now)
							 we have to join the channel to make the ban "permanent" */
							else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers == 1 && getStayChannel(thread,thread->c.chan.name)==NULL){
								do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
					        	do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
					        	csjoined=1;
							}
							
							do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",Akick.row[1]);
							do_kick_nopart(thread,cs->nick,thread->c.chan.name,nick,"Akicked: %s",Akick.row[2]);
							//thread->c.chan.countusers--;
						}
			        }
				}
				mysql_free_result_myr(Akick.res);
				/* ************************************************************************************ */
				
				if(stoi(thread->c.chan.access.targetlevel)<=thread->c.chan.levels.nojoin){
					/* Make the ban stay in channel - We proof the List for such channels later */

					/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
					if(thread->c.chan.settings&CS_INHABIT_BIT){
						Users 		*my_chanserv 		= GetUser(cs->nick);
						Uchannels 	*my_chanserv_channel= NULL;

						if(my_chanserv!=NULL)
							my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

						/* WOOHOOO ChanServ isn't in the Channel yet - if we will set a ban now, it will disappear without any user ...
						 and since ChanServ should be a channel inhabitant according to the channel settings -> it joins the channel here ;) */
						if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
							do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
							do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
						}
					}

					/* ChanServ isn't inhabitant, so if there is only one user in the channel (the one we want kick now)
					 we have to join the channel to make the ban "permanent" */
					else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers == 1 && getStayChannel(thread,thread->c.chan.name)==NULL){
						do_join(thread,FAKETYPE_SERVICE,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
					    do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
					    csjoined=1;
					}
					
					do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",BuildBanMask(thread,&thread->c.Cdata,CSAnalyseBantype(thread->c.chan.settings)));
					do_kick_nopart(thread,cs->nick,thread->c.chan.name,nick,"matched NOJOIN levelsetting");
					//thread->c.chan.countusers--;
				}else{
					dlog(thread,2, "MODE TEIL\n");
					dlog(thread,2, "stoi(thread->c.chan.access.targetlevel) == %d\n",stoi(thread->c.chan.access.targetlevel));
					
					if(stoi(thread->c.chan.access.targetlevel)>IDENTIFIED_NO_ACCESS)
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%d' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),mysql_quote(thread,thread->c.chan.name));
					

					if(stoi(thread->c.chan.access.targetlevel)>=ACCESS_FOUNDER && ~thread->sql.access.settings&NS_SET_INCOGNITO)
						do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+qao %s %s %s",nick,nick,nick);
					else if(streqnc(nick,thread->c.chan.founder) && ~thread->sql.access.settings&NS_SET_INCOGNITO)
						do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+qao %s %s %s",nick,nick,nick);
						
					else if(stoi(thread->c.chan.access.targetlevel)>=thread->c.chan.levels.autoop && ((thread->sql.access.settings&NS_SET_INCOGNITO && thread->c.chan.levels.autoop<=IDENTIFIED_NO_ACCESS) || ~thread->sql.access.settings&NS_SET_INCOGNITO)){
						if(stoi(thread->c.chan.access.targetlevel)>=thread->c.chan.levels.autoprotect && ((thread->sql.access.settings&NS_SET_INCOGNITO && thread->c.chan.levels.autoprotect<=IDENTIFIED_NO_ACCESS) || ~thread->sql.access.settings&NS_SET_INCOGNITO))
							do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oa %s %s",nick,nick);
						else do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+o %s",nick);
					}

					else if(stoi(thread->c.chan.access.targetlevel)>=thread->c.chan.levels.autohalfop && ((thread->sql.access.settings&NS_SET_INCOGNITO && thread->c.chan.levels.autohalfop<=IDENTIFIED_NO_ACCESS) || ~thread->sql.access.settings&NS_SET_INCOGNITO)){
						if(stoi(thread->c.chan.access.targetlevel)>=thread->c.chan.levels.autoprotect && ((thread->sql.access.settings&NS_SET_INCOGNITO && thread->c.chan.levels.autoprotect<=IDENTIFIED_NO_ACCESS) || ~thread->sql.access.settings&NS_SET_INCOGNITO))
							do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+ha %s %s",nick,nick);
						else do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+h %s",nick);
					}

					else if(stoi(thread->c.chan.access.targetlevel)>=thread->c.chan.levels.autovoice && ((thread->sql.access.settings&NS_SET_INCOGNITO && thread->c.chan.levels.autovoice<=IDENTIFIED_NO_ACCESS) || ~thread->sql.access.settings&NS_SET_INCOGNITO))
						do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+v %s",nick);
				}
			}
			if(thread->c.chan.fields && thread->c.chan.welcome[0]!='\0')
				msend(thread,0,cs->nick,nick,"[%s] %s",thread->c.chan.name,thread->c.chan.welcome);
			if(~thread->c.chan.settings&CS_SUSPEND_BIT && thread->c.chan.settings&CS_KEEPTOPIC_BIT && thread->c.chan.last_topic[0]!='\0' && thread->c.chan.countusers==1)
				do_topic(thread, cs->nick, thread->c.chan.last_topic_setter[0]=='\0' ? cs->nick : thread->c.chan.last_topic_setter, thread->c.chan.name, time(NULL), thread->c.chan.last_topic);
		}
		
		/* Manage our Bot */
		if(csjoined && thread->c.chan.countusers <= 0)
		{
			addStayChannel(thread,thread->c.chan.name,cs->nick,thread->c.chan.botinchan,csjoined);
		}
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * cs_on_valid_fantasymessage: ChanServ Events: On valid Fantasy Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL cs_on_valid_fantasymessage(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Now get the command parameter without ":" */
	char *cmd = &param[3][1];

	if(GetUser(nick)==NULL)
	    return MOD_CONT;
	    
	if(str_start(cmd,m->BSFantasyCharacter)){
		ModCheck modcheck;
    	ModCo *modul = NULL;
    	
    	/* Now get the real command */
    	cmd    = &param[3][dynlen(m->BSFantasyCharacter)+1];
    	
    	bzero(&modcheck,sizeof(modcheck));
    	
    	bzero(&thread->sql,sizeof(SQL));
		sql_mainAAC(thread,nick);
		
    	/* Since language == NULL, howtosend = 0, snick == NULL - we wake them up now ;) */
    	snick		= cs->nick;
    	howtosend	= thread->sql.access.settings&NS_SET_MSG ? 1 : 0;
    	language	= languages[thread->sql.access.language];
    	/*********************************************************************************/
    	
	/* CS Commands */
		/* If help for a specific Command was requested */
    	if(streqnc(cmd,"help") && param[4]){
			
			/* Help with wildmasks */
    		if(strcor(param[4],"*"))
				call_wildview(thread, snick, nick, SERVICE, "help", param);
    		
			/* Normal help for a specific command (no wildmasks) */
			else{
    			modul 		= findModule(thread, SERVICE, param[4]);
  				modcheck 	= call_helpcommand(thread, modul, cmd, snick, nick, param);
  				
				if(!ModReply(thread, &modcheck, snick, nick, param))
					send_lang(thread, language, howtosend, snick, nick, "CS_UNKNOWN_COMMAND", snick);
    		}
    	}
    	
		/* If Command was only "help" */
		else if(streqnc(cmd,"help") && !param[4]){
    		modul 		= findModule(thread, SERVICE, cmd);
    		modcheck 	= call_helpcommand(thread, modul, cmd, snick, nick, param);
    		
			if(!ModReply(thread, &modcheck, snick, nick, param))
    			send_lang(thread, language, howtosend, snick, nick, "CS_UNKNOWN_COMMAND", snick);
    	}
    	
		/* All other commands which aren't "help" */
		else if(!streqnc(cmd,"help")){
			/* Let's create our fake PRIVMSG Message */
    		modul 		= findModule(thread, SERVICE, cmd);
			if(modul){
			// :shock PRIVMSG #deepirc :!access add shock 10
				char fake_ircd_msg[BUFSIZE];
				bzero(&fake_ircd_msg,sizeof(fake_ircd_msg));
						
				sprintf(fake_ircd_msg,"%s %s %s :%s %s",param[0],param[1],snick,&param[3][dynlen(m->BSFantasyCharacter)+1],param[2]);
		
				dlog(thread,2,"fake_ircd_msg: %s\n",fake_ircd_msg);
				straddc(fake_ircd_msg,' ');
				stradds(fake_ircd_msg,_dstrjoin(' ',param,4));
						
				dlog(thread,2,"fake_ircd_msg: %s\n",fake_ircd_msg);
				char *newparams[ARRAY_SIZE];
				strsplit(fake_ircd_msg,newparams,' ');
				/*********************************************************/
			
				modcheck = call_command(thread, modul, cmd, snick, nick, newparams);
			  	
				if(!ModReply(thread, &modcheck, snick, nick, newparams))
    				send_lang(thread, language, howtosend, snick, nick, "CS_UNKNOWN_COMMAND", snick);
			}
    	}
	}
	return MOD_CONT;
}


/*  ************************************************************************************************  */

/**
 * cs_on_message: ChanServ Events: On Message Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL cs_on_message(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* (language == NULL, howtosend = 0, snick == NULL) */

	if((m->UseStrictPrivMsg && streqnc(param[2],_dynadd("%s@%s",cs->nick,m->servername))) || (!m->UseStrictPrivMsg && (streqnc(param[2],cs->nick) || streqnc(param[2],_dynadd("%s@%s",cs->nick,m->servername))))){
		if(GetUser(nick)==NULL && (!thread->Webform && FindWebSocket(thread->Sock)==NULL))
	    	return MOD_CONT;

		ModCheck modcheck;
    	ModCo *modul = NULL;
    	char *cmd    = &param[3][1];
    	
    	bzero(&modcheck,sizeof(modcheck));
    	
    	bzero(&thread->sql,sizeof(SQL));
		sql_mainAAC(thread,nick);
		
    	/* Since language == NULL, howtosend = 0, snick == NULL - we wake them up now ;) */
    	snick		= cs->nick;
    	howtosend	= thread->sql.access.settings&NS_SET_MSG ? 1 : 0;
    	language	= languages[thread->sql.access.language];
    	/*********************************************************************************/
    	
	/* CS Commands */
	
		/* If help for a specific Command was requested */
    	if(streqnc(cmd,"help") && param[4]){
			
			/* Help with wildmasks */
    		if(strcor(param[4],"*"))
				call_wildview(thread, snick, nick, SERVICE, "help", param);
    		
			/* Normal help for a specific command (no wildmasks) */
			else{
    			modul 		= findModule(thread, SERVICE, param[4]);
  				modcheck 	= call_helpcommand(thread, modul, cmd, snick, nick, param);
  				
				if(!ModReply(thread, &modcheck, snick, nick, param))
					send_lang(thread, language, howtosend, snick, nick, "CS_UNKNOWN_COMMAND", snick);
    		}
    	}
		
		/* If Command was only "help" */
		else if(streqnc(cmd,"help") && !param[4]){
    		modul 		= findModule(thread, SERVICE, cmd);
    		modcheck 	= call_helpcommand(thread, modul, cmd, snick, nick, param);
    		
			if(!ModReply(thread, &modcheck, snick, nick, param))
    			send_lang(thread, language, howtosend, snick, nick, "CS_UNKNOWN_COMMAND", snick);
    	}
    	
		/* All other commands which aren't "help" */
		else if(!streqnc(cmd,"help")){
    		modul 		= findModule(thread, SERVICE, cmd);
  			modcheck 	= call_command(thread, modul, cmd, snick, nick, param);
			  	
			if(!ModReply(thread, &modcheck, snick, nick, param))
    			send_lang(thread, language, howtosend, snick, nick, "CS_UNKNOWN_COMMAND", snick);
    	}
		else send_lang(thread, language, howtosend, snick, nick, "CS_UNKNOWN_COMMAND", snick);
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return MOD_BOOL
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	if(Action == DO_UNLOAD){
		cs->loaded = 0;
		delStayNick(thread,cs->nick);
		do_quit(thread,cs->nick,"module unloaded");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */
