/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_getkey.c
 *   @brief This is the Module file for ChanServ GETKEY Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ GETKEY command.
 *
 *   The ChanServ GETKEY Command allows the user to get the channels key.<br/>
 *   This is a command mainly intended to be used by bots and/or scripts, so<br/>
 *   the output is in the following way: "KEY <channel> <key>"
 *
 *   Syntax is: /msg ChanServ GETKEY #channel
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "webform.h"
#include "global.h"

#define SERVICE "cs"
#define MODUL "getkey"
#define MODUL_FILE "cs_getkey"
#define MODUL_VERSION "1.2"

MOD_BOOL cs_getkey(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_getkey_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_getkey_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"getkey","getkey");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,WEBACCESS,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"getkey",cs_getkey,is_services_oper,is_services_user,4,4);
	addCommand(thread,&Module,"help getkey",cs_getkey_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_getkey_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ GETKEY Command.
 * 
 * The ChanServ GETKEY Command allows the user to get the channels key.<br/>
 *   this is a command mainly intended to be used by bots and/or scripts, so<br/>
 *   the output is in the following way: "KEY <channel> <key>"
 *
 * Syntax is: /msg ChanServ GETKEY #channel
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_getkey(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	int sender_level = 0;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * Getkey Command
	 */
 	// :shock PRIVMSG chanserv :aop #channel add nick
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"getkey")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
            sender_level = stoi(thread->c.chan.access.ownlevel);
			if(sender_level>=thread->c.chan.levels.getkey || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"getkey")){
				if(thread->c.chan.key[0]!='\0') msend(thread,howtosend,snick,nick,"KEY %s %s",param[4],thread->c.chan.key);
				else send_lang(thread,language,howtosend,snick,nick,"CS_GETKEY_NO_KEY_FOR_CHANNEL_X",thread->c.chan.name);
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_TOO_LOW",sender_level,param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_OR_NO_XOP",snick);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ GETKEY Command (/msg ChanServ HELP GETKEY)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP GETKEY,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP GETKEY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_getkey_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Getkey Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help getkey")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_GETKEY");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}

	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - GETKEY part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the GETKEY command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_getkey_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Getkey (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_GETKEY");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
