/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_register.c
 *   @brief This is the Module file for ChanServ REGISTER Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ REGISTER command.
 *
 *   The ChanServ REGISTER Command allows the user to register a channel.<br/>
 *   In order to use this command, you must first be a channel operator<br/>
 *   on the channel you're trying to register.
 *
 *   Syntax is: /msg ChanServ REGISTER #channel < description >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "webform.h"
#include "global.h"

#define SERVICE "cs"
#define MODUL "register"
#define MODUL_FILE "cs_register"
#define MODUL_VERSION "1.2"

MOD_BOOL cs_register(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_register_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_register_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"register","register");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"register",cs_register,is_services_admin,is_services_user,5,0);
	addCommand(thread,&Module,"help register",cs_register_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_register_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ REGISTER Command.
 * 
 * The ChanServ REGISTER Command allows the user to register a channel.<br/>
 *   In order to use this command, you must first be a channel operator<br/>
 *   on the channel you're trying to register.
 *
 * Syntax is: /msg ChanServ REGISTER #channel < description >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_register(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//Channel c;
	int settings = 0;
	int botsettings = 0;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Register Command
	 */
 	// :shock PRIVMSG chanserv :register #channel description
	 
	if(!thread->Webform && thread->sql.Cdata.localtime+m->CSWaitRegister>time(NULL)){
		send_lang(thread,language,howtosend,snick,nick,"CHANNEL_REGISTER_NOT_LONG_ENOUGH_CONNECTED",m->CSWaitRegister);
		return MOD_CONT;
	}
 	
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"register")){
		if(param[4][0]=='#'){
			/* Updates the Threads Channel Management */
			UpdateChannelInformation(thread,param[4],NULL);
			dynncpy(thread->c.chan.name,param[4],sizeof(thread->c.chan.name));
	    	
	    	if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
	    	else if(thread->c.chan.fields && thread->c.chan.serv[0]=='\0'){
				
				if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_chan_users` WHERE `nick`='%s' AND `channel`='%s' AND `modes` LIKE '%c%c%c'",mysql_quote(thread,nick),mysql_quote(thread,param[4]),'%',IRC.OpMode,'%'))){
		    		if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_cs_core` WHERE `founder`='%s' OR `successor`='%s'",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,thread->sql.aliases.authnick)))<m->CSMaxReg || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"register")){
						do_mode(thread,cs->nick,param[4],thread->c.chan.datecreate,IRC.DefaultMLOCK);
			    		do_mode(thread,cs->nick,param[4],thread->c.chan.datecreate,"+q %s",nick);
			    		
			    		domysql(thread,"INSERT INTO `orion_cs_access` (`serv`,`authnick`,`channel`,`level`) VALUES ('ChanServ','%s','%s','%d')",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,param[4]),ACCESS_FOUNDER);
			    		
						settings    = CSCheckSettings();
						botsettings = BSCheckSettings();
						settings    = CSAddBanType(settings);
						
						/* Update Memory, for caching */
						thread->c.chan.levels.set 			= CS_SET;
						thread->c.chan.levels.autodeop 		= CS_AUTODEOP;
						thread->c.chan.levels.autohalfop 	= CS_AUTOHALFOP;
						thread->c.chan.levels.autoop 		= CS_AUTOOP;
						thread->c.chan.levels.autoprotect 	= CS_AUTOPROTECT;
						thread->c.chan.levels.autovoice 	= CS_AUTOVOICE;
						thread->c.chan.levels.nojoin 		= CS_NOJOIN;
						thread->c.chan.levels.signkick 		= CS_SIGNKICK;
						thread->c.chan.levels.acclist 		= CS_ACCESS_LIST;
						thread->c.chan.levels.accchange 	= CS_ACCESS_CHANGE;
						thread->c.chan.levels.akick 		= CS_AKICK;
						thread->c.chan.levels.ban 			= CS_BAN;
						thread->c.chan.levels.banme 		= CS_BANME;
						thread->c.chan.levels.clear 		= CS_CLEAR;
						thread->c.chan.levels.getkey 		= CS_GETKEY;
						thread->c.chan.levels.halfop 		= CS_HALFOP;
						thread->c.chan.levels.halfopme 		= CS_HALFOPME;
						thread->c.chan.levels.info 			= CS_INFO;
						thread->c.chan.levels.kick 			= CS_KICK;
						thread->c.chan.levels.kickme 		= CS_KICKME;
						thread->c.chan.levels.invite 		= CS_INVITE;
						thread->c.chan.levels.opdeop 		= CS_OPDEOP;
						thread->c.chan.levels.opdeopme 		= CS_OPDEOPME;
						thread->c.chan.levels.protect 		= CS_PROTECT;
						thread->c.chan.levels.protectme 	= CS_PROTECTME;
						thread->c.chan.levels.topic 		= CS_TOPIC;
						thread->c.chan.levels.unban 		= CS_UNBAN;
						thread->c.chan.levels.voice 		= CS_VOICE;
						thread->c.chan.levels.voiceme 		= CS_VOICEME;
						thread->c.chan.levels.memo 			= CS_MEMO;
						thread->c.chan.levels.assign 		= CS_ASSIGN;
						thread->c.chan.levels.badwords 		= CS_BADWORDS;
						thread->c.chan.levels.fantasia 		= CS_FANTASIA;
						thread->c.chan.levels.greet 		= CS_GREET;
						thread->c.chan.levels.nokick 		= CS_NOKICK;
						thread->c.chan.levels.say 			= CS_SAY;
						
						thread->c.chan.settings 			= settings;
						thread->c.chan.botserv.set 			= botsettings;
						thread->c.chan.botserv.caps_ttb 	= -1;
						thread->c.chan.botserv.caps_min 	= -1;
						thread->c.chan.botserv.caps_per 	= -1;
						thread->c.chan.botserv.flood_ttb 	= -1;
						thread->c.chan.botserv.flood_ln 	= -1;
						thread->c.chan.botserv.flood_secs 	= -1;
						thread->c.chan.botserv.repeat_ttb 	= -1;
						thread->c.chan.botserv.repeat	 	= -1;
						thread->c.chan.botserv.colors	 	= -1;
						thread->c.chan.botserv.underlines 	= -1;
						thread->c.chan.botserv.reverses	 	= -1;
						thread->c.chan.botserv.badwords	 	= -1;
						thread->c.chan.botserv.bolds	 	= -1;
						
						dynncpy(thread->c.chan.serv,"ChanServ",sizeof(thread->c.chan.serv));
						dynncpy(thread->c.chan.founder,thread->sql.aliases.authnick,sizeof(thread->c.chan.founder));
						dynncpy(thread->c.chan.access.ownauthnick,thread->sql.aliases.authnick,sizeof(thread->c.chan.access.ownauthnick));
						dynncpy(thread->c.chan.access.ownlevel,_dynadd("%d",ACCESS_FOUNDER),sizeof(thread->c.chan.access.ownlevel));
						dynncpy(thread->c.chan.description,_dstrjoin(' ',param,5),sizeof(thread->c.chan.description));
						
						bzero(&thread->c.chan.successor,sizeof(thread->c.chan.successor));
						bzero(&thread->c.chan.reason,sizeof(thread->c.chan.reason));
						
						thread->c.chan.maxmemos  = m->maxmemos;
						thread->c.chan.regdate   = time(NULL);
						thread->c.chan.last_used = time(NULL);
						thread->c.chan.expire    = 1;
						
						/* Update SQL as Live Update */
				    	domysql(thread,
							"UPDATE `orion_cs_core` SET "
							
							"`serv`='ChanServ',`description`='%s',`founder`='%s',`expire`='1',`regdate`='%ld',`last_used`='%ld',`maxmemos`='%d',`settings`='%d',"
							"`cs_set`='%d',`cs_autodeop`='%d',`cs_autohalfop`='%d',`cs_autoop`='%d',`cs_autoprotect`='%d',`cs_autovoice`='%d',"
							"`cs_nojoin`='%d',`cs_signkick`='%d',`cs_acclist`='%d',`cs_accchange`='%d',`cs_akick`='%d',`cs_ban`='%d',`cs_banme`='%d',"
							"`cs_cclear`='%d',`cs_getkey`='%d',`cs_halfop`='%d',`cs_halfopme`='%d',`cs_info`='%d',`cs_kick`='%d',`cs_kickme`='%d',`cs_invite`='%d',"
							"`cs_opdeop`='%d',`cs_opdeopme`='%d',`cs_protect`='%d',`cs_protectme`='%d',`cs_topic`='%d',`cs_unban`='%d',`cs_voice`='%d',"
							"`cs_voiceme`='%d',`cs_memo`='%d',`cs_botassign`='%d',`cs_badwords`='%d',`cs_fantasia`='%d',`cs_greet`='%d',`cs_nokick`='%d',"
							"`cs_say`='%d',"
							"`bs_set`='%d',"
							"`bs_caps_ttb`='-1',`bs_caps_min`='-1',`bs_caps_per`='-1',`bs_flood_ttb`='-1',`bs_flood_ln`='-1',`bs_flood_secs`='-1',`bs_repeat_ttb`='-1',`bs_repeat`='-1',`bs_colors`='-1',"
							"`bs_underlines`='-1',`bs_reverses`='-1',`bs_badwords`='-1',`bs_bolds`='-1'"
							
							" WHERE `channel`='%s'",
							
							mysql_quote(thread,thread->c.chan.description),mysql_quote(thread,thread->sql.aliases.authnick),time(NULL),time(NULL),m->maxmemos,settings,
							CS_SET,CS_AUTODEOP,CS_AUTOHALFOP,CS_AUTOOP,CS_AUTOPROTECT,CS_AUTOVOICE,
							CS_NOJOIN,CS_SIGNKICK,CS_ACCESS_LIST,CS_ACCESS_CHANGE,CS_AKICK,CS_BAN,CS_BANME,
							CS_CLEAR,CS_GETKEY,CS_HALFOP,CS_HALFOPME,CS_INFO,CS_KICK,CS_KICKME,CS_INVITE,
							CS_OPDEOP,CS_OPDEOPME,CS_PROTECT,CS_PROTECTME,CS_TOPIC,CS_UNBAN,CS_VOICE,
							CS_VOICEME,CS_MEMO,CS_ASSIGN,CS_BADWORDS,CS_FANTASIA,CS_GREET,CS_NOKICK,
							CS_SAY,botsettings,
							mysql_quote(thread,param[4])
						);
						
						if(settings&CS_INHABIT_BIT){
							/* Make ChanServ join the channel */
							do_services_join(thread,cs->nick,param[4],thread->c.chan.datecreate == 0 ? time(NULL) : thread->c.chan.datecreate,2);
						}
						
						if(!thread->Webform)
							do_logchan(thread,snick,NULL,NULL,NULL,"Channel: \'%s\' registered by %s!%s@%s (email: %s)",param[4],nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,thread->sql.access.email);
						else
							do_logchan(thread,snick,NULL,NULL,NULL,"WEB: Channel: \'%s\' registered by %s (email: %s)",param[4],nick,thread->sql.access.email);
								
			    		send_lang(thread,language,howtosend,snick,nick,"CS_REGISTER_CHAN_REGISTERED",param[4],nick);
			    		
			    		/* Load channel again, if needed */
			    		/* ! highly important !
							THIS IS REALLY NECESSARY, DON'T DELETE IF YOU DON'T KNOW WHAT YOU ARE DOING
							MIGHT GIVE STRANGE RESULTS IN CHANSERV JOINING CAUZE BY RESTRICT OR SOME STUFF
	  					*/
			    		bzero(&thread->c,sizeof(thread->c));
			    		
			    		setWebSuccess(thread); /* Indentify webform that changes were applied */
					}else send_lang(thread,language,howtosend,snick,nick,"REGISTER_LIMIT_X_FOR_NICK_X_AS_FOUNDER_REACHED",m->CSMaxReg,thread->sql.aliases.authnick);
				}else send_lang(thread,language,howtosend,snick,nick,"NOT_A_CHANNELOP_OR_ON_CHANNEL",param[4]);
            }
			else if(!thread->c.chan.fields) send_lang(thread,language,howtosend,snick,nick,"NOT_A_CHANNELOP_OR_ON_CHANNEL",param[4]);
			else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_ALREADY_REGISTERED",thread->c.chan.serv);
        }else send_lang(thread,language,howtosend,snick,nick,"CS_REGISTER_SYNTAX",snick);
	}else{
	/*
	 *  Error Messages
	 */
	 	if(thread->sql.access.rights<ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"register"))	send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");
	 	else{
			send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
			send_lang(thread,language,howtosend,snick,nick,"CS_REGISTER_SYNTAX",snick);
		}
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ REGISTER Command (/msg ChanServ HELP REGISTER)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP REGISTER,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP REGISTER
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_register_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Register Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help register")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_REGISTER",snick,snick,snick,ns->nick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}

	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - REGISTER part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the REGISTER command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_register_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Register (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_REGISTER");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_BOOL
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
