/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ms_del.c
 *   @brief This is the Module file for MemoServ DEL Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the MemoServ DEL command.
 *
 *   The MemoServ DEL Command allows the user to delete a memo from<br/>
 *   his memos or from a channel for which he is able to manage the Memos.
 *
 *   Syntax is: /msg MemoServ DEL [#channel] < memo-number | LAST | ALL >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "webform.h"
#include "global.h"

#define SERVICE "ms"
#define MODUL "del"
#define MODUL_FILE "ms_del"
#define MODUL_VERSION "1.2"

MOD_BOOL ms_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_del_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_del_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"del","del");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"del",ms_del,is_services_admin,is_services_user,4,5);
	addCommand(thread,&Module,"help del",ms_del_help,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help",ms_del_help_listing,is_services_admin,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

//:shock PRIVMSG MemoServ :cancel shock
/*  ************************************************************************************************  */

/**
 * @brief Main function for the MemoServ DEL Command.
 * 
 * The MemoServ DEL Command allows the user to delete a memo from<br/>
 *   his memos or from a channel for which he is able to manage the Memos.
 *
 * Syntax is: /msg MemoServ DEL [#channel] < memo-number | LAST | ALL >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL memos;
	Channel c;
	char *sender = NULL;
	int i=0;
	
	/* Initialise Variables to (NULL) */
	bzero(&memos,sizeof(memos));
	bzero(&c,sizeof(c));
	
	/*
	 * Delete Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"del")){
		if(param[4][0]=='#' && digitcol(param[5])){ //Channel
			c = sql_chan(thread,param[4]);

			if(c.serv[0]=='\0') send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
			else if(c.serv[0]!='\0' && stoi(param[5])>0){

				memos.res    = doSelectMySQL(thread,"SELECT `time` FROM `orion_ms_core` WHERE `receiver`='%s' ORDER BY `time` ASC",mysql_quote(thread,c.name));
				memos.fields = mysql_num_rows(memos.res);

				for(i=0;i<memos.fields;i++){
					memos.row = mysql_fetch_row(memos.res);
					
					if(i==stoi(param[5])-1){
						domysql(thread,"DELETE FROM `orion_ms_core` WHERE `receiver`='%s' AND `time`='%ld'",mysql_quote(thread,c.name),(time_t)stol(memos.row[0]));
						send_lang(thread,language,howtosend,snick,nick,"MS_DELETE_DELETED_MEMO_FOR_X_MEMO_X",c.name,i+1);
					}
				}
				/* mysql_free_result_myr(); - memos */
				mysql_free_result_myr(memos.res);
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else send_lang(thread,language,howtosend,snick,nick,"MEMO_LESS_0");
		}else if(digitcol(param[4]) && !param[5]){
			if(thread->sql.aliases.authnick[0]=='\0')
				send_lang(thread,language,howtosend,snick,nick,"NO_AUTHNICK");
				
			memos.res    = doSelectMySQL(thread,"SELECT `time` FROM `orion_ms_core` WHERE `receiver`='%s' ORDER BY `time` ASC",mysql_quote(thread,thread->sql.aliases.authnick));
			memos.fields = mysql_num_rows(memos.res);
			if(!memos.fields)
				send_lang(thread,language,howtosend,snick,nick,"MEMO_NO_MEMOS");
				
			else if(thread->sql.aliases.authnick[0]!='\0' && stoi(param[4])>0){
				for(i=0;i<memos.fields;i++){
					memos.row = mysql_fetch_row(memos.res);
					if(i==stoi(param[4])-1){
						sender = mysql_sel_var(thread,"SELECT `sender` FROM `orion_ms_core` WHERE `time`='%ld'",(time_t)stol(memos.row[0]));

						if(sender!=NULL){
							domysql(thread,"DELETE FROM `orion_ms_core` WHERE `receiver`='%s' AND `time`='%ld'",mysql_quote(thread,thread->sql.aliases.authnick),(time_t)stol(memos.row[0]));
							send_lang(thread,language,howtosend,snick,nick,"MS_DELETE_DELETED_MEMO_FROM_X_MEMO_X",sender,i+1);
							
							setWebSuccess(thread); /* Indentify webform that changes were applied */
						}
					}
				}
			}
			else send_lang(thread,language,howtosend,snick,nick,"MEMO_LESS_0");
			/* mysql_free_result_myr(); - memos */
			mysql_free_result_myr(memos.res);
		}else if(streqnc(param[4],"all") && !param[5]){
			if(thread->sql.aliases.authnick[0]=='\0')
				send_lang(thread,language,howtosend,snick,nick,"NO_AUTHNICK");
			
			if(!stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_ms_core` WHERE `receiver`='%s'",mysql_quote(thread,thread->sql.aliases.authnick))) )
				send_lang(thread,language,howtosend,snick,nick,"MEMO_NO_MEMOS");
				
			else if(thread->sql.aliases.authnick[0]!='\0'){
				domysql(thread,"DELETE FROM `orion_ms_core` WHERE `receiver`='%s'",mysql_quote(thread,thread->sql.aliases.authnick));
				send_lang(thread,language,howtosend,snick,nick,"MEMO_NO_MEMOS");
			}
			
			else send_lang(thread,language,howtosend,snick,nick,"MEMO_LESS_0");
        }
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the MemoServ DEL Command (/msg MemoServ HELP DEL)
 * 
 * When the user tries to get help for this command, using /msg MemoServ HELP DEL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg MemoServ HELP DEL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_del_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Delete Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help del")){
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_DEL");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}

	else send_lang(thread,language,howtosend,snick,nick,"MS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief MemoServ Help List Command - DEL part (/msg MemoServ help)
 * 
 * When the user tries to get general MemoServ help, using /msg MemoServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the DEL command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_del_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_MS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Delete (Help listing)
	 */
	if(Users_MS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_CMD_DEL");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
