/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ms_limit.c
 *   @brief This is the Module file for MemoServ LIMIT Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the MemoServ LIMIT command.
 *
 *   The MemoServ LIMIT Command allows the user to modify his / a channels memo-limit.<br/>
 *   (Can't be greater than HARD-LIMIT - a setting in the services.conf)
 *
 *   Syntax is: /msg MemoServ LIMIT [ #channel | nick ] < limit-number >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "webform.h"
#include "global.h"

#define SERVICE "ms"
#define MODUL "limit"
#define MODUL_FILE "ms_limit"
#define MODUL_VERSION "1.2"

MOD_BOOL ms_limit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_limit_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_limit_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"limit","limit");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"limit",ms_limit,is_services_admin,is_services_user,4,5);
	addCommand(thread,&Module,"help limit",ms_limit_help,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help",ms_limit_help_listing,is_services_admin,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

//:shock PRIVMSG MemoServ :limit shock 20
/*  ************************************************************************************************  */

/**
 * @brief Main function for the MemoServ LIMIT Command.
 * 
 * The MemoServ LIMIT Command allows the user to modify his / a channels memo-limit.<br/>
 *   (Can't be greater than HARD-LIMIT - a setting in the services.conf)
 *
 * Syntax is: /msg MemoServ LIMIT [ #channel | nick ] < limit-number >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_limit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	ChannelDB c;
	Aliases aliases5;
	int limit=0;
	
	/* Initialise Variables to (NULL) */
	bzero(&c,sizeof(c));
	bzero(&aliases5,sizeof(aliases5));
	
	/*
	 * Limit Command
	 */
	 //:shock PRIVMSG MemoServ :limit #bla 50
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"limit")){
		if(param[4][0]=='#' && digitcol(param[5])){ //Channel
			c = sql_chan_db(thread,param[4]);
			
			if(c.serv[0]!='\0'){
    			if(stoi(param[5]) <= m->maxmemos && (stoi(acclevel(thread,"ChanServ",c.name,thread->sql.aliases.authnick))>=c.cs_memo || strcor(acclevel(thread,"Q",c.name,thread->sql.aliases.authnick),"o") || strcor(acclevel(thread,"L",c.name,thread->sql.aliases.authnick),"o") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"limit") || streqnc(thread->sql.aliases.authnick,c.founder))){
    				domysql(thread,"UPDATE `orion_cs_core` SET `maxmemos`='%d' WHERE `channel`='%s' AND `serv`!=''",stoi(param[5]),mysql_quote(thread,c.name));
    				send_lang(thread,language,howtosend,snick,nick,"MS_LIMIT_SET_TO_X_FOR_CHANNEL_X",stoi(param[5]),c.name);
    				
    				setWebSuccess(thread); /* Indentify webform that changes were applied */
    			}
                else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_LOW",c.name);
            }else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
		}else if(param[4]){

			if(param[5] && digitcol(param[5]) && thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"limit")){
				aliases5 = sql_aliases(thread,param[4]);
				limit    = stoi(param[5]);
			}
			else{
				aliases5 = sql_aliases(thread,nick);
				limit    = stoi(param[4]);
				if(limit > m->maxmemos)
					limit = m->maxmemos;
			}
			
			if(aliases5.authnick[0]!='\0'){
    			domysql(thread,"UPDATE `orion_ns_core` SET `maxmemos`='%d' WHERE `authnick`='%s'",limit,mysql_quote(thread,aliases5.authnick));
    			send_lang(thread,language,howtosend,snick,nick,"MS_LIMIT_SET_TO_X_FOR_USER_X",limit,aliases5.authnick);
    			
    			setWebSuccess(thread); /* Indentify webform that changes were applied */
    		}else send_lang(thread,language,howtosend,snick,nick,"NO_AUTHNICK");
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the MemoServ LIMIT Command (/msg MemoServ HELP LIMIT)
 * 
 * When the user tries to get help for this command, using /msg MemoServ HELP LIMIT,<br/>
 * this function is executed.
 * 
 * Syntax: /msg MemoServ HELP LIMIT
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_limit_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Limit Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help limit")){
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_LIMIT",m->maxmemos);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}

	else send_lang(thread,language,howtosend,snick,nick,"MS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief MemoServ Help List Command - LIMIT part (/msg MemoServ help)
 * 
 * When the user tries to get general MemoServ help, using /msg MemoServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the LIMIT command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_limit_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_MS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Limit (Help listing)
	 */
	if(Users_MS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_CMD_LIMIT");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
