/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_getemail.c
 *   @brief This is the Module file for NickServ GETEMAIL Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ GETEMAIL command.
 *
 *   The NickServ GETEMAIL Command allows the user to dump all emails which match the pattern.
 *
 *   Syntax is: /msg NickServ GETEMAIL <*emailpattern*>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "ns"
#define MODUL "getemail"
#define MODUL_FILE "ns_getemail"
#define MODUL_VERSION "1.2"

MOD_BOOL ns_getemail(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_getemail_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_getemail_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"getemail","getemail");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"getemail",ns_getemail,is_services_admin,INFINIT,4,4);
	addCommand(thread,&Module,"help getemail",ns_getemail_help,is_services_admin,INFINIT,4,4);
	addCommand(thread,&Module,"help",ns_getemail_help_listing,is_services_admin,INFINIT,3,3); //100 - each Admin is able to see it.

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ GETEMAIL Command.
 * 
 * The NickServ GETEMAIL Command allows the user to dump all emails which match the pattern.
 *
 * Syntax is: /msg NickServ GETEMAIL <*emailpattern*>
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_getemail(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
    deepSQL select;
	char enick   [MAX_NICK_LEN];
	char pattern [BUFSIZE];
	int counter = 0;
	int i 		= 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&select,sizeof(select));
	bzero(&enick,sizeof(enick));
	bzero(&pattern,sizeof(pattern));
	
	/*
	 * Getemail Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"getemail") && strcor(param[4],"*")){
		dynncpy(pattern,param[4],sizeof(pattern));
		
		strrepc(pattern,'*','%');
		select.res    = doSelectMySQL(thread,"SELECT `authnick`,`password`,`email` FROM `orion_ns_core` WHERE `email` LIKE '%s'",mysql_quote(thread,pattern));
		select.fields = mysql_num_rows(select.res);
		
		if(select.fields){
			for(i=0;i<select.fields;i++){
				select.row = mysql_fetch_row(select.res);
				
    			counter++;
    			bzero(&enick,sizeof(enick));
    			dynncpy(enick,mysql_sel_var(thread,"SELECT `nick` FROM `orion_ns_aliases` WHERE `display`='%s'",select.row[0]),sizeof(enick));
    			
                if(thread->sql.access.fields && thread->sql.access.olevel>=1000){
    				if(enick[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"NS_GETEMAIL_LIST_FOUND_EMAILUSER_AS_LOGGED_IN_USER_SEE_PASSWORDS",enick,select.row[0],select.row[2],select.row[1]);
    				else send_lang(thread,language,howtosend,snick,nick,"NS_GETEMAIL_LIST_EMAILUSER_IS_NOT_LOGGED_IN_SEE_PASSWORDS",select.row[0],select.row[2],select.row[1]);
    			}
                else{
    				if(enick[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"NS_GETEMAIL_LIST_FOUND_EMAILUSER_AS_LOGGED_IN_USER",enick,select.row[0],select.row[2]);
    				else send_lang(thread,language,howtosend,snick,nick,"NS_GETEMAIL_LIST_EMAILUSER_IS_NOT_LOGGED_IN",select.row[0],select.row[2]);
    			}
    		}

    		send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
    		//strrepc(param[4],'%','*');
    		if(counter != 0) send_lang(thread,language,howtosend,snick,nick,"NS_GETEMAIL_FOUND_X_MATCHING_EMAILS_BY_SEARCHING_WITH_PATTERN",counter,param[4]);
    		else send_lang(thread,language,howtosend,snick,nick,"NS_GETEMAIL_FOUND_NOTHING_BY_SEARCHING_WITH_PATTERN");
    		
    		setWebSuccess(thread); /* Indentify webform that changes were applied */
        }else send_lang(thread,language,howtosend,snick,nick,"NS_GETEMAIL_FOUND_NOTHING_BY_SEARCHING_WITH_PATTERN");
        
		/* mysql_free_result_myr(); - select */
		mysql_free_result_myr(select.res);
	}else{
		/*
		 *  Error Messages
		 */
		if(!strcor(param[4],"*"))	send_lang(thread,language,howtosend,snick,nick,"WRONG_INPUT_PATTERN_INPUT",param[4]);
		else 						send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ GETEMAIL Command (/msg NickServ HELP GETEMAIL)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP GETEMAIL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP GETEMAIL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_getemail_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Getemail Help
	 */ 
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help getemail")){
		send_lang(thread,language,howtosend,snick,nick,"NS_SERVADMIN_HELP_GETEMAIL");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - GETEMAIL part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the GETEMAIL command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_getemail_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Getemail (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_GETEMAIL");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
