/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_saset.c
 *   @brief This is the Module file for NickServ SASET Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ SASET command.
 *
 *   The NickServ SASET Command allows an admin to set various SET-Options<br/>
 *   for other users. (Controls DISPLAY/PASSWORD etc.)
 *
 *   Syntax is: /msg NickServ SASET < nick > < option > < value ... >
 *
 *<table border="0">
 *    <tr><td> DISPLAY	</td><td> Syntax is: /msg NickServ SASET < nick > DISPLAY	</td><td> < new-display >									</td></tr>
 *    <tr><td> PASSWORD	</td><td> Syntax is: /msg NickServ SASET < nick > PASSWORD	</td><td> < new-password >									</td></tr>
 *    <tr><td> LANGUAGE	</td><td> Syntax is: /msg NickServ SASET < nick > LANGUAGE	</td><td> { 1-5 }											</td></tr>
 *    <tr><td> URL		</td><td> Syntax is: /msg NickServ SASET < nick > URL		</td><td> < new-Url >										</td></tr>
 *    <tr><td> EMAIL	</td><td> Syntax is: /msg NickServ SASET < nick > EMAIL		</td><td> < new-Email >										</td></tr>
 *    <tr><td> ICQ		</td><td> Syntax is: /msg NickServ SASET < nick > ICQ		</td><td> < IcqUIN >										</td></tr>
 *    <tr><td> GREET	</td><td> Syntax is: /msg NickServ SASET < nick > GREET		</td><td> < New-GreetMessage >								</td></tr>
 *    <tr><td> KILL		</td><td> Syntax is: /msg NickServ SASET < nick > KILL		</td><td> { ON | QUICK | IMMED | OFF }						</td></tr>
 *    <tr><td> SECURE	</td><td> Syntax is: /msg NickServ SASET < nick > SECURE	</td><td> { ON | OFF }										</td></tr>
 *    <tr><td> PRIVATE	</td><td> Syntax is: /msg NickServ SASET < nick > PRIVATE	</td><td> { ON | OFF }										</td></tr>
 *    <tr><td> HIDE		</td><td> Syntax is: /msg NickServ SASET < nick > HIDE		</td><td> { EMAIL | USERMASK | STATUS | QUIT } { ON | OFF }	</td></tr>
 *    <tr><td> MSG		</td><td> Syntax is: /msg NickServ SASET < nick > MSG		</td><td> { ON | OFF }										</td></tr>
 *    <tr><td> AUTOOP	</td><td> Syntax is: /msg NickServ SASET < nick > AUTOOP	</td><td> { ON | OFF }										</td></tr>
 *    <tr><td> INCOGNITO</td><td> Syntax is: /msg NickServ SASET < nick > INCOGNITO	</td><td> { ON | OFF }										</td></tr>
 *    <tr><td> NOEXPIRE	</td><td> Syntax is: /msg NickServ SASET < nick > NOEXPIRE	</td><td> { ON | OFF }										</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "webform.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "saset"
#define MODUL_FILE "ns_saset"
#define MODUL_VERSION "1.2"

MOD_BOOL ns_saset_display(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_password(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_language(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_url(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_email(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_greet(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_icq(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_kill(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
//MOD_BOOL ns_saset_secure(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_private(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_hide(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_msg(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_autoop(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_incognito(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_noexpire(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
//--------------------------------------------------------------------------------------

MOD_BOOL ns_saset_display_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_password_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_language_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_url_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_email_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_greet_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_icq_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_kill_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
//MOD_BOOL ns_saset_secure_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_private_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_hide_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_msg_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_autoop_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_incognito_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_noexpire_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL ns_saset_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_saset_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"saset * option[1]","saset option[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,5,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"saset",is_services_admin,INFINIT); /* just to add a displaylevel for /msg ns help
															  this should always be the lowest used Module level! */

	addCommand(thread,&Module,"saset display",ns_saset_display,is_services_admin,INFINIT,6,6);
	addCommand(thread,&Module,"saset password",ns_saset_password,is_services_admin,INFINIT,6,6);
	addCommand(thread,&Module,"saset language",ns_saset_language,is_services_admin,INFINIT,6,6);
	addCommand(thread,&Module,"saset url",ns_saset_url,is_services_admin,INFINIT,5,6);
	addCommand(thread,&Module,"saset email",ns_saset_email,is_services_admin,INFINIT,6,6);
	addCommand(thread,&Module,"saset greet",ns_saset_greet,is_services_admin,INFINIT,5,0);
	addCommand(thread,&Module,"saset icq",ns_saset_icq,is_services_admin,INFINIT,5,6);
	addCommand(thread,&Module,"saset kill",ns_saset_kill,is_services_admin,INFINIT,6,6);
	//addCommand(thread,&Module,"saset secure",ns_saset_secure,is_services_admin,INFINIT,6,6);
	addCommand(thread,&Module,"saset private",ns_saset_private,is_services_admin,INFINIT,6,6);
	addCommand(thread,&Module,"saset hide",ns_saset_hide,is_services_admin,INFINIT,7,7);
	addCommand(thread,&Module,"saset msg",ns_saset_msg,is_services_admin,INFINIT,6,6);
	addCommand(thread,&Module,"saset autoop",ns_saset_autoop,is_services_admin,INFINIT,6,6);
	addCommand(thread,&Module,"saset incognito",ns_saset_incognito,is_services_admin,INFINIT,6,6);
	addCommand(thread,&Module,"saset noexpire",ns_saset_noexpire,is_services_admin,INFINIT,6,6);

	addCommand(thread,&Module,"help saset",ns_saset_help,is_services_admin,INFINIT,4,4);
	addCommand(thread,&Module,"help saset display",ns_saset_display_help,is_services_admin,INFINIT,5,5);
	addCommand(thread,&Module,"help saset password",ns_saset_password_help,is_services_admin,INFINIT,5,5);
	addCommand(thread,&Module,"help saset language",ns_saset_language_help,is_services_admin,INFINIT,5,5);
	addCommand(thread,&Module,"help saset url",ns_saset_url_help,is_services_admin,INFINIT,5,5);
	addCommand(thread,&Module,"help saset email",ns_saset_email_help,is_services_admin,INFINIT,5,5);
	addCommand(thread,&Module,"help saset greet",ns_saset_greet_help,is_services_admin,INFINIT,5,5);
	addCommand(thread,&Module,"help saset icq",ns_saset_icq_help,is_services_admin,INFINIT,5,5);
	addCommand(thread,&Module,"help saset kill",ns_saset_kill_help,is_services_admin,INFINIT,5,5);
	//addCommand(thread,&Module,"help saset secure",ns_saset_secure_help,is_services_admin,INFINIT,5,5);
	addCommand(thread,&Module,"help saset private",ns_saset_private_help,is_services_admin,INFINIT,5,5);
	addCommand(thread,&Module,"help saset hide",ns_saset_hide_help,is_services_admin,INFINIT,5,5);
	addCommand(thread,&Module,"help saset msg",ns_saset_msg_help,is_services_admin,INFINIT,5,5);
	addCommand(thread,&Module,"help saset autoop",ns_saset_autoop_help,is_services_admin,INFINIT,5,5);
	addCommand(thread,&Module,"help saset incognito",ns_saset_incognito_help,is_services_admin,INFINIT,5,5);
	addCommand(thread,&Module,"help saset noexpire",ns_saset_noexpire_help,is_services_admin,INFINIT,5,5);
	
	addCommand(thread,&Module,"help",ns_saset_help_listing,is_services_admin,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Display function for the NickServ SASET Command.
 * 
 * The NickServ SASET Display Command allows an admin change a users<br/>
 *   Main-GroupNick to another Nick in the users NickGroup.
 *  
 * Syntax is: /msg NickServ SASET < nick > DISPLAY < newdisplay >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_display(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	Aliases aliases5;
	Access access5;
	
	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
    
    /*
     * SaSet Display Command
     */
	aliases5  = sql_aliases(thread,param[4]);
	access5   = sql_access(thread,aliases5.authnick);
	
	//:shock PRIVMSG NickServ :SaSet target DISPLAY newdisp
	if(aliases5.nick[0]!='\0' && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"saset display") && access5.rights<is_services_admin){
		if(thread->sql.aliases.active){
			if(!streqnc(accessselect(thread,"authnick",param[6]),param[6])){
				if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_ns_aliases` WHERE `nick`='%s' AND `display`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,aliases5.authnick)))){
					domysql(thread,"UPDATE `orion_ns_core` LEFT OUTER JOIN `orion_ms_core` ON (`orion_ms_core`.`receiver`=`orion_ns_core`.`authnick`) SET `orion_ns_core`.`authnick`='%s',`orion_ms_core`.`receiver`='%s' WHERE `orion_ns_core`.`authnick`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,param[6]),mysql_quote(thread,aliases5.authnick));
					//domysql(thread,"UPDATE `orion_cs_access` SET `authnick`='%s' WHERE `authnick`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,aliases5.authnick));
					
					//domysql(thread,"UPDATE `orion_cs_core` SET `founder`='%s' WHERE `founder`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,aliases5.authnick));
					//domysql(thread,"UPDATE `orion_cs_core` SET `successor`='%s' WHERE `successor`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,aliases5.authnick));
					/* 
					InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore
					domysql(thread,"UPDATE `orion_ns_aliases` SET `display`='%s' WHERE `display`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,aliases5.authnick));
					*/
					
					send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_DISPLAY",aliases5.authnick,param[6]);
					do_logchan(thread,snick,NULL,NULL,NULL,"%s changed %s nickname group display to %s",nick,aliases5.authnick,param[6]);
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
					
				}else send_lang(thread,language,howtosend,snick,nick,"NS_SET_DISPLAYNICK_IS_NOT_IN_YOUR_GROUP");
			}else send_lang(thread,language,howtosend,snick,nick,"NS_SET_DISPLAYNICK_X_ALREADY_USED",param[6]);
		}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN");
	}else{
	/*
	 *  Error Messages
	 */
		if(aliases5.nick[0]=='\0')					send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		else if(access5.rights>=is_services_admin)	send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
		else 										send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Password function for the NickServ SASET Command.
 * 
 * The NickServ SASET Password Command allows an admin change a users<br/>
 *   Password to a NewPassword!
 *  
 * Syntax is: /msg NickServ SASET < nick > PASSWORD < new-password >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_password(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	Aliases aliases5;
	Access access5;
	char tmppasswd[MAX_PASSWORD_LEN + 1];
	
	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
	bzero(&tmppasswd,sizeof(tmppasswd));
    
    /*
     * SaSet Password Command
     */
	aliases5  = sql_aliases(thread,param[4]);
	access5   = sql_access(thread,aliases5.authnick);
	
	if((!m->NSSecureAdmins || (m->NSSecureAdmins && access5.rights<=is_services_oper)) && aliases5.nick[0]!='\0' && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"saset password")){
        if(dynlen(param[6])>=ns->min_passwordlen && dynlen(param[6])<=MAX_PASSWORD_LEN){ 
  			if(is_valid_chars(param[6],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+-.:<>|^!\"$%&/()=?*'#,;~{[]}\\`")){
				send_lang(thread,language,howtosend,snick,nick,"INVALID_PASSWORD_CHARS","0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+-.:<>|^!\"$%&/()=?*'#,;~{[]}\\`");
				return MOD_CONT;
			}
   
    		dynncpy(tmppasswd,param[6],sizeof(tmppasswd));

    		char enc_password[ENC_PASSMAX+1];
			bzero(&enc_password,sizeof(enc_password));
    		if (orion_encrypt_specific(newest_encryption, tmppasswd, strlen(tmppasswd), enc_password, ENC_PASSMAX) < 0) {
    			//memset(tmppasswd, 0, PASSWORD_LEN);
    			alog(thread,"%s: Failed to encrypt password for %s (set password)",
    				ns->nick, nick);
    			send_lang(thread,language,howtosend,snick,nick,"REGISTRATION_FAILED");
    			return MOD_CONT;
    		}
    		access5.settings |= NS_NEW_ENCRYPTION;
			
    		domysql(thread,"UPDATE `orion_ns_core` SET `password`='%s', `settings`=%d WHERE `authnick`='%s'",mysql_quote(thread,enc_password),access5.settings,mysql_quote(thread,aliases5.authnick));

			send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_PASSWORD",aliases5.authnick,tmppasswd);
			do_logchan(thread,snick,NULL,NULL,NULL,"%s changed %s's password. (e-mail: %s)",nick,aliases5.authnick,access5.email);
			
			if(m->WallSetpass)
				do_global(thread,snick,"%s changed %s's password. (e-mail: %s)",nick,aliases5.authnick,access5.email);
				
			setWebSuccess(thread); /* Indentify webform that changes were applied */
			
		}else send_lang(thread,language,howtosend,snick,nick,"PASSWORD_TOO_SHORT",ns->min_passwordlen);   
	}else{
	/*
	 *  Error Messages
	 */
		if(aliases5.nick[0]=='\0')					send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		else if(access5.rights>=is_services_admin)	send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
		else 										send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Language function for the NickServ SASET Command.
 * 
 * The NickServ SASET Language Command allows an admin change a users<br/>
 *   Language!
 *  
 * Syntax is: /msg NickServ SASET < nick > LANGUAGE { 1-5 }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_language(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	Aliases aliases5;
	Access access5;
	void *define = NULL;
	
	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
    
    /*
     * SaSet Language Command
     */
	aliases5  = sql_aliases(thread,param[4]);
	access5   = sql_access(thread,aliases5.authnick);
    
	if(aliases5.nick[0]!='\0' && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"saset language") && access5.rights<is_services_admin){
		if(digitcol(param[6]) && (stoi(param[6])==1 || stoi(param[6])==2)){
			domysql(thread,"UPDATE `orion_ns_core` SET `language`='%d' WHERE `authnick`='%s'",stoi(param[6])-1,mysql_quote(thread,aliases5.authnick));
			define=dynsym(thread,languages[stoi(param[6])-1],"LANGUAGE");
			send_lang(thread,languages[stoi(param[6])-1],howtosend,snick,nick,"NS_SASET_CHANGED_LANGUAGE",aliases5.authnick,define);
			
			setWebSuccess(thread); /* Indentify webform that changes were applied */
        }else send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[5]);
	}else{
	/*
	 *  Error Messages
	 */
		if(aliases5.nick[0]=='\0')					send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		else if(access5.rights>=is_services_admin)	send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
		else 										send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Url function for the NickServ SASET Command.
 * 
 * The NickServ SASET Url Command allows an admin change a users<br/>
 *   Url (Homepage)!
 *  
 * Syntax is: /msg NickServ SASET < nick > URL < new-Url >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_url(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	Aliases aliases5;
	Access access5;
	
	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
    
    /*
     * SaSet Url Command
     */
	aliases5  = sql_aliases(thread,param[4]);
	access5   = sql_access(thread,aliases5.authnick);
    
	if(aliases5.nick[0]!='\0' && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"saset url") && access5.rights<is_services_admin){
        if(!param[6]){
    		domysql(thread,"UPDATE `orion_ns_core` SET `url`='' WHERE `authnick`='%s'",mysql_quote(thread,aliases5.authnick));
    		send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_URL_NONE",aliases5.authnick);
    		
    		setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else if(!is_valid_chars(param[6],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+-_.:/%")){
    		domysql(thread,"UPDATE `orion_ns_core` SET `url`='%s' WHERE `authnick`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,aliases5.authnick));
    		send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_URL",aliases5.authnick,param[6]);
    		
    		setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[6]);
	}else{
	/*
	 *  Error Messages
	 */
		if(aliases5.nick[0]=='\0')					send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		else if(access5.rights>=is_services_admin)	send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
		else 										send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Email function for the NickServ SASET Command.
 * 
 * The NickServ SASET Email Command allows an admin change a users<br/>
 *   Email-Address!
 *  
 * Syntax is: /msg NickServ SASET < nick > EMAIL < new-Email >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_email(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	Aliases aliases5;
	Access  access5;
	char From[80];
	char Subject[80];
	char passcode[PASSWORD_LEN];
	
	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
	bzero(&From,sizeof(From));
	bzero(&Subject,sizeof(Subject));
	bzero(&passcode,sizeof(passcode));
    
    /*
     * SaSet Email Command
     */
	aliases5 = sql_aliases(thread,param[4]);
	access5  = sql_access(thread,aliases5.authnick);
    
	if(aliases5.nick[0]!='\0' && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"saset email") && access5.rights<is_services_admin){
		/*dynncpy(email,param[6],sizeof(email));
		
		if(!strcsea(email,"@&.")){
			send_lang(thread,language,howtosend,snick,nick,"NS_SET_EMAIL_INVALID_EMAIL");
			return MOD_CONT;
		}
		
		strsplit(email,domain,'@');
		dlog(thread,3,"domain[0] = %s\n",domain[0]);
		dlog(thread,3,"domain[1] = %s\n",domain[1]);
		strsplit(domain[1],parts,'.');
		
		if(domain[0]=='\0' || parts[0]=='\0' || parts[1]=='\0' || is_valid_chars(parts[0],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_.") || is_valid_chars(parts[1],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_.") || is_valid_chars(domain[0],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_.")){
			dlog(thread,3,"parts[0] validation = %d\n",is_valid_chars(parts[0],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_."));
			dlog(thread,3,"parts[1] validation = %d\n",is_valid_chars(parts[1],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_."));
			dlog(thread,3,"domain[0] validation = %d\n",is_valid_chars(domain[0],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_."));

			dlog(thread,3,"parts[0] = %s\n",parts[0]);
			dlog(thread,3,"parts[1] = %s\n",parts[1]);
			dlog(thread,3,"domain[0] = %s\n",domain[0]);

			send_lang(thread,language,howtosend,snick,nick,"NS_SET_EMAIL_INVALID_EMAIL");
			return MOD_CONT;
		}
		
		else */
		if(MailValidate(param[6])/*dynlen(parts[0])>=ns->min_emaildomainlen && dynlen(parts[1])>=ns->min_emaildomainlen*/){ //email ok and accepted
			if(m->NSEmailReg){
				randomstring(passcode,sizeof(passcode)-1);
				
				if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_ns_request` WHERE `display`='%s'",mysql_quote(thread,aliases5.authnick)))==0){
					domysql(thread,"INSERT INTO `orion_ns_request` (`display`,`passcode`,"
											  	"`email`,`reqtime`,`reqsent`,`emailchange`)"
										"VALUES ('%s','%s','%s','%s','%ld','%ld','1')",
							mysql_quote(thread,access5.authnick),mysql_quote(thread,passcode),mysql_quote(thread,param[6]),time(NULL),time(NULL));

					if(access5.email[0]!='\0' && aliases5.authnick[0]!='\0' && m->UseMail){
						dynncpy(From,_dynadd("%s Network (%s) <%s>",m->network,ns->nick,m->SendFrom),sizeof(From));
						dynncpy(Subject,_dynadd("Your %s account (Your requested email change)",snick),sizeof(Subject));

						MailInfo *mail = o_email(thread,From,nick,param[6],Subject);
						//Email	
						if(m->UseWebForm)
							s_email(thread,language,mail->pipe,"NS_CHANGE_EMAIL_REQUEST_FOR_WEBFORM",aliases5.authnick,passcode,ns->nick,0xA0,snick,ns->host,passcode,passcode,m->LinkToWebform,m->NSExpire,m->homepage);
						else
							s_email(thread,language,mail->pipe,"NS_CHANGE_EMAIL_REQUEST",aliases5.authnick,passcode,ns->nick,0xA0,snick,ns->host,passcode,m->NSExpire,m->homepage);
						c_email(thread,mail);
					}

					do_logchan(thread,snick,NULL,NULL,NULL,"Nick %s has requested an email change (%s -> %s) - awaiting validation",aliases5.nick,access5.email,param[6]);
					send_lang(thread,language,howtosend,snick,nick,"NS_CHANGED_EMAIL_VALIDATION",aliases5.authnick,param[6],(m->NSRExpire / 3600) % 24);
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else send_lang(thread,language,howtosend,snick,nick,"NS_EMAIL_CHANGE_ALREADY_IN_PROGRESS");
			}else{
				domysql(thread,"UPDATE `orion_ns_core` SET `email`='%s' WHERE `authnick`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,aliases5.authnick));
				send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_EMAIL",aliases5.authnick,param[6]);
				do_logchan(thread,snick,NULL,NULL,NULL,"%s changed %s's (e-mail: %s) email to email: %s",nick,aliases5.authnick,access5.email,param[6]);
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}
		}
		else send_lang(thread,language,howtosend,snick,nick,"NS_SET_EMAIL_INVALID_EMAIL");
	}else{
	/*
	 *  Error Messages
	 */
		if(aliases5.nick[0]=='\0')					send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		else if(access5.rights>=is_services_admin)	send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
		else 										send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Icq function for the NickServ SASET Command.
 * 
 * The NickServ SASET Icq Command allows an admin change a users<br/>
 *   ICQ UIN!
 *  
 * Syntax is: /msg NickServ SASET < nick > ICQ < IcqUIN >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_icq(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	Aliases aliases5;
	Access access5;
	
	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
    
    /*
     * SaSet ICQ Command
     */
	aliases5  = sql_aliases(thread,param[4]);
	access5   = sql_access(thread,aliases5.authnick);
    
	if(aliases5.nick[0]!='\0' && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"saset icq") && access5.rights<is_services_admin){
        if(!param[6]){
    		domysql(thread,"UPDATE `orion_ns_core` SET `icq`='' WHERE `authnick`='%s'",mysql_quote(thread,aliases5.authnick));
    		send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_ICQ_NONE",aliases5.authnick);
    		
    		setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else if(digitcol(param[6]) && !is_valid_chars(param[6],"0123456789")){
			domysql(thread,"UPDATE `orion_ns_core` SET `icq`='%ld' WHERE `authnick`='%s'",stol(param[6]),mysql_quote(thread,aliases5.authnick));
			send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_ICQ",aliases5.authnick,stol(param[6]));
			
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[6]);
	}else{
	/*
	 *  Error Messages
	 */
		if(aliases5.nick[0]=='\0')					send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		else if(access5.rights>=is_services_admin)	send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
		else 										send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Greet function for the NickServ SASET Command.
 * 
 * The NickServ SASET Greet Command allows an admin change a users<br/>
 *   Greet-Message!
 *  
 * Syntax is: /msg NickServ SASET < nick > GREET < New-Greet-Message >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_greet(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	Aliases aliases5;
	Access access5;
	char *greet = NULL;
	
	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
    
    /*
     * SaSet Greet Command
     */
	aliases5  = sql_aliases(thread,param[4]);
	access5   = sql_access(thread,aliases5.authnick);
    
	if(aliases5.nick[0]!='\0' && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"saset greet") && access5.rights<is_services_admin){
		if(!param[6]){
    		domysql(thread,"UPDATE `orion_ns_core` SET `greet`='' WHERE `authnick`='%s'",mysql_quote(thread,aliases5.authnick));
    		send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_GREET_NONE",aliases5.authnick);
    		
    		setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else{
			greet = _dstrjoin(' ',param,6);
    		domysql(thread,"UPDATE `orion_ns_core` SET `greet`='%s' WHERE `authnick`='%s'",mysql_quote(thread,greet),mysql_quote(thread,aliases5.authnick));
    		send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_GREET",aliases5.authnick,greet);
    		
    		setWebSuccess(thread); /* Indentify webform that changes were applied */
		}
	}else{
	/*
	 *  Error Messages
	 */
		if(aliases5.nick[0]=='\0')					send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		else if(access5.rights>=is_services_admin)	send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
		else 										send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Kill function for the NickServ SASET Command.
 * 
 * The NickServ SASET Kill Command allows an admin change a users<br/>
 *   Kill-Settings!
 *  
 * Syntax is: /msg NickServ SASET < nick > KILL { ON | QUICK | IMMED | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_kill(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	Aliases aliases5;
	Access access5;
	Users *User = NULL;
	
	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
    
    /*
     * SaSet Kill Command
     */
	aliases5	= sql_aliases(thread,param[4]);
	access5		= sql_access(thread,aliases5.authnick);
	User        = GetUser(param[4]);
    
	if(aliases5.nick[0]!='\0' && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"saset kill") && access5.rights<is_services_admin){
		if(streqnc(param[6],"immed") && m->NSImmedKill>0){
			access5.settings |= NS_SET_KILL_IMMED;
			access5.settings &= ~NS_SET_KILL_QUICK;
			access5.settings &= ~NS_SET_KILLPROTECT;

			if(User != NULL){
#ifdef USE_THREADS
				my_pthread_mutex_lock(&users_lock);
#endif
				User->settings = access5.settings;
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&users_lock);
#endif
			}
			
			domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",access5.settings,mysql_quote(thread,aliases5.authnick));
			send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_KILL",aliases5.authnick,_uc(param[6]));
			
			setWebSuccess(thread); /* Indentify webform that changes were applied */
			return MOD_CONT;
		}else if(streqnc(param[6],"quick") && m->NSQuickKill>0){
			access5.settings |= NS_SET_KILL_QUICK;
			access5.settings &= ~NS_SET_KILL_IMMED;
			access5.settings &= ~NS_SET_KILLPROTECT;

			/* Update user-struct, we have to keep us up to date! */
			if(User != NULL){
#ifdef USE_THREADS
				my_pthread_mutex_lock(&users_lock);
#endif
				User->settings = access5.settings;
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&users_lock);
#endif
			}
			
			domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",access5.settings,mysql_quote(thread,aliases5.authnick));
			send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_KILL",aliases5.authnick,_uc(param[6]));
			
			setWebSuccess(thread); /* Indentify webform that changes were applied */
			return MOD_CONT;
		}else if(streqnc(param[6],"on")){
			access5.settings |= NS_SET_KILLPROTECT;
			access5.settings &= ~NS_SET_KILL_IMMED;
			access5.settings &= ~NS_SET_KILL_QUICK;

			/* Update user-struct, we have to keep us up to date! */
			if(User != NULL){
#ifdef USE_THREADS
				my_pthread_mutex_lock(&users_lock);
#endif
				User->settings = access5.settings;
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&users_lock);
#endif
			}
			
			domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",access5.settings,mysql_quote(thread,aliases5.authnick));
			send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_KILL",aliases5.authnick,_uc(param[6]));
			
			setWebSuccess(thread); /* Indentify webform that changes were applied */
			return MOD_CONT;
		}else if(streqnc(param[6],"off")){
			access5.settings &= ~NS_SET_KILLPROTECT;
			access5.settings &= ~NS_SET_KILL_IMMED;
			access5.settings &= ~NS_SET_KILL_QUICK;

			/* Update user-struct, we have to keep us up to date! */
			if(User != NULL){
#ifdef USE_THREADS
				my_pthread_mutex_lock(&users_lock);
#endif
				User->settings = access5.settings;
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&users_lock);
#endif
			}
			
			domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",access5.settings,mysql_quote(thread,aliases5.authnick));
			send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_KILL",aliases5.authnick,_uc(param[6]));
			
			setWebSuccess(thread); /* Indentify webform that changes were applied */
			return MOD_CONT;
		}else send_lang(thread,language,howtosend,snick,nick,"NS_SET_KILL_NO_SUCH_FUNCTION");
	}else{
	/*
	 *  Error Messages
	 */
		if(aliases5.nick[0]=='\0')					send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		else if(access5.rights>=is_services_admin)	send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
		else 										send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Secure function for the NickServ SASET Command.
 * 
 * The NickServ SASET Secure Command allows an admin change a users<br/>
 *   Secure State!
 *  
 * Syntax is: /msg NickServ SASET < nick > SECURE { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_secure(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	Aliases aliases5;
	Access access5;
	Users *User = NULL;
	
	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
    
    /*
     * SaSet Secure Command
     */
	aliases5	= sql_aliases(thread,param[4]);
	access5		= sql_access(thread,aliases5.authnick);
	User        = GetUser(param[4]);
    
	if(aliases5.nick[0]!='\0' && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"saset secure") && access5.rights<is_services_admin){
		if(streqnc(param[6],"on"))			access5.settings |= NS_SET_SECURE;
		else if(streqnc(param[6],"off"))	access5.settings &= ~NS_SET_SECURE;
		else return MOD_CONT;
		
		/* Update user-struct, we have to keep us up to date! */
		if(User != NULL){
#ifdef USE_THREADS
			my_pthread_mutex_lock(&users_lock);
#endif
			User->settings = access5.settings;
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&users_lock);
#endif
		}
			    
		domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",access5.settings,mysql_quote(thread,aliases5.authnick));
		send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_SECURE",aliases5.authnick,_uc(param[6]));
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		if(aliases5.nick[0]=='\0')					send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		else if(access5.rights>=is_services_admin)	send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
		else 										send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Private function for the NickServ SASET Command.
 * 
 * The NickServ SASET Private Command allows an admin change a users<br/>
 *   Private State!
 *  
 * Syntax is: /msg NickServ SASET < nick > PRIVATE { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_private(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	Aliases aliases5;
	Access access5;
	Users *User = NULL;
	
	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
    
    /*
     * SaSet Private Command
     */
	aliases5	= sql_aliases(thread,param[4]);
	access5		= sql_access(thread,aliases5.authnick);
	User        = GetUser(param[4]);
    
	if(aliases5.nick[0]!='\0' && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"saset private") && access5.rights<is_services_admin){
		if(streqnc(param[6],"on"))			access5.settings |= NS_SET_PRIVATE;
		else if(streqnc(param[6],"off"))	access5.settings &= ~NS_SET_PRIVATE;
		else return MOD_CONT;
		
		/* Update user-struct, we have to keep us up to date! */
		if(User != NULL){
#ifdef USE_THREADS
			my_pthread_mutex_lock(&users_lock);
#endif
			User->settings = access5.settings;
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&users_lock);
#endif
		}

		domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",access5.settings,mysql_quote(thread,aliases5.authnick));
		send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_PRIVATE",aliases5.authnick,_uc(param[6]));
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		if(aliases5.nick[0]=='\0')					send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		else if(access5.rights>=is_services_admin)	send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
		else 										send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Hide function for the NickServ SASET Command.
 * 
 * The NickServ SASET Hide Command allows an admin change a users<br/>
 *   Hided Options (parameters)!
 *  
 * Syntax is: /msg NickServ SASET < nick > HIDE { EMAIL | USERMASK | STATUS | QUIT } { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_hide(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	Aliases aliases5;
	Access access5;
	Users *User = NULL;
	
	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
    
    /*
     * SaSet Hide Command
     */
	aliases5	= sql_aliases(thread,param[4]);
	access5		= sql_access(thread,aliases5.authnick);
	User        = GetUser(param[4]);
    
	if(aliases5.nick[0]!='\0' && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"saset hide") && access5.rights<is_services_admin){
	//0001 0x01 - QUIT		NS_SET_HIDE_QUIT
	//0010 0x02 - USERMASK	NS_SET_HIDE_USERMASK
	//0100 0x04 - EMAIL		NS_SET_HIDE_EMAIL
		if(streqnc(param[7],"on")){
			if(streqnc(param[6],"email")){
				access5.settings |= NS_SET_HIDE_EMAIL;
				send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_HIDE_EMAIL_ON",aliases5.authnick,snick);
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else if(streqnc(param[6],"usermask")){
				access5.settings |= NS_SET_HIDE_MASK;
				send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_HIDE_USERMASK_ON",aliases5.authnick,snick);
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else if(streqnc(param[6],"quit")){
				access5.settings |= NS_SET_HIDE_QUIT;
				send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_HIDE_QUIT_ON",aliases5.authnick,snick);
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else if(streqnc(param[6],"status")){
				access5.settings |= NS_SET_HIDE_STATUS;
				send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_HIDE_STATUS_ON",aliases5.authnick,snick);
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else{
                send_lang(thread,language,howtosend,ns->nick,nick,"NS_SET_HIDE_NO_SUCH_FUNCTION_OR_ALREADY_SET");
                return MOD_CONT;
            }
            
			/* Update user-struct, we have to keep us up to date! */
			if(User != NULL){
#ifdef USE_THREADS
				my_pthread_mutex_lock(&users_lock);
#endif
				User->settings = access5.settings;
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&users_lock);
#endif
			}
			
			domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",access5.settings,mysql_quote(thread,aliases5.authnick));
			//send_lang(thread,language,howtosend,snick,nick,"NS_COMMAND_DONE");
		}else if(streqnc(param[7],"off")){
			if(streqnc(param[6],"email")){
				access5.settings &= ~NS_SET_HIDE_EMAIL;
				send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_HIDE_EMAIL_OFF",aliases5.authnick,snick);
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else if(streqnc(param[6],"usermask")){
				access5.settings &= ~NS_SET_HIDE_MASK;
				send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_HIDE_USERMASK_OFF",aliases5.authnick,snick);
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else if(streqnc(param[6],"quit")){
				access5.settings &= ~NS_SET_HIDE_QUIT;
				send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_HIDE_QUIT_OFF",aliases5.authnick,snick);
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else if(streqnc(param[6],"status")){
				access5.settings &= ~NS_SET_HIDE_STATUS;
				send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_HIDE_STATUS_OFF",aliases5.authnick,snick);
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else{
                send_lang(thread,language,howtosend,snick,nick,"NS_SET_HIDE_NO_SUCH_FUNCTION_OR_A_NOT_SET_HIDE_OPTION");
                return MOD_CONT;
            }
            
			/* Update user-struct, we have to keep us up to date! */
			if(User != NULL){
#ifdef USE_THREADS
				my_pthread_mutex_lock(&users_lock);
#endif
				User->settings = access5.settings;
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&users_lock);
#endif
			}
            
			domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",access5.settings,mysql_quote(thread,aliases5.authnick));
		}else send_lang(thread,language,howtosend,snick,nick,"NS_SET_HIDE_NO_SUCH_FUNCTION",param[7]);
	}else{
	/*
	 *  Error Messages
	 */
		if(aliases5.nick[0]=='\0')						send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		else if(access5.rights>=is_services_admin)		send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
		else 											send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Msg function for the NickServ SASET Command.
 * 
 * The NickServ SASET Msg Command allows an admin change a users<br/>
 *   Msg Options (Decides weather to PRIVMSG or NOTICE that User)!<br/>
 *   If it's set to "off" NickServ will NOTICE you, otherwise he uses PRIVMSG
 *  
 * Syntax is: /msg NickServ SASET < nick > MSG { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_msg(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	Aliases aliases5;
	Access access5;
	Users *User = NULL;
	
	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
    
    /*
     * SaSet Msg Command
     */
	aliases5	= sql_aliases(thread,param[4]);
	access5		= sql_access(thread,aliases5.authnick);
	User        = GetUser(param[4]);
    
	if(aliases5.nick[0]!='\0' && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"saset msg") && access5.rights<is_services_admin){
		if(streqnc(param[6],"on"))			access5.settings |= NS_SET_MSG;
		else if(streqnc(param[6],"off"))	access5.settings &= ~NS_SET_MSG;
		else return MOD_CONT;
		
		/* Update user-struct, we have to keep us up to date! */
		if(User != NULL){
#ifdef USE_THREADS
			my_pthread_mutex_lock(&users_lock);
#endif
			User->settings = access5.settings;
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&users_lock);
#endif
		}
		
		domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",access5.settings,mysql_quote(thread,aliases5.authnick));
		send_lang(thread,language,access5.settings&NS_SET_MSG ? 1 : 0,snick,nick,"NS_SASET_CHANGED_MSG",aliases5.authnick,_uc(param[6]));
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		if(aliases5.nick[0]=='\0')					send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		else if(access5.rights>=is_services_admin)	send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
		else 										send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Autoop function for the NickServ SASET Command.
 *
 * The NickServ SASET Msg Command allows an admin change a users<br/>
 *   Msg Options (Decides weather to PRIVMSG or NOTICE that User)!<br/>
 *   If it's set to "off" NickServ will NOTICE you, otherwise he uses PRIVMSG
 *
 * Syntax is: /msg NickServ SASET < nick > AUTOOP { ON | OFF }
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_autoop(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	Aliases aliases5;
	Access access5;
	Users *User = NULL;

	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));

    /*
     * SaSet AutoOp Command
     */
	aliases5	= sql_aliases(thread,param[4]);
	access5		= sql_access(thread,aliases5.authnick);
	User        = GetUser(param[4]);

	if(aliases5.nick[0]!='\0' && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"saset autoop") && access5.rights<is_services_admin){
		if(streqnc(param[6],"on"))			access5.settings |= NS_SET_AUTOOP;
		else if(streqnc(param[6],"off"))	access5.settings &= ~NS_SET_AUTOOP;
		else return MOD_CONT;
		
		/* Update user-struct, we have to keep us up to date! */
		if(User != NULL){
#ifdef USE_THREADS
			my_pthread_mutex_lock(&users_lock);
#endif
			User->settings = access5.settings;
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&users_lock);
#endif
		}
		
		domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",access5.settings,mysql_quote(thread,aliases5.authnick));
		send_lang(thread,language,access5.settings&NS_SET_MSG ? 1 : 0,snick,nick,"NS_SASET_CHANGED_AUTOOP",aliases5.authnick,_uc(param[6]));
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		if(aliases5.nick[0]=='\0')					send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		else if(access5.rights>=is_services_admin)	send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
		else 										send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Incognito function for the NickServ SASET Command.
 *
 * The NickServ SASET Incognito Command allows an admin change a users<br/>
 *   Incognito Option<br/>
 *   If it's set to "on" NickServ won't op you on identify and<br/>
 *   you won't get op on channels you have proper rights to<br/>
 *
 * Syntax is: /msg NickServ SASET < nick > INCOGNITO { ON | OFF }
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_incognito(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	Aliases aliases5;
	Access access5;
	Users *User = NULL;

	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));

    /*
     * SaSet Incognito Command
     */
	aliases5	= sql_aliases(thread,param[4]);
	access5		= sql_access(thread,aliases5.authnick);
	User        = GetUser(param[4]);

	if(aliases5.nick[0]!='\0' && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"saset incognito") && access5.rights<is_services_admin){
		if(streqnc(param[6],"on"))			access5.settings |= NS_SET_AUTOOP;
		else if(streqnc(param[6],"off"))	access5.settings &= ~NS_SET_AUTOOP;
		else return MOD_CONT;
		
		/* Update user-struct, we have to keep us up to date! */
		if(User != NULL){
#ifdef USE_THREADS
			my_pthread_mutex_lock(&users_lock);
#endif
			User->settings = access5.settings;
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&users_lock);
#endif
		}
		
		domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",access5.settings,mysql_quote(thread,aliases5.authnick));
		send_lang(thread,language,access5.settings&NS_SET_MSG ? 1 : 0,snick,nick,"NS_SASET_CHANGED_INCOGNITO",aliases5.authnick,_uc(param[6]));
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		if(aliases5.nick[0]=='\0')					send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		else if(access5.rights>=is_services_admin)	send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
		else 										send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET NoExpire function for the NickServ SASET Command.
 * 
 * The NickServ SASET NoExpire Command allows an admin change a users<br/>
 *   NoExpire Options (If NoExpire is set, that NickName will never Expire)!<br/>
 *   This is useful for Bots and IRCOps!
 *  
 * Syntax is: /msg NickServ SASET < nick > NOEXPIRE { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_noexpire(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	Aliases aliases5;
	Access access5;
	bool value=0;
	
	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
    
    /*
     * SaSet NoExpire Command
     */
	aliases5  = sql_aliases(thread,param[4]);
	access5   = sql_access(thread,aliases5.authnick);
    
	if(aliases5.nick[0]!='\0' && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"saset noexpire")){
		if(streqnc(param[6],"on")) value=0;
		else if(streqnc(param[6],"off")) value=1;
		
		domysql(thread,"UPDATE `orion_ns_aliases` SET `expire`='%d' WHERE `display`='%s'",value,mysql_quote(thread,aliases5.authnick));
		send_lang(thread,language,howtosend,snick,nick,"NS_SASET_CHANGED_NOEXPIRE",aliases5.authnick,_uc(param[6]));
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		if(aliases5.nick[0]=='\0')					send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		else if(access5.rights>=is_services_admin)	send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
		else 										send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ SASET Command (/msg NickServ HELP SASET)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SASET,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SASET
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * SaSet Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help saset")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SASET",snick,snick,snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Display Help function for the NickServ SASET Command (/msg NickServ HELP SASET DISPLAY)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SASET DISPLAY,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SASET DISPLAY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_display_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * SaSet Display Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help saset display")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SASET_DISPLAY");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Password Help function for the NickServ SASET Command (/msg NickServ HELP SASET PASSWORD)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SASET PASSWORD,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SASET PASSWORD
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_password_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * SaSet Password Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help saset password")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SASET_PASSWORD");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Language Help function for the NickServ SASET Command (/msg NickServ HELP SASET LANGUAGE)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SASET LANGUAGE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SASET LANGUAGE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_language_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * SaSet Language Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help saset language")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SASET_LANGUAGE");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Url Help function for the NickServ SASET Command (/msg NickServ HELP SASET URL)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SASET URL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SASET URL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_url_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * SaSet Url Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help saset url")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SASET_URL");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Email Help function for the NickServ SASET Command (/msg NickServ HELP SASET EMAIL)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SASET EMAIL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SASET EMAIL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_email_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * SaSet Email Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help saset email")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SASET_EMAIL");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Icq Help function for the NickServ SASET Command (/msg NickServ HELP SASET ICQ)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SASET ICQ,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SASET ICQ
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_icq_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * SaSet Icq Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help saset icq")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SASET_ICQ");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Greet Help function for the NickServ SASET Command (/msg NickServ HELP SASET GREET)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SASET GREET,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SASET GREET
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_greet_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * SaSet Greet Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help saset greet")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SASET_GREET");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Kill Help function for the NickServ SASET Command (/msg NickServ HELP SASET KILL)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SASET KILL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SASET KILL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_kill_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * SaSet Kill Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help saset kill")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SASET_KILL",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Secure Help function for the NickServ SASET Command (/msg NickServ HELP SASET SECURE)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SASET SECURE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SASET SECURE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_secure_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * SaSet Secure Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help saset secure")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SASET_SECURE",snick,snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Private Help function for the NickServ SASET Command (/msg NickServ HELP SASET PRIVATE)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SASET PRIVATE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SASET PRIVATE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_private_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * SaSet Private Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help saset private")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SASET_PRIVATE",snick,snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Hide Help function for the NickServ SASET Command (/msg NickServ HELP SASET HIDE)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SASET HIDE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SASET HIDE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_hide_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * SaSet Hide Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help saset hide")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SASET_HIDE",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Autoop Help function for the NickServ SASET Command (/msg NickServ HELP SASET AUTOOP)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SASET AUTOOP,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SASET AUTOOP
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_autoop_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * SaSet AutoOp Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help saset autoop")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SASET_AUTOOP");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Incognito Help function for the NickServ SASET Command (/msg NickServ HELP SASET INCOGNITO)
 *
 * When the user tries to get help for this command, using /msg NickServ HELP SASET INCOGNITO,<br/>
 * this function is executed.
 *
 * Syntax: /msg NickServ HELP SASET INCOGNITO
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_incognito_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * SaSet Incognito Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help saset incognito")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SASET_INCOGNITO");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET Msg Help function for the NickServ SASET Command (/msg NickServ HELP SASET MSG)
 *
 * When the user tries to get help for this command, using /msg NickServ HELP SASET MSG,<br/>
 * this function is executed.
 *
 * Syntax: /msg NickServ HELP SASET MSG
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_msg_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * SaSet Msg Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help saset msg")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SASET_MSG",snick,snick,snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SASET NoExpire Help function for the NickServ SASET Command (/msg NickServ HELP SASET NOEXPIRE)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SASET NOEXPIRE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SASET NOEXPIRE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_noexpire_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * SaSet NoExpire Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help saset noexpire")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SASET_NOEXPIRE");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - SASET part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SASET command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_saset_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * SaSet (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_SASET");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
