/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_events.c
 *   @brief This is the Module file for O EVENTS Command.
 *
 *   It contains all functions which are neccessary to initialize and use 
 *   the O EVENTS command.
 *
 *   Main functions - EVENTS ONLY
 *
 *   Syntax is: /msg O EVENTS
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"
#include "webform.h"

#define SERVICE "O"
#define MODUL "events"
#define MODUL_FILE "o_events"
#define MODUL_VERSION "1.2"

/*  ******************************************* Event List *****************************************  */

MOD_BOOL o_on_message(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL o_primary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL o_secondary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL o_squit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

/*  **************************************** End of Event List **************************************  */

MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,EVENT_PROTOTYPE,EVENT_PROTOTYPE);

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addAccess (thread,&Module,NOONE);
	addMODType(thread,&Module,EVENT);
	
	addEventHook(thread,&Module,"on_message",o_on_message);

	addEventHook(thread,&Module,"on_connect_primary"	,o_primary);
	addEventHook(thread,&Module,"on_connect_secondary"	,o_secondary);
	addEventHook(thread,&Module,"on_squit"				,o_squit);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);

/* Things we want to process "on load" */
	O->loaded = 1;

	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the O PRIMARY Command.
 * 
 * The O PRIMARY Command allows the user....
 *  
 * Syntax is: /msg O PRIMARY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL o_primary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    do_kill(thread,m->servername,O->nick,"nick used by orion");
	do_sqline(thread,m->servername,O->nick,0,time(NULL),"nick used by orion");
	createuser(thread,FAKETYPE_SERVICE,O->nick,O->ident,O->host,IRC.OSMode,O->realname);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the O SECONDARY Command.
 * 
 * The O SECONDARY Command allows the user....
 *  
 * Syntax is: /msg O SECONDARY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL o_secondary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	if(m->logchan[0] != '\0'){
		do_services_join(thread,O->nick,m->logchan,time(NULL),FAKETYPE_SERVICE);
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the O SQUIT Command.
 * 
 * The O SQUIT Command allows the user....
 *  
 * Syntax is: /msg O SQUIT
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL o_squit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * o_on_nick: O Events: On Nick Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL o_on_nick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * o_on_message: O Events: On Message Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL o_on_message(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* (language == NULL, howtosend = 0, snick == NULL) */

	if((m->UseStrictPrivMsg && streqnc(param[2],_dynadd("%s@%s",O->ident,m->servername))) || (!m->UseStrictPrivMsg && (streqnc(param[2],O->nick) || streqnc(param[2],_dynadd("%s@%s",O->ident,m->servername))))){
		if(GetUser(nick)==NULL && (!thread->Webform && FindWebSocket(thread->Sock)==NULL))
	    	return MOD_CONT;

		ModCheck modcheck;
    	ModCo *modul = NULL;
    	char *cmd    = &param[3][1];
    	
    	bzero(&modcheck,sizeof(modcheck));
    	
    	bzero(&thread->sql,sizeof(SQL));
		sql_mainAAC(thread,nick);
		
    	/* Since language == NULL, howtosend = 0, snick == NULL - we wake them up now ;) */
    	snick		= O->nick;
    	howtosend	= thread->sql.access.settings&NS_SET_MSG ? 1 : 0;
    	language	= languages[thread->sql.access.language];
    	/*********************************************************************************/
    	
	/* O Commands */
	
		/* If help for a specific Command was requested */
    	if(streqnc(cmd,"help") && param[4]){
			
			/* Help with wildmasks */
    		if(strcor(param[4],"*"))
				call_wildview(thread, snick, nick, SERVICE, "help", param);
    		
			/* Normal help for a specific command (no wildmasks) */
			else{
    			modul 		= findModule(thread, SERVICE, param[4]);
  				modcheck 	= call_helpcommand(thread, modul, cmd, snick, nick, param);
  				
				if(!ModReply(thread, &modcheck, snick, nick, param))
					send_lang(thread, language, howtosend, snick, nick, "O_UNKNOWN_COMMAND", snick);
    		}
    	}
		
		/* If Command was only "help" */
		else if(streqnc(cmd,"help") && !param[4]){
    		modul 		= findModule(thread, SERVICE, cmd);
    		modcheck 	= call_helpcommand(thread, modul, cmd, snick, nick, param);
    		
			if(!ModReply(thread, &modcheck, snick, nick, param))
    			send_lang(thread, language, howtosend, snick, nick, "O_UNKNOWN_COMMAND", snick);
    	}
    	
		/* All other commands which aren't "help" */
		else if(!streqnc(cmd,"help")){
    		modul 		= findModule(thread, SERVICE, cmd);
  			modcheck 	= call_command(thread, modul, cmd, snick, nick, param);
			  	
			if(!ModReply(thread, &modcheck, snick, nick, param))
    			send_lang(thread, language, howtosend, snick, nick, "O_UNKNOWN_COMMAND", snick);
    	}
		else send_lang(thread, language, howtosend, snick, nick, "O_UNKNOWN_COMMAND", snick);
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return MOD_BOOL
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
    if(Action == DO_UNLOAD){
		O->loaded = 0;
		do_quit(thread,O->nick,"module unloaded");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */
