/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_addchan.c
 *   @brief This is the Module file for Q ADDCHAN Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q ADDCHAN command.
 *
 *   The Q ADDCHAN Command allows the user to set register<br/>
 *   a channel.
 *
 *   Syntax is: /msg Q ADDCHAN #channel [ < url / Homepage > ] [ < chantype > ]
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

/* Define Q statics */
#define OWNER_MODE "ahmnoptv"

#define SERVICE "Q"
#define MODUL "addchan"
#define MODUL_FILE "q_addchan"
#define MODUL_VERSION "1.2"

MOD_BOOL Q_addchan(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_addchan_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_addchan_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"addchan","addchan");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"addchan",Q_addchan,550,is_services_user,4,0);
	addCommand(thread,&Module,"help addchan",Q_addchan_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"showcommands",Q_addchan_showcommands,is_services_user,is_services_user,3,3); //100 - each Admin is able to see it.

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q ADDCHAN Command.
 * 
 * The Q ADDCHAN Command allows the user to set register<br/>
 *   a channel.
 *
 * Syntax is: /msg Q ADDCHAN #channel [ < url / Homepage > ] [ < chantype > ]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_addchan(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global variables */
	int settings 	= 0;
	int botsettings = 0;

	/* Initialise Variables to (NULL) */

	/*
	 * Addchan Command
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"addchan")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);

		if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT))			send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT))	send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (streqnc(thread->c.chan.serv,"q") || streqnc(thread->c.chan.serv,"chanserv")))
			send_lang(thread,language,howtosend,snick,nick,"CHANNEL_ALREADY_REGISTERED",thread->c.chan.serv);

		else if(!thread->Webform && thread->sql.Cdata.localtime+m->CSWaitRegister>time(NULL)){
			send_lang(thread,language,howtosend,snick,nick,"CHANNEL_REGISTER_NOT_LONG_ENOUGH_CONNECTED",m->CSWaitRegister);
			return MOD_CONT;
		}else if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_cs_core` WHERE `founder`='%s' OR `successor`='%s'",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,thread->sql.aliases.authnick)))>=m->CSMaxReg)
			send_lang(thread,language,howtosend,snick,nick,"REGISTER_LIMIT_X_FOR_NICK_X_AS_FOUNDER_REACHED",m->CSMaxReg,thread->sql.aliases.authnick);

		else if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_chan_users` WHERE `nick`='%s' AND `channel`='%s' AND `faketype`='0' AND `modes` lIKE '%c%c%c'",mysql_quote(thread,nick),mysql_quote(thread,param[4]),'%',IRC.OpMode,'%'))>0){

			if(thread->c.chan.fields && thread->c.chan.serv[0]=='\0' && streqnc(thread->c.chan.name,param[4])){

				settings    = CSCheckSettings();
				botsettings = BSCheckSettings();
				settings    = CSAddBanType(settings);

				/* Update Memory, for caching */
#ifdef USE_THREADS
				my_pthread_mutex_lock(&threads_lock);
#endif
				thread->c.chan.levels.set 			= CS_SET;
				thread->c.chan.levels.autodeop 		= CS_AUTODEOP;
				thread->c.chan.levels.autohalfop 	= CS_AUTOHALFOP;
				thread->c.chan.levels.autoop 		= CS_AUTOOP;
				thread->c.chan.levels.autoprotect 	= CS_AUTOPROTECT;
				thread->c.chan.levels.autovoice 	= CS_AUTOVOICE;
				thread->c.chan.levels.nojoin 		= CS_NOJOIN;
				thread->c.chan.levels.signkick 		= CS_SIGNKICK;
				thread->c.chan.levels.acclist 		= CS_ACCESS_LIST;
				thread->c.chan.levels.accchange 	= CS_ACCESS_CHANGE;
				thread->c.chan.levels.akick 		= CS_AKICK;
				thread->c.chan.levels.ban 			= CS_BAN;
				thread->c.chan.levels.banme 		= CS_BANME;
				thread->c.chan.levels.clear 		= CS_CLEAR;
				thread->c.chan.levels.getkey 		= CS_GETKEY;
				thread->c.chan.levels.halfop 		= CS_HALFOP;
				thread->c.chan.levels.halfopme 		= CS_HALFOPME;
				thread->c.chan.levels.info 			= CS_INFO;
				thread->c.chan.levels.kick 			= CS_KICK;
				thread->c.chan.levels.kickme 		= CS_KICKME;
				thread->c.chan.levels.invite 		= CS_INVITE;
				thread->c.chan.levels.opdeop 		= CS_OPDEOP;
				thread->c.chan.levels.opdeopme 		= CS_OPDEOPME;
				thread->c.chan.levels.protect 		= CS_PROTECT;
				thread->c.chan.levels.protectme 	= CS_PROTECTME;
				thread->c.chan.levels.topic 		= CS_TOPIC;
				thread->c.chan.levels.unban 		= CS_UNBAN;
				thread->c.chan.levels.voice 		= CS_VOICE;
				thread->c.chan.levels.voiceme 		= CS_VOICEME;
				thread->c.chan.levels.memo 			= CS_MEMO;
				thread->c.chan.levels.assign 		= CS_ASSIGN;
				thread->c.chan.levels.badwords 		= CS_BADWORDS;
				thread->c.chan.levels.fantasia 		= CS_FANTASIA;
				thread->c.chan.levels.greet 		= CS_GREET;
				thread->c.chan.levels.nokick 		= CS_NOKICK;
				thread->c.chan.levels.say 			= CS_SAY;

				thread->c.chan.settings 			= settings;
				thread->c.chan.botserv.set 			= botsettings;
				thread->c.chan.botserv.caps_ttb 	= -1;
				thread->c.chan.botserv.caps_min 	= -1;
				thread->c.chan.botserv.caps_per 	= -1;
				thread->c.chan.botserv.flood_ttb 	= -1;
				thread->c.chan.botserv.flood_ln 	= -1;
				thread->c.chan.botserv.flood_secs 	= -1;
				thread->c.chan.botserv.repeat_ttb 	= -1;
				thread->c.chan.botserv.repeat	 	= -1;
				thread->c.chan.botserv.colors	 	= -1;
				thread->c.chan.botserv.underlines 	= -1;
				thread->c.chan.botserv.reverses	 	= -1;
				thread->c.chan.botserv.badwords	 	= -1;
				thread->c.chan.botserv.bolds	 	= -1;

				dynncpy(thread->c.chan.serv,"Q",sizeof(thread->c.chan.serv));
				dynncpy(thread->c.chan.founder,thread->sql.aliases.authnick,sizeof(thread->c.chan.founder));
				dynncpy(thread->c.chan.access.ownauthnick,thread->sql.aliases.authnick,sizeof(thread->c.chan.access.ownauthnick));
				dynncpy(thread->c.chan.access.ownlevel,OWNER_MODE,sizeof(thread->c.chan.access.ownlevel));

				bzero(&thread->c.chan.successor,sizeof(thread->c.chan.successor));
				bzero(&thread->c.chan.reason,sizeof(thread->c.chan.reason));

				thread->c.chan.maxmemos = m->maxmemos;
				thread->c.chan.regdate   = time(NULL);
				thread->c.chan.last_used = time(NULL);
				thread->c.chan.expire    = 1;
				
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&threads_lock);
#endif

				if(param[5]==NULL)
					dynncpy(thread->c.chan.url,"not-set",sizeof(thread->c.chan.url));
				else
					dynncpy(thread->c.chan.url,param[5],sizeof(thread->c.chan.url));

				if(param[6]==NULL){
					
					/* Update Memory, for caching */
#ifndef USE_THREADS
					dynncpy(thread->c.chan.description,"not-set",sizeof(thread->c.chan.description));
#endif
					
					/* Update SQL as Live Update */
				    domysql(thread,
						"UPDATE `orion_cs_core` SET "
							
						"`serv`='Q',`description`='not-set',`founder`='%s',`successor`=NULL,`expire`='1',`regdate`='%ld',`last_used`='%ld',`maxmemos`='%d',`settings`='%d',"
						"`url`='%s',`reason`='',"
						"`cs_set`='%d',`cs_autodeop`='%d',`cs_autohalfop`='%d',`cs_autoop`='%d',`cs_autoprotect`='%d',`cs_autovoice`='%d',"
						"`cs_nojoin`='%d',`cs_signkick`='%d',`cs_acclist`='%d',`cs_accchange`='%d',`cs_akick`='%d',`cs_ban`='%d',`cs_banme`='%d',"
						"`cs_cclear`='%d',`cs_getkey`='%d',`cs_halfop`='%d',`cs_halfopme`='%d',`cs_info`='%d',`cs_kick`='%d',`cs_kickme`='%d',`cs_invite`='%d',"
						"`cs_opdeop`='%d',`cs_opdeopme`='%d',`cs_protect`='%d',`cs_protectme`='%d',`cs_topic`='%d',`cs_unban`='%d',`cs_voice`='%d',"
						"`cs_voiceme`='%d',`cs_memo`='%d',`cs_botassign`='%d',`cs_badwords`='%d',`cs_fantasia`='%d',`cs_greet`='%d',`cs_nokick`='%d',"
						"`cs_say`='%d',"
						"`bs_set`='%d',"
						"`bs_caps_ttb`='-1',`bs_caps_min`='-1',`bs_caps_per`='-1',`bs_flood_ttb`='-1',`bs_flood_ln`='-1',`bs_flood_secs`='-1',`bs_repeat_ttb`='-1',`bs_repeat`='-1',`bs_colors`='-1',"
						"`bs_underlines`='-1',`bs_reverses`='-1',`bs_badwords`='-1',`bs_bolds`='-1'"
		
						" WHERE `channel`='%s'",
								
						mysql_quote(thread,thread->sql.aliases.authnick),time(NULL),time(NULL),m->maxmemos,settings,
						mysql_quote(thread,thread->c.chan.url),
						CS_SET,CS_AUTODEOP,CS_AUTOHALFOP,CS_AUTOOP,CS_AUTOPROTECT,CS_AUTOVOICE,
						CS_NOJOIN,CS_SIGNKICK,CS_ACCESS_LIST,CS_ACCESS_CHANGE,CS_AKICK,CS_BAN,CS_BANME,
						CS_CLEAR,CS_GETKEY,CS_HALFOP,CS_HALFOPME,CS_INFO,CS_KICK,CS_KICKME,CS_INVITE,
						CS_OPDEOP,CS_OPDEOPME,CS_PROTECT,CS_PROTECTME,CS_TOPIC,CS_UNBAN,CS_VOICE,
						CS_VOICEME,CS_MEMO,CS_ASSIGN,CS_BADWORDS,CS_FANTASIA,CS_GREET,CS_NOKICK,
						CS_SAY,botsettings,
						mysql_quote(thread,thread->c.chan.name)
					);

					domysql(thread,"INSERT INTO `orion_cs_access` (`serv`,`authnick`,`channel`,`level`) VALUES ('Q','%s','%s','%s')",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,thread->c.chan.name),mysql_quote(thread,thread->c.chan.access.ownlevel));
					do_join(thread,1,snick,thread->c.chan.name,thread->c.chan.datecreate);
					do_mode(thread,snick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",snick,snick,snick);
					send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else{
					/* Update Memory, for caching */
#ifndef USE_THREADS
					dynncpy(thread->c.chan.description,_dstrjoin(' ',param,6),sizeof(thread->c.chan.description));
#endif

					/* Update SQL as Live Update */
				    domysql(thread,
						"UPDATE `orion_cs_core` SET "
							
						"`serv`='Q',`description`='%s',`founder`='%s',`successor`=NULL ,`expire`='1',`regdate`='%ld',`last_used`='%ld',`maxmemos`='%d',`settings`='%d',"
						"`url`='%s',`reason`='',"
						"`cs_set`='%d',`cs_autodeop`='%d',`cs_autohalfop`='%d',`cs_autoop`='%d',`cs_autoprotect`='%d',`cs_autovoice`='%d',"
						"`cs_nojoin`='%d',`cs_signkick`='%d',`cs_acclist`='%d',`cs_accchange`='%d',`cs_akick`='%d',`cs_ban`='%d',`cs_banme`='%d',"
						"`cs_cclear`='%d',`cs_getkey`='%d',`cs_halfop`='%d',`cs_halfopme`='%d',`cs_info`='%d',`cs_kick`='%d',`cs_kickme`='%d',`cs_invite`='%d',"
						"`cs_opdeop`='%d',`cs_opdeopme`='%d',`cs_protect`='%d',`cs_protectme`='%d',`cs_topic`='%d',`cs_unban`='%d',`cs_voice`='%d',"
						"`cs_voiceme`='%d',`cs_memo`='%d',`cs_botassign`='%d',`cs_badwords`='%d',`cs_fantasia`='%d',`cs_greet`='%d',`cs_nokick`='%d',"
						"`cs_say`='%d',"
						"`bs_set`='%d',"
						"`bs_caps_ttb`='-1',`bs_caps_min`='-1',`bs_caps_per`='-1',`bs_flood_ttb`='-1',`bs_flood_ln`='-1',`bs_flood_secs`='-1',`bs_repeat_ttb`='-1',`bs_repeat`='-1',`bs_colors`='-1',"
						"`bs_underlines`='-1',`bs_reverses`='-1',`bs_badwords`='-1',`bs_bolds`='-1'"
		
						" WHERE `channel`='%s'",
								
						mysql_quote(thread,thread->c.chan.description),mysql_quote(thread,thread->sql.aliases.authnick),time(NULL),time(NULL),m->maxmemos,settings,
						mysql_quote(thread,thread->c.chan.url),
						CS_SET,CS_AUTODEOP,CS_AUTOHALFOP,CS_AUTOOP,CS_AUTOPROTECT,CS_AUTOVOICE,
						CS_NOJOIN,CS_SIGNKICK,CS_ACCESS_LIST,CS_ACCESS_CHANGE,CS_AKICK,CS_BAN,CS_BANME,
						CS_CLEAR,CS_GETKEY,CS_HALFOP,CS_HALFOPME,CS_INFO,CS_KICK,CS_KICKME,CS_INVITE,
						CS_OPDEOP,CS_OPDEOPME,CS_PROTECT,CS_PROTECTME,CS_TOPIC,CS_UNBAN,CS_VOICE,
						CS_VOICEME,CS_MEMO,CS_ASSIGN,CS_BADWORDS,CS_FANTASIA,CS_GREET,CS_NOKICK,
						CS_SAY,botsettings,
						mysql_quote(thread,thread->c.chan.name)
					);
					
					domysql(thread,"INSERT INTO `orion_cs_access` (`serv`,`authnick`,`channel`,`level`) VALUES ('Q','%s','%s','%s')",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,thread->c.chan.name),mysql_quote(thread,thread->c.chan.access.ownlevel));				
					do_join(thread,1,snick,thread->c.chan.name,thread->c.chan.datecreate);
					do_mode(thread,snick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",snick,snick,snick);
					send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}
				do_mode(thread,snick,thread->c.chan.name,thread->c.chan.datecreate,IRC.DefaultMLOCK);
				do_mode(thread,snick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",nick,nick,nick);
				
				if(!thread->Webform)
					do_logchan(thread,snick,NULL,NULL,NULL,"Channel: \'%s\' registered by %s!%s@%s (Q-Channel) (email: %s)",thread->c.chan.name,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,thread->sql.access.email);
				else
					do_logchan(thread,snick,NULL,NULL,NULL,"Channel: \'%s\' registered (Q-Channel) (email: %s)",thread->c.chan.name,nick,thread->sql.access.email);
					
			}else if(param[4][0]!='#') send_lang(thread,language,howtosend,snick,nick,"CHANNEL_INPUT_INVALID");
			else send_lang(thread,language,howtosend,snick,nick,"Q_ADDCHAN_CANT_ADD_AN_EMPTY_CHANNEL");
		}else send_lang(thread,language,howtosend,snick,nick,"NOT_A_CHANNELOP_OR_ON_CHANNEL",param[4]);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Q ADDCHAN Command (/msg Q HELP ADDCHAN)
 * 
 * When the user tries to get help for this command, using /msg Q HELP ADDCHAN,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP ADDCHAN
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_addchan_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Addchan Help
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help addchan")){
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_ADDCHAN",snick,snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"Q_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SHOWCOMMANDS Command - ADDCHAN part (/msg Q showcommands)
 * 
 * When the user tries to get general Q help, using /msg Q showcommands, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to<br/>
 * include the information for the ADDCHAN command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_addchan_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=0, Users_Q_Level=0;
	command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"showcommands");
	Users_Q_Level=(thread->sql.aliases.active>0 ? thread->sql.access.qlevel : is_user);
	/*
	 * Addchan (Showcommands)
	 */
	if(Users_Q_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_ADDCHAN",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"addchan"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
