/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_adduser.c
 *   @brief This is the Module file for Q ADDUSER Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q ADDUSER command.
 *
 *   The Q ADDUSER Command allows the user to add another<br/>
 *   user to his channel's access list. (with default rights: "aov")
 *
 *   Syntax is: /msg Q ADDUSER #channel < nick >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

/* Q mode by using adduser */
#define ADDUSER_MODE "aov"

#define SERVICE "Q"
#define MODUL "adduser"
#define MODUL_FILE "q_adduser"
#define MODUL_VERSION "1.2"

MOD_BOOL Q_adduser(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_adduser_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_adduser_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"adduser","adduser");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"adduser",Q_adduser,550,1,5,5);
	addCommand(thread,&Module,"help adduser",Q_adduser_help,100,1,4,4);
	addCommand(thread,&Module,"showcommands",Q_adduser_showcommands,100,1,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q ADDUSER Command.
 * 
 * The Q ADDUSER Command allows the user to add another<br/>
 *   user to his channel's access list. (with default rights: "aov")
 *
 * Syntax is: /msg Q ADDUSER #channel < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_adduser(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global variables */

	/* Initialise variables to (NULL) */

	/*
	 * Adduser Command
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"adduser")){
		/* Updates the Threads Channel-Management */
		UpdateChannelInformation(thread,param[4],param[5]);

		if(thread->c.chan.fields && streqnc(thread->c.chan.serv,"q")){
			if((thread->c.chan.settings & CS_FORBID_BIT)) 		send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
			else if((thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);

			else if(thread->sql.access.qlevel>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"adduser") || strcor(thread->c.chan.access.ownlevel,"nm")){
				//aliases5=sql_aliases(thread,param[5]); //Logins aliases5 - STRUCT
				if(thread->c.aliases.fields && thread->c.aliases.nick[0]!='\0' && thread->c.aliases.authnick[0]!='\0'){
					printf("thread->c.chan.access.targetauthnick = %s\n",thread->c.chan.access.targetauthnick);
					
					if(thread->c.chan.access.targetauthnick[0]!='\0')
						send_lang(thread,language,howtosend,snick,nick,"Q_ADDUSER_ACCOUNT_ALREADY_EXISTS");
					else{
						/* Update memory, for caching */
#ifndef USE_THREADS
						bzero(&thread->c.chan.access.targetlevel,sizeof(thread->c.chan.access.targetauthnick));
						dynncpy(thread->c.chan.access.targetauthnick,ADDUSER_MODE,sizeof(thread->c.chan.access.targetauthnick));
#endif

						/* Update SQL as live update */
						domysql(thread,"INSERT INTO `orion_cs_access` (`serv`,`authnick`,`channel`,`level`) VALUES ('Q','%s','%s','%s')",mysql_quote(thread,thread->c.aliases.authnick),mysql_quote(thread,param[4]),mysql_quote(thread,ADDUSER_MODE));
						
						send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
						setWebSuccess(thread); /* Indentify webform that changes were applied */
					}
				}else send_lang(thread,language,howtosend,snick,nick,"NO_AUTHNICK");
			}else send_lang(thread,language,howtosend,snick,nick,"Q_RIGHT_MASTER_TO_CHANGE_FLAGS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_EXISTING");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Q ADDUSER Command (/msg Q HELP ADDUSER)
 * 
 * When the user tries to get help for this command, using /msg Q HELP ADDUSER,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP ADDUSER
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_adduser_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Adduser Help
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help adduser")){
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_ADDUSER",snick,snick,snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}

	else send_lang(thread,language,howtosend,snick,nick,"Q_NO_HELP_AVAIL");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SHOWCOMMANDS Command - ADDUSER part (/msg Q showcommands)
 * 
 * When the user tries to get general Q help, using /msg Q showcommands, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to<br/>
 * include the information for the ADDUSER command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_adduser_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=0, Users_Q_Level=0;
	command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"showcommands");
	Users_Q_Level=(thread->sql.aliases.active>0 ? thread->sql.access.qlevel : is_user);
	/*
	 * Adduser (Showcommands)
	 */
	if(Users_Q_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_ADDUSER",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"adduser"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
