/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_spewchan.c
 *   @brief This is the Module file for Q SPEWCHAN Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q SPEWCHAN command.
 *
 *   The Q SPEWCHAN Command allows an admin search for channels<br/>
 *   which match a given < *channel-pattern* >.
 *
 *   Syntax is: /msg Q SPEWCHAN < *channel-pattern* >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "Q"
#define MODUL "spewchan"
#define MODUL_FILE "q_spewchan"
#define MODUL_VERSION "1.2"

MOD_BOOL Q_spewchan(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_spewchan_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_spewchan_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"spewchan","spewchan");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"spewchan",Q_spewchan,950,INFINIT,4,4);
	addCommand(thread,&Module,"help spewchan",Q_spewchan_help,950,INFINIT,4,4);
	addCommand(thread,&Module,"showcommands",Q_spewchan_showcommands,950,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q SPEWCHAN Command.
 * 
 * The Q SPEWCHAN Command allows an admin search for channels<br/>
 *   which match a given < *channel-pattern* >.
 *
 * Syntax is: /msg Q SPEWCHAN < *channel-pattern* >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_spewchan(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global variables */
	deepSQL chans;
	char channel[BUFSIZE];
	int counter1 = 0, counter2 = 0;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&chans,sizeof(chans));
	bzero(&channel,sizeof(channel));

	/*
	 * Spewchan Command
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"spewchan")){
		dynncpy (channel,param[4],sizeof(channel));
		strrepc (channel,'*','%');
		
		chans.res    = doSelectMySQL(thread,"SELECT `channel`,`settings`,`flags` FROM `orion_cs_core` WHERE `channel` LIKE '%s' AND `serv`='Q'",mysql_quote(thread,channel));
		chans.fields = mysql_num_rows(chans.res);

		for(i=0;i<chans.fields;i++){
			chans.row = mysql_fetch_row(chans.res);
			
			counter1++;
			if(!streqnc(chans.row[2],"")){
				if(stoi(chans.row[1]) & CS_SUSPEND_BIT){
					counter2++;
					send_lang(thread,language,howtosend,snick,nick,"Q_SPEWCHAN_SUSPENDED_CHANNEL_USING_CHANFLAGS",counter1,chans.row[0],chans.row[2]);
				}
				else
					send_lang(thread,language,howtosend,snick,nick,"Q_SPEWCHAN_CHANNEL_USING_CHANFLAGS",counter1,chans.row[0],chans.row[2]);
			}
			else{
				if(stoi(chans.row[1]) & CS_SUSPEND_BIT){
					counter2++;
					send_lang(thread,language,howtosend,snick,nick,"Q_SPEWCHAN_SUSPENDED_CHANNEL_NOT_USING_CHANFLAGS",counter1,chans.row[0]);
				}
				else
					send_lang(thread,language,howtosend,snick,nick,"Q_SPEWCHAN_CHANNEL_NOT_USING_CHANFLAGS",counter1,chans.row[0]);
			}
		}
		/* mysql_free_result_myr(); - chans */
		mysql_free_result_myr(chans.res);

		//strdelc(param[4],'%');

		if(counter1 != 0){
			send_lang(thread,language,howtosend,snick,nick,"Q_SPEWCHAN_STATISTICS");
			send_lang(thread,language,howtosend,snick,nick,"Q_SPEWCHAN_THERE_ARE_X_MATCHING",counter1);
		}
		if(counter2 != 0){
			msend(thread,howtosend,snick,nick,"---------------------------------");
			send_lang(thread,language,howtosend,snick,nick,"Q_SPEWCHAN_THERE_ARE_X_SUSPENDED_OF_X_MATCHING",counter2,counter1);
			send_lang(thread,language,howtosend,snick,nick,"Q_SPEWCHAN_X_MATCHING_ARE_NOT_SUSPENDED",counter1-counter2,counter1);
		}
		if(counter1 != 0)
			send_lang(thread,language,howtosend,snick,nick,"Q_SPEWCHAN_USE_CHANLEV_FOR_MORE_INFO",snick);
		else
			send_lang(thread,language,howtosend,snick,nick,"Q_SPEWCHAN_FOUND_NOTHING",param[4]);
			
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Q SPEWCHAN Command (/msg Q HELP SPEWCHAN)
 * 
 * When the user tries to get help for this command, using /msg Q HELP SPEWCHAN,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP SPEWCHAN
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_spewchan_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Spewchan Help
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help spewchan")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"spewchan"));
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_SPEWCHAN");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"Q_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SHOWCOMMANDS Command - SPEWCHAN part (/msg Q showcommands)
 * 
 * When the user tries to get general Q help, using /msg Q showcommands, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to<br/>
 * include the information for the SPEWCHAN command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_spewchan_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"showcommands");
	int Users_Q_Level=(thread->sql.aliases.active>0 ? thread->sql.access.qlevel : is_user);
	/*
	 * Spewchan (Showcommands)
	 */
	if(Users_Q_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_SPEWCHAN",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"spewchan"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
