/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file link.c
 *   @brief This is the Module file for LINK (Server Side) linking process.
 *
 *   It contains all functions which are neccessary to process
 *   Serverlinks!
 *
 *   The LINK File is used to process Serverlinks between other servers
 *   and us with our servers as well. (Protocol-Process-File -> PPF)
 *   (Link functions for the server - manages the DB Status (sockscontrol).)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"
#include "version.h"
#include "webform.h"

/**
 * @brief on_linkdone:  Is Executed if we receive any Information about a finished Link
 *               Only manages the database and sends out an EVENT for the ProgrammingArea.
 ****************************************************************************************
 *                               DON'T EDIT THIS PART
 ****************************************************************************************
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_linkdone(Threads *thread){
	if(DuringServerlink==false)
		return CMD_EXECUTED;
		
	DuringServerlink = false;
	ExpireCheckTime  = time(NULL);

#ifdef USE_THREADS
	my_pthread_mutex_lock(&globvar_lock);
#endif

	/* Server which was linking was OUR SERVICES-SERVER */
	if(m->UseWhowas && streqnc(linkserver,m->servername)){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&globvar_lock);
#endif
		domysql(thread,"INSERT INTO `orion_core_condata_log` SELECT * FROM `orion_core_condata`");
	}
	
	/* Server which was linking was NOT our services-server */
	else if(m->UseWhowas){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&globvar_lock);
#endif
		domysql(thread,"INSERT INTO `orion_core_condata_tmp` SELECT * FROM `orion_core_condata`");
		domysql(thread,"INSERT INTO `orion_core_condata_log` SELECT * FROM `orion_core_condata`");
		
#ifdef SQL_CONDATA_USE_INNODB
		domysql(thread,"DELETE FROM `orion_core_condata`");
#else
		domysql(thread,"TRUNCATE `orion_core_condata`");
#endif

		domysql(thread,"INSERT INTO `orion_core_condata` SELECT * FROM `orion_core_condata_tmp`");
	}
#ifdef USE_THREADS
	else my_pthread_mutex_unlock(&globvar_lock);
#endif

	if(m->MySQLSuperUser){
		domysql(thread,"ALTER TABLE `orion_core_condata` ENABLE KEYS");	
		domysql(thread,"FLUSH QUERY CACHE");
		domysql(thread,"RESET QUERY CACHE");
	}

#ifdef USE_THREADS
	my_pthread_mutex_lock(&globvar_lock);
#endif
	if(m->logfile[0]!='\0' && logging==0) logging    = 1;
    if(m->logchan[0]!='\0' && logchan==0) logchan    = 1;
    if(m->sqllogfile[0]!='\0')            sqllogging = 1;
    
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&globvar_lock);
#endif

	/* This indicates that "linkserver" finished linking, fully connected now */
	_ExecuteEvent(NULL,NULL,"on_linkdone");
	
	bzero(&linkserver,sizeof(linkserver));
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */

/**
 * @brief on_link:  Is Executed if we receive any Information about a Link
 *           Only manages the database and sends out an EVENT for the ProgrammingArea.
 *************************************************************************************
 *                               DON'T EDIT THIS PART
 *************************************************************************************
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param LinkedServer The Linking Server
 * @param Uplink The Server where it linked
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_link(Threads *thread, char *LinkedServer, char *Uplink){
	if(DuringServerlink)
		return CMD_EXECUTED;

	if(m->MySQLSuperUser){
		domysql(thread,"FLUSH QUERY CACHE");
		domysql(thread,"RESET QUERY CACHE");
		domysql(thread,"ALTER TABLE `orion_core_condata` DISABLE KEYS");
	}

//> :razorblade.dev.hub.deepirc.net SMO o :(link) Link razorblade.dev.hub.deepirc.net -> CServe.deepirc.net[@127.0.0.1.21828] established
//> :razorblade.dev.hub.deepirc.net SMO o :(sync) Link CServe.deepirc.net -> razorblade.dev.hub.deepirc.net is now synced [secs: 1 recv: 3.764 sent: 2.482]


	DuringServerlink 	= true;
    logging				= false;
    sqllogging			= false;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&globvar_lock);
#endif
	/* Linking server isn't our ServicesServer */
	if(!streqnc(LinkedServer,m->servername)){

		orion_WRCtranslate_cmd_ping(thread,m->servername,LinkedServer);

		if(m->UseWhowas){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&globvar_lock);
#endif
			domysql(thread,"DELETE FROM `orion_core_condata_tmp`");
			domysql(thread,"INSERT INTO `orion_core_condata_tmp` SELECT * FROM `orion_core_condata`");
			domysql(thread,"DELETE FROM `orion_core_condata`");
		}
#ifdef USE_THREADS
		else my_pthread_mutex_unlock(&globvar_lock);
#endif
		/* Execute EVENT that A server (not really we) is linking */
		_ExecuteEvent(NULL,NULL,"on_link");

		//domysql(thread,"DELETE FROM `orion_core_servers` WHERE `server`='%s'",LinkedServer);

#ifdef USE_THREADS
		my_pthread_mutex_lock(&globvar_lock);
#endif
		dynncpy(linkserver,LinkedServer,sizeof(linkserver));
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&globvar_lock);
#endif
	}
	/* If WE are the currently linking server */
	else if(streqnc(LinkedServer,m->servername)){
		if(m->UseWhowas)
			domysql(thread,"DELETE FROM `orion_core_condata_log`");
			
		dynncpy(linkserver,LinkedServer,sizeof(linkserver));
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&globvar_lock);
#endif
		//[11:03] -tfnnet.hub.eu.deepirc.net- (link) Link developer.hub.eu.deepirc.net -> CServe.deepirc.net[@81.169.179.172.40847] established
	}
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */
