/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file movedb.c
 *   @brief This is the Module file for moving the Database Information.
 *
 *   It contains all functions which are neccessary to
 *   move the AnopeDB to the orionDB. (converts a few things)
 *   NOTE: ALL MLOCK DATA IS LOST!
 *
 *	 MoveDB File - Moves the Information from the AnopeDB to the orionDB
 */
/*  ************************************************************************************************  */
 
#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <ctype.h>
#include <time.h>

#ifndef _WIN32
#include <unistd.h>
#else
#include "sysconf.h"
#include <windows.h>
#endif

/* Retain topic even after last person leaves channel */
#define ANOPE_CI_KEEPTOPIC			0x00000001
/* Don't allow non-authorized users to be opped */
#define ANOPE_CI_SECUREOPS			0x00000002
/* Hide channel from ChanServ LIST command */
#define ANOPE_CI_PRIVATE			0x00000004
/* Topic can only be changed by SET TOPIC */
#define ANOPE_CI_TOPICLOCK			0x00000008
/* Those not allowed ops are kickbanned */
#define ANOPE_CI_RESTRICTED			0x00000010
/* Don't allow ChanServ and BotServ commands to do bad things to bigger levels */
#define ANOPE_CI_PEACE  			0x00000020
/* Don't allow any privileges unless a user is IDENTIFY'd with NickServ */
#define ANOPE_CI_SECURE				0x00000040
/* Don't allow the channel to be registered or used */
#define ANOPE_CI_VERBOTEN			0x00000080
/* Channel password is encrypted */
#define ANOPE_CI_ENCRYPTEDPW		0x00000100
/* Channel does not expire */
#define ANOPE_CI_NO_EXPIRE			0x00000200
/* Channel memo limit may not be changed */
#define ANOPE_CI_MEMO_HARDMAX		0x00000400
/* Send notice to channel on use of OP/DEOP */
#define ANOPE_CI_OPNOTICE			0x00000800
/* Stricter control of channel founder status */
#define ANOPE_CI_SECUREFOUNDER 		0x00001000
/* Always sign kicks */
#define ANOPE_CI_SIGNKICK 			0x00002000
/* Sign kicks if level is < than the one defined by the SIGNKICK level */
#define ANOPE_CI_SIGNKICK_LEVEL 	0x00004000
/* Use the xOP lists */
#define ANOPE_CI_XOP 				0x00008000
/* Channel is suspended */
#define ANOPE_CI_SUSPENDED 			0x00010000

/* TEMPORARY - ChanServ is on the channel. */
#define ANOPE_CI_INHABIT 			0x80000000

/* Indices for cmd_access[]: */
#define ANOPE_CA_INVITE				0
#define ANOPE_CA_AKICK				1
#define ANOPE_CA_SET				2	/* but not FOUNDER or PASSWORD */
#define ANOPE_CA_UNBAN				3
#define ANOPE_CA_AUTOOP				4
#define ANOPE_CA_AUTODEOP			5	/* Maximum, not minimum */
#define ANOPE_CA_AUTOVOICE			6
#define ANOPE_CA_OPDEOP				7	/* ChanServ commands OP and DEOP */
#define ANOPE_CA_ACCESS_LIST		8
#define ANOPE_CA_CLEAR				9
#define ANOPE_CA_NOJOIN				10	/* Maximum */
#define ANOPE_CA_ACCESS_CHANGE 		11
#define ANOPE_CA_MEMO				12
#define ANOPE_CA_ASSIGN       		13  /* BotServ ASSIGN command */
#define ANOPE_CA_BADWORDS     		14  /* BotServ BADWORDS command */
#define ANOPE_CA_NOKICK       		15  /* Not kicked by the bot */
#define ANOPE_CA_FANTASIA   		16
#define ANOPE_CA_SAY				17
#define ANOPE_CA_GREET            	18
#define ANOPE_CA_VOICEME			19
#define ANOPE_CA_VOICE				20
#define ANOPE_CA_GETKEY           	21
#define ANOPE_CA_AUTOHALFOP			22
#define ANOPE_CA_AUTOPROTECT		23
#define ANOPE_CA_OPDEOPME         	24
#define ANOPE_CA_HALFOPME         	25
#define ANOPE_CA_HALFOP           	26
#define ANOPE_CA_PROTECTME        	27
#define ANOPE_CA_PROTECT          	28
#define ANOPE_CA_KICKME           	29
#define ANOPE_CA_KICK             	30
#define ANOPE_CA_SIGNKICK			31
#define ANOPE_CA_BANME            	32
#define ANOPE_CA_BAN              	33
#define ANOPE_CA_TOPIC            	34
#define ANOPE_CA_INFO             	35

#define ANOPE_CA_SIZE				36

/* BotServ SET flags */
#define ANOPE_BS_DONTKICKOPS      	0x00000001
#define ANOPE_BS_DONTKICKVOICES   	0x00000002
#define ANOPE_BS_FANTASY          	0x00000004
#define ANOPE_BS_SYMBIOSIS        	0x00000008
#define ANOPE_BS_GREET            	0x00000010
#define ANOPE_BS_NOBOT            	0x00000020

/* BotServ Kickers flags */
#define ANOPE_BS_KICK_BOLDS 		0x80000000
#define ANOPE_BS_KICK_COLORS 		0x40000000
#define ANOPE_BS_KICK_REVERSES		0x20000000
#define ANOPE_BS_KICK_UNDERLINES	0x10000000
#define ANOPE_BS_KICK_BADWORDS		0x08000000
#define ANOPE_BS_KICK_CAPS			0x04000000
#define ANOPE_BS_KICK_FLOOD			0x02000000
#define ANOPE_BS_KICK_REPEAT		0x01000000

/* Indices for TTB (Times To Ban) */
#define ANOPE_TTB_BOLDS 			0
#define ANOPE_TTB_COLORS 			1
#define ANOPE_TTB_REVERSES			2
#define ANOPE_TTB_UNDERLINES  		3
#define ANOPE_TTB_BADWORDS			4
#define ANOPE_TTB_CAPS				5
#define ANOPE_TTB_FLOOD				6
#define ANOPE_TTB_REPEAT			7
#define ANOPE_TTB_SIZE 				8

/* Nickname setting flags: */
#define ANOPE_NI_KILLPROTECT		0x00000001  /* Kill others who take this nick */
#define ANOPE_NI_SECURE				0x00000002  /* Don't recognize unless IDENTIFY'd */
#define ANOPE_NI_MSG            	0x00000004  /* Use PRIVMSGs instead of NOTICEs */
#define ANOPE_NI_MEMO_HARDMAX		0x00000008  /* Don't allow user to change memo limit */
#define ANOPE_NI_MEMO_SIGNON		0x00000010  /* Notify of memos at signon and un-away */
#define ANOPE_NI_MEMO_RECEIVE		0x00000020  /* Notify of new memos when sent */
#define ANOPE_NI_PRIVATE			0x00000040  /* Don't show in LIST to non-servadmins */
#define ANOPE_NI_HIDE_EMAIL			0x00000080  /* Don't show E-mail in INFO */
#define ANOPE_NI_HIDE_MASK			0x00000100  /* Don't show last seen address in INFO */
#define ANOPE_NI_HIDE_QUIT			0x00000200  /* Don't show last quit message in INFO */
#define ANOPE_NI_KILL_QUICK			0x00000400  /* Kill in 20 seconds instead of 60 */
#define ANOPE_NI_KILL_IMMED			0x00000800  /* Kill immediately instead of in 60 sec */
#define ANOPE_NI_SERVICES_OPER 		0x00001000  /* User is a Services operator */
#define ANOPE_NI_SERVICES_ADMIN		0x00002000  /* User is a Services admin */
#define ANOPE_NI_ENCRYPTEDPW		0x00004000  /* Nickname password is encrypted */
#define ANOPE_NI_SERVICES_ROOT  	0x00008000  /* User is a Services root */
#define ANOPE_NI_MEMO_MAIL      	0x00010000  /* User gets email on memo */
#define ANOPE_NI_HIDE_STATUS    	0x00020000  /* Don't show services access status */
#define ANOPE_NI_SUSPENDED			0x00040000  /* Nickname is suspended */
#define ANOPE_NI_AUTOOP				0x00080000  /* Autoop nickname in channels */

#define ANOPE_NS_VERBOTEN			0x0002      /* Nick may not be registered or used */
#define ANOPE_NS_NO_EXPIRE			0x0004      /* Nick never expires */

#define ANOPE_LANG_EN_US              0	/* United States English */
#define ANOPE_LANG_JA_JIS             1	/* Japanese (JIS encoding) */
#define ANOPE_LANG_JA_EUC             2	/* Japanese (EUC encoding) */
#define ANOPE_LANG_JA_SJIS            3	/* Japanese (SJIS encoding) */
#define ANOPE_LANG_ES                 4	/* Spanish */
#define ANOPE_LANG_PT                 5	/* Portugese */
#define ANOPE_LANG_FR                 6	/* French */
#define ANOPE_LANG_TR                 7	/* Turkish */
#define ANOPE_LANG_IT                 8	/* Italian */
#define ANOPE_LANG_DE                 9   	/* German */
#define ANOPE_LANG_CAT               10   	/* Catalan */
#define ANOPE_LANG_GR                11   	/* Greek */
#define ANOPE_LANG_NL                12   	/* Dutch */
#define ANOPE_LANG_RU                13   	/* Russian */
#define ANOPE_LANG_HUN               14   	/* Hungarian */
#define ANOPE_LANG_PL                15   	/* Polish */


#define ANOPE_BI_PRIVATE			0x0001

#define NICK_DB_1      "db/nick.db"
#define CHAN_DB_1      "db/chan.db"
#define BOT_DB_1       "db/bot.db"
#define HOST_DB_1      "db/hosts.db"

/* END OF CONFIGURATION BLOCK */

#ifndef _WIN32
#define C_LBLUE "\033[1;34m"
#define C_NONE "\033[m"
#else
#define C_LBLUE ""
#define C_NONE ""
#endif

#define getc_db(f)		(fgetc((f)->fp))
#define HASH(nick)	((tolower((nick)[0])&31)<<5 | (tolower((nick)[1])&31))
#define HASH2(chan)	((chan)[1] ? ((chan)[1]&31)<<5 | ((chan)[2]&31) : 0)
#define read_buffer(buf,f)	(read_db((f),(buf),sizeof(buf)) == sizeof(buf))
#define write_buffer(buf,f)	(write_db((f),(buf),sizeof(buf)) == sizeof(buf))
#define read_db(f,buf,len)	(fread((buf),1,(len),(f)->fp))
#define write_db(f,buf,len)	(fwrite((buf),1,(len),(f)->fp))
#define read_int8(ret,f)	((*(ret)=fgetc((f)->fp))==EOF ? -1 : 0)
#define write_int8(val,f)	(fputc((val),(f)->fp)==EOF ? -1 : 0)
#define SAFE(x) do { \
    if ((x) < 0) { \
	printf("Error, the database is broken, trying to continue... no guarantee.\n"); \
    } \
} while (0)
#define READ(x) do { \
    if ((x) < 0) { \
	    printf("Error, the database is broken, trying to continue... no guarantee.\n"); \
        exit(0); \
    } \
} while (0)

typedef struct memo_ Memo;
typedef struct dbFILE_ dbFILE;
typedef struct nickalias_ NickAlias;
typedef struct nickcore_ NickCore;
typedef struct chaninfo_ ChannelInfo;
typedef struct botinfo_ BotInfo;
typedef struct badword_ BadWord;
typedef struct hostcore_ HostCore;

struct memo_ {
    uint32 number;      /* Index number -- not necessarily array position! */
    uint16 flags;            /* Flags */
    time_t time;          /* When was it sent? */
    char sender[32];   /* Name of the sender */
    char *text;
};

struct dbFILE_ {
    int mode;                   /* 'r' for reading, 'w' for writing */
    FILE *fp;                    /* The normal file descriptor */
    char filename[1024];  /* Name of the database file */
};

typedef struct {
    int16 memocount;  /* Current # of memos */
    int16 memomax;    /* Max # of memos one can hold*/
    Memo *memos;     /* Pointer to original memos */
} MemoInfo;

typedef struct {
    uint16 in_use;         /* 1 if this entry is in use, else 0 */
    int16 level;
    NickCore *nc;       /* Guaranteed to be non-NULL if in use, NULL if not */
    time_t last_seen;
} AChanAccess;

typedef struct {
    int16 in_use;         /* Always 0 if not in use */
    int16 is_nick;         /* 1 if a regged nickname, 0 if a nick!user@host mask */
	uint16 flags;
    union {
		char *mask;      /* Guaranteed to be non-NULL if in use, NULL if not */
		NickCore *nc;   /* Same */
    } u;
    char *reason;
    char *creator;
    time_t addtime;
} AutoKick;

struct nickalias_ {
    NickAlias *next, *prev;
    char *nick;                    /* Nickname */
    char *last_quit;             /* Last quit message */
    char *last_realname;     /* Last realname */
    char *last_usermask;     /* Last usermask */
    time_t time_registered;  /* When the nick was registered */
    time_t last_seen;           /* When it was seen online for the last time */
    uint16 status;                  /* See NS_* below */
    NickCore *nc;               /* I'm an alias of this */
};

struct nickcore_ {
	NickCore *next, *prev;

    char *display;           /* How the nick is displayed */
    char pass[32];               /* Password of the nicks */
    char *email;              /* E-mail associated to the nick */
    char *greet;              /* Greet associated to the nick */
    uint32 icq;                 /* ICQ # associated to the nick */
    char *url;                  /* URL associated to the nick */
    uint32 flags;                /* See NI_* below */
    uint16 language;        /* Language selected by nickname owner (LANG_*) */
    uint16 accesscount;     /* # of entries */
    char **access;          /* Array of strings */
    MemoInfo memos;     /* Memo information */
    uint16 channelcount;  /* Number of channels currently registered */
    uint16 channelmax;   /* Maximum number of channels allowed */
    int unused;                /* Used for nick collisions */
    int aliascount;            /* How many aliases link to us? Remove the core if 0 */
};

struct chaninfo_ {
    ChannelInfo *next, *prev;

    char name[64];                     /* Channel name */
    char *founder;                      /* Who registered the channel */
    char *successor;                   /* Who gets the channel if the founder nick is dropped or expires */
    char founderpass[32];            /* Channel password */
    char *desc;                           /* Description */
    char *url;                              /* URL */
    char *email;                          /* Email address */
    time_t time_registered;          /* When was it registered */
    time_t last_used;                   /* When was it used hte last time */
    char *last_topic;	                    /* Last topic on the channel */
    char last_topic_setter[32];      /* Who set the last topic */
    time_t last_topic_time;           /* When the last topic was set */
    uint32 flags;                          /* Flags */
    char *forbidby;                      /* if forbidden: who did it */
    char *forbidreason;                /* if forbidden: why */
    int16 bantype;                       /* Bantype */
    int16 *levels;                        /* Access levels for commands */
    uint16 accesscount;                 /* # of pple with access */
    AChanAccess *access;             /* List of authorized users */
    uint16 akickcount;                    /* # of akicked pple */
    AutoKick *akick;                    /* List of users to kickban */
    uint32 mlock_on, mlock_off;   /* See channel modes below */
    uint32 mlock_limit;                /* 0 if no limit */
    char *mlock_key;                  /* NULL if no key */
    char *mlock_flood;	                /* NULL if no +f */
    char *mlock_redirect;            /* NULL if no +L */
    char *entry_message;           /* Notice sent on entering channel */
    MemoInfo memos;                 /* Memos */
    char *bi;                               /* Bot used on this channel */
    uint32 botflags;                      /* BS_* below */
    int16 *ttb;                             /* Times to ban for each kicker */
    uint16 bwcount;                       /* Badword count */
    BadWord *badwords;             /* For BADWORDS kicker */
    int16 capsmin, capspercent;    /* For CAPS kicker */
    int16 floodlines, floodsecs;      /* For FLOOD kicker */
    int16 repeattimes;                 /* For REPEAT kicker */
};

struct botinfo_ {
 	BotInfo *next, *prev;
 	char *nick;            /* Nickname of the bot */
 	char *user;            /* Its user name */
 	char *host;            /* Its hostname */
 	char *real;            /* Its real name */
 	int16 flags;            /* Bot flags */
 	time_t created;      /* Birth date */
	int16 chancount;	    /* Number of channels that use the bot. */
};

struct badword_ {
	uint16 in_use;
	char *word;
	uint16 type;
};

struct hostcore_ {
    HostCore *next, *last;
    char *nick;        /* Owner of the vHost */
    char *vIdent;    /* vIdent for the user */
    char *vHost;     /* Vhost for this user */
    char *creator;   /* Oper Nick of the oper who set the vhost */
    time_t time;      /* Date/Time vHost was set */
};

dbFILE *open_db_write(const char *service, const char *filename, int version);
dbFILE *open_db_read(const char *service, const char *filename, int version);
NickCore *findcore(const char *nick, int version);
NickCore *myfindcore(const char *nick);
NickAlias *findnick(const char *nick);
BotInfo *findbot(char *nick);
ChannelInfo *cs_findchan(const char *chan);
char *strscpy(char *d, const char *s, size_t len);
int write_file_version(dbFILE * f, uint32 version);
int mystricmp(const char *s1, const char *s2);
int delnick(NickAlias *na, int donttouchthelist);
int write_string(const char *s, dbFILE * f);
int write_ptr(const void *ptr, dbFILE * f);
int read_int16(int16 * ret, dbFILE * f);
int read_int32(int32 * ret, dbFILE * f);
int read_uint16(uint16 * ret, dbFILE * f);
int read_uint32(uint32 * ret, dbFILE * f);
int read_string(char **ret, dbFILE * f);
int write_int16(uint16 val, dbFILE * f);
int write_int32(uint32 val, dbFILE * f);
int read_ptr(void **ret, dbFILE * f);
int delcore(NickCore *nc);
void alpha_insert_chan(ChannelInfo * ci);
void insert_bot(BotInfo * bi);
void close_db(dbFILE * f);

ChannelInfo *chanlists[256];
NickAlias *nalists[1024];
NickCore *nclists[1024];
BotInfo *botlists[256];

int nonick = 0, nochan = 0, nobot = 0, nohost = 0;

/**
 * @brief anope_to_orion:  Moves the DatabaseContent from Anope's DB to Orion's
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param blob The BLOB Identifier (NS passwords stored as BLOB @ AnopeDB -> 1, otherwise 0)
 * @return void
 **/
void anope_to_orion(Threads *thread, bool blob){

	/* Main MySQL */
	deepSQL channels;
	int i = 0;
	int maxmemos = 0;
	int tmplevel = 0;
	
	/* Moving ChannelDB */
	char reason[256];
	char botnick[MAX_NICK_LEN];
	char founder[MAX_NICK_LEN];
	char successor[MAX_NICK_LEN];
	char ns_password[ENC_PASSMAX+1];
	int chansettings = 0;
	int botsettings  = 0;
	int forbid		 = 0;
	
	/* Moving Chanlevels to ChannelDB */
	char currchan[MAX_CHANNELNAME_LEN];
	int cs_set 			= CS_AUTOOP;
	int cs_autodeop 	= CS_AUTODEOP;
	int cs_autohalfop 	= CS_AUTOHALFOP;
	int cs_autoop 		= CS_AUTOOP;
	int cs_autoprotect 	= CS_AUTOPROTECT;
	int cs_autovoice 	= CS_AUTOVOICE;
	int cs_nojoin 		= CS_NOJOIN;
	int cs_signkick 	= CS_SIGNKICK;
	int cs_acclist 		= CS_ACCESS_LIST;
	int cs_accchange	= CS_ACCESS_CHANGE;
	int cs_akick 		= CS_AKICK;
	int cs_ban 			= CS_BAN;
	int cs_banme 		= CS_BANME;
	int cs_cclear 		= CS_CLEAR;
	int cs_getkey 		= CS_GETKEY;
	int cs_halfop 		= CS_HALFOP;
	int cs_halfopme 	= CS_HALFOPME;
	int cs_info 		= CS_INFO;
	int cs_kick 		= CS_KICK;
	int cs_kickme 		= CS_KICKME;
	int cs_invite 		= CS_INVITE;
	int cs_opdeop 		= CS_OPDEOP;
	int cs_opdeopme 	= CS_OPDEOPME;
	int cs_protect 		= CS_PROTECT;
	int cs_protectme 	= CS_PROTECTME;
	int cs_topic 		= CS_TOPIC;
	int cs_unban 		= CS_UNBAN;
	int cs_voice 		= CS_VOICE;
	int cs_voiceme 		= CS_VOICEME;
	int cs_memo 		= CS_MEMO;
	int cs_botassign 	= CS_ASSIGN;
	int cs_badwords 	= CS_BADWORDS;
	int cs_fantasia 	= CS_FANTASIA;
	int cs_greet 		= CS_GREET;
	int cs_nokick 		= CS_NOKICK;
	int cs_say 			= CS_SAY;
	
	/* Moving BotKickers to ChannelDB */
	//char currchan[MAX_CHANNELNAME_LEN]; - Used for ChanLevels and BotKickers
	int bs_bolds_ttb 		= -1;
	int bs_colors_ttb 		= -1;
	int bs_reverses_ttb 	= -1;
	int bs_underlines_ttb 	= -1;
	int bs_badwords_ttb 	= -1;
	int bs_caps_ttb 		= -1;
	int bs_flood_ttb 		= -1;
	int bs_repeat_ttb 		= -1;
	
	/* Moving Vhosts to VhostDB */
	int vhosted = 0;
	char vhost[MAX_HOST_LEN+MAX_IDENT_LEN+2];
	
	/* Moving Access to AccessDB (DEPENDENCY for 2used_blob_pass") */
	int language=0;
	int flags   =0;
	int qlevel  =0;
	int olevel  =0;
	int slevel  =0;
	int llevel  =0;
	int rights  =0;
	
	/* Moving Aliases to AliasesDB */
	int expire = 0;
	//int forbid = 0;
	
	open_logs(thread);
	
	if(copyconfig.MysqlMove[0] == '\0'){
		dlog(thread,0,"You forgot to define \"MysqlMove\", this is REQUIRED in that case!, define it - Config Error\n");
    	_orion_exit();
	}
	
	if(OrionLoadModule(thread,_dynadd("enc_%s",m->MysqlMove)).ModErr == MOD_ERR_OK){
		dlog(thread,0,"  *  ERROR: Loading Encryption Module (\"MysqlMove\" parameter in the Config -> Check the Config!)...\n");
		orion_exit(thread);
	}
	
	mysql_connect(thread);
	
/* Moving Memos from anope_os_exceptions to orion_os_trusts */
	domysql(thread,"INSERT INTO `orion_os_trusts` (`rhost`,`limit`,`description`) SELECT `mask`,`lim`,`reason` FROM `anope_os_exceptions`");
	domysql(thread,"UPDATE `orion_os_trusts` SET `art`='1'");
	
/* Moving BotDB */
	domysql(thread,"INSERT INTO `orion_bs_core` (`botnick`,`botident`,`bothost`,`botrealname`,`private`,`time`) SELECT `nick`,`user`,`host`,`rname`,`flags`,`created` FROM `anope_bs_core`");	
	
/* Moving Access from anope_ns_core to access - DEPENDENCY for using BLOB in DB or not!? */
	/*if(blob)
		channels.res    = doSelectMySQL(thread,"SELECT `display`,CAST(`pass` AS CHAR),`email`,`icq`,`url`,`flags`,`language`,`greet` FROM `anope_ns_core`");
	else*/
	channels.res    = doSelectMySQL(thread,"SELECT `display`,`pass`,`email`,`icq`,`url`,`flags`,`language`,`greet` FROM `anope_ns_core`");
	
	channels.fields = mysql_num_rows(channels.res);
	if(channels.fields){
		for(i=0;i<channels.fields;i++){
			channels.row = mysql_fetch_row(channels.res);
			
			language=0;
			flags   =0;
			qlevel  =0;
			olevel  =0;
			slevel  =0;
			llevel  =0;
			rights  =0;
			bzero(&ns_password,sizeof(ns_password));
			
			if(stoi(channels.row[5])&ANOPE_NI_KILLPROTECT) flags |= NS_SET_KILLPROTECT;
			if(stoi(channels.row[5])&ANOPE_NI_KILL_QUICK) flags |= NS_SET_KILL_QUICK;
			if(stoi(channels.row[5])&ANOPE_NI_KILL_IMMED) flags |= NS_SET_KILL_IMMED;
			if(stoi(channels.row[5])&ANOPE_NI_MSG) flags |= NS_SET_MSG;
			if(stoi(channels.row[5])&ANOPE_NI_SECURE) flags |= NS_SET_SECURE;
			if(stoi(channels.row[5])&ANOPE_NI_PRIVATE) flags |= NS_SET_PRIVATE;
			if(stoi(channels.row[5])&ANOPE_NI_HIDE_EMAIL) flags |= NS_SET_HIDE_EMAIL;
			if(stoi(channels.row[5])&ANOPE_NI_HIDE_MASK) flags |= NS_SET_HIDE_MASK;
			if(stoi(channels.row[5])&ANOPE_NI_HIDE_QUIT) flags |= NS_SET_HIDE_QUIT;
			if(stoi(channels.row[5])&ANOPE_NI_HIDE_STATUS) flags |= NS_SET_HIDE_STATUS;
			if(stoi(channels.row[5])&ANOPE_NI_SUSPENDED) flags |= NS_SUSPENDED;
		
			if(stoi(channels.row[5])&ANOPE_NI_SERVICES_ROOT){
				rights = is_services_root;
				qlevel = Q_SRA_LEVEL;
				olevel = O_SRA_LEVEL;
				slevel = S_SRA_LEVEL;
				llevel = 256;
			}else if(stoi(channels.row[5])&ANOPE_NI_SERVICES_ADMIN){
				rights = is_services_admin;
				qlevel = Q_SA_LEVEL;
				olevel = O_SA_LEVEL;
				slevel = S_SA_LEVEL;
				llevel = 254;
			}else if(stoi(channels.row[5])&ANOPE_NI_SERVICES_OPER){
				rights = is_services_oper;
				qlevel = Q_SO_LEVEL;
				olevel = O_SO_LEVEL;
				slevel = S_SO_LEVEL;
				llevel = 254;
			}else{
				rights = is_services_user;
				qlevel = 1;
				olevel = 0;
				slevel = 0;
				llevel = 1;
			}
			
			if(!streqnc(orion_check_encryption(),"none")){
				orion_encrypt(channels.row[1], strlen(channels.row[1]), ns_password, ENC_PASSMAX);	
			}
			else{
				dynncpy(ns_password,channels.row[1],sizeof(channels.row[1]));
			}

			switch(stoi(channels.row[6])){
				case ANOPE_LANG_EN_US: language = EN;
					break;
				case ANOPE_LANG_JA_JIS: language = EN;
					break;
				case ANOPE_LANG_JA_EUC: language = EN;
					break;
				case ANOPE_LANG_JA_SJIS: language = EN;
					break;
				case ANOPE_LANG_ES: language = EN;
					break;
				case ANOPE_LANG_PT: language = EN;
					break;
				case ANOPE_LANG_FR: language = EN;
					break;
				case ANOPE_LANG_TR: language = EN;
					break;
				case ANOPE_LANG_IT: language = EN;
					break;
				case ANOPE_LANG_DE: language = EN;
					break;
				case ANOPE_LANG_CAT: language = EN;
					break;
				case ANOPE_LANG_GR: language = EN;
					break;
				case ANOPE_LANG_NL: language = EN;
					break;
				case ANOPE_LANG_RU: language = EN;
					break;
				case ANOPE_LANG_HUN: language = EN;
					break;
				case ANOPE_LANG_PL: language = EN;
					break;
			}
			//`display`,`pass`,`email`,`icq`,`url`,`flags`,`language`,`greet`
			domysql(thread,"INSERT INTO `orion_ns_core` (`authnick`,`password`,`email`,`icq`,`url`,`settings`,`language`,`greet`,`rights`,`qlevel`,`olevel`,`slevel`,`llevel`) VALUES ('%s','%s','%s','%s','%s','%d','%d','%s','%d','%d','%d','%d','%d')",
				mysql_quote(thread,channels.row[0]),mysql_quote(thread,ns_password),mysql_quote(thread,channels.row[2]),
				mysql_quote(thread,channels.row[3]),mysql_quote(thread,channels.row[4]),flags,language,mysql_quote(thread,channels.row[7]),
				rights,qlevel,olevel,slevel,llevel
			);
		}
	}
	mysql_free_quotes(thread);
	mysql_free_result_myr(channels.res);
	bzero(&channels,sizeof(channels));

/* Moving Aliases from anope_ns_alias to aliases */
	channels.res    = doSelectMySQL(thread,"SELECT `display`,`nick`,`time_registered`,`last_seen`,`last_usermask`,`last_realname`,`last_quit`,`status` FROM `anope_ns_alias`");
	channels.fields = mysql_num_rows(channels.res);
	if(channels.fields){
		for(i=0;i<channels.fields;i++){
			channels.row = mysql_fetch_row(channels.res);

			expire = 1;
			forbid = 0;

			if(stoi(channels.row[7])&ANOPE_NS_NO_EXPIRE) expire=0;
			if(stoi(channels.row[7])&ANOPE_NS_VERBOTEN) forbid=1;
			
			domysql(thread,"INSERT INTO `orion_ns_aliases` (`display`,`nick`,`time_linked`,`last_used`,`last_usermask`,`last_realname`,`last_quit`,`expire`,`active`,`linked`,`forbid`) "
					"VALUES ('%s','%s','%ld','%ld','%s','%s','%s','%d','0','0','%d')",
					mysql_quote(thread,channels.row[0]),mysql_quote(thread,channels.row[1]),stol(channels.row[2])==0 ? (long)time(NULL) : stol(channels.row[2]),
					stol(channels.row[3])==0 ? (long)time(NULL) : stol(channels.row[3]),mysql_quote(thread,channels.row[4]),mysql_quote(thread,channels.row[5]),
					mysql_quote(thread,channels.row[6]),expire,forbid
					);
		}
	}
	mysql_free_quotes(thread);
	mysql_free_result_myr(channels.res);
	bzero(&channels,sizeof(channels));
	
/* Moving ChannelDB */
	channels.res = doSelectMySQL(thread,"SELECT "
		  "`name`,"
		  "`founder`,"
		  "`successor`,"
		  "`descr`,"
		  "`url`,"
		  "`email`,"
		  "`time_registered`,"
		  "`last_used`,"
		  "`last_topic`,"
		  "`last_topic_setter`,"
		  "`last_topic_time`,"
		  "`flags`,"
		  "`forbidby`,"
		  "`forbidreason`,"
		  "`bantype`,"
		  "`entry_message`,"
		  "`memomax`,"
		  "`botnick`,"
		  
		  "`botflags`,"
		  
		  "`capsmin`,"
		  "`capspercent`,"
		  "`floodlines`,"
		  "`floodsecs`,"
		  "`repeattimes`"
		  
		  " FROM `anope_cs_info`"
	);
		  
	channels.fields = mysql_num_rows(channels.res);
	if(channels.fields){
		for(i=0;i<channels.fields;i++){
			channels.row = mysql_fetch_row(channels.res);
			
			dlog(thread,0,"Moving channel = \"%s\"\n",channels.row[0]);
			
			chansettings = 0;
			botsettings  = 0;
			forbid		 = 0;
			expire		 = 1;
			bzero(&reason,sizeof(reason));
			bzero(&botnick,sizeof(botnick));
			bzero(&founder,sizeof(founder));
			bzero(&successor,sizeof(successor));
			
			maxmemos = stoi(channels.row[16]);
			
			if(stoi(channels.row[11])>0){		
				if(stoi(channels.row[11]) & ANOPE_CI_KEEPTOPIC) chansettings |= CS_KEEPTOPIC_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_PRIVATE) chansettings |= CS_PRIVATE_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_TOPICLOCK) chansettings |= CS_TOPICLOCK_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_RESTRICTED) chansettings |= CS_RESTRICTED_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_PEACE) chansettings |= CS_PEACE_BIT;
				//if(stoi(channels.row[11]) & ANOPE_CI_SECURE) chansettings |= CS_SECURE_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_SECUREFOUNDER) chansettings |= CS_SECUREFOUNDER_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_SECUREOPS) chansettings |= CS_SECUREOPS_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_VERBOTEN) chansettings |= CS_FORBID_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_NO_EXPIRE) expire=0;
				if(stoi(channels.row[11]) & ANOPE_CI_MEMO_HARDMAX) maxmemos=m->maxmemos;
				if(stoi(channels.row[11]) & ANOPE_CI_OPNOTICE) chansettings |= CS_OPNOTICE_BIT;

				if(stoi(channels.row[11]) & ANOPE_CI_SIGNKICK_LEVEL){
					chansettings |= CS_SIGNKICK_1_BIT;
					chansettings |= CS_SIGNKICK_2_BIT;
				}else if(stoi(channels.row[11]) & ANOPE_CI_SIGNKICK){
					chansettings |= CS_SIGNKICK_1_BIT;
					chansettings &= ~CS_SIGNKICK_2_BIT;
				}

				if(stoi(channels.row[11]) & ANOPE_CI_XOP) chansettings |= CS_XOP_BIT;
				if(stoi(channels.row[11]) & ANOPE_CI_SUSPENDED) chansettings |= CS_SUSPEND_BIT;
				
				if(stoi(channels.row[11]) & ANOPE_BS_DONTKICKVOICES) botsettings |= BS_SET_DONTKICKVOICES_BIT;
				if(stoi(channels.row[11]) & ANOPE_BS_FANTASY) botsettings |= BS_SET_FANTASY_BIT;
				if(stoi(channels.row[11]) & ANOPE_BS_SYMBIOSIS) botsettings |= BS_SET_SYMBIOSIS_BIT;
				if(stoi(channels.row[11]) & ANOPE_BS_GREET) botsettings |= BS_SET_GREET_BIT;
				if(stoi(channels.row[11]) & ANOPE_BS_NOBOT) chansettings |= BS_SET_NOBOT_BIT;
			}
			
			if(channels.row[13][0]!='\0')
				dynncpy(reason,channels.row[13],sizeof(reason));
				
			if(stoi(channels.row[18])>0){
				if(stoi(channels.row[18]) & ANOPE_BS_KICK_BOLDS) botsettings |= BS_KICK_BOLDS_BIT;
				if(stoi(channels.row[18]) & ANOPE_BS_KICK_COLORS) botsettings |= BS_KICK_COLORS_BIT;
				if(stoi(channels.row[18]) & ANOPE_BS_KICK_REVERSES) botsettings |= BS_KICK_REVERSES_BIT;
				if(stoi(channels.row[18]) & ANOPE_BS_KICK_UNDERLINES) botsettings |= BS_KICK_UNDERLINES_BIT;
				if(stoi(channels.row[18]) & ANOPE_BS_KICK_BADWORDS) botsettings |= BS_KICK_BADWORDS_BIT;
				if(stoi(channels.row[18]) & ANOPE_BS_KICK_CAPS) botsettings |= BS_KICK_CAPS_BIT;
				if(stoi(channels.row[18]) & ANOPE_BS_KICK_FLOOD) botsettings |= BS_KICK_FLOOD_BIT;
				if(stoi(channels.row[18]) & ANOPE_BS_KICK_REPEAT) botsettings |= BS_KICK_REPEAT_BIT;
			}
			
			if(stoi(channels.row[14]) == 0){
				if(chansettings&CS_BANTYPE_1_BIT) chansettings &= ~CS_BANTYPE_1_BIT;
				if(chansettings&CS_BANTYPE_2_BIT) chansettings &= ~CS_BANTYPE_2_BIT;
			}else if(stoi(channels.row[14]) == 1){
				if(~chansettings&CS_BANTYPE_1_BIT) chansettings |= CS_BANTYPE_1_BIT;
				if(chansettings&CS_BANTYPE_2_BIT)  chansettings &= ~CS_BANTYPE_2_BIT;
			}else if(stoi(channels.row[14]) == 2){
				if(chansettings&CS_BANTYPE_1_BIT)  chansettings &= ~CS_BANTYPE_1_BIT;
				if(~chansettings&CS_BANTYPE_2_BIT) chansettings |= CS_BANTYPE_2_BIT;
			}else if(stoi(channels.row[14]) == 3){
				if(~chansettings&CS_BANTYPE_1_BIT) chansettings |= CS_BANTYPE_1_BIT;
				if(~chansettings&CS_BANTYPE_2_BIT) chansettings |= CS_BANTYPE_2_BIT;
			}

			if(channels.row[17][0]=='\0'){
				dynncpy(botnick,"NULL",sizeof(botnick));
			}else
				snprintf(botnick,sizeof(botnick),"%c%s%c",toascii(39),mysql_quote(thread,channels.row[17]),toascii(39));
				
			if(channels.row[1][0]=='\0'){
				dynncpy(founder,"NULL",sizeof(founder));
			}else
				snprintf(founder,sizeof(botnick),"%c%s%c",toascii(39),mysql_quote(thread,channels.row[1]),toascii(39));
				
			if(channels.row[2][0]=='\0'){
				dynncpy(successor,"NULL",sizeof(successor));
			}else
				snprintf(successor,sizeof(botnick),"%c%s%c",toascii(39),mysql_quote(thread,channels.row[2]),toascii(39));

			domysql(thread,"INSERT INTO `orion_cs_core` "
				"(`serv`,`channel`,`botnick`,`founder`,`successor`,`email`,`settings`, "
				"`description`,`last_topic`,`last_topic_setter`,`last_topic_time`,`url`,`welcome`, "
				"`last_used`,`regdate`,`expire`,`maxmemos`,`bs_set`, "
					
				"`bs_caps_min`,"
				"`bs_caps_per`,"
				"`bs_flood_ln`,"
				"`bs_flood_secs`,"
				"`bs_repeat`,"
				
				"`reason`"
					
				") "
				"VALUES "
				"('ChanServ','%s',%s,%s,%s,'%s','%d',"
				"'%s','%s','%s','%s','%s','%s',"
				"'%ld','%ld','%d','%d',%ld,"
				"'%s',"
				"'%s',"
				"'%s',"
				"'%s',"
				"'%s',"
				"'%s')",
				mysql_quote(thread,channels.row[0]),botnick,founder,successor,mysql_quote(thread,channels.row[5]),chansettings,
				mysql_quote(thread,channels.row[3]),mysql_quote(thread,channels.row[8]),mysql_quote(thread,channels.row[9]),mysql_quote(thread,channels.row[10]),mysql_quote(thread,channels.row[4]),mysql_quote(thread,channels.row[15]),
				stol(channels.row[7])==0 ? (long)time(NULL) : stol(channels.row[7]),stol(channels.row[6]),expire,maxmemos,botsettings,
				mysql_quote(thread,channels.row[19]),
				mysql_quote(thread,channels.row[20]),
				mysql_quote(thread,channels.row[21]),
				mysql_quote(thread,channels.row[22]),
				mysql_quote(thread,channels.row[23]),
				mysql_quote(thread,reason)
			);
		}
	}
	mysql_free_quotes(thread);
	mysql_free_result_myr(channels.res);
	bzero(&channels,sizeof(channels));

/* Moving ChanLevels to ChannelDB */
	channels.res    = doSelectMySQL(thread,"SELECT `channel`,`position`,`level` FROM `anope_cs_levels`");
	channels.fields = mysql_num_rows(channels.res);
	
	if(channels.fields){
		for(i=0;i<channels.fields;i++){
			channels.row = mysql_fetch_row(channels.res);

			if(stoi(channels.row[2])<-ACCESS_SUCCESSOR)
				tmplevel = stoi(channels.row[2])+9000;
			else if(stoi(channels.row[2])>ACCESS_SUCCESSOR)
				tmplevel = stoi(channels.row[2])-9000;
			else
				tmplevel = stoi(channels.row[2]);
				
			if(stoi(channels.row[1])==ANOPE_CA_INVITE) cs_invite=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_AKICK) cs_akick=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_SET) cs_set=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_UNBAN) cs_unban=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_AUTOOP) cs_autoop=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_AUTODEOP) cs_autodeop=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_AUTOVOICE) cs_autovoice=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_OPDEOP) cs_opdeop=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_ACCESS_LIST) cs_acclist=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_CLEAR) cs_cclear=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_NOJOIN) cs_nojoin=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_ACCESS_CHANGE) cs_accchange=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_MEMO) cs_memo=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_ASSIGN) cs_botassign=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_BADWORDS) cs_badwords=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_NOKICK) cs_nokick=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_FANTASIA) cs_fantasia=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_SAY) cs_say=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_GREET) cs_greet=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_VOICEME) cs_voiceme=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_VOICE) cs_voice=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_GETKEY) cs_getkey=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_AUTOHALFOP) cs_autohalfop=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_AUTOPROTECT) cs_autoprotect=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_OPDEOPME) cs_opdeopme=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_HALFOPME) cs_halfopme=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_HALFOP) cs_halfop=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_PROTECTME) cs_protectme=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_PROTECT) cs_protect=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_KICKME) cs_kickme=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_KICK) cs_kick=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_SIGNKICK) cs_signkick=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_BANME) cs_banme=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_BAN) cs_ban=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_TOPIC) cs_topic=tmplevel;
			else if(stoi(channels.row[1])==ANOPE_CA_INFO) cs_info=tmplevel;
			
			if(!streqnc(currchan,channels.row[0])){
				if(currchan[0]!='\0'){
					domysql(thread,
						"UPDATE `orion_cs_core` SET "
							"`cs_set`='%d',"
							"`cs_autodeop`='%d',"
							"`cs_autohalfop`='%d',"
							"`cs_autoop`='%d',"
							"`cs_autoprotect`='%d',"
							"`cs_autovoice`='%d',"
							"`cs_nojoin`='%d',"
							"`cs_signkick`='%d',"
							"`cs_acclist`='%d',"
							"`cs_accchange`='%d',"
							"`cs_akick`='%d',"
							"`cs_ban`='%d',"
							"`cs_banme`='%d',"
							"`cs_cclear`='%d',"
							"`cs_getkey`='%d',"
							"`cs_halfop`='%d',"
							"`cs_halfopme`='%d',"
							"`cs_info`='%d',"
							"`cs_kick`='%d',"
							"`cs_kickme`='%d',"
							"`cs_invite`='%d',"
							"`cs_opdeop`='%d',"
							"`cs_opdeopme`='%d',"
							"`cs_protect`='%d',"
							"`cs_protectme`='%d',"
							"`cs_topic`='%d',"
							"`cs_unban`='%d',"
							"`cs_voice`='%d',"
							"`cs_voiceme`='%d',"
							"`cs_memo`='%d',"
							"`cs_botassign`='%d',"
							"`cs_badwords`='%d',"
							"`cs_fantasia`='%d',"
							"`cs_greet`='%d',"
							"`cs_nokick`='%d',"
							"`cs_say`='%d'"
						" WHERE `channel`='%s'",
						cs_set,
						cs_autodeop,
						cs_autohalfop,
						cs_autoop,
						cs_autoprotect,
						cs_autovoice,
						cs_nojoin,
						cs_signkick,
						cs_acclist,
						cs_accchange,
						cs_akick,
						cs_ban,
						cs_banme,
						cs_cclear,
						cs_getkey,
						cs_halfop,
						cs_halfopme,
						cs_info,
						cs_kick,
						cs_kickme,
						cs_invite,
						cs_opdeop,
						cs_opdeopme,
						cs_protect,
						cs_protectme,
						cs_topic,
						cs_unban,
						cs_voice,
						cs_voiceme,
						cs_memo,
						cs_botassign,
						cs_badwords,
						cs_fantasia,
						cs_greet,
						cs_nokick,
						cs_say,
						currchan
					);
					cs_set = CS_AUTOOP;
					cs_autodeop = CS_AUTODEOP;
					cs_autohalfop = CS_AUTOHALFOP;
					cs_autoop = CS_AUTOOP;
					cs_autoprotect = CS_AUTOPROTECT;
					cs_autovoice = CS_AUTOVOICE;
					cs_nojoin = CS_NOJOIN;
					cs_signkick = CS_SIGNKICK;
					cs_acclist = CS_ACCESS_LIST;
					cs_accchange = CS_ACCESS_CHANGE;
					cs_akick = CS_AKICK;
					cs_ban = CS_BAN;
					cs_banme = CS_BANME;
					cs_cclear = CS_CLEAR;
					cs_getkey = CS_GETKEY;
					cs_halfop = CS_HALFOP;
					cs_halfopme = CS_HALFOPME;
					cs_info = CS_INFO;
					cs_kick = CS_KICK;
					cs_kickme = CS_KICKME;
					cs_invite = CS_INVITE;
					cs_opdeop = CS_OPDEOP;
					cs_opdeopme = CS_OPDEOPME;
					cs_protect = CS_PROTECT;
					cs_protectme = CS_PROTECTME;
					cs_topic = CS_TOPIC;
					cs_unban = CS_UNBAN;
					cs_voice = CS_VOICE;
					cs_voiceme = CS_VOICEME;
					cs_memo = CS_MEMO;
					cs_botassign = CS_ASSIGN;
					cs_badwords = CS_BADWORDS;
					cs_fantasia = CS_FANTASIA;
					cs_greet = CS_GREET;
					cs_nokick = CS_NOKICK;
					cs_say = CS_SAY;

					bzero(&currchan,sizeof(currchan));
					dynncpy(currchan,channels.row[0],sizeof(currchan));
				}else{
					bzero(&currchan,sizeof(currchan));
					dynncpy(currchan,channels.row[0],sizeof(currchan));
				}
			}
		}
	}
	mysql_free_result_myr(channels.res);
	
	bzero(&channels,sizeof(channels));
	bzero(&currchan,sizeof(currchan));
	
/* Moving BotKickers to ChannelDB */
	channels.res    = doSelectMySQL(thread,"SELECT `channel`,`ttb_id`,`value` FROM `anope_cs_ttb`");
	channels.fields = mysql_num_rows(channels.res);
	if(channels.fields){
		for(i=0;i<channels.fields;i++){
			channels.row = mysql_fetch_row(channels.res);

			if(stoi(channels.row[1])==ANOPE_TTB_BOLDS) bs_bolds_ttb=stoi(channels.row[2]);
			else if(stoi(channels.row[1])==ANOPE_TTB_COLORS) bs_colors_ttb=stoi(channels.row[2]);
			else if(stoi(channels.row[1])==ANOPE_TTB_REVERSES) bs_reverses_ttb=stoi(channels.row[2]);
			else if(stoi(channels.row[1])==ANOPE_TTB_UNDERLINES) bs_underlines_ttb=stoi(channels.row[2]);
			else if(stoi(channels.row[1])==ANOPE_TTB_BADWORDS) bs_badwords_ttb=stoi(channels.row[2]);
			else if(stoi(channels.row[1])==ANOPE_TTB_CAPS) bs_caps_ttb=stoi(channels.row[2]);
			else if(stoi(channels.row[1])==ANOPE_TTB_FLOOD) bs_flood_ttb=stoi(channels.row[2]);
			else if(stoi(channels.row[1])==ANOPE_TTB_REPEAT) bs_repeat_ttb=stoi(channels.row[2]);
			
			if(!streqnc(currchan,channels.row[0])){
				if(currchan[0]!='\0'){
					domysql(thread,
						"UPDATE `orion_cs_core` SET "
							"`bs_bolds`='%d',"
							"`bs_colors`='%d',"
							"`bs_reverses`='%d',"
							"`bs_underlines`='%d',"
							"`bs_badwords`='%d',"
							"`bs_caps_ttb`='%d',"
							"`bs_flood_ttb`='%d',"
							"`bs_repeat_ttb`='%d'"
						" WHERE `channel`='%s'",
						bs_bolds_ttb,
						bs_colors_ttb,
						bs_reverses_ttb,
						bs_underlines_ttb,
						bs_badwords_ttb,
						bs_caps_ttb,
						bs_flood_ttb,
						bs_repeat_ttb,
						currchan
					);
					
					bzero(&currchan,sizeof(currchan));
					dynncpy(currchan,channels.row[0],sizeof(currchan));
					
				}else{
					bzero(&currchan,sizeof(currchan));
					dynncpy(currchan,channels.row[0],sizeof(currchan));
				}
			}
		}
	}
	mysql_free_result_myr(channels.res);
	bzero(&channels,sizeof(channels));

/* Moving ChanAccess from anope_cs_access to chanaccess */
	channels.res    = doSelectMySQL(thread,"SELECT `display`,`channel`,`level` FROM `anope_cs_access`");
	channels.fields = mysql_num_rows(channels.res);
	if(channels.fields){
		for(i=0;i<channels.fields;i++){
			channels.row = mysql_fetch_row(channels.res);

			dlog(thread,0,"Moving user = \"%s\", channel = \"%s\"\n",channels.row[0],channels.row[1]);

			if(stoi(channels.row[2])>ACCESS_SUCCESSOR)
				tmplevel = 1000;
			else
				tmplevel = stoi(channels.row[2]);

			domysql(thread,"INSERT INTO `orion_cs_access` (`serv`,`authnick`,`channel`,`level`) VALUES ('ChanServ','%s','%s','%d')",
			mysql_quote(thread,channels.row[0]),mysql_quote(thread,channels.row[1]),tmplevel);
		}
	}
	mysql_free_quotes(thread);
	mysql_free_result_myr(channels.res);
	bzero(&channels,sizeof(channels));		
	
/* Moving Badwords from anope_cs_badwords to botbadwords */
	domysql(thread,"INSERT INTO `orion_bs_badwords` (`art`,`channel`,`badword`) SELECT `type`,`channel`,`word` FROM `anope_cs_badwords`");
	
/* Moving Akicks from anope_cs_akicks to akick */
	domysql(thread,"INSERT INTO `orion_cs_akick` (`channel`,`flags`,`mask`,`reason`,`creator`,`addtime`) SELECT `channel`,`flags`,`dmask`,`reason`,`creator`,`addtime` FROM `anope_cs_akicks` WHERE `dmask`!=''");

/* Moving Memos from anope_ms_info to memos */
	domysql(thread,"INSERT INTO `orion_ms_core` (`receiver`,`sender`,`memo`,`mode`,`time`) SELECT `receiver`,`sender`,`text`,`flags`,`time` FROM `anope_ms_info`");

/* Moving Vhosts from anope_hs_core to vhosts */
	channels.res    = doSelectMySQL(thread,"SELECT `anope_hs_core`.`nick`,`anope_ns_alias`.`display`,`anope_hs_core`.`vident`,`anope_hs_core`.`vhost`,`anope_hs_core`.`creator`,`anope_hs_core`.`time` FROM `anope_hs_core` LEFT OUTER JOIN `anope_ns_alias` ON (`anope_ns_alias`.`nick` = `anope_hs_core`.`nick`)");
	channels.fields = mysql_num_rows(channels.res);
	if(channels.fields){
		for(i=0;i<channels.fields;i++){
			channels.row = mysql_fetch_row(channels.res);

			if(channels.row[1]==NULL)
				continue;

			vhosted = 0;
			bzero(&vhost,sizeof(vhost));
			if(channels.row[2][0]!='\0'){
				straddns(vhost,channels.row[2],sizeof(vhost));
				vhosted = 1;
			}
			if(channels.row[3][0]!='\0'){
				if(vhosted)
					straddnc(vhost,'@',sizeof(vhost));
				straddns(vhost,channels.row[3],sizeof(vhost));
			}
			
			dlog(thread,0,"Moving vhost for nick = \"%s\" Groupnick: \"%s\"\n",channels.row[0],channels.row[1]);
			
			domysql(thread,
						"INSERT INTO `orion_hs_core` (`nick`,`display`,`nick_type`,`vhost`,`authorized`,`changetime`,`active`) VALUES "
						"('%s','%s','0','%s','%s','%ld','0')",
						mysql_quote(thread,channels.row[0]),mysql_quote(thread,channels.row[1]),mysql_quote(thread,vhost),mysql_quote(thread,channels.row[4]),stol(channels.row[5])
					);
			mysql_free_quotes(thread);
		}
	}
	mysql_free_result_myr(channels.res);
	bzero(&channels,sizeof(channels));

	printf(" [*] MoveDB: Database moved successful ...\n");
	mysql_disconnect(thread);
}

/**
 * @brief anopeFiles_to_orion:  Moves the Filecontent from Anope's DBFiles to Orion's SQL Database
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return void
 **/
void anopeFiles_to_orion(Threads *thread){
    printf("Started...\n");
	/* Main MySQL */
	int i = 0;
	int j = 0;
	int maxmemos = 0;
	int tmplevel = 0;

	/* Moving ChannelDB */
	char reason[256];
	char botnick[MAX_NICK_LEN];
	char founder[MAX_NICK_LEN];
	char successor[MAX_NICK_LEN];
	char ns_password[ENC_PASSMAX];
	int chansettings = 0;
	int botsettings  = 0;
	int forbid		 = 0;

	/* Moving Vhosts to VhostDB */
	int vhosted = 0;
	char vhost[MAX_HOST_LEN+MAX_IDENT_LEN+2];

	/* Moving Access to AccessDB (DEPENDENCY for 2used_blob_pass") */
	int language=0;
	int flags   =0;
	int qlevel  =0;
	int olevel  =0;
	int slevel  =0;
	int llevel  =0;
	int rights  =0;

	/* Moving Aliases to AliasesDB */
	int expire = 0;
	//int forbid = 0;

	open_logs(thread);
    printf("opened logs...\n");

	if(copyconfig.MysqlMove[0] == '\0'){
        printf("MysqlMove Error...\n");
		dlog(thread,0,"You forgot to define \"MysqlMove\", this is REQUIRED in that case!, define it - Config Error\n");
    	_orion_exit();
	}

	if(OrionLoadModule(thread,_dynadd("enc_%s",m->MysqlMove)).ModErr != MOD_ERR_OK){
        printf("OrionLoadModule error...\n");
		dlog(thread,0,"  *  ERROR: Loading Encryption Module (\"MysqlMove\" parameter in the Config -> Check the Config!)...\n");
		_orion_exit();
	}

	mysql_connect(thread);

	/* NEW PART */
    dbFILE *f;
    HostCore *firsthc = NULL;

    printf("\n"C_LBLUE"DB Importer v0.1 beta for Orion IRC Services"C_NONE"\n\n");

    /* Section I: Nicks */
    /* Ia: First database */
    if ((f = open_db_read("NickServ", NICK_DB_1, 14))) {

        NickAlias *na, **nalast, *naprev;
        NickCore *nc, **nclast, *ncprev;
        int16 tmp16;
        int32 tmp32;
        int i, j, c;

        printf("Reading Nicks...\n");

        /* Nick cores */
        for (i = 0; i < 1024; i++) {
            nclast = &nclists[i];
            ncprev = NULL;

            while ((c = getc_db(f)) == 1) {
                if (c != 1) {
                    printf("Invalid format in %s.\n", NICK_DB_1);
                    exit(0);
                }

                nc = (NickCore*)calloc(1, sizeof(NickCore));
                nc->aliascount = 0;
                nc->unused = 0;

                *nclast = nc;
                nclast = &nc->next;
                nc->prev = ncprev;
                ncprev = nc;

                READ(read_string(&nc->display, f));
                READ(read_buffer(nc->pass, f));
                READ(read_string(&nc->email, f));
                READ(read_string(&nc->greet, f));
                READ(read_uint32(&nc->icq, f));
                READ(read_string(&nc->url, f));
                READ(read_uint32(&nc->flags, f));
                READ(read_uint16(&nc->language, f));
                READ(read_uint16(&nc->accesscount, f));
                if (nc->accesscount) {
                    char **access;
                    access = (char**)calloc(sizeof(char *) * nc->accesscount, 1);
                    nc->access = access;
                    for (j = 0; j < nc->accesscount; j++, access++)
                        READ(read_string(access, f));
                }
                READ(read_int16(&nc->memos.memocount, f));
                READ(read_int16(&nc->memos.memomax, f));
                if (nc->memos.memocount) {
                    Memo *memos;
                    memos = (Memo*)calloc(sizeof(Memo) * nc->memos.memocount, 1);
                    nc->memos.memos = memos;
                    for (j = 0; j < nc->memos.memocount; j++, memos++) {
                        READ(read_uint32(&memos->number, f));
                        READ(read_uint16(&memos->flags, f));
                        READ(read_int32(&tmp32, f));
                        memos->time = tmp32;
                        READ(read_buffer(memos->sender, f));
                        READ(read_string(&memos->text, f));
                    }
                }
                READ(read_uint16(&nc->channelcount, f));
                READ(read_int16(&tmp16, f));
            } /* getc_db() */
            *nclast = NULL;
        } /* for() loop */

        /* Nick aliases */
        for (i = 0; i < 1024; i++) {
            char *s = NULL;

            nalast = &nalists[i];
            naprev = NULL;

            while ((c = getc_db(f)) == 1) {
                if (c != 1) {
                    printf("Invalid format in %s.\n", NICK_DB_1);
                    exit(0);
                }

                na = (NickAlias*)calloc(1, sizeof(NickAlias));

                READ(read_string(&na->nick, f));
                READ(read_string(&na->last_usermask, f));
                READ(read_string(&na->last_realname, f));
                READ(read_string(&na->last_quit, f));

                READ(read_int32(&tmp32, f));
                na->time_registered = tmp32;
                READ(read_int32(&tmp32, f));
                na->last_seen = tmp32;
                READ(read_uint16(&na->status, f));
                READ(read_string(&s, f));
                na->nc = findcore(s, 0);
                na->nc->aliascount++;
                free(s);

                *nalast = na;
                nalast = &na->next;
                na->prev = naprev;
                naprev = na;
            } /* getc_db() */
            *nalast = NULL;
        } /* for() loop */
        close_db(f); /* End of section Ia */
    } else
        nonick = 1;

    /* Ic: Saving */
    if (!nonick) {
		printf("Saving Nicks ...\n");
        NickAlias *na;
        NickCore *nc;
        Memo *memos;
        int i, j;

        /* Nick cores */
        for (i = 0; i < 1024; i++) {
			//printf("Running (%i)\n",i);
            for (nc = nclists[i]; nc; nc = nc->next) {
				bzero(&ns_password,sizeof(ns_password));
				//printf("Saving %s\n",nc->display);
				if(nc->flags&ANOPE_NI_KILLPROTECT) flags |= NS_SET_KILLPROTECT;
				if(nc->flags&ANOPE_NI_KILL_QUICK) flags |= NS_SET_KILL_QUICK;
				if(nc->flags&ANOPE_NI_KILL_IMMED) flags |= NS_SET_KILL_IMMED;
				if(nc->flags&ANOPE_NI_MSG) flags |= NS_SET_MSG;
				if(nc->flags&ANOPE_NI_SECURE) flags |= NS_SET_SECURE;
				if(nc->flags&ANOPE_NI_PRIVATE) flags |= NS_SET_PRIVATE;
				if(nc->flags&ANOPE_NI_HIDE_EMAIL) flags |= NS_SET_HIDE_EMAIL;
				if(nc->flags&ANOPE_NI_HIDE_MASK) flags |= NS_SET_HIDE_MASK;
				if(nc->flags&ANOPE_NI_HIDE_QUIT) flags |= NS_SET_HIDE_QUIT;
				if(nc->flags&ANOPE_NI_HIDE_STATUS) flags |= NS_SET_HIDE_STATUS;
				if(nc->flags&ANOPE_NI_SUSPENDED) flags |= NS_SUSPENDED;

				if(nc->flags&ANOPE_NI_SERVICES_ROOT){
					rights = is_services_root;
					qlevel = Q_SRA_LEVEL;
					olevel = O_SRA_LEVEL;
					slevel = S_SRA_LEVEL;
					llevel = 256;
				}else if(nc->flags&ANOPE_NI_SERVICES_ADMIN){
					rights = is_services_admin;
					qlevel = Q_SA_LEVEL;
					olevel = O_SA_LEVEL;
					slevel = S_SA_LEVEL;
					llevel = 254;
				}else if(nc->flags&ANOPE_NI_SERVICES_OPER){
					rights = is_services_oper;
					qlevel = Q_SO_LEVEL;
					olevel = O_SO_LEVEL;
					slevel = S_SO_LEVEL;
					llevel = 254;
				}else{
					rights = is_services_user;
					qlevel = 1;
					olevel = 0;
					slevel = 0;
					llevel = 1;
				}

				if(!streqnc(orion_check_encryption(),"none")){
					orion_encrypt(nc->pass, strlen(nc->pass), ns_password, ENC_PASSMAX);
				}
				else{
					dynncpy(ns_password,nc->pass,sizeof(nc->pass));
				}

				switch(nc->language){
					case ANOPE_LANG_EN_US: language = EN;
						break;
					case ANOPE_LANG_JA_JIS: language = EN;
						break;
					case ANOPE_LANG_JA_EUC: language = EN;
						break;
					case ANOPE_LANG_JA_SJIS: language = EN;
						break;
					case ANOPE_LANG_ES: language = EN;
						break;
					case ANOPE_LANG_PT: language = EN;
						break;
					case ANOPE_LANG_FR: language = EN;
						break;
					case ANOPE_LANG_TR: language = EN;
						break;
					case ANOPE_LANG_IT: language = EN;
						break;
					case ANOPE_LANG_DE: language = EN;
						break;
					case ANOPE_LANG_CAT: language = EN;
						break;
					case ANOPE_LANG_GR: language = EN;
						break;
					case ANOPE_LANG_NL: language = EN;
						break;
					case ANOPE_LANG_RU: language = EN;
						break;
					case ANOPE_LANG_HUN: language = EN;
						break;
					case ANOPE_LANG_PL: language = EN;
						break;
				}
				domysql(thread,"INSERT IGNORE INTO `orion_ns_core` "
					"(`authnick`,`password`,`email`,`icq`,`url`,`settings`,`language`,`greet`,`rights`,`qlevel`,`olevel`,`slevel`,`llevel`)"
					" VALUES ('%s','%s','%s',%d,'%s',%d,%d,'%s',%d,%d,%d,%d,%d)",
					mysql_quote(thread,nc->display),mysql_quote(thread,ns_password),mysql_quote(thread,nc->email),
					nc->icq,mysql_quote(thread,nc->url),flags,language,mysql_quote(thread,nc->greet),
					rights,qlevel,olevel,slevel,llevel
				);

				memos = nc->memos.memos;

				for (j = 0; j < nc->memos.memocount; j++, memos++) {
					domysql(thread,"INSERT IGNORE INTO `orion_ms_core` (`receiver`,`sender`,`memo`,`mode`,`time`) "
						"VALUES ('%s','%s','%s',%d,%ld)", //SELECT `receiver`,`sender`,`text`,`flags`,`time` FROM `anope_ms_info`");
						mysql_quote(thread,nc->display),
						mysql_quote(thread,memos->sender),
						mysql_quote(thread,memos->text),
						memos->flags,
						memos->time
					);
				}
            }
		}// for (i)

        // Nick aliases
		for (i = 0; i < 1024; i++) {
			for (na = nalists[i]; na; na = na->next) {
				expire = 1;
				forbid = 0;

				if(na->status&ANOPE_NS_NO_EXPIRE) expire=0;
				if(na->status&ANOPE_NS_VERBOTEN) forbid=1;

				domysql(thread,"INSERT IGNORE INTO `orion_ns_aliases` (`display`,`nick`,`time_linked`,`last_used`,`last_usermask`,`last_realname`,`last_quit`,`expire`,`active`,`linked`,`forbid`) "
					"VALUES ('%s','%s','%d','%d','%s','%s','%s','%d','0','0','%d')",
					mysql_quote(thread,na->nc->display),mysql_quote(thread,na->nick),na->time_registered==0 ? (long)time(NULL) : na->time_registered,
					na->last_seen==0 ? (long)time(NULL) : na->last_seen,na->last_usermask == NULL ? "" : mysql_quote(thread,na->last_usermask),na->last_realname == NULL ? "" : mysql_quote(thread,na->last_realname),
					na->last_quit == NULL ? "" : mysql_quote(thread,na->last_quit),expire,forbid
				);
			} // for (na)
        } // for (i)
		printf("NickServ-database moved to sql.\n");
    } /* End of section I */

    /* Section II: Bots */
    /* IIa: First database */
    if ((f = open_db_read("Botserv", BOT_DB_1, 10))) {
        BotInfo *bi;
        int c;
        int32 tmp32;
        int16 tmp16;

        printf("Reading Bots...\n");

        while ((c = getc_db(f)) == 1) {
            if (c != 1) {
                printf("Invalid format in %s.\n", BOT_DB_1);
                exit(0);
            }

            bi = (BotInfo*)calloc(sizeof(BotInfo), 1);
            READ(read_string(&bi->nick, f));
            READ(read_string(&bi->user, f));
            READ(read_string(&bi->host, f));
            READ(read_string(&bi->real, f));
            SAFE(read_int16(&tmp16, f));
            bi->flags = tmp16;
            READ(read_int32(&tmp32, f));
            bi->created = tmp32;
            READ(read_int16(&tmp16, f));
            bi->chancount = tmp16;
            insert_bot(bi);
        }
        close_db(f);
    } else
        nobot = 1;

    /* IIb: Saving */
    if (!nobot) {
		BotInfo *bi;
		printf("Saving Bots ...\n");
		for (i = 0; i < 256; i++) {
			for (bi = botlists[i]; bi; bi = bi->next) {
                domysql(thread,"INSERT IGNORE INTO `orion_bs_core` (`botnick`,`botident`,`bothost`,`botrealname`,`private`,`time`) "
					"VALUES ('%s','%s','%s','%s',%d,%d)",	//SELECT `nick`,`user`,`host`,`rname`,`flags`,`created` FROM `anope_bs_core`");
					mysql_quote(thread,bi->nick),
					mysql_quote(thread,bi->user),
					mysql_quote(thread,bi->host),
					mysql_quote(thread,bi->real),
					bi->flags,
					bi->created
				);
			}
		}
        printf("Bot-database moved to sql\n");
    } /* End of section II */
    
    /* Section III: Chans */
    /* IIIa: First database */
    if ((f = open_db_read("ChanServ", CHAN_DB_1, 16)) && !nonick) {
        ChannelInfo *ci, **last, *prev;
        int c;

        printf("Reading channels...\n");

        for (i = 0; i < 256; i++) {
            int16 tmp16;
            int32 tmp32;
            int n_levels;
            char *s;
            int n_ttb;
			/* Unused variable - why? -GD
            int J;
			*/

            last = &chanlists[i];
            prev = NULL;

            while ((c = getc_db(f)) == 1) {
                int j;

                if (c != 1) {
                    printf("Invalid format in %s.\n", CHAN_DB_1);
                    exit(0);
                }

                ci = (ChannelInfo*)calloc(sizeof(ChannelInfo), 1);
                *last = ci;
                last = &ci->next;
                ci->prev = prev;
                prev = ci;
                READ(read_buffer(ci->name, f));
                READ(read_string(&ci->founder, f));
                READ(read_string(&ci->successor, f));
                READ(read_buffer(ci->founderpass, f));
                READ(read_string(&ci->desc, f));
                if (!ci->desc)
                    ci->desc = strdup("");
                READ(read_string(&ci->url, f));
                READ(read_string(&ci->email, f));
                READ(read_int32(&tmp32, f));
                ci->time_registered = tmp32;
                READ(read_int32(&tmp32, f));
                ci->last_used = tmp32;
                READ(read_string(&ci->last_topic, f));
                READ(read_buffer(ci->last_topic_setter, f));
                READ(read_int32(&tmp32, f));
                ci->last_topic_time = tmp32;
                READ(read_uint32(&ci->flags, f));
                /* Temporary flags cleanup */
                ci->flags &= ~0x80000000;
                READ(read_string(&ci->forbidby, f));
                READ(read_string(&ci->forbidreason, f));
                READ(read_int16(&tmp16, f));
                ci->bantype = tmp16;
                READ(read_int16(&tmp16, f));
                n_levels = tmp16;
                ci->levels = (int16*)calloc(36 * sizeof(*ci->levels), 1);
                for (j = 0; j < n_levels; j++) {
                    if (j < 36)
                        READ(read_int16(&ci->levels[j], f));
                    else
                        READ(read_int16(&tmp16, f));
                }
                READ(read_uint16(&ci->accesscount, f));
                if (ci->accesscount) {
                    ci->access = (AChanAccess*)calloc(ci->accesscount, sizeof(AChanAccess));
                    for (j = 0; j < ci->accesscount; j++) {
                        READ(read_uint16(&ci->access[j].in_use, f));
                        if (ci->access[j].in_use) {
                            READ(read_int16(&ci->access[j].level, f));
                            READ(read_string(&s, f));
                            if (s) {
                                ci->access[j].nc = findcore(s, 0);
                                free(s);
                            }
                            if (ci->access[j].nc == NULL)
                                ci->access[j].in_use = 0;
                            READ(read_int32(&tmp32, f));
                            ci->access[j].last_seen = tmp32;
                        }
                    }
                } else {
                    ci->access = NULL;
                }
                READ(read_uint16(&ci->akickcount, f));
                if (ci->akickcount) {
                    ci->akick = (AutoKick*)calloc(ci->akickcount, sizeof(AutoKick));
                    for (j = 0; j < ci->akickcount; j++) {
                        SAFE(read_uint16(&ci->akick[j].flags, f));
                        if (ci->akick[j].flags & 0x0001) {
                            SAFE(read_string(&s, f));
                            if (ci->akick[j].flags & 0x0002) {
                                ci->akick[j].u.nc = findcore(s, 0);
                                if (!ci->akick[j].u.nc)
                                    ci->akick[j].flags &= ~0x0001;
                                free(s);
                            } else {
                                ci->akick[j].u.mask = s;
                            }
                            SAFE(read_string(&s, f));
                            if (ci->akick[j].flags & 0x0001)
                                ci->akick[j].reason = s;
                            else if (s)
                                free(s);
                            SAFE(read_string(&s, f));
                            if (ci->akick[j].flags & 0x0001) {
                                ci->akick[j].creator = s;
                            } else if (s) {
                                free(s);
                            }
                            SAFE(read_int32(&tmp32, f));
                            if (ci->akick[j].flags & 0x0001)
                                ci->akick[j].addtime = tmp32;
                        }
                    }
                } else {
                    ci->akick = NULL;
                }
                READ(read_uint32(&ci->mlock_on, f));
                READ(read_uint32(&ci->mlock_off, f));
                READ(read_uint32(&ci->mlock_limit, f));
                READ(read_string(&ci->mlock_key, f));
                READ(read_string(&ci->mlock_flood, f));
                READ(read_string(&ci->mlock_redirect, f));
                READ(read_int16(&ci->memos.memocount, f));
                READ(read_int16(&ci->memos.memomax, f));
                if (ci->memos.memocount) {
                    Memo *memos;
                    memos = (Memo*)calloc(sizeof(Memo) * ci->memos.memocount, 1);
                    ci->memos.memos = memos;
                    for (j = 0; j < ci->memos.memocount; j++, memos++) {
                        READ(read_uint32(&memos->number, f));
                        READ(read_uint16(&memos->flags, f));
                        READ(read_int32(&tmp32, f));
                        memos->time = tmp32;
                        READ(read_buffer(memos->sender, f));
                        READ(read_string(&memos->text, f));
                    }
                }
                READ(read_string(&ci->entry_message, f));

                /* BotServ options */
                READ(read_string(&ci->bi, f));
                READ(read_int32(&tmp32, f));
                ci->botflags = tmp32;
                READ(read_int16(&tmp16, f));
                n_ttb = tmp16;
                ci->ttb = (int16*)calloc(2 * 8, 1);
                for (j = 0; j < n_ttb; j++) {
                    if (j < 8)
                        READ(read_int16(&ci->ttb[j], f));
                    else
                        READ(read_int16(&tmp16, f));
                }
                for (j = n_ttb; j < 8; j++)
                    ci->ttb[j] = 0;
                READ(read_int16(&tmp16, f));
                ci->capsmin = tmp16;
                READ(read_int16(&tmp16, f));
                ci->capspercent = tmp16;
                READ(read_int16(&tmp16, f));
                ci->floodlines = tmp16;
                READ(read_int16(&tmp16, f));
                ci->floodsecs = tmp16;
                READ(read_int16(&tmp16, f));
                ci->repeattimes = tmp16;

                READ(read_uint16(&ci->bwcount, f));
                if (ci->bwcount) {
                    ci->badwords = (BadWord*)calloc(ci->bwcount, sizeof(BadWord));
                    for (j = 0; j < ci->bwcount; j++) {
                        SAFE(read_uint16(&ci->badwords[j].in_use, f));
                        if (ci->badwords[j].in_use) {
                            SAFE(read_string(&ci->badwords[j].word, f));
                            SAFE(read_uint16(&ci->badwords[j].type, f));
                        }
                    }
                } else {
                    ci->badwords = NULL;
                }
            } /* getc_db() */
            *last = NULL;
        } /* for() loop */
        close_db(f);
    } else
        nochan = 1;

    /* IIIb: Saving */
    if (!nochan) {
		printf("Saving Channels ...\n");
        ChannelInfo *ci;
        Memo *memos;
			/* Unused variable - why? -GD
            static time_t lastwarn = 0;
			*/

        for (i = 0; i < 256; i++) {
			for (ci = chanlists[i]; ci; ci = ci->next) {
				chansettings = 0;
				botsettings  = 0;
				forbid		 = 0;
				expire		 = 1;
				bzero(&reason,sizeof(reason));
				bzero(&botnick,sizeof(botnick));
				bzero(&founder,sizeof(founder));
				bzero(&successor,sizeof(successor));

				maxmemos = ci->memos.memomax;

				if(ci->flags>0){
					if(ci->flags & ANOPE_CI_KEEPTOPIC) chansettings |= CS_KEEPTOPIC_BIT;
					if(ci->flags & ANOPE_CI_PRIVATE) chansettings |= CS_PRIVATE_BIT;
					if(ci->flags & ANOPE_CI_TOPICLOCK) chansettings |= CS_TOPICLOCK_BIT;
					if(ci->flags & ANOPE_CI_RESTRICTED) chansettings |= CS_RESTRICTED_BIT;
					if(ci->flags & ANOPE_CI_PEACE) chansettings |= CS_PEACE_BIT;
					//if(ci->flags & ANOPE_CI_SECURE) chansettings |= CS_SECURE_BIT;
					if(ci->flags & ANOPE_CI_SECUREFOUNDER) chansettings |= CS_SECUREFOUNDER_BIT;
					if(ci->flags & ANOPE_CI_SECUREOPS) chansettings |= CS_SECUREOPS_BIT;
					if(ci->flags & ANOPE_CI_VERBOTEN) chansettings |= CS_FORBID_BIT;
					if(ci->flags & ANOPE_CI_NO_EXPIRE) expire=0;
					if(ci->flags & ANOPE_CI_MEMO_HARDMAX) maxmemos=m->maxmemos;
					if(ci->flags & ANOPE_CI_OPNOTICE) chansettings |= CS_OPNOTICE_BIT;

					if(ci->flags & ANOPE_CI_SIGNKICK_LEVEL){
						chansettings |= CS_SIGNKICK_1_BIT;
						chansettings |= CS_SIGNKICK_2_BIT;
					}else if(ci->flags & ANOPE_CI_SIGNKICK){
						chansettings |= CS_SIGNKICK_1_BIT;
						chansettings &= ~CS_SIGNKICK_2_BIT;
					}

					if(ci->flags & ANOPE_CI_XOP) chansettings |= CS_XOP_BIT;
					if(ci->flags & ANOPE_CI_SUSPENDED) chansettings |= CS_SUSPEND_BIT;

					if(ci->flags & ANOPE_BS_DONTKICKVOICES) botsettings |= BS_SET_DONTKICKVOICES_BIT;
					if(ci->flags & ANOPE_BS_FANTASY) botsettings |= BS_SET_FANTASY_BIT;
					if(ci->flags & ANOPE_BS_SYMBIOSIS) botsettings |= BS_SET_SYMBIOSIS_BIT;
					if(ci->flags & ANOPE_BS_GREET) botsettings |= BS_SET_GREET_BIT;
					if(ci->flags & ANOPE_BS_NOBOT) chansettings |= BS_SET_NOBOT_BIT;
				}

				if(ci->forbidreason!=NULL)
					dynncpy(reason,ci->forbidreason,sizeof(reason));

				if(ci->botflags>0){
					if(ci->botflags & ANOPE_BS_KICK_BOLDS) botsettings |= BS_KICK_BOLDS_BIT;
					if(ci->botflags & ANOPE_BS_KICK_COLORS) botsettings |= BS_KICK_COLORS_BIT;
					if(ci->botflags & ANOPE_BS_KICK_REVERSES) botsettings |= BS_KICK_REVERSES_BIT;
					if(ci->botflags & ANOPE_BS_KICK_UNDERLINES) botsettings |= BS_KICK_UNDERLINES_BIT;
					if(ci->botflags & ANOPE_BS_KICK_BADWORDS) botsettings |= BS_KICK_BADWORDS_BIT;
					if(ci->botflags & ANOPE_BS_KICK_CAPS) botsettings |= BS_KICK_CAPS_BIT;
					if(ci->botflags & ANOPE_BS_KICK_FLOOD) botsettings |= BS_KICK_FLOOD_BIT;
					if(ci->botflags & ANOPE_BS_KICK_REPEAT) botsettings |= BS_KICK_REPEAT_BIT;
				}

				if(ci->bantype == 0){
					if(chansettings&CS_BANTYPE_1_BIT) chansettings &= ~CS_BANTYPE_1_BIT;
					if(chansettings&CS_BANTYPE_2_BIT) chansettings &= ~CS_BANTYPE_2_BIT;
				}else if(ci->bantype == 1){
					if(~chansettings&CS_BANTYPE_1_BIT) chansettings |= CS_BANTYPE_1_BIT;
					if(chansettings&CS_BANTYPE_2_BIT)  chansettings &= ~CS_BANTYPE_2_BIT;
				}else if(ci->bantype == 2){
					if(chansettings&CS_BANTYPE_1_BIT)  chansettings &= ~CS_BANTYPE_1_BIT;
					if(~chansettings&CS_BANTYPE_2_BIT) chansettings |= CS_BANTYPE_2_BIT;
				}else if(ci->bantype == 3){
					if(~chansettings&CS_BANTYPE_1_BIT) chansettings |= CS_BANTYPE_1_BIT;
					if(~chansettings&CS_BANTYPE_2_BIT) chansettings |= CS_BANTYPE_2_BIT;
				}

				for (j = 0; j < 36; j++){
					if(ci->levels[j]<-ACCESS_SUCCESSOR)
						ci->levels[j] = ci->levels[j]+9000;
					else if(ci->levels[j]>ACCESS_SUCCESSOR)
						ci->levels[j] = ci->levels[j]-9000;
				}

				if(ci->bi==NULL){
					dynncpy(botnick,"NULL",sizeof(botnick));
				}else
					snprintf(botnick,sizeof(botnick),"%c%s%c",toascii(39),mysql_quote(thread,ci->bi),toascii(39));

				if(ci->founder==NULL){
					dynncpy(founder,"NULL",sizeof(founder));
				}else
					snprintf(founder,sizeof(founder),"%c%s%c",toascii(39),mysql_quote(thread,ci->founder),toascii(39));

				if(ci->successor==NULL){
					dynncpy(successor,"NULL",sizeof(successor));
				}else
					snprintf(successor,sizeof(botnick),"%c%s%c",toascii(39),mysql_quote(thread,ci->successor),toascii(39));

				domysql(thread,"INSERT IGNORE INTO `orion_cs_core` "
					"(`serv`,`channel`,`botnick`,`founder`,`successor`,`email`,`settings`, "
					"`description`,`last_topic`,`last_topic_setter`,`last_topic_time`,`url`,`welcome`, "
					"`last_used`,`regdate`,`expire`,`maxmemos`,`bs_set`, "

					"`bs_caps_min`,`bs_caps_per`,`bs_flood_ln`,`bs_flood_secs`,`bs_repeat`,"

					"`reason`,"

					"`cs_set`,`cs_autodeop`,`cs_autohalfop`,`cs_autoop`,`cs_autoprotect`,`cs_autovoice`,`cs_nojoin`,"
					"`cs_signkick`,`cs_acclist`,`cs_accchange`,`cs_akick`,`cs_ban`,`cs_banme`,`cs_cclear`,`cs_getkey`,"
					"`cs_halfop`,`cs_halfopme`,`cs_info`,`cs_kick`,`cs_kickme`,`cs_invite`,`cs_opdeop`,`cs_opdeopme`,"
					"`cs_protect`,`cs_protectme`,`cs_topic`,`cs_unban`,`cs_voice`,`cs_voiceme`,`cs_memo`,`cs_botassign`,"
					"`cs_badwords`,`cs_fantasia`,`cs_greet`,`cs_nokick`,`cs_say`,"

					"`bs_bolds`,`bs_colors`,`bs_reverses`,`bs_underlines`,`bs_badwords`,`bs_caps_ttb`,`bs_flood_ttb`,`bs_repeat_ttb`) "
					"VALUES "

					"('ChanServ','%s',%s,%s,%s,'%s','%d',"
					"'%s','%s','%s',%d,'%s','%s',"
					"'%ld','%ld','%d','%d',%ld,"

					"%d,%d,%d,%d,%d,"

					"'%s',"// reason ?

					"%d,%d,%d,%d,%d,%d,%d,"
					"%d,%d,%d,%d,%d,%d,%d,%d,"
					"%d,%d,%d,%d,%d,%d,%d,%d,"
					"%d,%d,%d,%d,%d,%d,%d,%d,"
					"%d,%d,%d,%d,%d,"

					"%d,%d,%d,%d,%d,%d,%d,%d)",
					mysql_quote(thread,ci->name),botnick,founder,successor,ci->email == NULL ? "" : mysql_quote(thread,ci->email),chansettings,
					mysql_quote(thread,ci->desc),ci->last_topic == NULL ? "" : mysql_quote(thread,ci->last_topic),ci->last_topic_setter == NULL ? "" : mysql_quote(thread,ci->last_topic_setter),ci->last_topic_time,ci->url == NULL ? "" : mysql_quote(thread,ci->url),ci->entry_message == NULL ? "" : mysql_quote(thread,ci->entry_message),
					ci->last_used==0 ? (long)time(NULL) : ci->last_used,ci->time_registered,expire,maxmemos,botsettings,

					ci->capsmin,ci->capspercent,ci->floodlines,ci->floodsecs,ci->repeattimes,

					mysql_quote(thread,reason),

					ci->levels[ANOPE_CA_SET],
					ci->levels[ANOPE_CA_AUTODEOP],
					ci->levels[ANOPE_CA_AUTOHALFOP],
					ci->levels[ANOPE_CA_AUTOOP],
					ci->levels[ANOPE_CA_AUTOPROTECT],
					ci->levels[ANOPE_CA_AUTOVOICE],
					ci->levels[ANOPE_CA_NOJOIN],
					ci->levels[ANOPE_CA_SIGNKICK],
					ci->levels[ANOPE_CA_ACCESS_LIST],
					ci->levels[ANOPE_CA_ACCESS_CHANGE],
					ci->levels[ANOPE_CA_AKICK],
					ci->levels[ANOPE_CA_BAN],
					ci->levels[ANOPE_CA_BANME],
					ci->levels[ANOPE_CA_CLEAR],
					ci->levels[ANOPE_CA_GETKEY],
					ci->levels[ANOPE_CA_HALFOP],
					ci->levels[ANOPE_CA_HALFOPME],
					ci->levels[ANOPE_CA_INFO],
					ci->levels[ANOPE_CA_KICK],
					ci->levels[ANOPE_CA_KICKME],
					ci->levels[ANOPE_CA_INVITE],
					ci->levels[ANOPE_CA_OPDEOP],
					ci->levels[ANOPE_CA_OPDEOPME],
					ci->levels[ANOPE_CA_PROTECT],
					ci->levels[ANOPE_CA_PROTECTME],
					ci->levels[ANOPE_CA_TOPIC],
					ci->levels[ANOPE_CA_UNBAN],
					ci->levels[ANOPE_CA_VOICE],
					ci->levels[ANOPE_CA_VOICEME],
					ci->levels[ANOPE_CA_MEMO],
					ci->levels[ANOPE_CA_ASSIGN],
					ci->levels[ANOPE_CA_BADWORDS],
					ci->levels[ANOPE_CA_FANTASIA],
					ci->levels[ANOPE_CA_GREET],
					ci->levels[ANOPE_CA_NOKICK],
					ci->levels[ANOPE_CA_SAY],

					ci->ttb[ANOPE_TTB_BOLDS],
					ci->ttb[ANOPE_TTB_COLORS],
					ci->ttb[ANOPE_TTB_REVERSES],
					ci->ttb[ANOPE_TTB_UNDERLINES],
					ci->ttb[ANOPE_TTB_BADWORDS],
					ci->ttb[ANOPE_TTB_CAPS],
					ci->ttb[ANOPE_TTB_FLOOD],
					ci->ttb[ANOPE_TTB_REPEAT]
				);
        	    for (j = 0; j < ci->accesscount; j++) {
					if(ci->access[j].in_use){
						if(ci->access[j].level>ACCESS_SUCCESSOR)
							tmplevel = 1000;
						else
							tmplevel = ci->access[j].level;

						domysql(thread,"INSERT IGNORE INTO `orion_cs_access` (`serv`,`authnick`,`channel`,`level`) VALUES ('ChanServ','%s','%s',%d)",
							mysql_quote(thread,ci->access[j].nc->display),mysql_quote(thread,ci->name),tmplevel);
					}
				}

    	        for (j = 0; j < ci->akickcount; j++) {
        	        //domysql(thread,"INSERT INTO `orion_cs_akick` (`channel`,`flags`,`mask`,`reason`,`creator`,`addtime`) SELECT `channel`,`flags`,`dmask`,`reason`,`creator`,`addtime` FROM `anope_cs_akicks` WHERE `dmask`!=''");
					if(ci->akick[j].flags & 0x0001){
						domysql(thread,
							"INSERT IGNORE INTO `orion_cs_akick` (`channel`,`flags`,`mask`,`reason`,`creator`,`addtime`) "
							"VALUES ('%s',%d,'%s','%s','%s',%ld)",mysql_quote(thread,ci->name),
						
							ci->akick[j].flags,
							ci->akick[j].flags & 0x0002 ? mysql_quote(thread,ci->akick[j].u.nc->display) : mysql_quote(thread,ci->akick[j].u.mask),
							ci->akick[j].reason == NULL ? "" : mysql_quote(thread,ci->akick[j].reason),
							mysql_quote(thread,ci->akick[j].creator),
							ci->akick[j].addtime
						);
					}
				}

				memos = ci->memos.memos;

				for (j = 0; j < ci->memos.memocount; j++, memos++) {
					domysql(thread,"INSERT IGNORE INTO `orion_ms_core` (`receiver`,`sender`,`memo`,`mode`,`time`) "
						"VALUES ('%s','%s','%s',%d,%ld)", //SELECT `receiver`,`sender`,`text`,`flags`,`time` FROM `anope_ms_info`");
						mysql_quote(thread,ci->name),
						mysql_quote(thread,memos->sender),
						mysql_quote(thread,memos->text),
						memos->flags,
						memos->time
					);
				}

				for (j = 0; j < ci->bwcount; j++) {
					if(ci->badwords[j].in_use){
						domysql(thread,"INSERT IGNORE INTO `orion_bs_badwords` (`art`,`channel`,`badword`) "
							"VALUES (%d,'%s','%s')", //SELECT `type`,`channel`,`word` FROM `anope_cs_badwords`");
							ci->badwords[j].type,
							mysql_quote(thread,ci->name),
							mysql_quote(thread,ci->badwords[j].word)
						);
					}
				}
            } /* for (i) */
		}
        printf("Channel-database file moved to sql\n");
    } /* End of section III */

    /* Section IV: Hosts */
    /* IVa: First database */
    if ((f = open_db_read("HostServ", HOST_DB_1, 3)) && !nonick) {
        HostCore *hc;
        int c;
        int32 tmp32;

        printf("Reading Hosts...\n");

        while ((c = getc_db(f)) == 1) {
            if (c != 1) {
                printf("Invalid format in %s.\n", HOST_DB_1);
                exit(0);
            }
            hc = (HostCore*)calloc(1, sizeof(HostCore));
            READ(read_string(&hc->nick, f));
            READ(read_string(&hc->vIdent, f));
            READ(read_string(&hc->vHost, f));
            READ(read_string(&hc->creator, f));
            READ(read_int32(&tmp32, f));
            hc->time = tmp32;
            hc->next = firsthc;
            if (firsthc)
                firsthc->last = hc;
            hc->last = NULL;
            firsthc = hc;
        }
        close_db(f);
    } else
        nohost = 1;

    /* IVc: Saving */
    if (!nohost) {
        HostCore *hcptr;
        printf("Saving Hosts ...\n");
		for (hcptr = firsthc; hcptr; hcptr = hcptr->next) {
			if(myfindcore(hcptr->nick) == NULL)
				continue;

			vhosted = 0;
			bzero(&vhost,sizeof(vhost));
			if(hcptr->vIdent!=NULL){
				straddns(vhost,hcptr->vIdent,sizeof(vhost));
				vhosted = 1;
			}
			if(hcptr->vHost!=NULL){
				if(vhosted)
					straddnc(vhost,'@',sizeof(vhost));
				straddns(vhost,hcptr->vHost,sizeof(vhost));
			}

			domysql(thread,
						"INSERT IGNORE INTO `orion_hs_core` (`nick`,`display`,`nick_type`,`vhost`,`authorized`,`changetime`,`active`) VALUES "
						"('%s','%s','0','%s','%s','%ld','0')",
						mysql_quote(thread,hcptr->nick),mysql_quote(thread,myfindcore(hcptr->nick)->display),mysql_quote(thread,vhost),mysql_quote(thread,hcptr->creator),hcptr->time
					);
		}
		printf("HostServ-database moved to sql\n");
    } /* End of section IV */

    /* MERGING DONE \o/ HURRAY! */
	mysql_disconnect(thread);
    printf("\n\nAnope's *.db files were ported to orion's sql tables successfully. (There is NO guarantee for your DBs)\n");
	return;
} /* End of main() */

/* Open a database file for reading and check for the version */
dbFILE *open_db_read(const char *service, const char *filename, int version)
{
    dbFILE *f;
    FILE *fp;
    int myversion;

    f = (dbFILE*)calloc(sizeof(*f), 1);
    if (!f) {
        printf("Can't allocate memory for %s database %s.\n", service, filename);
        exit(0);
    }
    strscpy(f->filename, filename, sizeof(f->filename));
    f->mode = 'r';
    fp = fopen(f->filename, "rb");
    if (!fp) {
        printf("Can't read %s database %s.\n", service, f->filename);
        free(f);
        return NULL;
    }
    f->fp = fp;
    myversion = fgetc(fp) << 24 | fgetc(fp) << 16 | fgetc(fp) << 8 | fgetc(fp);
    if (feof(fp)) {
        printf("Error reading version number on %s: End of file detected.\n", f->filename);
        exit(0);
    } else if (myversion < version) {
        printf("Unsuported database version (%d) on %s.\n", myversion, f->filename);
        exit(0);
    }
    return f;
}

/* Open a database file for reading and check for the version */
dbFILE *open_db_write(const char *service, const char *filename, int version)
{
    dbFILE *f;
    int fd;

    f = (dbFILE*)calloc(sizeof(*f), 1);
    if (!f) {
        printf("Can't allocate memory for %s database %s.\n", service, filename);
        exit(0);
    }
    strscpy(f->filename, filename, sizeof(f->filename));
    filename = f->filename;
#ifndef _WIN32
    unlink(filename);
#else
    DeleteFile(filename);
#endif
    f->mode = 'w';
#ifndef _WIN32
    fd = open(filename, O_WRONLY | O_CREAT | O_EXCL, 0666);
#else
    fd = open(filename, O_WRONLY | O_CREAT | O_EXCL | _O_BINARY, 0666);
#endif
    f->fp = fdopen(fd, "wb");   /* will fail and return NULL if fd < 0 */
    if (!f->fp || !write_file_version(f, version)) {
        printf("Can't write to %s database %s.\n", service, filename);
        if (f->fp) {
            fclose(f->fp);
#ifndef _WIN32
            unlink(filename);
#else
            DeleteFile(filename);
#endif
        }
        free(f);
        return NULL;
    }
    return f;
}

/* Close it */
void close_db(dbFILE * f)
{
    fclose(f->fp);
    free(f);
}

int read_int16(int16 * ret, dbFILE * f)
{
    int c1, c2;

    c1 = fgetc(f->fp);
    c2 = fgetc(f->fp);
    if (c1 == EOF || c2 == EOF)
        return -1;
    *ret = c1 << 8 | c2;
    return 0;
}

int read_uint16(uint16 * ret, dbFILE * f)
{
    int c1, c2;

    c1 = fgetc(f->fp);
    c2 = fgetc(f->fp);
    if (c1 == EOF || c2 == EOF)
        return -1;
    *ret = c1 << 8 | c2;
    return 0;
}


int write_int16(uint16 val, dbFILE * f)
{
    if (fputc((val >> 8) & 0xFF, f->fp) == EOF
        || fputc(val & 0xFF, f->fp) == EOF)
        return -1;
    return 0;
}

int read_int32(int32 * ret, dbFILE * f)
{
    int c1, c2, c3, c4;

    c1 = fgetc(f->fp);
    c2 = fgetc(f->fp);
    c3 = fgetc(f->fp);
    c4 = fgetc(f->fp);
    if (c1 == EOF || c2 == EOF || c3 == EOF || c4 == EOF)
        return -1;
    *ret = c1 << 24 | c2 << 16 | c3 << 8 | c4;
    return 0;
}

int read_uint32(uint32 * ret, dbFILE * f)
{
    int c1, c2, c3, c4;

    c1 = fgetc(f->fp);
    c2 = fgetc(f->fp);
    c3 = fgetc(f->fp);
    c4 = fgetc(f->fp);
    if (c1 == EOF || c2 == EOF || c3 == EOF || c4 == EOF)
        return -1;
    *ret = c1 << 24 | c2 << 16 | c3 << 8 | c4;
    return 0;
}

int write_int32(uint32 val, dbFILE * f)
{
    if (fputc((val >> 24) & 0xFF, f->fp) == EOF)
        return -1;
    if (fputc((val >> 16) & 0xFF, f->fp) == EOF)
        return -1;
    if (fputc((val >> 8) & 0xFF, f->fp) == EOF)
        return -1;
    if (fputc((val) & 0xFF, f->fp) == EOF)
        return -1;
    return 0;
}


int read_ptr(void **ret, dbFILE * f)
{
    int c;

    c = fgetc(f->fp);
    if (c == EOF)
        return -1;
    *ret = (c ? (void *) 1 : (void *) 0);
    return 0;
}

int write_ptr(const void *ptr, dbFILE * f)
{
    if (fputc(ptr ? 1 : 0, f->fp) == EOF)
        return -1;
    return 0;
}


int read_string(char **ret, dbFILE * f)
{
    char *s;
    uint16 len;

    if (read_uint16(&len, f) < 0)
        return -1;
    if (len == 0) {
        *ret = NULL;
        return 0;
    }
    s = (char *)calloc(len, 1);
    if (len != fread(s, 1, len, f->fp)) {
        free(s);
        return -1;
    }
    *ret = s;
    return 0;
}

int write_string(const char *s, dbFILE * f)
{
    uint32 len;

    if (!s)
        return write_int16(0, f);
    len = strlen(s);
    if (len > 65534)
        len = 65534;
    if (write_int16((uint16) (len + 1), f) < 0)
        return -1;
    if (len > 0 && fwrite(s, 1, len, f->fp) != len)
        return -1;
    if (fputc(0, f->fp) == EOF)
        return -1;
    return 0;
}

NickCore *findcore(const char *nick, int unused)
{
    NickCore *nc;

    for (nc = nclists[HASH(nick)]; nc; nc = nc->next) {
        if (!mystricmp(nc->display, nick))
            if ((nc->unused && unused) || (!nc->unused && !unused))
                return nc;
    }

    return NULL;
}

NickCore *myfindcore(const char *nick)
{
    NickCore *nc;

    for (nc = nclists[HASH(nick)]; nc; nc = nc->next) {
        if (!mystricmp(nc->display, nick))
            return nc;
    }

    return NULL;
}

NickAlias *findnick(const char *nick)
{
    NickAlias *na;

    for (na = nalists[HASH(nick)]; na; na = na->next) {
        if (!mystricmp(na->nick, nick))
            return na;
    }

    return NULL;
}

int write_file_version(dbFILE * f, uint32 version)
{
    FILE *fp = f->fp;
    if (fputc(version >> 24 & 0xFF, fp) < 0 ||
        fputc(version >> 16 & 0xFF, fp) < 0 ||
        fputc(version >> 8 & 0xFF, fp) < 0 ||
        fputc(version & 0xFF, fp) < 0) {
            printf("Error writing version number on %s.\n", f->filename);
            exit(0);
    }
    return 1;
}

/* strscpy:  Copy at most len-1 characters from a string to a buffer, and
 *           add a null terminator after the last character copied.
 */

char *strscpy(char *d, const char *s, size_t len)
{
    char *d_orig = d;

    if (!len)
        return d;
    while (--len && (*d++ = *s++));
    *d = '\0';
    return d_orig;
}

int mystricmp(const char *s1, const char *s2)
{
    register int c;

    while ((c = tolower(*s1)) == tolower(*s2)) {
        if (c == 0)
            return 0;
        s1++;
        s2++;
    }
    if (c < tolower(*s2))
        return -1;
    return 1;
}

int delnick(NickAlias *na, int donttouchthelist)
{
    if (!donttouchthelist) {
        /* Remove us from the aliases list */
        if (na->next)
            na->next->prev = na->prev;
        if (na->prev)
            na->prev->next = na->next;
        else
            nalists[HASH(na->nick)] = na->next;
    }

    /* free() us */
    free(na->nick);
    if (na->last_usermask)
        free(na->last_usermask);
    if (na->last_realname)
        free(na->last_realname);
    if (na->last_quit)
        free(na->last_quit);
    free(na);
    return 1;
}

int delcore(NickCore *nc)
{
    int i;
    /* Remove the core from the list */
    if (nc->next)
        nc->next->prev = nc->prev;
    if (nc->prev)
        nc->prev->next = nc->next;
    else
        nclists[HASH(nc->display)] = nc->next;

    free(nc->display);
    if (nc->pass)
        free(nc->pass);
    if (nc->email)
        free(nc->email);
    if (nc->greet)
        free(nc->greet);
    if (nc->url)
        free(nc->url);
    if (nc->access) {
        for (i = 0; i < nc->accesscount; i++) {
            if (nc->access[i])
                free(nc->access[i]);
        }
        free(nc->access);
    }
    if (nc->memos.memos) {
        for (i = 0; i < nc->memos.memocount; i++) {
            if (nc->memos.memos[i].text)
                free(nc->memos.memos[i].text);
        }
        free(nc->memos.memos);
    }
    free(nc);
    return 1;
}

void insert_bot(BotInfo * bi)
{
    BotInfo *ptr, *prev;

    for (prev = NULL, ptr = botlists[tolower(*bi->nick)];
         ptr != NULL && mystricmp(ptr->nick, bi->nick) < 0;
         prev = ptr, ptr = ptr->next);
    bi->prev = prev;
    bi->next = ptr;
    if (!prev)
        botlists[tolower(*bi->nick)] = bi;
    else
        prev->next = bi;
    if (ptr)
        ptr->prev = bi;
}

BotInfo *findbot(char *nick)
{
    BotInfo *bi;

    for (bi = botlists[tolower(*nick)]; bi; bi = bi->next)
        if (!mystricmp(nick, bi->nick))
            return bi;

    return NULL;
}

ChannelInfo *cs_findchan(const char *chan)
{
    ChannelInfo *ci;
    for (ci = chanlists[tolower(chan[1])]; ci; ci = ci->next) {
        if (!mystricmp(ci->name, chan))
            return ci;
    }
    return NULL;
}

void alpha_insert_chan(ChannelInfo * ci)
{
    ChannelInfo *ptr, *prev;
    char *chan = ci->name;

    for (prev = NULL, ptr = chanlists[tolower(chan[1])];
         ptr != NULL && mystricmp(ptr->name, chan) < 0;
         prev = ptr, ptr = ptr->next);
    ci->prev = prev;
    ci->next = ptr;
    if (!prev)
        chanlists[tolower(chan[1])] = ci;
    else
        prev->next = ci;
    if (ptr)
        ptr->prev = ci;
}
