/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file socket.c
 *   @brief This is the Module file for SOCKET (Server Side) receiving.
 *
 *   It contains all functions which are neccessary to process
 *   Inputs from ANYWHERE!
 *
 *   The SOCKET File is used to process ALL inputs FROM EVERYWHERE.
 *   (Protocol-Process-File -> PPF)
 *   (Socket is the FIRST function (used to split Serverinputs!) - this is where the servermessages are received.)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"
#include "webform.h"

/* Main connection socket */

/**
 * @brief irc_check_command_r:  Called when Information was read from the Socket (FIRST STEP BY PROCESSING INCOMING DATA!)
 *                                  This is the ThreadSafe version of irc_check_command();
 *
 * @param in_thread The thread-struct (passed from active thread)- holds the threads information
 * @param receive_in The Information read from the Socket (this Information is going to be split to the ParameterArray here)
 * @return void
 **/
#ifdef USE_THREADS
void *irc_check_command_r(void *in_thread){
	my_pthread_mutex_lock(&mysql_lock);
	mysql_thread_init(); /* Initialize MySQL for Threading */
	my_pthread_mutex_unlock(&mysql_lock);
	
	Threads *thread = (Threads*)in_thread;
	char *param  [ARRAY_SIZE];
	char receive [BUFSIZE];
	
	size_t paramvalue = 0;

	/* Now initialize our Thread */
	//my_pthread_mutex_lock(&threads_lock);
	thread->Sock 			= IRCThread->Sock; // Takeover main IRCSock int! (necessary for sockwrites)
	thread->debughandle 	= IRCThread->debughandle;
	thread->sqlloghandle 	= IRCThread->sqlloghandle;
	thread->loghandle 		= IRCThread->loghandle;
	//my_pthread_mutex_unlock(&threads_lock);
	/* Done */
														
	if(m->debug>=1){
		dlog(thread,1,"> %s\n", thread->buf);
		
	}else if(!m->debug && dofork){
		alog(thread,"> %s\n", thread->buf);
	}

	/* receive
		includes the Information given from the socket to this void.
		Example: ":segfault.fl.us.deepirc.net SQLINE O :nick used by orion"
		without "".
	*/

	bzero(&receive,sizeof(receive));
	bzero(param,sizeof(param));
	
	dynncpy              (receive,thread->buf,sizeof(receive));			/* Function of string.h */
	paramvalue = strsplit(receive,param,' ');		/* splitdata (splitdata.c line 79) */
	/* BITS
		bits contains the same information as param
		but without ':' and the string is saved
		as a lowercase string, all charakters are used
		with lower case.
	*/
	/** PERFORM OUR MAIN PROTOCOL **/
	if(thread->Webform)
		on_protocol(thread,&WebForm,paramvalue,param);
	else
		on_protocol(thread,&IRC,paramvalue,param);
	

	/* clear Quotes array */
	mysql_free_quotes(thread);
	/**********************/
	
	/* End Thread */
	KillThread(thread); //Thread Safe
	mysql_thread_end();
	pthread_exit(NULL);
	return NULL;
}
#endif

/*  ************************************************************************************************  */

/**
 * @brief irc_check_command:  Called when Information was read from the Socket (FIRST STEP BY PROCESSING INCOMING DATA!)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param receive_in The Information read from the Socket (this Information is going to be split to the ParameterArray here)
 * @return void
 **/
void irc_check_command(Threads *thread,char *receive_in){
	char *param  [ARRAY_SIZE];
	char receive [BUFSIZE];
	
	size_t paramvalue = 0;		
														
	if(m->debug>=1){
		dlog(thread,1,"> %s\n", receive_in);
		
	}else if(!m->debug && dofork){
		alog(thread,"> %s\n", receive_in);
	}

	/* receive
		includes the Information given from the socket to this void.
		Example: ":segfault.fl.us.deepirc.net SQLINE O :nick used by orion"
		without "".
	*/
   	//dlog(thread,3,"irc_check_command() - receive_in = %s\n",receive_in);
	bzero(&receive,sizeof(receive));
	bzero(param,sizeof(param));
	
	dynncpy              (receive,receive_in,sizeof(receive));			/* Function of string.h */
	paramvalue = strsplit(receive,param,' ');		/* splitdata (splitdata.c line 79) */
	/* BITS
		bits contains the same information as param
		but without ':' and the string is saved
		as a lowercase string, all charakters are used
		with lower case.
	*/
	/** PERFORM OUR MAIN PROTOCOL **/
	if(thread->Webform)
		on_protocol(thread,&WebForm,paramvalue,param);
	else
		on_protocol(thread,&IRC,paramvalue,param);

	/* clear Quotes array */
	mysql_free_quotes(thread);
	/**********************/
	
    /* Finally, clear the first byte of `inbuf' to signal that we're
     * finished processing. */
    //*inbuf = 0;
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief web_check_command:  Called when Information was read from the WebSocket (FIRST STEP BY PROCESSING INCOMING DATA!)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param receive_in The Information read from the Socket (this Information is going to be split to the ParameterArray here)
 * @return void
 **/
void web_check_command(Threads *thread, char *receive_in){
	char *param  [ARRAY_SIZE];
	char receive [BUFSIZE];
	
	size_t paramvalue = 0;		
														
	if(m->debug>=1){
		dlog(thread,1,"W> %s\n", receive_in);
		
	}else if(!m->debug && dofork){
		alog(thread,"W> %s\n", receive_in);
	}

	/* Set our output to nothing, make us ready for reply */
	bzero(&thread->webformOutput,sizeof(thread->webformOutput));
	thread->webSuccess = false;
	
	/* receive
		includes the Information given from the socket to this void.
		Example: ":segfault.fl.us.deepirc.net SQLINE O :nick used by orion"
		without "".
	*/
   	//dlog(thread,3,"web_check_command() - receive_in = %s\n",receive_in);
	bzero(&receive,sizeof(receive));
	bzero(param,sizeof(param));
	
	dynncpy              (receive,receive_in,sizeof(receive));			/* Function of string.h */
	paramvalue = strsplit(receive,param,' ');		/* splitdata (splitdata.c line 79) */
	/* BITS
		bits contains the same information as param
		but without ':' and the string is saved
		as a lowercase string, all charakters are used
		with lower case.
	*/
	
	/*if(paramvalue>0 && param[0][0]=='!' && param[0][1]==':'){
		main_raw(IRCThread,&receive_in[1]);
	}else{*/
		/** PERFORM OUR MAIN PROTOCOL **/
		if(thread->Webform)
			on_protocol(thread,&WebForm,paramvalue,param);
		else
			on_protocol(thread,&IRC,paramvalue,param);

		/* clear Quotes array */
		mysql_free_quotes(thread);
		/**********************/
	//}
	
    /* Finally, clear the first byte of `inbuf' to signal that we're
     * finished processing. */
    //*inbuf = 0;
    
    buildAndSendWebformMessage(thread);
	return;
}

/*  ************************************************************************************************  */
