/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file stats.c
 *   @brief This is the Module file for STATS (Server Side) Command.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the STATS command.
 *
 *   The STATS File is used to process STATS requests sent from Servers
 *   and to Servers. (Protocol-Process-File -> PPF)
 *   (Stats is used to send status information depending on the irc-protocol /stats u)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "osocket.h"

/**
 * @brief on_stats:  Receives a STATS Request (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param user The Nick which requested the STATS Information
 * @param what The kind of STATS Report the User requested
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_stats(Threads *thread, char *who, char *user, char *what){
	if(streqnc("u", what)){
		time_t t      = time(NULL);
		time_t uptime = t-m->starttime;
		
		main_raw(thread,":%s 242 %s :Services up: %d day%s, %02d:%02d:%02d", who, user, uptime / 86400,(uptime / 86400 == 1) ? "" : "s", (uptime / 3600) % 24, (uptime / 60) % 60, uptime % 60);
		main_raw(thread,":%s 219 %s u End of /STATS report", who, user);
		return CMD_EXECUTED;
	}
	return CMD_ERROR;
}

/*  ************************************************************************************************  */
