/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file webform.c
 *   @brief This is the Module file for communication with the WebForm.
 *
 *   It contains all functions which are neccessary for
 *   the Communication with the Webform. (Translates it to Web and IRC)
 *
 *   The WEBFORM File is used to communicate with the PHP-Webform.
 */
/*  ************************************************************************************************  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <fcntl.h>

#include <sys/types.h>
#include <sys/stat.h>

#ifndef _WIN32
  #include <unistd.h>
  #include <sys/socket.h>
  #include <netinet/in.h>
  #include <arpa/inet.h>
#endif

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "osocket.h"
#include "webform.h"

static Socket *WebSockets = NULL;

#ifdef USE_THREADS
/* master file descriptor list */
fd_set master;

/* temp file descriptor list for select() */
fd_set read_fds;

/* server address */
struct sockaddr_in serveraddr;

/* client address */
struct sockaddr_in clientaddr;
#endif

void delWebSocket(Socket *s){
	Socket *ptr = NULL;

	for (ptr = WebSockets; ptr; ptr = ptr->next) {
		if (ptr == s)
			break;
	}
	if (!ptr)
		return;
	if (s->prev)
		s->prev->next = s->next;
	else
		WebSockets = s->next;
	if (s->next)
		s->next->prev = s->prev;
	free(s);
	return;
}

Socket *addWebSocket(orion_socket_t Sock){
	Socket *s = (Socket*)malloc(sizeof(Socket));
	bzero(s,sizeof(Socket));

	s->Sock			= Sock;
	s->Identified	= 0;
	bzero(&s->Ip,sizeof(s->Ip));

	s->next = WebSockets;

	s->prev = NULL;
	if (WebSockets)
		WebSockets->prev = s;
	WebSockets = s;
	return s;
}

Socket *FindWebSocket(orion_socket_t Sock){
	Socket *s = NULL;
	for (s = WebSockets; s; s = s->next){
		if(s->Sock == Sock)
			return s;
	}
	return NULL;
}

void addWebSockets(int fdmax){
	int i = 0;
	for(i = 0; i < fdmax; i++)
		addWebSocket(0);

	return;
}

void delWebSockets(void){
	Socket *s = NULL;
	for (s = WebSockets; s; s = s->next)
		delWebSocket(s);

	return;
}

void WebSocketClose(Threads *thread, Socket *s){
	thread->Sock = 0;
	/* close it... */
	close(s->Sock);
	/* remove from master set */
	FD_CLR(s->Sock, &master);

	dlog(thread,0," [*] WebForm - Socket %d (%s) closed connection\n", s->Sock, inet_ntoa(s->Ip));
	delWebSocket(s);
	return;
}

bool WebCheckIp(Socket *s){
	if(streqnc(inet_ntoa(s->Ip),m->WebFormIp))
		return true;

	return false;
}

void setWebSuccess(Threads *thread){
	if(thread->Webform)
		thread->webSuccess = 1;
}

#ifdef USE_THREADS
void *WebSocket (void *in_thread) {
	Threads *thread = (Threads *)in_thread;
	
	/* maximum file descriptor number */
	orion_socket_t fdmax;

	/* for setsockopt() SO_REUSEADDR, below */
	int    			yes			= 1;
	int    			web_read	= 0;
	orion_socket_t	save_fd		= 0;
	orion_socket_t  i			= 0;
	unsigned int addrsize;
	/****************************************/

	pthread_setcancelstate (PTHREAD_CANCEL_ENABLE, NULL);
	pthread_cleanup_push   (EndThread,(void *)thread);

	/* Connect the Thread to SQL */
	//mysql_connect(thread);
	my_pthread_mutex_lock(&mysql_lock);
	mysql_thread_init(); /* Initialize MySQL for Threading */
	my_pthread_mutex_unlock(&mysql_lock);

	/* Open our logfiles for the Thread */
	open_logs(thread);

	/* Indicate that it's the WebForm application */
	thread->Webform = true;

	/* clear the master and temp sets */
	FD_ZERO(&master);
	FD_ZERO(&read_fds);

	/* get the listener */
	if((thread->MSock = socket(AF_INET, SOCK_STREAM, 0)) == -1){
		dlog(thread,0,"  *  ERROR: WebForm - Server-socket() failed to establish connection\n");
		return NULL;
	}

	/* "address already in use" error message */
	if(setsockopt(thread->MSock, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int)) == -1){
		dlog(thread,0,"  *  ERROR: WebForm - Server-setsockopt()\n");
		return NULL;
	}
	dlog(thread,0," [*] WebForm - Server-setsockopt() is OK...\n");

	/* bind */
	serveraddr.sin_family = AF_INET;
	serveraddr.sin_addr.s_addr = INADDR_ANY;
	serveraddr.sin_port = htons(m->WebPort);
	memset(&(serveraddr.sin_zero), '\0', 8);

	if(bind(thread->MSock, (struct sockaddr *)&serveraddr, sizeof(serveraddr)) == -1){
		return NULL;
	}
	dlog(thread,0," [*] WebForm - Server-bind() is OK...\n");

	/* listen */
	if(listen(thread->MSock, 10) == -1){
		dlog(thread,0,"  *  ERROR: WebForm - Server-listen() error!\n");
		return NULL;
	}
	dlog(thread,0," [*] WebForm - Server-listen() is OK...\n");

#ifndef _WIN32
	save_fd = fcntl( thread->MSock, F_GETFL );
	save_fd |= O_NONBLOCK;
	fcntl( thread->MSock, F_SETFL, save_fd );
#else
	thread->MSock =  orion_socksetnonb(save_fd);
#endif

	dlog(thread,0," [*] WebForm - Server NONBLOCKING is OK...\n");

	/* add the listener to the master set */
	FD_SET(thread->MSock, &master);

	/* keep track of the biggest file descriptor */
	fdmax = thread->MSock; /* so far, it's this one*/

	/* loop */
	for(;;){
		/* copy it */
		read_fds = master;

		if(select(fdmax+1, &read_fds, NULL, NULL, NULL) == -1){
			dlog(thread,0,"  *  ERROR: WebForm - for select()\n");
			exit(1);
		}
		dlog(thread,0," [*] WebForm - Server-select() is OK...\n");

 		/*run through the existing connections looking for data to be read*/
		for(i = 0; i <= fdmax; i++){
			if(FD_ISSET(i, &read_fds)){ /* we got one... */

				thread->Sock = i;
				Socket *s = FindWebSocket(i);
				if(s == NULL)
					s = addWebSocket(i);

				if(s->Sock == thread->MSock){
					/* handle new connections */
					addrsize = (unsigned int)sizeof(clientaddr);
					if((s->Sock = accept(thread->MSock, (struct sockaddr *)&clientaddr, &addrsize)) == -1){
						dlog(thread,0,"  *  ERROR: WebForm - Server-accept() error\n");
					}
					else{
						//printf("Server-accept() is OK...\n");
						dlog(thread,0," [*] WebForm - Server-accept() is OK...\n");
						FD_SET(s->Sock, &master); /* add to master set */
						if(s->Sock > fdmax){ /* keep track of the maximum */
							fdmax = s->Sock;
						}

#ifndef _WIN32
						save_fd = fcntl( s->Sock, F_GETFL );
						save_fd |= O_NONBLOCK;
						fcntl( s->Sock, F_SETFL, save_fd );
#else
						s->Sock =  orion_socksetnonb(save_fd);
#endif

						s->Ip = clientaddr.sin_addr;
						dlog(thread,0," [*] WebForm - New connection from %s on socket %d\n",inet_ntoa(s->Ip), s->Sock);

						if(WebCheckIp(s)==false)
							WebSocketClose(thread,s);
						/* Check if Ip was blacklisted (not existing atm) */
						//else if(WebFormBlackList(s->Ip))
						//	WebSocketClose(WebFormSocketThread,s);
					}
				}

				else{
					/* handle data from a client */
					web_read = (int) (long) sgets2(thread->inbuf, sizeof(thread->inbuf), thread);
					printf("inbuf: %s length: %d\n",thread->inbuf, (int)dynlen(thread->inbuf));
					if((web_read > 0) || (web_read < (-1))){	/* Read from Socket */
						/* we got some data from a client - we send it back */
			        	bzero(thread->buf,sizeof(thread->buf));
			        	dynncpy(thread->buf,thread->inbuf,513);
			        	
			        	/* Senden an interpreter */
			        	irc_check_command(thread,thread->buf);
					}
					else{
						/* got error or connection closed by client */
						printf("joined web_read <= 0: %d\n", web_read);
						if(web_read == 0){
							/* connection closed */
							dlog(thread,0," [*] WebForm - Socket %d hung up\n", thread->Sock);
						}
						else
							dlog(thread,0,"  *  ERROR: WebForm - Receive error\n");

						WebSocketClose(thread,s);
					}
				}
			}
		}
	}
	/* End Thread */
	mysql_thread_end();
	pthread_cleanup_pop(0);
	pthread_exit(NULL);
	return NULL;
}
#endif

PROTOCOL_BOOL orion_WRCtranslate_cmd_initialize(Threads *thread){
	/*if(thread->Webform)
		WebForm.orion_cmd_initialize(thread);*/
	return IRC.orion_cmd_initialize(IRCThread);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_add_account(Threads *thread, char *uid, char *authnick){
	/*if(thread->Webform)
		WebForm.orion_cmd_add_account(thread,uid,authnick);*/
	return IRC.orion_cmd_add_account(IRCThread,uid,authnick);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_del_account(Threads *thread, char *uid){
	/*if(thread->Webform)
		WebForm.orion_cmd_del_account(thread,uid);*/
	return IRC.orion_cmd_del_account(IRCThread,uid);
}

PROTOCOL_BOOL orion_WRCtranslate_cmd_topic(Threads *thread, char *whosets, char *chan, char *whosetit, char *topic, time_t when){
	/*if(thread->Webform)
		WebForm.orion_cmd_topic(thread,whosets,chan,whosetit,topic,when);*/
	return IRC.orion_cmd_topic(IRCThread,whosets,chan,whosetit,topic,when);
}

PROTOCOL_BOOL orion_WRCtranslate_cmd_gline(Threads *thread, char *user, char *host, char *who, time_t when, time_t expires, char *reason){
	/*if(thread->Webform)
		WebForm.orion_cmd_gline(thread,user,host,who,when,expires,reason);*/
	return IRC.orion_cmd_gline(IRCThread,user,host,who,when,expires,reason);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_ungline(Threads *thread, char *unsetter, char *host, char *user, time_t time_in){
	/*if(thread->Webform)
		WebForm.orion_cmd_ungline(thread,unsetter,host,user,time_in);*/
	return IRC.orion_cmd_ungline(IRCThread,unsetter,host,user,time_in);
}

PROTOCOL_BOOL orion_WRCtranslate_cmd_svskill(Threads *thread, char *source, char *user, char *buf){
	/*if(thread->Webform)
		WebForm.orion_cmd_svskill(thread,source,user,buf);*/
	return IRC.orion_cmd_svskill(IRCThread,source,user,buf);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_kill(Threads *thread, char *source, char *user, char *buf){
	/*if(thread->Webform)
		WebForm.orion_cmd_kill(thread,source,user,buf);*/
	return IRC.orion_cmd_kill(IRCThread,source,user,buf);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_rawsvsmode(Threads *thread, char *source, char *nick, char *modes){
	/*if(thread->Webform)
		WebForm.orion_cmd_rawsvsmode(thread,source,nick,modes);*/
	return IRC.orion_cmd_rawsvsmode(IRCThread,source,nick,modes);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_svsmode(Threads *thread, char *nick, char *target, char *modes){
	/*if(thread->Webform)
		WebForm.orion_cmd_svsmode(thread,nick,target,modes);*/
	return IRC.orion_cmd_svsmode(IRCThread,nick,target,modes);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_svs2mode(Threads *thread, char *nick, char *target, char *modes){
	/*if(thread->Webform)
		WebForm.orion_cmd_svs2mode(thread,nick,target,modes);*/
	return IRC.orion_cmd_svs2mode(IRCThread,nick,target,modes);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_nick(Threads *thread, char *nick, char *ident, char *host, char *name, char *modes){
	/*if(thread->Webform)
		WebForm.orion_cmd_nick(thread,nick,ident,host,name,modes);*/
	return IRC.orion_cmd_nick(IRCThread,nick,ident,host,name,modes);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_mode(Threads *thread, char *source, char *dest, time_t time_in, char *buf){
	/*if(thread->Webform)
		WebForm.orion_cmd_mode(thread,source,dest,time_in,buf);*/
	return IRC.orion_cmd_mode(IRCThread,source,dest,time_in,buf);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_rawmode(Threads *thread, char *source, char *dest, time_t time_in, char *buf){
	/*if(thread->Webform)
		WebForm.orion_cmd_rawmode(thread,source,dest,time_in,buf);*/
	return IRC.orion_cmd_rawmode(IRCThread,source,dest,time_in,buf);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_kick(Threads *thread, char *source, char *chan, char *user, char *buf){
	/*if(thread->Webform)
		WebForm.orion_cmd_kick(thread,source,chan,user,buf);*/
	return IRC.orion_cmd_kick(IRCThread,source,chan,user,buf);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_notice(Threads *thread, char *source, char *dest, char *buf){
	/*if(thread->Webform)
		WebForm.orion_cmd_notice(thread,source,dest,buf);*/
	return IRC.orion_cmd_notice(IRCThread,source,dest,buf);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_privmsg(Threads *thread, char *source, char *dest, char *buf){
	/*if(thread->Webform)
		WebForm.orion_cmd_privmsg(thread,source,dest,buf);*/
	return IRC.orion_cmd_privmsg(IRCThread,source,dest,buf);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_serv_notice(Threads *thread, char *source, char *dest, char *msg){
	/*if(thread->Webform)
		WebForm.orion_cmd_serv_notice(thread,source,dest,msg);*/
	return IRC.orion_cmd_serv_notice(IRCThread,source,dest,msg);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_serv_privmsg(Threads *thread, char *source, char *dest, char *msg){
	/*if(thread->Webform)
		WebForm.orion_cmd_serv_privmsg(thread,source,dest,msg);*/
	return IRC.orion_cmd_serv_privmsg(IRCThread,source,dest,msg);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_quit(Threads *thread, char *source, char *buf){
	/*if(thread->Webform)
		WebForm.orion_cmd_quit(thread,source,buf);*/
	return IRC.orion_cmd_quit(IRCThread,source,buf);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_protoctl(Threads *thread){
	/*if(thread->Webform)
		WebForm.orion_cmd_protoctl(thread);*/
	return IRC.orion_cmd_protoctl(IRCThread);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_pass(Threads *thread, char *pass){
	/*if(thread->Webform)
		WebForm.orion_cmd_pass(thread,pass);*/
	return IRC.orion_cmd_pass(IRCThread,pass);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_server(Threads *thread, char *servname, int hop, char *descript){
	/*if(thread->Webform)
		WebForm.orion_cmd_server(thread,servname,hop,descript);*/
	return IRC.orion_cmd_server(IRCThread,servname,hop,descript);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_pong(Threads *thread, char *servname, char *who){
	/*if(thread->Webform)
		WebForm.orion_cmd_pong(thread,servname,who);*/
	return IRC.orion_cmd_pong(IRCThread,servname,who);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_ping(Threads *thread, char *servname, char *who){
	/*if(thread->Webform)
		WebForm.orion_cmd_ping(thread,servname,who);*/
	return IRC.orion_cmd_ping(IRCThread,servname,who);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_join(Threads *thread, char *user, char *channel, time_t chantime){
	/*if(thread->Webform)
		WebForm.orion_cmd_join(thread,user,channel,chantime);*/
	return IRC.orion_cmd_join(IRCThread,user,channel,chantime);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_services_join(Threads *thread, char *user, char *channel, time_t chantime, int fake){
	/*if(thread->Webform)
		WebForm.orion_cmd_services_join(thread,user,channel,chantime,fake);*/
	return IRC.orion_cmd_services_join(IRCThread,user,channel,chantime,fake);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_chghost(Threads *thread, char *source, char *nick, char *vhost){
	/*if(thread->Webform)
		WebForm.orion_cmd_chghost(thread,source,nick,vhost);*/
	return IRC.orion_cmd_chghost(IRCThread,source,nick,vhost);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_chgident(Threads *thread, char *source, char *nick, char *vIdent){
	/*if(thread->Webform)
		WebForm.orion_cmd_chgident(thread,source,nick,vIdent);*/
	return IRC.orion_cmd_chgident(IRCThread,source,nick,vIdent);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_invite(Threads *thread, char *source, char *chan, char *nick){
	/*if(thread->Webform)
		WebForm.orion_cmd_invite(thread,source,chan,nick);*/
	return IRC.orion_cmd_invite(IRCThread,source,chan,nick);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_part(Threads *thread, char *nick, char *chan, char *buf){
	/*if(thread->Webform)
		WebForm.orion_cmd_part(thread,nick,chan,buf);*/
	return IRC.orion_cmd_part(IRCThread,nick,chan,buf);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_wallops(Threads *thread, char *source, char *fmt){
	/*if(thread->Webform)
		WebForm.orion_cmd_wallops(thread,source,fmt);*/
	return IRC.orion_cmd_wallops(IRCThread,source,fmt);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_global(Threads *thread, char *source, char *buf){
	/*if(thread->Webform)
		WebForm.orion_cmd_global(thread,source,buf);*/
	return IRC.orion_cmd_global(IRCThread,source,buf);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_sqline(Threads *thread, char *sender, char *mask, time_t time_in, time_t expire, char *reason){
	/*if(thread->Webform)
		WebForm.orion_cmd_sqline(thread,sender,mask,time_in,expire,reason);*/
	return IRC.orion_cmd_sqline(IRCThread,sender,mask,time_in,expire,reason);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_unsqline(Threads *thread, char *servername, char *nick, char *snick){
	/*if(thread->Webform)
		WebForm.orion_cmd_unsqline(thread,servername,nick,snick);*/
	return IRC.orion_cmd_unsqline(IRCThread,servername,nick,snick);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_squit(Threads *thread, char *servname, char *message){
	/*if(thread->Webform)
		WebForm.orion_cmd_squit(thread,servname,message);*/
	return IRC.orion_cmd_squit(IRCThread,servname,message);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_svso(Threads *thread, char *source, char *nick, char *flag){
	/*if(thread->Webform)
		WebForm.orion_cmd_svso(thread,source,nick,flag);*/
	return IRC.orion_cmd_svso(IRCThread,source,nick,flag);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_chg_nick(Threads *thread, char *oldnick, char *newnick){
	/*if(thread->Webform)
		WebForm.orion_cmd_chg_nick(thread,oldnick,newnick);*/
	return IRC.orion_cmd_chg_nick(IRCThread,oldnick,newnick);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_svsnick(Threads *thread, char *source, char *oldnick, char *newnick, time_t when){
	/*if(thread->Webform)
		WebForm.orion_cmd_svsnick(thread,source,oldnick,newnick,when);*/
	return IRC.orion_cmd_svsnick(IRCThread,source,oldnick,newnick,when);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_connect(Threads *thread, bool how){
	/*if(thread->Webform)
		WebForm.orion_cmd_connect(thread,how);*/
	return IRC.orion_cmd_connect(IRCThread,how);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_svshold(Threads *thread, char *nick){
	/*if(thread->Webform)
		WebForm.orion_cmd_svshold(thread,nick);*/
	return IRC.orion_cmd_svshold(IRCThread,nick);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_release_svshold(Threads *thread, char *nick){
	/*if(thread->Webform)
		WebForm.orion_cmd_release_svshold(thread,nick);*/
	return IRC.orion_cmd_release_svshold(IRCThread,nick);
}

PROTOCOL_BOOL orion_WRCtranslate_cmd_szline(Threads *thread, char *servername, char *mask, char *snick, time_t when, time_t expire, char *reason){
	/*if(thread->Webform)
		WebForm.orion_cmd_szline(thread,servername,mask,snick,when,expire,reason);*/
	return IRC.orion_cmd_szline(IRCThread,servername,mask,snick,when,expire,reason);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_unszline(Threads *thread, char *servername, char *mask, char *snick, time_t in_time){
	/*if(thread->Webform)
		WebForm.orion_cmd_unszline(thread,servername,mask,snick,in_time);*/
	return IRC.orion_cmd_unszline(IRCThread,servername,mask,snick,in_time);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_unban(Threads *thread, char *name, char *nick){
	/*if(thread->Webform)
		WebForm.orion_cmd_unban(thread,name,nick);*/
	return IRC.orion_cmd_unban(IRCThread,name,nick);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_svsmode_chan(Threads *thread, char *name, char *mode, char *nick){
	/*if(thread->Webform)
		WebForm.orion_cmd_svsmode_chan(thread,name,mode,nick);*/
	return IRC.orion_cmd_svsmode_chan(IRCThread,name,mode,nick);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_svsjoin(Threads *thread, char *source, char *nick, char *chan, time_t date){
	/*if(thread->Webform)
		WebForm.orion_cmd_svsjoin(thread,source,nick,chan,date);*/
	return IRC.orion_cmd_svsjoin(IRCThread,source,nick,chan,date);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_svspart(Threads *thread, char *source, char *nick, char *chan){
	/*if(thread->Webform)
		WebForm.orion_cmd_svspart(thread,source,nick,chan);*/
	return IRC.orion_cmd_svspart(IRCThread,source,nick,chan);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_sapart(Threads *thread, char *source, char *nick, char *chan){
	/*if(thread->Webform)
		WebForm.orion_cmd_sapart(thread,source,nick,chan);*/
	return IRC.orion_cmd_sapart(IRCThread,source,nick,chan);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_sajoin(Threads *thread, char *source, char *nick, char *chan, time_t date){
	/*if(thread->Webform)
		WebForm.orion_cmd_sajoin(thread,source,nick,chan,date);*/
	return IRC.orion_cmd_sajoin(IRCThread,source,nick,chan,date);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_swhois(Threads *thread, char *source, char *who, char *mask){
	/*if(thread->Webform)
		WebForm.orion_cmd_swhois(thread,source,who,mask);*/
	return IRC.orion_cmd_swhois(IRCThread,source,who,mask);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_eob(Threads *thread){
	/*if(thread->Webform)
		WebForm.orion_cmd_eob(thread);*/
	return IRC.orion_cmd_eob(IRCThread);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_jupe(Threads *thread, char *jserver, char *who, char *reason){
	/*if(thread->Webform)
		WebForm.orion_cmd_jupe(thread,jserver,who,reason);*/
	return IRC.orion_cmd_jupe(IRCThread,jserver,who,reason);
}
PROTOCOL_BOOL orion_WRCtranslate_cmd_global_legacy(Threads *thread, char *source, char *fmt){
	/*if(thread->Webform)
		WebForm.orion_cmd_global_legacy(thread,source,fmt);*/
	return IRC.orion_cmd_global_legacy(IRCThread,source,fmt);
}

PROTOCOL_BOOL orion_WRCtranslate_cmd_raw(Threads *thread, char *param[]){
	/*if(thread->Webform)
		WebForm.orion_cmd_raw(thread,param);*/
	return IRC.orion_cmd_raw(IRCThread,param);
}
bool orion_WRCtranslate_valid_nick(Threads *thread, char *nick){
	/*if(thread->Webform)
		WebForm.orion_valid_nick(thread,nick);*/
	return IRC.orion_valid_nick(IRCThread,nick);
}
bool orion_WRCtranslate_valid_chan(Threads *thread, char *chan){
	/*if(thread->Webform)
		WebForm.orion_valid_chan(thread,chan);*/
	return IRC.orion_valid_chan(IRCThread,chan);
}
bool orion_WRCtranslate_flood_mode_check(Threads *thread, char *value){
	/*if(thread->Webform)
		WebForm.orion_flood_mode_check(thread,value);*/
	return IRC.orion_flood_mode_check(IRCThread,value);
}
